<?php
class DomainFilter extends sfFilter
{
  public function execute($filterChain)
  {
    if (!$this->isFirstCall())
    {
      $filterChain->execute();
    }

    $domain = $this->getContext()->getUser()->getUserDomain();
    $response = $this->getContext()->getResponse();

    $baseUrl = $this->getContext()->getController()->genUrl('@homepage', true);
    $url = str_replace($baseUrl, '', $this->getContext()->getRequest()->getUri());

    // try by url
    $meta = Doctrine::getTable('Meta')->findOneByUrl($domain, '/'.$url);

    // try by module/action
    if (!$meta)
    {
      $meta = Doctrine::getTable('Meta')->findOneByModuleAction($domain, $this->getContext()->getModuleName(), $this->getContext()->getActionName());
    }

    if ($meta)
    {
      $response->setTitle($meta->get('title'));
      $response->addMeta('description', $meta->get('description'));
      $response->addMeta('og:title', $meta->get('og_title'));
      $response->addMeta('og:image', $meta->get('og_image'));
      $response->addMeta('og:site_name', $meta->get('og_sitename'));
      $response->addMeta('og:description', $meta->get('og_description'));
    }
    else
    {
      $new = new DefaultMeta();

      $meta = $new->setDefaultMeta();
      $response = $this->getContext()->getResponse();

      foreach ($meta as $name => $value)
      {
        $response->addMeta($name, $value);
      }
    }

    $filterChain->execute();
  }
}