<?php


class ApiQueries
{
	const LISTING_QUERY = "SELECT 
	    date_format(l.created_at, '%Y-%m') as period,
	    \"Listings\" as \"booking_status\",
	    count(*) as \"[value]\"
	FROM 
	    listing l
	    INNER JOIN locations lp on l.pickup_location_id=lp.id
	    INNER JOIN locations ld on l.dropoff_location_id=ld.id
	    INNER JOIN location_category lc on lp.category_id=lc.id
	    INNER JOIN user ro on l.owner_id=ro.id
	WHERE lc.country_id=:country
	AND l.owner_id = :id
	and l.created_at >= :from_date 
	and DATE_FORMAT(l.created_at, '%Y-%m-%d') <= :to_date
	GROUP BY period
	ORDER BY period
	";

	const BOOKING_QUERY = "SELECT 
	    date_format(r.created_at, '%Y-%m') as period,
	    (CASE  
	    WHEN r.status=1 or (r.status=4 and r.no_show_fee = 0) THEN \"Bookings\"
	    ELSE \"Missed\"
	    END
	    
	    
	    ) as \"booking_status\",
	    count(*) as \"[value]\",
	    avg(l.price) as price
	FROM
	    request r
	    INNER JOIN listing l on l.id=r.listing_id
	    INNER JOIN locations lp on l.pickup_location_id=lp.id
	    INNER JOIN locations ld on l.dropoff_location_id=ld.id
	    INNER JOIN location_category lc on lp.category_id=lc.id
	    INNER JOIN user ro on l.owner_id=ro.id
	WHERE lc.country_id=:country
	AND l.owner_id = :id
	and r.created_at >= :from_date and  DATE_FORMAT(r.created_at, '%Y-%m-%d') <= :to_date
	GROUP BY period, booking_status
	ORDER BY period";


	const ROUTES_LISTINGS_QUERY = "SELECT lp.name as pickup, 
		ld.name as dropoff, 
		count(distinct l.id) as nb_listings
	FROM listing l
		INNER JOIN locations lp on l.pickup_location_id=lp.id
		INNER JOIN locations ld on l.dropoff_location_id=ld.id
		INNER JOIN location_category lc on lp.category_id=lc.id
	WHERE lc.country_id=:country
		AND l.owner_id = :id 
		AND l.created_at >= :from_date and  DATE_FORMAT(l.created_at, '%Y-%m-%d') <= :to_date
	GROUP BY pickup, dropoff
	ORDER BY nb_listings";

	const ROUTES_REQUESTS_QUERY = "SELECT lp.name as pickup, 
		ld.name as dropoff, 
		count(distinct r.id) nb_requests
	FROM listing l
		INNER JOIN locations lp on l.pickup_location_id=lp.id
		INNER JOIN locations ld on l.dropoff_location_id=ld.id
		INNER JOIN location_category lc on lp.category_id=lc.id
		INNER JOIN request r on l.id=r.listing_id
	WHERE lc.country_id=:country
		AND l.owner_id = :id 
		AND r.created_at >= :from_date and  DATE_FORMAT(r.created_at, '%Y-%m-%d') <= :to_date
	GROUP BY pickup, dropoff
	ORDER BY nb_listings";

	const ROUTES_BOOKINGS_QUERY = "SELECT lp.name as pickup, 
		ld.name as dropoff,
		count(distinct b.id) nb_bookings
	FROM listing l
		INNER JOIN locations lp on l.pickup_location_id=lp.id
		INNER JOIN locations ld on l.dropoff_location_id=ld.id
		INNER JOIN location_category lc on lp.category_id=lc.id
		INNER JOIN request b on (b.status in(1,4) and l.accepted_request_id=b.id)
	WHERE lc.country_id=:country
		AND l.owner_id = :id 
		AND b.when_accepted >= :from_date and  DATE_FORMAT(b.when_accepted, '%Y-%m-%d') <= :to_date
	GROUP BY pickup, dropoff
	ORDER BY nb_listings";


	const COMPANY_QUERY = "SELECT company FROM user WHERE id = :id LIMIT 1";
}

