<?php

ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
class ROApi
{
	private $conn;
    private $country_id;
	public function __construct($conn, $country_id)
	{
		$this->conn = $conn;
        $this->country_id = $country_id;
	}

	public function getListings($roId, $from_date = null, $to_date = null, $from_loc = null, $to_loc = null)
    {
        return $this->getResults(ApiQueries::LISTING_QUERY, $roId, $from_date, $to_date, $from_loc, $to_loc);
    }

    public function getBookings($roId, $from_date = null, $to_date = null, $from_loc = null, $to_loc = null)
    {
        return $this->getResults(ApiQueries::BOOKING_QUERY, $roId, $from_date, $to_date, $from_loc, $to_loc);
    }

    public function getRoutes($roId, $from_date = null, $to_date = null, $from_loc = null, $to_loc = null)
    {
        $callback = function($row) { return $row['pickup'].'-'.$row['dropoff']; };
        $listings = $this->map(
            $this->getResults(
                ApiQueries::ROUTES_LISTINGS_QUERY, 
                $roId, 
                $from_date, 
                $to_date, 
                $from_loc, 
                $to_loc), 
            $callback);

        $requests = $this->map(
            $this->getResults(
                ApiQueries::ROUTES_REQUESTS_QUERY,
                $roId,
                $from_date,
                $to_date,
                $from_loc,
                $to_loc), 
            $callback);

        $bookings = $this->map(
            $this->getResults(
                ApiQueries::ROUTES_BOOKINGS_QUERY, 
                $roId, 
                $from_date, 
                $to_date, 
                $from_loc, 
                $to_loc), 
            $callback);

        $routes = array_map(function($row) {
            return array_merge(['nb_listings' => 0,'nb_requests' => 0,'nb_bookings' => 0], $row);
        }, $this->merge($listings, $requests, $bookings));

        usort($routes, function($rowA, $rowB) {
            if ((int) $rowA['nb_listings'] === (int)$rowB['nb_listings']) {
                return 0;
            }

            return (int)$rowA['nb_listings'] < (int)$rowB['nb_listings'] ? -1 : 1;
        });

        return $routes;

    }

    private function map($data, $callback)
    {
        $return = [];
        foreach ($data as $row) {
            $return[$callback($row)] = $row;
        }

        return $return;
    }

    private function merge()
    {
        $arrays = func_get_args();
        $retval = [];
        foreach($arrays as $array) {
            foreach($array as $key => $row) {
                if (!isset($retval[$key])) {
                    $retval[$key] = [];
                }

                $retval[$key] = array_merge($retval[$key], $row);
            }
        }

        return array_values($retval);
    }

    public function getNPS($roId, $from_date = null, $to_date = null)
    {
        $stmt = $this->conn->prepare(ApiQueries::COMPANY_QUERY);
        $stmt->execute([
            'id' => $roId
        ]);
        $result = $stmt->fetch();

        $company = $result['company'];
        
        ini_set('memory_limit', -1);
        $json = json_decode(file_get_contents(__DIR__.'/nps.json'), true);
        $from_date = date('Y-m-d', strtotime('1 year ago'));
        $to_date = date('Y-m-d');

        $results = array_filter($json['data'], function($var) use ($company) {
            return $var['operator_c'] == $company;
        });

        $from_date_ts = $from_date ? strtotime($from_date) : null;
        $to_date_ts = $to_date ? strtotime($to_date) : null;

        $results = array_filter($results, function($var) use ($from_date_ts, $to_date_ts) {
            $keep = true;

            if ($from_date_ts && $from_date_ts > $var['responded']) {
                $keep = false;
            }

            if ($keep && $to_date_ts && $to_date_ts < $var['responded']) {
                $keep = false;
            }
            $score = (int) $var['answer'];
            if ($keep && ($score == -1)){
              $keep = false;
            } 

            return $keep;
        });

        $sumPromoter = array_reduce($results, function($carry, $item){
            if ((int) $item['answer'] > 8) {
                $carry++;
            }

            return $carry;
        }, 0);

        $sumDetractor = array_reduce($results, function($carry, $item){
            if ((int) $item['answer'] < 7) {
                $carry++;
            }

            return $carry;
        }, 0);

        $sumPassive = array_reduce($results, function($carry, $item){
            $score = (int) $item['answer'];
            if ($score == 7 || $score == 8) {
                $carry++;
            }

            return $carry;
        }, 0);

        $total = $sumPromoter + $sumDetractor+$sumPassive;
        $pctPromoter = $sumPromoter/$total;
        $pctDetractor = $sumDetractor/$total;

        $nps = round(($pctPromoter - $pctDetractor)*100, 1);

	return empty($results) ? 0 : $nps;
    }


    private function getResults($query, $roId, $from_date, $to_date, $from_loc, $to_loc)
    {
            $params = [
                    'country' => $this->country_id,
                    'id' => $roId,
                    'from_date' => $from_date ?: date('Y-m-d', strtotime('-1 year')),
                    'to_date' => $to_date ?: date('Y-m-d')
            ];

            if ($from_loc && strpos($from_loc, 'tc_') === 0) {
                $query = str_replace('WHERE ', 'WHERE lp.id = :pickup AND ', $query);
                $params['pickup'] = explode('_', $from_loc)[1];
            } elseif ($from_loc && strpos($from_loc, 'owner_') === 0) {
                $query = str_replace('WHERE ', ' INNER JOIN owner_branch pb ON l.owner_pickup_branch_id=pb.id WHERE pb.id = :pickup AND ', $query);
                $params['pickup'] = explode('_', $from_loc)[1];
            }
            if ($to_loc && strpos($to_loc, 'tc_') === 0) {
                $query = str_replace('WHERE ', 'WHERE ld.id = :dropoff AND ', $query);
                $params['dropoff'] = explode('_', $to_loc)[1];
            }elseif ($to_loc && strpos($to_loc, 'owner_') === 0) {
                $query = str_replace('WHERE ', ' INNER JOIN owner_branch pd ON l.owner_dropoff_branch_id=pb.id WHERE pb.id = :dropoff AND ', $query);
                $params['dropoff'] = explode('_', $to_loc)[1];
            }

            $stmt = $this->conn->prepare($query);
// print_r($query);print_r($params);
            $stmt->execute($params);

            return $stmt->fetchAll(PDO::FETCH_ASSOC);
    }
}
