<?php

use Transfercar\Geoloc\GoogleMapsDistancesMatrix;

class AdListingSearch
{
    public function findAds($pickup, $dropoff, $pickup_date, $vehicle_type, $age, $domain, $ip)
    {
        $ctAds = [];//$this->findCarTrawlerAds($pickup, $dropoff, $pickup_date, $age, $domain, $ip);
        $orAds = $this->findOnlineRepublicAds($pickup, $dropoff, $pickup_date, current((array)$vehicle_type), $age, $domain, $ip);
        $dbAds = $this->findRentalcarsAds($pickup, $dropoff, $pickup_date, $age, $domain, $ip);
	// $dbAds = $this->findDbAds($pickup, $dropoff, $pickup_date, current((array)$vehicle_type), $domain);

        //$this->updateStats($pickup, $dropoff, $orAds, $dbAds);

        return $this->merge($orAds, $ctAds, $dbAds);
    }

    private function merge($orAds, $ctAds, $dbAds)
    {
        $data = array_merge($orAds, $ctAds, $dbAds);

        usort($data, function ($a, $b) {
            return $a['total_charge'] - $b['total_charge'];
        });
        return $data;
    }


    private function findRentalcarsAds($pickup, $dropoff, $pickup_date, $age, $domain, $ip)
    {
        try {
            $this->advertiser = 'rentalcars.com';
            $pickup = RentalcarsComLocation::getLocation($pickup, 'pickup');
            $dropoff = RentalcarsComLocation::getLocation($dropoff, 'dropoff');

            if (!$pickup || !$dropoff) {
                return [];
            }

            list($days, $dropoff_date) = $this->getDropoffDate($pickup_date, $pickup, $dropoff, $domain);
            $gateway = new RentalcarsComGateway($days, null);
            $response = $gateway->find_vehicles($pickup->get('advertiser_location_id'),
                $pickup_date, $dropoff->get('advertiser_location_id'),
                $dropoff_date, $days, $age,
                $ip, strtoupper($domain.'D')
            );

            return $gateway->parseResponse((string) $response, [$pickup->get('advertiser_location_id'), $dropoff->get('advertiser_location_id')], [$this, 'importAds']);
        } catch (Exception $e) {
            error_log($e->getMessage());
            return [];
        }
    }


    private function findOnlineRepublicAds($pickup, $dropoff, $pickup_date, $vehicle_type, $age, $domain, $ip)
    {
        try {
            $this->advertiser = 'motorhomerepublic';

            $gateway = new OnlineRepublicGateway();
            $pickup = OnlineRepublicLocation::getLocation($pickup, 'pickup');
            $dropoff = OnlineRepublicLocation::getLocation($dropoff, 'dropoff');

            if (!$pickup || !$dropoff) {
                return [];
            }

            list($days, $dropoff_date) = $this->getDropoffDate($pickup_date, $pickup, $dropoff, $domain);
            $response = $gateway->find_vehicles(
                $pickup->get('advertiser_location_id'),
                $pickup_date,
                $dropoff->get('advertiser_location_id'),
                $dropoff_date,
                $days,
                $age,
                $domain,
                $ip,
                strtoupper($domain.'D')
            );

            return $gateway->parseResponse((string) $response, null, [$this, 'importAds']);
        } catch (\Exception $e) {
            return [];
        }
    }


    public function importAds($vehicles)
    {
        return $this->filter(array_map([$this, 'addRoute'], $vehicles));
    }

    private function filter($vehicles)
    {
        $filtered = [];
        foreach ($vehicles as $vehicle) {
            if (isset($filtered[$vehicle['vehicle_category']])) {
                if ($filtered[$vehicle['vehicle_category']]['total_charge'] < $vehicle['total_charge']) {
                    continue;
                }
            }

            $filtered[$vehicle['vehicle_category']] = $vehicle;
        }

        return array_slice($filtered, 0, 9);
    }

    private function map($locationId)
    {
        $map = Doctrine::getTable('AdvertiserLocationMap')
          ->createQuery('m')
          ->innerJoin('m.Advertiser a')
          ->where('a.username = ?', $this->advertiser)
          ->addWhere('advertiser_location_id = ?', $locationId)
          ->limit(1)
          ->fetchOne();

        if (!$map) {
          throw new RuntimeException("Unable to map $locationId for ".$this->advertiser, 1);
        }

        return $map->get('LocationNode');
    }

    private function getOrCreateRoute($pickup, $dropoff)
    {
        $pickup = $this->map($pickup);
        $dropoff = $this->map($dropoff);

        return ['pickup' => $pickup->get('name'), 'dropoff' => $dropoff->get('name')];
    }
    private function addRoute($vehicle)
    {
        $route = $this->getOrCreateRoute($vehicle['pickup_location_id'], $vehicle['return_location_id']);
        $vehicle['route'] = $route;

        return $vehicle;
    }

    private function getDropoffDate($pickup_date, $from, $to, $domain)
    {
        $api      = new GoogleMapsDistancesMatrix();
        $distance = $api->getDistances(
            $from->get('LocationNode')->getNameWithCountry('name'),
            $to->get('LocationNode')->getNameWithCountry('name'),
            $domain == 'us' ? 'imperial' : 'metric'
        );

        if (false === $distance)
        {
            return [7, date('Y-m-d\T10:00:00', strtotime('+7 days', strtotime($pickup_date)))];
        }

        $nb_days = (int) max(7, (int) ceil($distance/Settings::getDistanceByDay($domain)));

        return [$nb_days, date('Y-m-d\T10:00:00', strtotime('+'.max(7, $nb_days).' days', strtotime($pickup_date)))];
    }

    private function findDbAds($pickup, $dropoff, $pickup_date, $vehicle_type, $domain)
    {

        if (in_array($vehicle_type, array('motorhome', 'van', 'campervan'))) {
            return array();
        }

        $country = LocationCountry::retreiveByDomain($domain);
        $ads = Doctrine::getTable('AdListing')->findForFiltersQuery($country, array(
            'pickup'  => $pickup,
            'dropoff' => $dropoff,
            'pickup_date' => $pickup_date
        ));
        $ads->limit(3);
        $ads = $ads->execute();

        $dbAds = [];
        foreach ($ads as $ad) {
            $dbAds[] = [
                'name'                      => $ad->get('VehicleType')->get('description'),
                'code'                      => $ad->get('VehicleType')->get('description'),
                'vehicle_category'          => $ad->get('VehicleType')->get('description'),
                'picture_url'               => $ad->getImagePath(),
                'total_charge'              => $ad->get('price'),
                'transmission'              => 'varies',
                'passengers_quantity'       => 4,
                'vehicle_type'              => $ad->get('VehicleType')->get('description'),
                'pickup_location_name'      => $ad->get('Routes')->getFirst()->get('PickupLocation'),
                'return_location_name'      => $ad->get('Routes')->getFirst()->get('DropoffLocation'),
                'route'                     => ['pickup' => $ad->get('Routes')->getFirst()->get('PickupLocation')->get('name'), 'dropoff' => $ad->get('Routes')->getFirst()->get('DropoffLocation')->get('name')],
                'operator_logo'             => 'https://cdn.rcstatic.com/images/rclogo/blue/1x/wordmark-rclogo_122x26.png',
                'operator_name'             => $ad->get('Advertiser')->get('company'),
                'reference' =>  [
                    'url'        => '/adlisting/redirect/id/'.$ad->getId(),
                    'context'    => $ad->get('Advertiser')->get('company'),
                ]
              ];
        }

        return $dbAds;
    }
}
