<?php

use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Subscriber\Log\LogSubscriber;
use GuzzleHttp\Subscriber\Log\Formatter;
use Symfony\Component\DomCrawler\Crawler;

class CarTrawlerGateway
{
    private $logger, $client;
    public function __construct($logger)
    {
      $this->logger = $logger;
      $this->client = new HttpClient(array(
          'base_uri' => 'https://otageo.cartrawler.com/cartrawlerota'
      ));
    }

    public function ping($echo = 'ping')
    {
      $xml_query=<<<END_XML_QUERY
<?xml version="1.0" encoding="UTF-8"?>
        <OTA_PingRQ
            xmlns="http://www.opentravel.org/OTA/2003/05"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_PingRQ.xsd"
            Target="Test"
            Version="1.003">
            <EchoData>$echo</EchoData>
        </OTA_PingRQ>
END_XML_QUERY;

      return $this->invoke($xml_query);
    }

    public function find_vehicles($pickup_location_id, $pickup_datetime, $return_location_id, $return_datetime, $driver_age, $driver_country_code, $consumer_IP, $currency="NZD", $vehicle_class=3, $onResultParsed = null)
    {
      $cartrawler_client_id ='591840';

      $xml_query=<<<END_XML_QUERY
<?xml version="1.0" encoding="UTF-8"?>
<OTA_VehAvailRateRQ
    xmlns="http://www.opentravel.org/OTA/2003/05"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_VehAvailRateRQ.xsd"
    Target="Production"
    Version="1.005">
    <POS>
        <Source ISOCurrency="${currency}">
            <RequestorID Type="16" ID="${cartrawler_client_id}" ID_Context="CARTRAWLER" />
        </Source>
    </POS>
    <VehAvailRQCore Status="Available">
        <VehRentalCore PickUpDateTime="${pickup_datetime}" ReturnDateTime="${return_datetime}">
            <PickUpLocation CodeContext="CARTRAWLER" LocationCode="${pickup_location_id}" />
            <ReturnLocation CodeContext="CARTRAWLER" LocationCode="${return_location_id}" />
        </VehRentalCore>
        <DriverType Age='${driver_age}'/>
    </VehAvailRQCore>
    <VehAvailRQInfo>
      <Customer>
        <Primary>
          <CitizenCountryName Code='${driver_country_code}' />
        </Primary>
      </Customer>
      <TPA_Extensions>
        <ConsumerIP>${consumer_IP}</ConsumerIP>
      </TPA_Extensions>
    </VehAvailRQInfo>
</OTA_VehAvailRateRQ>
END_XML_QUERY;

      return $this->build_request($xml_query);
    }

    public function parseResponse($xml, $request, $onResultParsed)
    {
      $xml = str_replace([
        'xmlns="http://www.opentravel.org/OTA/2003/05"',
        'xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_VehAvailRateRS.xsd"'
      ], '', $xml);

      $reqXml = str_replace([
        'xmlns="http://www.opentravel.org/OTA/2003/05"',
        'xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_VehAvailRateRS.xsd"'
      ], '', (string) $request->getBody());

      $crawler = new Crawler($xml);
      $reqCrawler = new Crawler($reqXml);

      if ($crawler->filterXPath('//Errors')->count())
      {
        $pickup  = $reqCrawler->filterXPath('//PickUpLocation')->attr('LocationCode');
        $dropoff = $reqCrawler->filterXPath('//ReturnLocation')->attr('LocationCode');
        $pickup_time  = $reqCrawler->filterXPath('//VehRentalCore')->attr('PickUpDateTime');
        $dropoff_time = $reqCrawler->filterXPath('//VehRentalCore')->attr('ReturnDateTime');

        $message = sprintf('From %s to %s from dates %s to %s', $pickup, $dropoff, $pickup_time, $dropoff_time);
        throw new RuntimeException("Invalid request: ".$crawler->filterXPath('//Errors')->text().'. '.$message, 1);
      }

      $pickup_loc_name = $crawler->filterXPath("//VehRentalCore/PickUpLocation")->attr('Name');
      $pickup_loc_id   = $crawler->filterXPath("//VehRentalCore/PickUpLocation")->attr('LocationCode');
      $return_loc_name = $crawler->filterXPath("//VehRentalCore/ReturnLocation")->attr('Name');
      $return_loc_id   = $crawler->filterXPath("//VehRentalCore/ReturnLocation")->attr('LocationCode');

      $vendors = $crawler->filterXPath("//VehVendorAvail");

      $vehicles_array = [];
      foreach ($vendors as $vendorNode) {
          $vendor = new Crawler($vendorNode);
          $at_airport = (bool) $vendor->filterXPath('//LocationDetails')->first()->attr('atairport');

          $vehicles = $vendor->filterXPath('//VehAvail');

          $vehicleData = [];
          foreach ($vehicles as $vehicleNode) {
              $vehicle = new Crawler($vehicleNode);
              $ref = $vehicle->filterXPath("//Reference")->first();
              array_push($vehicles_array, [
                'name'                      => $vehicle->filterXPath("//VehMakeModel")->first()->attr('Name'),
                'code'                      => $vehicle->filterXPath("//VehMakeModel")->first()->attr('Code'),
                'picture_url'               => $vehicle->filterXPath("//PictureURL")->first()->text(),
                'total_charge'              => floatval($vehicle->filterXPath("//TotalCharge")->first()->attr('EstimatedTotalAmount'))/7,
                'total_charge_currency'     => $vehicle->filterXPath("//TotalCharge")->first()->attr('CurrencyCode'),
                'prepay_fee_amount'         => $vehicle->filterXPath("//Fee[@Purpose='22']")->first()->attr('Amount'),
                'prepay_fee_currency'       => $vehicle->filterXPath("//Fee[@Purpose='22']")->first()->attr('CurrencyCode'),
                'collection_fee_amount'     => $vehicle->filterXPath("//Fee[@Purpose='23']")->first()->attr('Amount'),
                'collection_fee_currency'   => $vehicle->filterXPath("//Fee[@Purpose='23']")->first()->attr('CurrencyCode'),
                'air_cond'                  => ($vehicle->filterXPath("//Vehicle")->first()->attr('AirconditionInd')=="true" ? "yes":"-"),
                'transmission'              => $vehicle->filterXPath("//Vehicle")->first()->attr('TransmissionType'),
                'passengers_quantity'       => $vehicle->filterXPath("//Vehicle")->first()->attr('PassengerQuantity'),
                'vehicle_asset_number'      => $vehicle->filterXPath("//VehIdentity")->attr("VehicleAssetNumber"),
                'vehicle_type'              => $vehicle->filterXPath("//VehClass")->first()->attr('Size'),
                'rate_qualifier'            => $vehicle->filterXPath("//RateQualifier")->first()->attr('RateQualifier'),
                'pickup_location_name'      => $pickup_loc_name,
                'pickup_location_id'        => $pickup_loc_id,
                'return_location_name'      => $return_loc_name,
                'return_location_id'        => $return_loc_id,
                'reference' =>  [
                    'at_airport' => $at_airport,
                    'type'       => $ref->attr('Type'),
                    'id'         => $ref->attr('ID'),
                    'uniqid'     => $ref->attr('ID').'_'.$pickup_loc_id.'_'.$return_loc_id,
                    'context'    => $ref->attr('ID_Context'),
                    'url'        => $this->build_url($reqCrawler, $crawler, $ref),
                    'datetime'   => $ref->attr('DateTime')
                ]
              ]);
          }
      }

      return $onResultParsed($vehicles_array);
    }

    private function build_url($request, $response, $ref)
    {
      $pickup       = $response->filterXPath("//VehRentalCore/PickUpLocation")->attr('LocationCode');
      $dropoff      = $response->filterXPath("//VehRentalCore/ReturnLocation")->attr('LocationCode');

      return strtr(
        'https://www.holidayautos.com/en/book?clientID=591840&pickupID=:pickup&returnID=:dropoff&pickupDateTime={puyear}-{pumonth}-{puday}T10:00:00Z&returnDateTime={doyear}-{domonth}-{doday}T10:00:00Z&age=30&curr=:currency&residenceID=:country&fs3=true',
        array(
          ':id'           => $ref->attr('ID'),
          ':pickup'       => $pickup,
          ':dropoff'      => $dropoff,
          ':currency'     => $request->filterXPath('//Source')->attr('ISOCurrency'),
          ':country'      => $request->filterXPath('//CitizenCountryName')->attr('Code')
        )
      );
    }

    # sends query to CarTrawlwer system and returns response
    private function build_request($xml_query)
    {
      return $this->client->createRequest('post', null, ['debug' => false, 'body' => $xml_query, 'future' => true]);
    }

    public function build_pool($requests, $onResultParsed)
    {
      return new GuzzleHttp\Pool($this->client, $requests, array(
        'pool_size' => 1,
        'before'   => function() {
          $this->logger->logSection('import', 'Start new request '.date('Y-m-d H:i:s'));
        },
        'complete' => function($event)  use($onResultParsed) {
            $response = $event->getResponse();

            if ($response->getStatusCode() != 200) {
                 throw new RuntimeException("Sorry, we are experiencing communication problems. Please try again later.");
            }

            return $this->parseResponse($response->getBody(), $event->getRequest(), $onResultParsed);
        },
        'error' => function($event)  use($onResultParsed) {
          $this->logger->logSection('error', get_class($event). ' '. $event->getException()->getMessage(), 1024, 'ERROR');
          $onResultParsed([]);
        }
      ));
    }

    private function invoke($xml_query, $onResponse)
    {
        return $this->_invoke($xml_query, $onResponse);
    }
}