<?php

use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Subscriber\Log\LogSubscriber;
use GuzzleHttp\Subscriber\Log\Formatter;
use Symfony\Component\DomCrawler\Crawler;

class OnlineRepublicGateway
{
    private $logger, $client, $days;
    public function __construct($logger = null)
    {
      $this->logger = $logger;
      $this->days = 1;
      $this->client = new HttpClient(array(
          'base_uri' => 'http://api.onlinerepublic.com/'
      ));
    }

    public function findLocations()
    {
      return file_get_contents('data/orlocation.xml');
    }

    public function find_vehicles($pickup_location_id, $pickup_datetime, $return_location_id, $return_datetime, $days, $driver_age, $driver_country_code, $consumer_IP, $currency="NZD", $vehicle_class=3, $onResultParsed = null)
    {
      $client_id ='238D4367';
      $driver_country_code = strtoupper($driver_country_code);
      $this->days = max(1, $days);
      $xml_query=<<<END_XML_QUERY
<?xml version="1.0" encoding="UTF-8"?>
<OTA_VehAvailRateRQ
    xmlns="http://www.opentravel.org/OTA/2003/05"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_VehAvailRateRQ.xsd"
    Target="Production"
    Version="1.0">
    <POS>
        <Source ISOCurrency="${currency}">
            <RequestorID Type="16" ID="${client_id}" ID_Context="ONELINEREPUBLIC" />
            <TPA_Extensions>
                <RemoteIP>${consumer_IP}</RemoteIP>
            </TPA_Extensions>
        </Source>
    </POS>
    <VehAvailRQCore Status="All">
        <VehRentalCore PickUpDateTime="${pickup_datetime}" ReturnDateTime="${return_datetime}">
            <PickUpLocation CodeContext="ONELINEREPUBLIC" LocationCode="${pickup_location_id}" />
            <ReturnLocation CodeContext="ONELINEREPUBLIC" LocationCode="${return_location_id}" />
        </VehRentalCore>
        <DriverType Age='${driver_age}'/>
    </VehAvailRQCore>
    <VehAvailRQInfo>
      <Customer>
        <Primary>
          <CitizenCountryName Code='${driver_country_code}' />
        </Primary>
      </Customer>
    </VehAvailRQInfo>
</OTA_VehAvailRateRQ>
END_XML_QUERY;

      return $this->build_request($xml_query);
    }

    public function parseResponse($xml, $request, $onResultParsed)
    {
      $xml = str_replace([
        'xmlns="http://www.opentravel.org/OTA/2003/05"',
        'xsi:schemaLocation="http://www.opentravel.org/OTA/2003/05 OTA_VehAvailRateRS.xsd"'
      ], '', $xml);

      $crawler = new Crawler($xml);

      if ($crawler->filterXPath('//Errors')->count())
      {
        $error = $crawler->filterXPath('//Error')->attr('ShortText');

        throw new RuntimeException("Invalid request: ".$error, 1);
      }

      $pickup_loc_name = $crawler->filterXPath("//VehRentalCore/PickUpLocation")->attr('Name');
      $pickup_loc_id   = $crawler->filterXPath("//VehRentalCore/PickUpLocation")->attr('LocationCode');
      $return_loc_name = $crawler->filterXPath("//VehRentalCore/ReturnLocation")->attr('Name');
      $return_loc_id   = $crawler->filterXPath("//VehRentalCore/ReturnLocation")->attr('LocationCode');

      $vendors = $crawler->filterXPath("//VehVendorAvail");

      $vehicles_array = [];
      foreach ($vendors as $vendorNode) {
          $vendor = new Crawler($vendorNode);
          $at_airport = (bool) $vendor->filterXPath('//LocationDetails')->first()->attr('AtAirport');

          $vehicles = $vendor->filterXPath('//VehAvail');

          $vehicleData = [];
          foreach ($vehicles as $vehicleNode) {
              $vehicle = new Crawler($vehicleNode);
              $ref = $vehicle->filterXPath("//Reference")->first();
              array_push($vehicles_array, [
		      'operator_name' =>  'Motorhome Republic',
		                      'operator_logo'             =>  '/images/motorhomerepublic.jpg',
                'name'                      => $vehicle->filterXPath("//VehMakeModel")->first()->attr('Name'),
                'code'                      => $vehicle->filterXPath("//VehMakeModel")->first()->attr('Code'),
                'picture_url'               => $vehicle->filterXPath("//PictureURL")->first()->text(),
                'total_charge'              => floatval($vehicle->filterXPath("//TotalCharge")->first()->attr('EstimatedTotalAmount'))/$this->days,
                'total_charge_currency'     => $vehicle->filterXPath("//TotalCharge")->first()->attr('CurrencyCode'),
                'prepay_fee_amount'         => $vehicle->filterXPath("//Fee[@Purpose='22']")->first()->attr('Amount'),
                'prepay_fee_currency'       => $vehicle->filterXPath("//Fee[@Purpose='22']")->first()->attr('CurrencyCode'),
                'collection_fee_amount'     => $vehicle->filterXPath("//Fee[@Purpose='23']")->first()->attr('Amount'),
                'collection_fee_currency'   => $vehicle->filterXPath("//Fee[@Purpose='23']")->first()->attr('CurrencyCode'),
                'air_cond'                  => ($vehicle->filterXPath("//Vehicle")->first()->attr('AirconditionInd')=="true" ? "yes":"-"),
                'transmission'              => $vehicle->filterXPath("//Vehicle")->first()->attr('TransmissionType'),
                'passengers_quantity'       => $vehicle->filterXPath("//Vehicle")->first()->attr('PassengerQuantity'),
                'vehicle_asset_number'      => $vehicle->filterXPath("//VehIdentity")->attr("VehicleAssetNumber"),
                'vehicle_category'          => $this->getCategory($vehicle->filterXPath("//Vehicle")->first()->attr('PassengerQuantity')),
                'vehicle_type'              => 'campervan',
                'pickup_location_name'      => $pickup_loc_name,
                'pickup_location_id'        => $pickup_loc_id,
                'return_location_name'      => $return_loc_name,
                'return_location_id'        => $return_loc_id,
                'reference' =>  [
                    'at_airport' => $at_airport,
                    'type'       => $ref->attr('Type'),
                    'id'         => $ref->attr('ID'),
                    'uniqid'     => $ref->attr('ID').'_'.$pickup_loc_id.'_'.$return_loc_id,
                    'context'    => 'Motorhome Republic',
                    'url'        => str_replace(['r=search', '&age=25'], ['affiliate=transfercar&r=search', ''], $ref->attr('URL')).'&category='.((string) $vehicle->filterXPath("//Vehicle")->first()->attr('PassengerQuantity')),
                    'datetime'   => $ref->attr('DateTime')
                ]
              ]);
          }
      }

      return $onResultParsed($vehicles_array);
    }

    private function getCategory($category)
    {
      if ((int) $category >= 5) {
        return 'Motorhome';
      }

      return 'Campervan';
    }

    private function build_url($url)
    {
      $url = preg_replace('/start_date=\d{4}-\d{2}-\d{2}/', 'start_date={puyear}-{pumonth}-{puday}', $url);
      $url = preg_replace('/start_date=\d{4}-\d{2}-\d{2}/', 'end_date={doyear}-{domonth}-{doday}', $url);

      return $url;
    }

    # sends query to CarTrawlwer system and returns response
    private function build_request($xml_query)
    {
      $cache = new \sfFileCache(array(
        'cache_dir' => sfConfig::get('sf_app_cache_dir').'/onlinerepublic',
        'lifetime' => 10800 // 3 hrs
      ));

      $key = sha1(preg_replace('#<RemoteIP>[^<]+</RemoteIP>|<VehRentalCore [^>]+>#', '', $xml_query));

      if (!$data = $cache->get($key))
      {
        $body = $this->client->request('POST', null, ['debug' => false, 'body' => $xml_query, 'future' => false]);
        $data = (string) $body->getBody();
        $cache->set($key, $data);
      }

      return $data;
    }

    public function build_pool($requests, $onResultParsed)
    {
      return new GuzzleHttp\Pool($this->client, $requests, array(
        'pool_size' => 1,
        'before'   => function() {
          $this->logger->logSection('import', 'Start new request '.date('Y-m-d H:i:s'));
        },
        'complete' => function($event)  use($onResultParsed) {
            $response = $event->getResponse();

            if ($response->getStatusCode() != 200) {
                 throw new RuntimeException("Sorry, we are experiencing communication problems. Please try again later.");
            }

            return $this->parseResponse($response->getBody(), $event->getRequest(), $onResultParsed);
        },
        'error' => function($event)  use($onResultParsed) {
          // throw $event->getException();
          $ex = $event->getException()->getPrevious();
          $this->logger->logSection('error', get_class($event). ' '. $ex->getMessage().' '.$ex->getFile().' '.$ex->getLine().' '.$ex->getTraceAsString(), 1024, 'ERROR');
          $onResultParsed([]);
        }
      ));
    }

    private function invoke($xml_query, $onResponse)
    {
        return $this->_invoke($xml_query, $onResponse);
    }
}
