<?php

use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Subscriber\Log\LogSubscriber;
use GuzzleHttp\Subscriber\Log\Formatter;
use Symfony\Component\DomCrawler\Crawler;

class RentalcarsComGateway
{
    private $logger, $client;
    public function __construct($logger)
    {
      $this->logger = $logger;
      $this->client = new HttpClient(array(
          'base_uri' => 'https://xml.rentalcars.com/service/ServiceRequest.do',
          'defaults' => [
            'headers' => [
                'User-Agent' => 'HTTPie/2.5.0'
            ]
          ]
      ));

    }

    public function findLocations($country, $cityName = 'Auckland')
    {
        $xml_query = <<<EOF
<PickUpCityListRQ>
    <Credentials username="transfercar" password="transfer123" remoteIp="74.207.251.84" />
    <Country>$country</Country>
    <!-- <City>$cityName</City> -->
</PickUpCityListRQ>
EOF;

        return $this->client->request('POST', '', ['body' => trim($xml_query)], false);
    }

    public function get_vehicles_extras($vehicleId)
    {
        $xml_query = <<<EOF
<ExtrasListRQ>
    <Credentials username="transfercar" password="transfer123" remoteIp="74.207.251.84" />
    <Vehicle id="$vehicleId"/>
</ExtrasListRQ>
EOF;
        return $this->client->send($this->build_request($xml_query, false))->getBody();
    }


    public function find_vehicles($pickup_location_id, $pickup_ts, $return_location_id, $return_ts, $days, $driver_age, $consumer_IP, $currency="EUR")
    {
        $pickup_year  = date('Y', strtotime($pickup_ts));
        $pickup_month = date('m', strtotime($pickup_ts));
        $pickup_day   = date('d', strtotime($pickup_ts));
        $pickup_hour  = date('H', strtotime($pickup_ts));

        $return_year  = date('Y', strtotime($return_ts));
        $return_month = date('m', strtotime($return_ts));
        $return_day   = date('d', strtotime($return_ts));
        $return_hour  = date('H', strtotime($return_ts));

        $xml_query = <<<EOF
<SearchRQ preflang="en" prefcurr="NZD">

<Credentials username="transfercar" password="transfer123" remoteIp="$consumer_IP" />

<PickUp>
  <Location country="New Zealand" city="$pickup_location_id" />
  <Date year="$pickup_year" month="$pickup_month" day="$pickup_day" hour="$pickup_hour" minute="00" />
</PickUp>

<DropOff>
  <Location country="New Zealand" city="$return_location_id" />
  <Date year="$return_year" month="$return_month" day="$return_day" hour="$return_hour" minute="00" />
</DropOff>

<DriverAge>30</DriverAge>

</SearchRQ>
EOF;
//file_put_contents('/tmp/debug-ads.log', $xml_query,\FILE_APPEND); 
//error_log(trim(preg_replace('/\s+/', ' ', $xml_query)));
        return $this->build_request($xml_query);
    }

    public function parseResponse($xml, $route, $onResultParsed)
    {
        // $reqXml = $request->getQuery()->get('xml');
        $crawler = new Crawler('<?xml version="1.0" encoding="UTF-8"?>'. (string) $xml);
        /* $request = new Crawler('<?xml version="1.0" encoding="UTF-8"?>'. (string) $reqXml);*/

        // $pickup  = $request->filterXPath('//PickUp/Location');
        // $dropoff = $request->filterXPath('//DropOff/Location');

        $matches = $crawler->filterXPath('//Match');
        $data = [];

        foreach ($matches as $matchNode) {


            $match = new Crawler($matchNode);
            $vehicle = $match->filterXPath('//Vehicle');
            $price    = $match->filterXPath('//Price');
            $pickup  = $match->filterXPath('//PickUp/Location');
            $dropoff = $match->filterXPath('//DropOff/Location');

            $data[] = [
                'name'                  => $vehicle->filterXPath("//Name")->text(),
                'code'                  => $vehicle->attr('group'),
                'picture_url'           => $vehicle->filterXPath("//ImageURL")->first()->text(),
                'total_charge'          => floatval(trim($price->text()))/7,
                'total_charge_currency' => $price->attr('currency'),
                'air_cond'              => $vehicle->attr('aircon'),
                'transmission'          => $vehicle->attr('automatic'),
                'passengers_quantity'   => $vehicle->attr('seats'),
                'vehicle_type'          => $vehicle->attr('group'),
                'pickup_location_name'  => $pickup->attr('locName'),
                'pickup_location_id'    => $route[0],
                'return_location_name'  => $dropoff->attr('locName'),
                'return_location_id'    => $route[1],
                'vehicle_category'      => 'sedan',
                'operator_logo'         => 'https://cdn.rcstatic.com/images/rclogo/blue/1x/wordmark-rclogo_122x26.png',
                'operator_name'         => 'From rentalcars.com',
                'reference' =>  [
                    'at_airport' => $pickup->attr('onAirport'),
                    'type'       => $vehicle->attr('group'),
                    'id'         => $vehicle->attr('id'),
                    'context'    => 'rentalcars.com',
                    'url'        => str_replace('http://www.rentalcars.com/', 'https://secure.rentalcars.com/', $match->filterXPath('//ForwardURL')->text()),
                    'datetime'   => ''
                ]
            ];
        }

        return $onResultParsed($data);
    }

    private function build_url($request, $match)
    {
        $pickup     = $match->filterXPath('//PickUp/Location');
        $pickupDate = $match->filterXPath('//PickUp/Date');

        $dropoff     = $match->filterXPath('//DropOff/Location');
        $dropoffDate = $match->filterXPath('//DropOff/Date')->text();

        $urlPattern = 'https://secure.rentalcars.com/DriverExtras.do?affiliateCode=:affiliate_code&'.
                      'vehicleInfo.vehicle.id=:id&location=:pickup_location_id'.
                      '&dropLocation=:return_location_id&driversAge=:driver_age'.
                      '&puYear={puyear}&puMonth={pumonth}&puDay={puday}&puHour=11'.
                      '&puMinute=30&doYear={doyear}&doMonth={domonth}&doDay={doday}'.
                      '&doHour=11&doMinute=30&prefcurrency=:currency&preflang=en'.
                      '&cor=GB';

        return strtr($urlPattern, array(
            ':id'                 => $match->filterXPath('//Vehicle')->attr('id'),
            ':pickup_location_id' => $match->filterXPath('//PickUp/Location')->attr('id'),
            ':return_location_id' => $match->filterXPath('//DropOff/Location')->attr('id'),
            ':driver_age'         => 25,//$match->filterXPath('//DriverAge')->text(),
            ':currency'           => $match->attr('prefcurr'),
            ':affiliate_code'     => 'transfercar'
        ));
    }

    private function build_request($xml_query)
    {
      $cache = new \sfFileCache(array(
        'cache_dir' => sfConfig::get('sf_app_cache_dir').'/onlinerepublic',
        'lifetime' => 10800 // 3 hrs
      ));

      $key = sha1(preg_replace('# remoteIp="[^"]+"#', '', $xml_query));

      if (!$data = $cache->get($key))
      {
        //$body = $this->client->request('POST', null, ['debug' => fopen('/var/www/linode_current/application/log/guzzle.log', 'wb'), 'proxy' => 'http://bcbxmape:w61bz43lastx@209.127.191.180:9279', 'body' => trim($xml_query)], false);
        // $body = $this->client->request('POST', null, ['debug' => false, 'body' => $xml_query, 'future' => false]);
        //$data = (string) $body->getBody();
        
        $data = $this->curl_request($xml_query);
        $cache->set($key, $data);
      }

      return $data;
    }


    private function curl_request($xml_query)
    {
      $url = 'https://xml.rentalcars.com/service/ServiceRequest.do';
      $proxy = '209.127.191.180:9279';
      $proxyauth = 'bcbxmape:w61bz43lastx';
      $headers = array(
         "Content-Type: application/xml",
         "Accept: application/xml",
      );

       
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL,$url);
      curl_setopt($ch, CURLOPT_POST, true);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $xml_query);
      #curl_setopt($ch, CURLOPT_PROXY, $proxy);
      #curl_setopt($ch, CURLOPT_PROXYUSERPWD, $proxyauth);
      curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      curl_setopt($ch, CURLOPT_HEADER, 0);
      curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
      $curl_scraped_page = curl_exec($ch);
      curl_close($ch);


#file_put_contents('/var/www/linode_current/application/log/guzzle.log', "\n".$curl_scraped_page, \FILE_APPEND);
      return $curl_scraped_page;
    }

    public function build_pool($requests, $onResultParsed)
    {
      return new GuzzleHttp\Pool($this->client, $requests, array(
        'pool_size' => 25,
        'complete' => function($event)  use($onResultParsed) {
            $response = $event->getResponse();
            $request  = $event->getRequest();

            if ($response->getStatusCode() != 200) {
                 throw new RuntimeException("Sorry, we are experiencing communication problems. Please try again later.");
            }

            return $this->parseResponse($response->getBody(), $request, $onResultParsed);
        },
        'error' => function($event)  use($onResultParsed) {
          $this->logger->logSection('error', get_class($event). ' '. $event->getException()->getMessage(), 1024, 'ERROR');
          $this->logger->log($event->hasResponse() ? (string) $event->getResponse() : 'No response');
          $onResultParsed([]);
        }
      ));
    }
}
