<?php

class RentalcarsComLocation
{
    public static function getLocation($tcLocationSlug)
    {
        if (!$tcLocationSlug) {
          $tcLocationSlug = self::getPopular($key);
        }

        $map = Doctrine::getTable('AdvertiserLocationMap')
          ->createQuery('m')
          ->innerJoin('m.Advertiser a')
          ->innerJoin('m.LocationNode ln')
          ->where('a.username = ?', 'rentalcars.com')
          ->addWhere('ln.slug = ?', $tcLocationSlug)
          ->limit(1)
          ->fetchOne();

        if (!$map) {
          $node = Doctrine::getTable('LocationNode')->findOneBySlug($tcLocationSlug);

          if ($node && $node->getType() == 'category') {
            $node = $node->getChildrenOfType('location')->getFirst();
          }

          if ($node && !$node->getParentOfType('departure')) {
            return false;
          }

          return $node ? self::getLocation($node->getParentOfType('departure')->get('slug')) : false;
        }

        return $map;
    }

    
    public static function getPopular($key = 'pickup')
    {
      $key = in_array($key, ['pickup', 'dropoff']) ? $key : 'pickup';

      $q = 'select ln.id, ln.slug, count(r.id) as c
      from location_node ln
      inner join locations l on ln.type=? and ln.type_id=l.id
      inner join listing li on li.'.$key.'_location_id=l.id
      inner join request r on r.listing_id=li.id
      inner join location_category lc on l.category_id=lc.id
      inner join advertiser_location_map alm on alm.tc_location_node_id=ln.id
      inner join advertiser a on alm.advertiser_id=a.id
      where lc.country_id=?
      and r.created_at > ?
      and a.username=?
      group by ln.id, ln.slug
      order by c desc
      limit 3';

      $conn = Doctrine_Manager::getInstance()->getCurrentConnection();

      $stmt = $conn->prepare($q);
      $stmt->execute(array('location', LocationCountry::retreiveByDomain($_SERVER['TRANSFER_CAR_COUNTRY']), date('Y-m-d', strtotime('-6 months')), 'rentalcars.com'));

      $data = $stmt->fetchAll();

      if (!empty($data)) {
        $row = $data[mt_rand(0, count($data) - 1)];

        return $row['slug'];
      }

      return false;
    }

    public static function getNzLocations($advertiser_id)
    {
        $mappings = Doctrine::getTable('AdvertiserLocationMap')
            ->createQuery()
            ->where('advertiser_id = ?', $advertiser_id)
            ->execute();

        $ids = [];
        foreach ($mappings as $mapping) {
            $ids[] = $mapping->get('advertiser_location_id');
        }

        return $ids;
    }
}
