<?php

class SearchFilterForm extends sfForm
{
    private $modified = array();

    public function __construct($defaults = array(), $options = array(), $CSRFSecret = null)
    {
      // disable csrf protection
      parent::__construct($defaults, $options, false);
    }

    public function configure()
    {
        $this->widgetSchema['pickup']  = new sfWidgetFormSelect(array('choices' => []));
        $this->widgetSchema['dropoff']    = new sfWidgetFormSelect(array('choices' => []));

        $this->widgetSchema['from_date']  = new sfWidgetFormInput();
        $this->widgetSchema['to_date']  = new sfWidgetFormInputHidden();
        $this->widgetSchema['area']       = new sfWidgetFormInputHidden();

        $this->setDefaults(array(
          'pickup'  => '',
          'dropoff'    => '',
          'from_date'  => '',
          'to_date'  => '',
          'travellers' => ''
        ));

        $this->widgetSchema['travellers'] = new sfWidgetFormChoice(array(
            'choices'  => array(
                ''=> 'Any',
                1 => '1+ person',
                2 => '2+ people',
                3 => '3+ people',
                4 => '4+ people',
                5 => '5+ people',
                6 => '6+ people'
            ),
            'expanded' => false,
            'multiple' => false
        ));

        $this->setDefault('travellers', '');

        $this->widgetSchema['transmission'] = new sfWidgetFormChoice(array(
            'choices' => [],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('transmission', array());

        $this->widgetSchema['vehicle_type'] = new sfWidgetFormChoice(array(
            'choices' => [],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('vehicle_type', array());

        $this->widgetSchema['country'] = new sfWidgetFormChoice(array(
            'choices' => [],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->widgetSchema['vehicle_type_filter'] = new sfWidgetFormChoice(array(
            'choices' => [],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('vehicle_type_filter', array());

        $this->setDefault('country', array());

        $this->widgetSchema['min_age'] = new sfWidgetFormInput();

        $this->setDefault('min_age', '');

        $this->widgetSchema['min_age_filter'] = new sfWidgetFormChoice(array(
            'choices' => [],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('min_age_filter', '');

        $this->widgetSchema['fuel'] = new sfWidgetFormChoice(array(
            'choices' => ['0' => 'Not included', '1' => 'Free tank', '2' => 'Partial refund'],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('fuel', array());

        $this->widgetSchema['ferry'] = new sfWidgetFormChoice(array(
            'choices' => [
              '1' => 'Vehicle only',
              '2' => 'Free for car + 1 adult',
              '3' => 'Free for car + 2 adults',
              '4' => 'Partial refund'],
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('ferry', array());

        $other_extras = [
          'petrol' => 'Fuel',
          'ferry' => 'Ferry',
          'expenses'   => 'Expenses'
        ];

        if ($this->getOption('country') != 1) {
          $other_extras['tolls'] = 'Road tolls';
        }

        $this->widgetSchema['other_extras'] = new sfWidgetFormChoice(array(
            'choices' => $other_extras,
            'expanded' => true,
            'multiple' => true,
            'renderer_options' => array('formatter' => array($this, 'formatter'))
        ));

        $this->setDefault('other_extras', array());

        $this->validatorSchema['pickup']    = new sfValidatorString(array('required' => false));
        $this->validatorSchema['dropoff']   = new sfValidatorString(array('required' => false));
        $this->validatorSchema['from_date'] = new sfValidatorAnd(array(
          new sfValidatorCallback(array('required' => false, 'callback' => function($validator, $value) {
            return str_replace(',' , ' ', $value);
          })),
          new sfValidatorDate(array('required' => false))
        ), array('required' => false));
        $this->validatorSchema['to_date'] = new sfValidatorAnd(array(
          new sfValidatorCallback(array('required' => false, 'callback' => function($validator, $value) {
            return str_replace(',' , ' ', $value);
          })),
          new sfValidatorDate(array('required' => false))
        ), array('required' => false));
        $this->validatorSchema['travellers']   = new tcValidatorFilterString(array('required' => false));
        $this->validatorSchema['transmission'] = new tcValidatorFilterString(array('required' => false, 'multiple' => true));
        $this->validatorSchema['vehicle_type'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        $this->validatorSchema['vehicle_type_filter'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        $this->validatorSchema['area']    = new sfValidatorCallback(array('required' => false, 'callback' => function($validator, $value){
          return explode(',', $value);
        }));
        
        $this->validatorSchema['fuel'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        
        $this->validatorSchema['ferry'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        
        $this->validatorSchema['other_extras'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        
        $this->validatorSchema['min_age'] = new tcValidatorFilterString(array('required' => false,  'multiple' => false));
        $this->validatorSchema['min_age_filter'] = new tcValidatorFilterString(array('required' => false,  'multiple' => true));
        $this->validatorSchema['country']   = new tcValidatorFilterString(array('required' => false));

        $this->setDefault('area', '');

        $this->validatorSchema->setOption('allow_extra_fields', true);
    }

    public function configureFromAggregations($aggregations)
    {
      //ini_set('memory_limit', -1);
      $i18n = $this->widgetSchema->getFormFormatter()->getTranslationCallable();

      foreach (array('pickup', 'dropoff') as $name)
      {
        $choices = $this->convertLocationBuckets($aggregations[$name]['buckets'], $aggregations['custom_'.$name][$name]['buckets'], $this->getValue($name));
        $choices[''] = call_user_func($i18n, ucfirst($name).' location');
        $this->widgetSchema[$name]->setOption('choices', $choices);
      }

      foreach ($aggregations as $name => $agg)
      {
        if (!isset($this[$name]) || in_array($name, array('pickup', 'dropoff')))
        {
          continue;
        }

        $choices = $this->convertBucketsToChoices($name, $agg['buckets'], $this->widgetSchema[$name]->getOption('multiple'));
        $this->widgetSchema[$name]->setOption('choices', $choices);
      }
    }

    public function hasFilters()
    {
      foreach (['vehicle_type', 'vehicle_type_filter', 'min_age', 'min_age_filter', 'fuel', 'ferry'] as $key) {
        if ($this->getValue($key)) {
          return true;
        }
      }
      return false;
    }


    private function convertBucketsToChoices($name, $buckets, $isMultiple, $anyKey = '', $anyLabel = 'Any')
    {
      $valueKey = 'travellers' == $name ? 'from' : 'key';
      $i18n = $this->widgetSchema->getFormFormatter()->getTranslationCallable();
      $choices = $isMultiple ? array() : array($anyKey => call_user_func($i18n, $anyLabel));

      foreach ($buckets as $key => $value)
      {
        $doc_count = $value['doc_count'];

        if (array_key_exists('sum', $value)) {
          $doc_count = array_key_exists('sum', $value['sum']) ? $value['sum']['sum']['value'] : $value['sum']['value'];
        }

        if (strpos($value[$valueKey], '_') !== false)
        {
          list($key, $label) = explode('_', $value[$valueKey], 2);
          $choices[$key] = sprintf('%s', call_user_func($i18n, $label), $doc_count);
        }
        elseif ('travellers' == $name)
        {
            $label = call_user_func($i18n, '[0] Any |[1] %s+ person (%s) |[2,5] %s+ people (%s) |[6,+Inf] %s or more people (%s)');
            $choice = new sfChoiceFormat();
            $retval = $choice->format($label, (int) $value[$valueKey]);
            $choices[$value[$valueKey]] = sprintf($retval, $value[$valueKey], $doc_count);
        }
        else
        {
          $choices[$value[$valueKey]] = sprintf('%s', ucfirst(call_user_func($i18n, $value[$valueKey])), $doc_count);
        }
      }

      return $choices;
    }

    private function convertLocationBuckets($locationBuckets, $customBuckets, $value)
    {
      $i18n = $this->widgetSchema->getFormFormatter()->getTranslationCallable();
      $locations = array('' => call_user_func($i18n, 'Any'));

      if ($value) {
        $locations[$value] = $this->getTextFor($value);
      }

      if (false && count($customBuckets))
      {
        $locations[call_user_func($i18n, 'Popular')] = $this->convertBucketsToChoices('popular', $customBuckets, true);
      }
      $allLocations = Doctrine::getTable('LocationNode')->findNestedByCountry($this->getOption('country'), 'slug');
      // echo '<pre>';
      // print_r($allLocations);
      foreach ($allLocations as $category => $locationNodes)
      {
        foreach ($locationNodes as $slug => $departure) {
          if (!isset($locations[call_user_func($i18n, $category)])) {
            $locations[$category] = [];
          }
          if (!isset($locations[call_user_func($i18n, $category)][$slug])) {
            $locations[$category][$slug] = (string) $departure;
          }
        }
      }
      
      // print_r($locations);
      // echo '</pre>';

      foreach ($locationBuckets as $name => $value)
      {
        list($key, $label) = explode('_', $value['key'], 2);
        if ($value['key'] == 'null_null') {
          continue;
          $label = 'Popular';
        }
        // translate state name
        $label = call_user_func($i18n, $label);
        $locations[$label] = $this->convertBucketsToChoices('locations', $value['locations']['buckets'], false, $key, call_user_func($i18n, 'Any from ').$label);
      }

      unset($locations['Popular']['null']);

      return $locations;
      return array_merge(array('' => call_user_func($i18n, 'Any')), isset($locations['Popular']) ? $locations['Popular'] : []);
    }

    public function formatter($widget, $inputs)
    {
      $rows = array();
      foreach ($inputs as $input)
      {
        $label = preg_replace(
          '/(<label for="[^"]+")>/',
          '\\1 class="checkbox">'.$input['input'],
          $input['label']
        );

        $rows[] = $widget->renderContentTag('div', 
          str_replace('<input ', '<input class="form-check-input" ', $input['input']).
          str_replace('<label ', '<label class="form-check-label" ', $input['label']), 
          ['class' => 'form-check form-check-inline']
        );
      }

      return implode('', $rows);
    }

    public function bind(array $taintedValues = null, array $taintedFiles = null)
    {
      $defaults = $this->getDefaults();

      foreach ($defaults as $name => $default) {
        if (!array_key_exists($name, $taintedValues)) {
          $taintedValues[$name] = $default;
        } elseif ($taintedValues[$name] != $default) {
          if ($this->validatorSchema[$name]->getOption('multiple') && !is_array($taintedValues[$name])) {
            // echo $name;var_dump($taintedValues[$name]);
            $taintedValues[$name] = [$taintedValues[$name]];
          }
          $this->modified[$name] = $taintedValues[$name];
        }
      }
      // print_r($taintedValues);
      return parent::bind($taintedValues, $taintedFiles);
    }

    public function getModifiedValues()
    {
      return $this->modified;
    }

    /**
     * Prevent returning default values and polluting the search query
     */
    public function getValuesForQuery()
    {
      return array_intersect_key($this->getValues(), $this->modified);
    }

    public function getText($name)
    {
      return $this[$name]->getValue() ? $this->getTextFor($this[$name]->getValue()) : '';
    }

    public function getTextFor($value)
    {
      $node = Doctrine::getTable('LocationNode')->findOneBySlug($value);

      return $node ? $node->get('name') : '';
    }

    public function fixDepartureNameParam($validator, $value, $args)
    {
      $parts = explode('-', $value);

      return $parts[count($parts) - 1];
    }

    public function getDepartureSlugFor($name)
    {
      $node = Doctrine::getTable('LocationNode')->findOneBySlug($this[$name]->getValue());

      return $node && $node->getDeparture() ? $node->getDeparture()->get('slug') : '';
    }

}

class tcValidatorFilterString extends sfValidatorBase
{
  protected function configure($options = array(), $messages = array())
  {
    $this->addOption('multiple', false);
  }

  protected function doClean($value)
  {
    if (is_string($value) && strpos($value, '|'))
    {
      return explode('|', $value);
    }
    elseif (is_array($value))
    {
      $explodedValues = array();
      foreach ($value as $v) {
        $cleaned = $this->doClean($v);

        if (is_array($cleaned))
        {
          $explodedValues = array_merge($cleaned, $explodedValues);
        }
        else
        {
          $explodedValues[] = $cleaned;
        }
      }

      $value = $explodedValues;
    }

    if ($this->getOption('multiple') && !is_array($value)) {
      $value = [$value];
    }

    return $value;
  }
}