<?php

class BaseForm extends sfForm
{
    protected $embeddedForms = array();

    public function embedForm($name, sfForm $form, $decorator = null)
    {
        parent::embedForm($name, $form, $decorator);
        $this->embeddedForms[$name] = $form;
    }

    public function getEmbeddedForms()
    {
        return $this->embeddedForms;
    }

    public function addCSRFProtection($secret = null)
    {
        parent::addCSRFProtection($secret);

        if (isset($this->validatorSchema[self::$CSRFFieldName]))
        {
            $this->validatorSchema[self::$CSRFFieldName]->setMessage('csrf_attack', 'This session has expired. Please <a href="?1">refresh</a> your browser to continue.');
            $this->getWidgetSchema()->getFormFormatter()->setNamedErrorRowFormatInARow("    <li>%error%</li>\n");
        }
    }

    public function getAllErrorsArray()
    {
        $ret = array();
        if($this->hasGlobalErrors())
        {
            $globalErrors = $this->getGlobalErrors();
            foreach($globalErrors as $id => $globalError)
            {
                $ret['globals'][$id] = $globalError->getMessage();
            }
        }

        if($this->getErrorSchema()->count())
        {
            $fields = $this->getWidgetSchema()->getFields();
            $errorSchema = $this->getErrorSchema();
            foreach($fields as $id => $value)
            {
                if($error = $errorSchema->offsetGet($id))
                {
                    if($id === '_csrf_token')
                    {
                        continue;
                    }

                    $widget = $this->widgetSchema[$id];
                    $label = $this->widgetSchema->getLabel($id);
                    if(!$label)
                    {
                        $label = $id;
                    }

                    $ret['named'][$id]['label'] = $label;
                    $err = $this->getWidgetSchema()->getFormFormatter()->translate($error->getMessageFormat(), $error->getArguments());
                    $ret['named'][$id]['message'] = $err;
                }
            }
        }
        return $ret;
    }

    public function getResultJson()
    {
        $resultArray = $this->getAllErrorArray();
        if($this->hasErrors() || $this->hasGlobalErrors())
        {
            $resultArray['has_error'] = true;
        }
        else
        {
            $resultArray['has_error'] = false;
        }
        return json_encode($resultArray);
    }
}
