<?php

/**
 * Sign up class
 * @author Daniel Piechocki Planetsoft
 * @author Pawel Smolinski
 * @version 0.1
 *
 */

class RegisterForm extends BaseForm
{
    public function __construct($defaults = array(), $options = array(), $CSRFSecret = null)
    {
        // disable CSRF
        parent::__construct($defaults, $options, false);
    }
    // Sign Up form

    public function configure()
    {

    // Set the form fields for SignIn

        $this->setWidgets(array(
            'firstname' => new sfWidgetFormInput(),
            'lastname'  => new sfWidgetFormInput(),
            // 'username'  => new sfWidgetFormInput(),
            'email'     => new sfWidgetFormInput(),
            'password'  => new sfWidgetFormInputPassword(),
            'mobile_phone' => new sfWidgetFormInput(),
            'email_newsletter' => new sfWidgetFormInputCheckbox(array(), array()),
        ));

        // Set the Labels for specific fields
        $this->widgetSchema->setLabels(array(
            'firstname' => 'First name:',
            'lastname' => 'Last name:',
            // 'username' => 'Username:',
            'password' => 'Password:',
            'email' => 'Email:',
            'mobile_phone' => 'Mobile phone:'
        ));

        $this->setDefault('email_newsletter', 1);

        // Use Validators to check if the data are correct
        $this->setValidators(array(
            'firstname' => new sfValidatorRegex(array(
                'required' => true,
                'pattern' => '#^[\p{Latin}[A-Za-z\s-]+$#'
            ), array('invalid' => 'Please use only alphabetical characters')),
            'lastname'  => new sfValidatorRegex(array(
                'required' => true,
                'pattern' => '#^[\p{Latin}[A-Za-z\s-]+$#'
            ), array('invalid' => 'Please use only alphabetical characters')),
            // 'username' => new sfValidatorDoctrineUnique(array(
            //     'model' => 'User',
            //     'column' => 'username',
            // ), array(
            //     'required' => 'This field is required',
            //     'invalid' => 'This name is already in use'
            // )),
            'password'  => new sfValidatorString(array('min_length' => '6')),
            'email'     => new myValidatorUniqueEmail(array(), array(
                'required' => 'This field is required',
                'invalid'  => 'This is not valid e-mail address',
                'unique'   => 'This e-mail is already in use'
            )),
            'mobile_phone' => new sfValidatorRegex(array(
                    'pattern'  => '/^\+?\d{6,15}$/',
                    'required' => false
                ), array(
                    'invalid'  => 'Invalid phone number %value% %clean% %pattern%.'
                )),
            'email_newsletter' => new sfValidatorBoolean(array("required" => false))
        ));

        $this->mergePostValidator(new sfValidatorCallback(array('callback' => array($this, 'checkBots'))));

        // Set the form names (posted) as an array

        $this->widgetSchema->setNameFormat('register[%s]');

        $formatter = $this->getWidgetSchema()->getFormFormatter();
        $formatter->setErrorListFormatInARow('<small>%errors%</small>');
        $formatter->setErrorRowFormatInARow('%error% ');

    }

    public function checkBots($validator, $values)
    {
        if ($values['firstname'] == $values['lastname']) {
            $error = new sfValidatorError($validator, 'invalid', array('values' => $values));
            throw new sfValidatorErrorSchema($validator, array('firstname' => $error));
        }

        return $values;
    }
}
