<?php


class RequestRentalForm extends BaseForm {

    public function configure()
    {
    	$rental = $this->getOption('rental');

    	foreach ($rental->get('RentalPickup') as $pickup) {
    		$pickups[$pickup->get('owner_pickup_branch_id')] = $pickup->get('Locations')->get('name');
    	}

        $this->widgetSchema['driver_firstname'] = new sfWidgetFormInput();
        $this->widgetSchema['driver_lastname'] = new sfWidgetFormInput();
        $this->widgetSchema['driver_email'] = new sfWidgetFormInput();
        $this->widgetSchema['driver_mobile_phone'] = new sfWidgetFormInput();
        $this->widgetSchema['driver_comment'] = new sfWidgetFormTextarea();

        $this->widgetSchema['insurance']    = new sfWidgetFormInputCheckbox();
        $this->widgetSchema['pickup'] 		= new sfWidgetFormChoice(array('choices' => $pickups));
        $this->widgetSchema['pickup_date']  = new tcWidgetFormInputDate();
        $this->widgetSchema['dropoff_date'] = new tcWidgetFormInputDate();


        $this->widgetSchema['pickup_time']  = new sfWidgetFormChoice(array('choices' => $this->getPickupTimes()));
        $this->widgetSchema['dropoff_time'] = new sfWidgetFormChoice(array('choices' => $this->getDropoffTimes()));

        $this->validatorSchema['driver_firstname'] = new sfValidatorString();
        $this->validatorSchema['driver_lastname'] = new sfValidatorString();
        $this->validatorSchema['driver_email'] = new sfValidatorString();
        $this->validatorSchema['driver_mobile_phone'] = new sfValidatorString();
        $this->validatorSchema['driver_comment'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['pickup_time']  = new sfValidatorInteger(array('required' => false));
        $this->validatorSchema['dropoff_time'] = new sfValidatorInteger(array('required' => false));
        
        $this->validatorSchema['pickup_date'] = new sfValidatorDate(array(
            'date_format' => '~(?P<day>\d{2})-(?P<month>\d{2})-(?P<year>\d{4})~'
        ));

        $this->validatorSchema['dropoff_date'] = new sfValidatorDate(array(
            'date_format' => '~(?P<day>\d{2})-(?P<month>\d{2})-(?P<year>\d{4})~'
        ));

        $this->validatorSchema['pickup'] = new sfValidatorChoice(['choices' => array_keys($pickups)]);
        $this->validatorSchema['insurance'] = new sfValidatorBoolean(['required' => false]);

        $this->widgetSchema->setNameFormat('request[%s]');
        //$this->mergeUserDetailsForm();


        $this->mergePostValidator(new sfValidatorSchemaCompare(
            'pickup_date',
            sfValidatorSchemaCompare::LESS_THAN_EQUAL,
            'dropoff_date',
            array(),
            array(
                'invalid' => 'Dropoff date can\'t be before the pickup date'
            )
        ));
    }

    public function hasMissingUserDetails()
    {
        return false;

        // (isset($this['user']['firstname']) || isset($this['user']['lastname']) || isset($this['user']['license']) || isset($this['user']['date_of_birth']) || isset($this['user']['sms_number_verification']) || isset($this['user']['email_number_verification']) || isset($this['user']['mobile_phone']));
    }

    public function renderHiddenFields()
    {
        $output = '';
        foreach ($this->getFormFieldSchema() as $name => $field)
        {
            if ($field->isHidden())
            {
                $output .= $field->render();
            }
        }

        return $output;
    }

    public function mergeUserDetailsForm(){
        if (!$this->getOption('user')) {
            return;
        }
    	$form = new  UserDetailsForm(array(), array(
            'user' => $this->getOption('user'),
            'phone_verification'  => false,
            'email_notifications' => true,
            'mobile_phone'        => true,
            'country'             => 'nz',
            'min_age'             => 21
        ), false);

        $this->embedForm('user', $form);
        $this->userDetailsForm = $form;
    }

    public function getNbDays()
    {
        $func = $this->isBound() ? 'getValue' : 'getDefault';
        $pickup = new DateTime($this->$func('pickup_date') ?: null);
        $dropoff = new DateTime($this->$func('dropoff_date') ?: null);
        $nbdays = 0;

        if ($pickup <= $dropoff) {
            $extraDay = $this->getOption('rental')->get('OwnerVehicleType')->get('rental_type') == 'day' ? 1 : 0;
            $diff = $dropoff->diff($pickup);
            $nbdays = $diff->days + $extraDay;
        }

        return $nbdays;
    }


    private function getTimes($earliest, $latest)
    {
      sfLoader::loadHelpers(array('Date', 'Transfercar'));
      $earliest = $earliest ? $earliest : 34200;
      $latest = $latest ? $latest : 59400;
      $time  = strtotime('today');
      $times = array('' => '');

      for ($seconds = $earliest; $seconds <= $latest; $seconds+=1800)
      {
        $times[$seconds] = tc_format_date($seconds, 'hours');
      }

      return $times;
    }


    private function getPickupTimes()
    {
        $rental = $this->getOption('rental');

        $earliest = 8*3600;
        $latest = 18*3600;

        if (!$this->getDefault('pickup_time'))
        {
            $this->setDefault('pickup_time', 36000);
        }

        return $this->getTimes((int) $earliest, (int) $latest);
    }

    private function getDropoffTimes()
    {
        $rental = $this->getOption('rental');

        $earliest = 8*3600;
        $latest = 18*3600;

        if (!$this->getDefault('dropoff_time'))
        {
            $this->setDefault('dropoff_time', 36000);
        }

        return $this->getTimes((int) $earliest, (int) $latest);
    }

    private function parseTime($time)
    {
        $hours = floor($time / 3600);
        $minutes = round(($time % 3600)/60);

        return array($hours, $minutes);
    }

    private function calculateOldPrice() {
    	$func = $this->isBound() ? 'getValue' : 'getDefault';
    	$insurance = (bool) $this->$func('insurance');
    	$nbdays = $this->getNbDays();
    	$insurance_price = $insurance ? $this->getOption('rental')->get('daily_insurance_rate') : 0;
        $monthdays = $this->getOption('rental')->get('Owner')->get('company') == 'Thrifty NZ' ? 31 : 30;

    	if ($nbdays < 7) {
        	$rental_price = ($nbdays *  $this->getOption('rental')->get('daily_rental_price'));
        } else if ($nbdays < $monthdays) {
        	$rental_price = floor($nbdays/7) * $this->getOption('rental')->get('weekly_rental_price') + $nbdays % 7 * $this->getOption('rental')->get('daily_rental_price');
        	$insurance_price = $insurance ? $this->getOption('rental')->get('weekly_insurance_rate') : 0;
        } else {
        	$rental_price = floor($nbdays / $monthdays) * $this->getOption('rental')->get('monthly_rental_price') + $nbdays % $monthdays * $this->getOption('rental')->get('extra_day_rental_price');
        	$insurance_price = $insurance ? $this->getOption('rental')->get('monthly_insurance_rate') : 0;
        }

        return [
            'total' => $rental_price + $nbdays * $insurance_price,
            'total_rental' => $rental_price,
            'total_insurance' => $nbdays * $insurance_price,
            'days' => $nbdays,
            'details'=> [['i' => 0, 'nb_days' => $nbdays, 'rate' => $rental_price/$nbdays]]
        ];
    }

    public function getPrice()
    {
        $rental = $this->getOption('rental');
        if (!$rental->get('rates')) {
            return $this->calculateOldPrice()['total'];
        }

        return $this->getPriceDetails()['total'];
    }

    public function getPriceDetails()
    {
        $rental = $this->getOption('rental');
        if (!$rental->get('rates')) {
            return $this->calculateOldPrice();
        }

        $func = $this->isBound() ? 'getValue' : 'getDefault';
        $insurance = $this->$func('insurance');
        $night = $rental->get('OwnerVehicleType')->get('rental_type') != 'day';

        return $this->calculatePrice(
            $this->$func('pickup_date'), 
            $this->$func('dropoff_date'), 
            $insurance, $night
        );
    }

    private function calculatePrice($from_date, $to_date, $insurance = false, $night = false)
    {
        $pickup_date  = new \DateTime($from_date);
        $dropoff_date = new \DateTime($to_date);
        $nbdays = $dropoff_date->diff($pickup_date)->days;
        $monthdays = $this->getOption('rental')->get('Owner')->get('company') == 'SIXT NZ' ? 31 : 30;
        $rate_type = ($nbdays < $monthdays ? 'weekly' : 'monthly').'_rate';
        $rate_divider = $rate_type == 'weekly_rate' ? 7 : $monthdays;

        $rates = $this->getOption('rental')->get('rates');

        $pricing = [];
        $i = 0;

        do {
            if ($i >= count($rates)) {
                $rate = end($rates);
                $rate['to_date'] = $dropoff_date->format('Y-m-d');
            } else {
                $rate = $rates[$i];
            }

            $daily_rate = $rate[$rate_type]/$rate_divider;
            $rate_to_date = new DateTime($rate['to_date']);


            if ($pickup_date > $rate_to_date) {
                $i++;
                continue;
            }

            // what's the maximum date for this date range
            $to_date = $dropoff_date > $rate_to_date ? $rate_to_date : $dropoff_date;
            // 
            $nb_days = max(1, $to_date->diff($pickup_date)->days) + (int)!$night;

            if ($nb_days >= $monthdays && empty($pricing) && $insurance) {
                $pricing[] = [
                    'i' => $i,
                    'from' => $pickup_date->format('Y-m-d'),
                    'to' => ($pickup_date->add(new DateInterval('P'.$monthdays.'D')))->format('Y-m-d'),
                    'nb_days' => $monthdays,
                    'rate' => $daily_rate,
                    // the 1st month always use the weekly price for insurance, subsequent months will get the discount
                    'insurance_rate' => $insurance ? $rate['insurance_weekly_rate'] : 0
                ];

                $nb_days2 = $nb_days - $monthdays;
                $nb_days -= $monthdays;

                if ($nb_days2 > 0) {
                    $pricing[] = [
                        'i' => $i.'.1',
                        'from' => ($pickup_date->add(new DateInterval('P'.((int)!$night).'D'))->format('Y-m-d')),
                        'to' => $to_date->format('Y-m-d'),
                        'nb_days' => ($d=max(0, $to_date->diff($pickup_date)->days + (int)!$night)),
                        'rate' => $daily_rate,
                        // the 1st month always use the weekly price for insurance',' subsequent months will get the discount
                        'insurance_rate' => $insurance ? $rate['insurance_weekly_rate'] : 0
                    ];
                    $nb_days-=$d;
                    $pickup_date->add(new DateInterval('P'.$d.'D'));
                }
            } else {
                $pricing[] = [
                    'i' => $i,
                    'from' => $pickup_date->format('Y-m-d'),
                    'to' => $to_date->format('Y-m-d'),
                    'nb_days' => $nb_days,
                    'rate' => $daily_rate,
                    // the 1st month always use the weekly price for insurance',' subsequent months will get the discount
                    'insurance_rate' => $insurance ? $rate['insurance_'.$rate_type] : 0
                ];
            }

            $pickup_date = $pickup_date->add(new DateInterval('P'.$nb_days.'D'));
            $i++;
        } while ($dropoff_date > $rate_to_date);


        $total_price = $total_days = 0;
        foreach ($pricing as $value) {
            $total_rental += $value['nb_days'] * $value['rate'];
            $total_insurance += $value['nb_days'] * $value['insurance_rate'];
            $total_days  += $value['nb_days'];
        }

        return [
            'pickup_date' => $from_date,
            'dropoff_date' => $to_date->format('Y-m-d'),
            'total' => $total_rental + $total_insurance, 
            'total_insurance' => $total_insurance, 
            'total_rental' => $total_rental, 
            'days' => $total_days, 
            'details' => $pricing
        ];
    }

    public function save(){
        $this->saveUser();

    	$enquiry = new Enquiry();
    	$enquiry->set('Rental', $this->getOption('rental'));
    	// $enquiry->set('driver_id', $this->getOption('user')->getId());
        $enquiry->set('driver_firstname',  $this->getValue('driver_firstname'));
        $enquiry->set('driver_lastname',  $this->getValue('driver_lastname'));
        $enquiry->set('driver_email',  $this->getValue('driver_email'));
        $enquiry->set('driver_mobile_phone',  $this->getValue('driver_mobile_phone'));
        $enquiry->set('driver_comment',  $this->getValue('driver_comment'));
    	$enquiry->set('owner_pickup_branch_id', $this->getValue('pickup'));
    	$enquiry->set('pickup_date', $this->getValue('pickup_date'));
    	$enquiry->set('dropoff_date', $this->getValue('dropoff_date'));
        $enquiry->set('pickup_time', $this->getValue('pickup_time'));
        $enquiry->set('dropoff_time', $this->getValue('dropoff_time'));
    	$enquiry->set('insurance', $this->getValue('insurance'));
    	$enquiry->set('price', $this->getPrice());
        $enquiry->set('created_at', date('Y-m-d H:i:s'));
        $enquiry->set('updated_at', date('Y-m-d H:i:s'));
    	$enquiry->save();

        //

        return $enquiry;
    }

    private function saveUser()
    {
        if ($this->hasMissingUserDetails())
        {
            $this->userDetailsForm->bind($this->getValue('user'));
            $this->userDetailsForm->save();
        }
    }
}

class tcWidgetFormInputDate extends sfWidgetFormInput {
    public function render($name, $value = null, $attributes = array(), $errors = array())
    {
        if ($value) {
            $date = new DateTime($value);
            $value = $date->format('d-m-Y');
        }

        return parent::render($name, $value, $attributes, $errors);

    }
}
