<?php

use Transfercar\Mailchimp\MailchimpApi;

class UserDetailsForm extends BaseForm
{
    public function configure()
    {
        $user = $this->getOption('user');
        $user->refresh();

        if (!$user->get('license')) {
            // country list
            sfLoader::loadHelpers(array('Array'));
            $countries    = COUNTRY_LIST();
            $countries[''] = 'Select country';
            $countryCodes = array_keys($countries);
            unset($countryCodes[0]);

            $this->widgetSchema['license'] = new sfWidgetFormSelect(array(
                'choices' => $countries
            ));
            $this->validatorSchema['license'] = new sfValidatorChoice(array(
                'choices' => $countryCodes
            ));
        }

        $checkName = function($name) {
            return $name && preg_match('#^[\p{Latin}[A-Za-z\s-]+$#', $name);
        };

        if (!$checkName($user->get('firstname')) || !$checkName($user->get('lastname'))) {
            $this->widgetSchema['firstname'] = new sfWidgetFormInput();
            $this->widgetSchema['lastname'] = new sfWidgetFormInput();

            $this->validatorSchema['firstname'] = new sfValidatorRegex(array(
                'required' => true,
                'pattern' => '#^[\p{Latin}[A-Za-z\s-]+$#'
            ), array('invalid' => 'Please use only alphabetical characters'));

            $this->validatorSchema['lastname'] = new sfValidatorRegex(array(
                'required' => true,
                'pattern' => '#^[\p{Latin}[A-Za-z\s-]+$#'
            ), array('invalid' => 'Please use only alphabetical characters'));

            $this->setDefault('firstname', '');
            $this->setDefault('lastname', '');
        }

        if (!$user->get('date_of_birth')) {
            $years = range(date('Y'), date('Y') - 120);
            $this->widgetSchema['date_of_birth'] = new sfWidgetFormI18nDate(array(
                'culture' => 'en_NZ',
                'years'  => array_combine($years, $years),
                'empty_values' => array(
                    'day'   => 'Day',
                    'month' => 'Month',
                    'year'  => 'Year'
                ),
                'format' => '<div class="col-4 col-lg-2">%day%</div><div class="col-4 col-lg-2">%month%</div><div class="col-4 col-lg-2">%year%</div>',
            ), array(
                'minAge' => $this->getOption('min_age')
            ));
            $this->validatorSchema['date_of_birth'] = new sfValidatorDate(array(
                'max' => $this->getOption('min_age') ? strtotime(sprintf('-%s years', $this->getOption('min_age'))) : null
            ), array(
                'max' => 'You must be at least '.$this->getOption('min_age').' to request this listing'
            ));

        }

        if ($this->getOption('phone_verification', true) && !$user->hasPhoneVerified())
        {
            $this->widgetSchema['sms_number_verification'] = new sfWidgetFormInput();
            $this->validatorSchema['sms_number_verification'] = new sfValidatorChoice(array('choices' => array($user->get('sms_number'))));
        }

        // existing users before we implemented the  must confirm their
        if (($this->getOption('mobile_phone', false) && !$user->get('mobile_phone')) || ($this->getOption('phone_verification', true) && $user->get('user_status') == 'active' && !$user->hasPhoneVerified() && strlen($user->get('sms_number')) != 6))
        {
            $smsForm = new SmsChangeForm();
            $this->widgetSchema['mobile_phone'] = new sfWidgetFormInput(array(), array('data-csrf-token' => $smsForm->getDefault('_csrf_token')));
            $this->validatorSchema['mobile_phone'] = new sfValidatorString();
            $this->setDefault('mobile_phone', $user->get('mobile_phone'));
        }

        if ($user->get('user_status') == 'inactive') {
            $this->widgetSchema['email_number_verification'] = new sfWidgetFormInput();
            $this->validatorSchema['email_number_verification'] = new sfValidatorChoice(array(
                'choices' => array(
                    $user->get('activation_code'),
                    $user->get('new_email_confirm')
                )
            ));
        }

        $this->widgetSchema['sms_notify_update_req'] = new sfWidgetFormInputCheckbox(array(), array());
        $this->validatorSchema['sms_notify_update_req'] = new sfValidatorBoolean(array("required" => false));

        if ($this->getOption('email_notifications'))
        {
            $this->widgetSchema['email_newsletter'] = new sfWidgetFormInputCheckbox(array(), array());
            $this->validatorSchema['email_newsletter'] = new sfValidatorBoolean(array("required" => false));
            $this->widgetSchema['email_notify_new_reloc'] = new sfWidgetFormInputCheckbox(array(), array());
            $this->validatorSchema['email_notify_new_reloc'] = new sfValidatorBoolean(array("required" => false));
        }

        if (!$user->get('source_information')) {
            $choices = [
                '',
                'Social media',
                'Email or newsletter',
                'Blog or article',
                'Search engine',
                'Word of mouth',
                'other'
            ];

            $this->widgetSchema['source_information'] = new sfWidgetFormSelect(array('choices' => array_combine($choices, $choices)));
            $this->validatorSchema['source_information'] = new sfValidatorChoice(array('required' => false, 'choices' => $choices));
        }

        $this->widgetSchema->setNameFormat('user_details[%s]');
        $this->validatorSchema->setOption('allow_extra_fields', true);
    }

    public function hasMissingDetails()
    {
        return isset($this['firstname']) || isset($this['lastname']) || isset($this['license']) || isset($this['date_of_birth']) || isset($this['sms_number_verification']) || isset($this['email_number_verification']);
    }

    public function save()
    {
        $user = $this->getOption('user');

        if ($this->getValue('firstname')) {
            $user->set('firstname', $this->getValue('firstname'));
        }

        if ($this->getValue('lastname')) {
            $user->set('lastname', $this->getValue('lastname'));
        }

        if ($this->getValue('username')) {
            $user->set('username', $this->getValue('username'));
        }

        if ($this->getValue('password')) {
            $user->set('password', sha1($this->getValue('password')));
        }
        if ($this->getValue('mobile_phone')) {
            $user->set('mobile_phone', $this->getValue('mobile_phone'));
        }

        if ($this->getValue('license')) {
            $user->set('license', $this->getValue('license'));
        }

        if ($this->getValue('date_of_birth')) {
            $user->set('date_of_birth', $this->getValue('date_of_birth'));
        }

        if ($this->getValue('sms_number_verification')) {
            $user->set('sms_number', null);
            $user->set('sms_number_confirmed', true);
        }

        if ($this->getValue('email_number_verification')) {
            $user->confirmEmail();
        }

        if ($this->getValue('sms_notify_update_req')) {
            $user->set('sms_notify_update_req', $this->getValue('sms_notify_update_req'));
        }

        if ($this->getValue('email_newsletter')) {
            $country = $this->getOption('country');
            $user->set('email_newsletter', $this->getValue('email_newsletter'));
            $user->set('email_country_newsletter', $country);

            $mc = new MailchimpApi();
            $mc->subscribe(sfConfig::get('app_mailchimp_'.$country.'_list_apikey'), $user->get('email'), $user->get('firstname'), $user->get('lastname'));
        }

        if ($this->getValue('email_notify_new_reloc')) {
            $user->set('email_notify_new_reloc', $this->getValue('email_notify_new_reloc'));
        }


        if ($this->getValue('source_information')) {
            $user->set('source_information', $this->getValue('source_information'));
        }

        $user->save();

        return $user;
    }
}
