<?php

class AdminCmsPageHeadElementForm extends BaseForm
{
    public function configure()
    {
        $type = $this->getOption('type');
        $id   = sprintf($this->getOption('id', 'new_%s'), uniqid());

        $this->setWidgets(array(
            'type' => new sfWidgetFormInputHidden()
        ));

        $propertiesForm = new BaseForm();

        switch ($type) {
            case 'title':
                $propertiesForm->widgetSchema['content'] = new sfWidgetFormTextarea();
                break;
            case 'meta_image':
                $propertiesForm->widgetSchema['name']    = new sfWidgetFormInputHidden();
                $propertiesForm->widgetSchema['content'] = new sfWidgetFormInput(array(), array('class' => $type));
                $this->defaults['properties']['name'] = 'og:image';
                $propertiesForm->widgetSchema->setLabel('content', ' ');
                break;
            case 'meta':
                $hidden = isset($this->defaults['properties']['name']) && $this->defaults['properties']['name'] != '';
                $propertiesForm->widgetSchema['name'] = $hidden ? new sfWidgetFormInputHidden() : new sfWidgetFormInput();
                $propertiesForm->widgetSchema['content'] = new sfWidgetFormTextarea(array(), array('rows' => 1, 'class' => $type));
                $propertiesForm->widgetSchema->setLabel('content', $hidden ? ' ' : 'Content');
                $propertiesForm->widgetSchema->setHelp('name', 'Tip: you can pick a name from <a href="http://code.lancepollard.com/complete-list-of-html-meta-tags/">this list</a>.');
                break;
            case 'hreflang':
                $propertiesForm->widgetSchema['hreflang'] = new sfWidgetFormInput();
                $propertiesForm->widgetSchema['href'] = new sfWidgetFormInput();
                break;
            case 'link':
                $propertiesForm->widgetSchema['rel'] = new sfWidgetFormInput();
                $propertiesForm->widgetSchema['href'] = new sfWidgetFormInput();
                break;
        }

        $oDecorator = new sfWidgetFormSchemaFormatterDiv($this->getWidgetSchema());
        $this->getWidgetSchema()->addFormFormatter('div', $oDecorator);
        $this->getWidgetSchema()->setFormFormatterName('div');

        $oDecorator = new sfWidgetFormSchemaFormatterDiv($propertiesForm->getWidgetSchema());
        $propertiesForm->getWidgetSchema()->addFormFormatter('div', $oDecorator);
        $propertiesForm->getWidgetSchema()->setFormFormatterName('div');

        $propertiesForm->setDefaults($this->getDefault('properties'));
        $this->embedForm('properties', $propertiesForm);
        $this->widgetSchema->setNameFormat('cms_page[HeadElements]['.$id.'][%s]');

        $this->setDefault('type', $type);

        $this->widgetSchema->setLabel('properties', ' ');
    }
}

class sfWidgetFormSchemaFormatterDiv extends sfWidgetFormSchemaFormatter
{
  protected
    $rowFormat       = "<div class=\"%is_error%\">\n  %label%\n  %error%\n    %field%\n%help%\n%hidden_fields%</div>\n",
    $errorRowFormat  = "%errors%\n",
    $helpFormat      = '<div class="form-help">%help%</div>',
    $decoratorFormat = "\n  %content%";

  public function formatRow($label, $field, $errors = array(), $help = '', $hiddenFields = null)
  {
    return strtr(parent::formatRow($label, $field, $errors, $help, $hiddenFields), array(
      '%is_error%'    => (count($errors) > 0) ? ' field_error' : '',
      //'%is_required%' => $field,
    ));
  }
}