<?php

use Transfercar\Mandrill\TcMandrill;
use Transfercar\Stripe\TcStripeForm;
use Transfercar\Stripe\TcStripe;

class ChargeFeeForm extends sfForm
{
    private $fee_error;

    public function configure()
    {
        $this->setWidgets(array(
            'description' => new sfWidgetFormInput()
        ));

        $this->setValidators(array(
            'description' => new sfValidatorString()
        ));

        $request = $this->getOption('request');

        $this->widgetSchema->setNameFormat('chargefee[%s]');
    }

    public function chargeMoney($request, $username)
    {
        if ($this->chargeCreditCard($request, $username))
        {
            $this->sendNotificationEmail($request);

            $request->addInternalStatus(Request::INTERNAL_STATUS_CHARGED, $this->getValue('description'));
            // remove the cc failed status if the previous cc processing failed
            $request->removeInternalStatus(Request::INTERNAL_STATUS_CC_FAILED);
            $request->save();
            return true;
        }

        $request->addInternalStatus(Request::INTERNAL_STATUS_CC_FAILED, 'Failed to process credit card');
        $request->save();

        return false;
    }

    public function sendNotificationEmail($request)
    {
        $domain = $request->getCountryDomain();
        $site = sfConfig::get('app_domain_' . $domain);

        $email_opt_link = 'http://' . $site . '/emailUnsubscribe/' . $request->get('Driver')->get('id') . '/update_request';
        $template_data = array(
            'driver_full_name' => $request->get('Driver')->getFullName(),
            'firstname'        => $request->get('Driver')->get('firstname'),
            'tcid'             => $request->getIncrementedId(),
            'cc_details'       => $request->get('CreditCard')->card_description(),
            'email_opt_link'   => $email_opt_link,
            'fee_amount'       => $request->get('no_show_fee_amount')
        );

        $mandrill = new TcMandrill($domain);
        $mandrill->sendTemplate(
            'Fee charged for no-show or late cancellation {{TCID}}',
            $request->get('Driver')->getEmailToParams(),
            $template_data,
            'fee-charged'
        );
    }

    public function getFeeError()
    {
        return $this->fee_error;
    }

    private function chargeCreditCard($request, $username)
    {
        if (strpos($request->get('CreditCard')->get('holder_name'), 'src_') === false) {
            return TcFlo2cash::chargeCreditCard($request, $username);
        }
        
        //Get accepted response
        $listing = $request->get('Listing');

        $form = new TcStripeForm([], [
            'user_id' => $request->get('driver_id'),
            'currency' => strtolower($listing->getCountryDomain().'d')
        ], false);
        $values = [
            'amount' => (float) $request->no_show_fee_amount > 0 ? (float) $request->no_show_fee_amount : 25
        ];

        $particular = substr($this->getValue('description'), 0, 22);
        $description = substr(sprintf('%s/%s/%s', $particular, $request->getIncrementedId(), $username), 0, 50);

        return $form->processPayment($description, $particular, $values, $request->CreditCard);
    }
}