<?php
/**
 * Form for edit user contact details
 *
 * @package transfercar
 * @subpackage form
 */
class contactDetailsForm extends transfercarForm
{
  public function configure()
  {
    sfLoader::loadHelpers(array('Array')); // we have countries list here

    // country list
    $countries    = COUNTRY_LIST();
    $countries[0] = ' ';
    $countryCodes = array_keys($countries);
    unset($countryCodes[0]);

    // location list
    $locations = Locations::locationList(sfContext::getInstance()->getUser()->getAttribute("country"), true, true, true);

    // years list (from current one down to 1900)
    $years = array();
    for($i = date('Y', time()); $i >= 1900; $i--){
        $years[$i] = $i;
    }

    $months = array(
      1  => 'January',
      2  => 'February',
      3  => 'March',
      4  => 'April',
      5  => 'May',
      6  => 'June',
      7  => 'July',
      8  => 'August',
      9  => 'September',
      10 => 'October',
      11 => 'November',
      12 => 'December'
    );

    // Set input fields
    // All fields have the same name as corresponding fields in database
    $this->setWidgets(array(
      'firstname'     => new sfWidgetFormInput(),
      'lastname'      => new sfWidgetFormInput(),
      'date_of_birth' => new sfWidgetFormInput(array('type' => 'date'), array(
        'type' => 'date'
      )),
      'mobile_phone'  => new sfWidgetFormInput(array(), array('minlength' => 8)),
      'license'       => new sfWidgetFormSelect(array('choices' => $countries)),
      'logo'          => new sfWidgetFormInputFile(),
    ));

    $this->widgetSchema->setNameFormat('profile[%s]');

    // Set labels
    $this->widgetSchema->setLabels(array(
      'firstname'     => 'first name',
      'lastname'      => 'last name',
      'date_of_birth' => 'date of birth',
      'mobile_phone'  => 'mobile phone',
      'license'       => 'drivers license',
    ));

    $messages = array('required' => 'This field is required', 'invalid' => 'Invalid value');

    $date_messages = array_merge($messages, array('max' => 'Date cannot be in the future'));
    $this->setValidators(array(
      'firstname'      => new sfValidatorString(array(), $messages),
      'lastname'       => new sfValidatorString(array(), $messages),
      'date_of_birth'  => new sfValidatorDate(array('max' => time()), $date_messages),
      'mobile_phone'   => new sfValidatorAnd(array(
          new sfValidatorCallback(array('callback' => array($this, 'cleanPhoneNumber'))),
          new sfValidatorMobilePhone(array(
            'required' => true
          ),
          array(
            'required' => 'This field is required',
            'invalid'  => 'The phone number is not valid. Please use numbers only and no spaces or +"'
          ))
      )),
      'license'        => new sfValidatorChoice(array(
        'choices' => $countryCodes, "required"=>true), $messages),
      'logo'    => new sfValidatorFile(array(
        'required' => false,
        'validated_file_class' => 'tcDriverLogoValidatedFile'
      ))
    ));

    $this->updateDefaultsFromUser($this->getOption('user'));
  }

  public function updateDefaultsFromUser(User $user = null)
  {
    if (!$user)
    {
      return;
    }

    // Get field list
    $field_names = $this->widgetSchema->getPositions();

    // For every field, if proper field in user object is set, set it as default value
    foreach($field_names as $field)
    {
        if($user->get($field))
        {
            $this->setDefault($field, $user->get($field));
        }
    }
  }

  public function update(User $user)
  {
    $this->saveThumbnail();
    $user->set('contact_updated_at', date('Y-m-d H:i:s'));

    if ($this->getValue('mobile_phone') != $user->get('mobile_phone'))
    {
      $user->set('sms_number_confirmed', false);
    }

    $user->fromArray($this->getValues());
    $user->save();
  }

  public function saveThumbnail()
  {
    if ($logo = $this->getValue('logo')) {
        $path = $logo->saveThumbnail();
        $this->values['logo'] = $path;

        if ($this->getOption('user')->get('logo')) {
            $filename = str_replace('/uploads', '', $this->getOption('user')->get('logo'));
            unlink(sfConfig::get('sf_upload_dir').$filename);
        }
    } else {
        unset($this->values['logo']);
    }
  }

  public function cleanPhoneNumber($validator, $value)
  {
    return preg_replace('/[a-z\s]+/', '', $value);
  }
}
