<?php

/**
 * AdListing form.
 *
 * @package    form
 * @subpackage AdListing
 * @version    SVN: $Id: sfPropelFormTemplate.php 6174 2007-11-27 06:22:40Z fabien $
 */
class AdListingForm extends BaseAdListingForm
{
  private $routeForm;

  public function configure()
  {
    unset(
      $this['id'],
      $this['status'],
      $this['advertiser_id'],
      $this['nb_clicks'],
      $this['nb_views'],
      $this['ad_listing_routes_list'],
      $this['url_parameters_defaults'],
      $this['country_id']
    );

    $this->routeForm = new BaseForm();

    if ($this->getObject()->get('Routes')->count() > 0)
    {
      foreach ($this->getObject()->get('Routes') as $id => $route)
      {
        $this->routeForm->embedForm($id, new AdListingRouteForm($route, array('country' => $this->getOption('country'))));
      }
    } else {
      $this->routeForm->embedForm('new', new AdListingRouteForm(null, array('country' => $this->getOption('country'))));
    }
    $this->embedForm('routes', $this->routeForm);

    $this->embedForm('url_parameters_defaults', new AdListingUrlParametersForm($this->getObject()->get('url_parameters_defaults')));

    $this->widgetSchema['travellers'] = new sfWidgetFormSelect(array(
      'choices' => array(
        '' => '',
        1 => '1 people',
        2 => '2 people',
        3 => '3 people',
        4 => '4 people',
        5 => '5 people',
        6 => '6+ people',
      )
    ));

    $this->widgetSchema['transmission_type'] = new sfWidgetFormSelect(array(
      'choices' => array_merge(array('' => ''), OwnerVehicleTypeTable::getTransmissionTypes())
    ));

    $this->widgetSchema['image'] = new sfWidgetFormInputFile();
    $this->validatorSchema['image'] = new sfValidatorFile(array(
            'required' => false,
            'validated_file_class' => 'tcAdvertiserLogoValidatedFile'
    ));

    $this->widgetSchema['vehicle_type_id']->setOption('criteria',
      Doctrine::getTable('VehicleType')
      ->createQuery()
      ->addWhere('country_id = ?', $this->getOption('country'))
      ->orderBy('description')
    );

    $this->widgetSchema['notes'] = new sfWidgetFormTextarea();

    $this->widgetSchema['destination_url'] = new sfWidgetFormTextarea(array(), array('rows' => 1));
    $this->validatorSchema['destination_url'] = new sfValidatorUrl(array(), array(
      'invalid' => 'Please enter a valid URL'
    ));

    $formatter = $this->getWidgetSchema()->getFormFormatter();
    $formatter->setErrorListFormatInARow('%errors%');
    $formatter->setErrorRowFormatInARow('<div>%error%</div> ');
  }

  public function bind(array $taintedValues = null, array $taintedFiles = null)
  {
    $this->routeForm = new BaseForm();

    foreach ($taintedValues['routes'] as $key => $value)
    {
      if (!isset($this['routes'][$name]))
      {
        $this->routeForm->embedForm((string) $key, new AdListingRouteForm(null, array('country' => $this->getOption('country'))));
      }
    }

    $this->embedForm('routes', $this->routeForm);

    parent::bind($taintedValues, $taintedFiles);
  }

  protected function doSave($con = null)
  {
    if (is_null($con))
    {
      $con = $this->getConnection();
    }

    $this->updateObject();
    $this->updateRoutes($this->object, $con);
    $this->object->save($con);
  }

  public function updateObject()
  {
    if ($image = $this->getValue('image')) {
        $path = $image->saveThumbnail();
        $this->values['image'] = $path;

        if ($this->getObject()->get('image')) {
            $filename = str_replace('/uploads', '', $this->getObject()->get('image'));
            unlink(sfConfig::get('sf_upload_dir').$filename);
        }
    } else {
        unset($this->values['image']);
    }

    $adListing = parent::updateObject();
    $adListing->set('status', AdListing::LISTED);

    return $adListing;
  }

  private function updateRoutes(AdListing $adListing, $con)
  {
    $this->object->get('AdListingRoutes')->delete($con);
    $this->object->get('AdListingRoutes')->clear();
    $values = $this->getValues();

    foreach ($this->routeForm->getEmbeddedForms() as $name => $form)
    {
      $route = $form->getObject();
      $route->fromArray($values['routes'][$name]);
      $route = $this->getOrCreateRoute($route);
      $adListingRoutes = new AdListingRoutes();
      $adListingRoutes->set('AdListingRoute', $route);
      $adListing->get('AdListingRoutes')->add($adListingRoutes);
    }
  }

  private function getOrCreateRoute($route)
  {
    $existingRoute = Doctrine::getTable('AdListingRoute')->createQuery()
      ->where('pickup_location_id = ?',  $route->get('pickup_location_id'))
      ->addWhere('dropoff_location_id = ?', $route->get('dropoff_location_id'))
      ->limit(1)
      ->fetchOne();

    return $existingRoute ?: $route;
  }

}

class AdListingUrlParametersForm extends BaseForm
{
  public function configure()
  {
    $choices = array(
      '' => '',
      0 => '0 day',
      1 => '+1 days',
      2 => '+2 days',
      3 => '+3 days',
      4 => '+4 days',
      5 => '+5 days',
      6 => '+6 days',
      7 => '+7 days'
    );

    $dochoices = $choices;
    for ($i = 8; $i <= 20; $i++)
    {
      $dochoices[$i] = '+'.$i.' days';
    }

    $this->setWidgets(array(
      'pickup_date' => new sfWidgetFormSelect(array('choices' => $choices)),
      'dropoff_date' => new sfWidgetFormSelect(array('choices' => $dochoices))
    ));



    $this->setValidators(array(
      'pickup_date' => new sfValidatorChoice(array('choices' => range(0, 5), 'required' => false)),
      'dropoff_date' => new sfValidatorChoice(array('choices' => range(0, 20), 'required' => false))
    ));
  }
}

