<?php

/**
 * NotifyRoute form.
 *
 * @package    form
 * @subpackage NotifyRoute
 * @version    SVN: $Id: sfPropelFormTemplate.php 6174 2007-11-27 06:22:40Z fabien $
 */
class NotificationRouteForm extends BaseForm
{
  public function configure()
  {
    $format = '<div class="row"><div class="col-xs-4">%month%</div><div class="col-xs-4">%day%</div><div class="col-xs-4">%year%</div></div>';
    $this->setWidgets(array(
        'email' => new sfWidgetFormInput(),
        'pickup_date' => new sfWidgetFormInput(array(), array('class' => 'tc-datepicker')),
        'dropoff_date' => new sfWidgetFormInput(array(), array('class' => 'tc-datepicker')),
        'pickup' => new sfWidgetFormInputHidden(),
        'dropoff' => new sfWidgetFormInputHidden(),
        'source' => new sfWidgetFormInputHidden(),
    ));


    $this->setValidators(array(
        'email' => new sfValidatorEmail(),
        'pickup' => new sfValidatorString(array('required' => false)),
        'dropoff' => new sfValidatorString(array('required' => false)),
        'source' => new sfValidatorString(array('required' => false)),
        'pickup_date' => new sfValidatorDate(array(
          'required' => false, 
          'date_format' => '~(?P<day>\d{2})[-/](?P<month>\d{2})[-/](?P<year>\d{4})~', 
          'date_format_error' => 'd/m/Y'
        ), array('bad_format' => 'Invalid date')),
        'dropoff_date' => new sfValidatorDate(array(
          'required' => false, 
          'date_format' => '~(?P<day>\d{2})[-/](?P<month>\d{2})[-/](?P<year>\d{4})~', 
          'date_format_error' => 'd/m/Y'
        ), array('bad_format' => 'Invalid date'))
    ));

    if (!$this->getDefault('pickup_date')) {
      $this->setDefault('pickup_date', date('d-m-Y'));
    }



    $this->validatorSchema->setOption('allow_extra_fields', true);

    $this->mergePreValidator(new sfValidatorCallback(array('callback' => function($validator, $values){
      if (isset($values['dropoff_date']) && $values['dropoff_date']) {
        $this->mergePostValidator(new sfValidatorDateCompare('dropoff_date', 'greater_than', 'pickup_date'));
      }
    })));

    $formatter = $this->getWidgetSchema()->getFormFormatter();
    $formatter->setErrorListFormatInARow('<small>%errors%</small>');
    $formatter->setErrorRowFormatInARow('%error% ');
  }

  public function save()
  {
    $email = $this->getValue('email');

    $pickup  = $this->getLocationNode($this->getValue('pickup'));
    $dropoff = $this->getLocationNode($this->getValue('dropoff'));

    $account = Doctrine::getTable('NotificationUser')->findAccountByEmail($email);
    $account->set('country_id', $this->getOption('country'));
    $account->set('frequence', 'daily');

    if ($pickup || $dropoff)
    {
        $id = $this->getOption('id');
        $route = $id ? Doctrine::getTable('NotifyRoute')->find($id) : new NotifyRoute();
        $route->set('LocationFrom', $pickup ?: null);
        $route->set('LocationTo', $dropoff ?: null);
        $route->set('date_from', $this->getValue('pickup_date') ?: null);
        $route->set('date_to', $this->getValue('dropoff_date') ?: null);
        $route->set('alert_type', $this->getOption('alert_type', 'email'));
        $route->set('User', $account);
        $route->save();
    }
    else
    {
        $account->save();
    }
  }

  private function getLocationNode($slug)
  {
    if (!$slug) {
      return false;
    }

    return Doctrine::getTable('LocationNode')->findOneBySlug($slug);
  }
}