<?php
/**
 * Form for adding/editing listings
 *
 * @package transfercar
 * @subpackage form
 */
class editListingForm extends transfercarForm
{
   /**
     * Prepare form
     */
    public function configure()
    {
        // Load required helpers (we need push_to_begin() function)
        sfLoader::loadHelpers(array('Array'));

        // Get all data we need from db

        // Location names
        $locations = Locations::locationListFull();
        $locations_with_empty = $locations;
        $locations_with_empty = push_to_begin($locations, 'Please choose...');

        // Vehicle types
        $vehicles = VehicleType::typeList();
        $vehicles_with_empty = $vehicles;
        $vehicles_with_empty = push_to_begin($vehicles, 'Please choose...');

		// Owner names
        $owners = array();// Listing::ownerList();
        $owners_with_empty = $owners;
        $owners_with_empty = push_to_begin($owners, 'Please choose...');

        // Options for yes/no radiobuttons
        $yes_no_opts = array('choices' => array(1 => 'Yes', 0 => 'No'));

        // Fields
        $this->setWidgets(array(
		        'id' => new sfWidgetFormInput(array(), array(
                    'readonly' => 'readonly', 'style' => 'border: none',
                    'onfocus' => 'this.blur();')),
                'owner_id' => new tcWidgetFormAutocomplete(array(
                    'url' => $this->getOption('url'))),
                'pickup_location_id' => new sfWidgetFormSelect(array(
                    'choices' => $locations_with_empty)),
                'dropoff_location_id' => new sfWidgetFormSelect(array(
                    'choices' => $locations_with_empty)),
                'price' => new sfWidgetFormInput(array(), array(
                    'readonly' => 'readonly', 'style' => 'border: none',
                    'onfocus' => 'this.blur();')),
                'date_available_from' => new myWidgetFormDateWithPicker(),
                'date_return_by' => new myWidgetFormDateWithPicker(),
                'vehicle_type_id' => new sfWidgetFormSelect(array(
                    'choices' => $vehicles_with_empty)),
                'kms_allowed' => new sfWidgetFormInput(),
                'days_allowed' => new sfWidgetFormInput(),
                'paid_days_allowed' => new sfWidgetFormInput(),
                'price_per_day' => new sfWidgetFormInput(),
                'petrol_included' => new myWidgetFormRadio($yes_no_opts),
                'ferry_included' => new myWidgetFormRadio($yes_no_opts),
                'expenses_included' => new myWidgetFormRadio($yes_no_opts),
                'insurance_included' => new myWidgetFormRadio($yes_no_opts),
                'auto_accept' => new sfWidgetFormInputCheckbox(
                    array(), array('value' => 1)),
                'comment' => new sfWidgetFormTextarea()
            ));

        // Set default values for drop-down lists
        $this->setDefaults(array(
                'pickup_location_id' => 0,
                'dropoff_location_id' => 0,
                'vehicle_type_id' => 0,
                'date_available_from' => date('d-m-Y', time()),
                'date_return_by' => date('d-m-Y', time())
            ));

        // Field names
        $this->widgetSchema->setLabels(array(
		        'id' => 'ListingID',
                'owner_id' => 'Owner',
                'pickup_location_id' => 'Pickup location',
                'dropoff_location_id' => 'Dropoff location',
                'price' => 'Price(credits)',
                'date_available_from' => 'Available from',
                'date_return_by' => 'Return by',
                'vehicle_type_id' => 'Vehicle type',
                'kms_allowed' => 'Kms allowed',
                'days_allowed' => 'Days allowed',
                'petrol_included' => 'Fuel included',
                'expenses_included' => 'Expenses included',
                'ferry_included' => 'Ferry included',
                'insurance_included' => 'Insurance included',
                'auto_accept' => 'Automatically accept request',
                'comment' => 'Comment',
                'paid_days_allowed' => '# of extra days',
                'price_per_day' => 'Price per day ($)'
            ));

        $this->widgetSchema->setNameFormat('listing[%s]');

        // CSRF protection
        $this->addCSRFProtection(sfConfig::get('sf_csrf_secret'));


        // Validators
        $this->setValidators(array(

				// Owner - should be on the lists generated above
                'owner_id' => new sfValidatorCallback(array('callback' => array($this, 'validateOwner')), array('invalid' => 'Please choose owner')),
                // Pickup and drop-off location - should be on the lists generated above
                'pickup_location_id' => new sfValidatorChoice(array(
                    'choices' => array_keys($locations)), array('invalid' => 'Please choose location')),
                'dropoff_location_id' => new sfValidatorAnd(array(
                    new sfValidatorChoice(array('choices' => array_keys($locations)),
                        array('invalid' => 'Please choose location')),
                    new myValidatorSchemaCompareInListing(
                            'dropoff_location_id',
                            sfValidatorSchemaCompare::NOT_EQUAL,
                            'pickup_location_id',
                            array(),
                            array('invalid' => 'Dropoff location can\'t be the same as the pickup location'))
                    )),

                // Price - non-negative integer value
                'price' => new sfValidatorPass(array('required' => false)),

                // Availability date - can't be in the past (86400sec = 1day)
                'date_available_from' => new sfValidatorDate(array('min' => time() - 86400), array(
                        'min' => 'Date cannot be in the past',
                        'invalid' => 'Invalid value'
                    )),
                // Return date - can't be in the future and return date >= availability date
                'date_return_by' => new sfValidatorAnd(array(
                        new sfValidatorDate(array('min' => time()), array(
                            'min' => 'Date must be in the future',
                            'invalid' => 'Invalid value'
                        )),
                        new myValidatorDatesInListing()
                    )),
                // Vehicle type - should be
                'vehicle_type_id' => new sfValidatorChoice(array(
                        'choices' => array_keys($vehicles)),
                        array('invalid' => 'Please choose vehicle type')),
                'kms_allowed' => new sfValidatorInteger(array('min' => 1, 'required' => false),
                        array('invalid' => 'Invalid value', 'min' => 'Invalid value')),
                'days_allowed' => new sfValidatorInteger(
                        array('min' => 1, 'required' => false),
                        array('invalid' => 'Invalid value', 'min' => 'Invalid value',
                            'max' => 'Value is too big')),
                'petrol_included' => new sfValidatorPass(array('required' => false)),
                'expenses_included' => new sfValidatorPass(array('required' => false)),
                'ferry_included' => new sfValidatorPass(array('required' => false)),
                'insurance_included' => new sfValidatorPass(array('required' => false)),
                'auto_accept' => new sfValidatorPass(array('required' => false)),
                'comment' => new sfValidatorPass(array('required' => false))
            ));


		//allow extra fields with no validation
		$this->validatorSchema->setOption('allow_extra_fields', true);
		$this->validatorSchema->setOption('filter_extra_fields', false);
    }

    public function validateOwner($validator, $value)
    {
        $user = Doctrine::getTable('User')->findOneByUsername($value);

        if (!$user) {
            throw new sfValidatorError($this, 'invalid', array('value' => $value));
        }

        return $user->get('id');
    }
}
