<?php
/**
 * Form for edit user contact details
 *
 * @package transfercar
 * @subpackage form
 */
class editUserForm extends transfercarForm
{
  public function configure()
  {
    sfLoader::loadHelpers(array('Array')); // we have countries list here

    // country list
    $countries = COUNTRY_LIST();
    $countries[0] = ' ';
    $listVariants = sfConfig::get('app_users_listAnswerVariants', "");

    // years list (from current one down to 1900)
    $years = array();
    for($i = date('Y', time()); $i >= 1900; $i--){
        $years[$i] = $i;
    }

  // Set input fields
  // All fields have the same name as corresponding fields in database
    $this->setWidgets(array(
      'user_status' => new sfWidgetFormSelect(array('choices' => array(
        'inactive'  => 'inactive',
        'active'    => 'active',
        'blocked'   => 'blocked',
        'subscribe' => 'subscribe',
        'deleted'   => 'deleted'
      ))),
      'id' => new sfWidgetFormInput(array(), array(
                    'readonly' => 'readonly', 'style' => 'border: none',
                    'onfocus' => 'this.blur();')),
      'username' => new sfWidgetFormInput(),
      'is_owner' => new sfWidgetFormChoice(array(
          "choices" => array(User::USER_ONLY_DRIVER => "Driver", User::USER_OWNER => "Owner"),
          'expanded' => true,
          'multiple' => false
      )),
      'email' => new sfWidgetFormInput(),
      'source_information' => new sfWidgetFormSelect(array('choices' => $listVariants), array("onchange"=>"selectAnswer(''); return false;")),
      'password' => new sfWidgetFormInputPassword(),
      'firstname'  => new sfWidgetFormInput(),
      'lastname'  => new sfWidgetFormInput(),
      'date_of_birth' => new sfWidgetFormDate(array(
                    'can_be_empty' => true,
                    'format' => '%day%%month%%year%',
                    'years' => $years
                ), array(
                    'class' => 'select_date'
                )),
      'company' => new sfWidgetFormInput(),
      'contact_phone' => new sfWidgetFormInput(),
      'mobile_phone' => new sfWidgetFormInput(),
      'sms_number_confirmed' => new sfWidgetFormInputCheckbox(),
      'address_line1' => new sfWidgetFormInput(),
      'address_line2' => new sfWidgetFormInput(),
      'suburb' => new sfWidgetFormInput(),
      'city' => new sfWidgetFormInput(),
      'postcode' => new sfWidgetFormInput(),
      'country' => new sfWidgetFormSelect(array('choices' => $countries)),
      'license' => new sfWidgetFormSelect(array('choices' => $countries)),
      'payment_cc' => new sfWidgetFormInputCheckbox(),
      'relo_price_per_day' => new sfWidgetFormInput(),
      'require_bond' => new sfWidgetFormInputCheckbox(),
      'bond_amounts' => new sfWidgetFormInput(),
      'boost_ranking' => new sfWidgetFormInputCheckbox(),
      'can_create_airport_relocs' => new sfWidgetFormInputCheckbox(),
    ));

    $form = new BaseForm();
    $form->setWidgets(array(
      'nz' => new sfWidgetFormInput(),
      'au' => new sfWidgetFormInput(),
      'us' => new sfWidgetFormInput()
    ));
    $form->setValidators(array(
      'nz' => new sfValidatorString(),
      'au' => new sfValidatorString(),
      'us' => new sfValidatorString()
    ));
    $this->embedForm('bond_amounts', $form);

    $form = new BaseForm();
    $form->setWidgets(array(
      'nz' => new sfWidgetFormInputCheckbox(array('value_attribute_value' => 1)),
      'au' => new sfWidgetFormInputCheckbox(array('value_attribute_value' => 1)),
      'us' => new sfWidgetFormInputCheckbox(array('value_attribute_value' => 1))
    ));
    $form->setValidators(array(
      'nz' => new sfValidatorBoolean(array('required' => false)),
      'au' => new sfValidatorBoolean(array('required' => false)),
      'us' => new sfValidatorBoolean(array('required' => false))
    ));
    $this->embedForm('boost_ranking', $form);

    $this->widgetSchema->setNameFormat('user[%s]');

    // Set labels
    $this->widgetSchema->setLabels(array(
      'user_status' => 'Status',
      'id' => 'UserID:',
      'username' => 'Username',
      'is_owner' => 'Is owner?',
      'email' => 'Email',
      'source_information' => 'How did you <br/>hear about us?',
      'password' => 'Password',
      'firstname' => 'First name',
      'lastname' => 'Last name',
      'date_of_birth' => 'Date of birth',
      'company' => 'Company',
      'contact_phone' => 'Contact Phone',
      'mobile_phone' => 'Mobile Phone',
      'sms_number_confirmed' => 'Mobile phone number confirmed',
      'address_line1' => 'Street address',
      'address_line2' => '&nbsp;',
      'suburb' => 'Suburb',
      'city' => 'City',
      'postcode' => 'Postcode',
      'country' => 'Country',
      'license' => 'Drivers license',
      'payment_cc' => 'Payment by credit card',
      'relo_price_per_day' => 'Price per allowed day of relocations',
      'require_bond' => 'Bond required',
      'bond_amounts' => 'Bond amounts',
      'boost_ranking' => 'Boost search ranking',
      'can_create_airport_relocs' => 'Can create airport relocations'
    ));

    // Set default values from logged in user object
    $user = sfContext::getInstance()->getUser()->getAttribute('profile');
    if($user instanceof User){
        // Get field list
        $field_names = $this->widgetSchema->getPositions();

        // For every field, if proper field in user object is set, set it as default value
        foreach($field_names as $field){
            if($user->get($field)){
                $this->setDefault($field, $user->get($field));
            }
        }
    }

        $messages = array('required' => 'This field is required', 'invalid' => 'Invalid value');
        $date_messages = array_merge($messages, array('max' => 'Date cannot be in the future'));
        $username_messages = array_merge($messages, array('invalid' => 'This username is used by other user'));

        $this->setValidators(array(
          'username'   => new myValidatorUsernameUnique(array(), $username_messages),
          'is_owner' => new sfValidatorString(),
          'email' => new sfValidatorEmail(array(), $messages),
          'source_information' => new sfValidatorChoice(array("choices" => array_keys($listVariants), "required" => false)),
          'password' => new sfValidatorString(array(), $messages),
          'firstname'  => new sfValidatorString(array(), $messages),
          'lastname'  => new sfValidatorString(array(), $messages),
          'date_of_birth'  => new sfValidatorDate(array('required' => false, 'max' => time()), $date_messages),
          'company'  => new sfValidatorPass(array('required' => false)),
          'contact_phone'  => new sfValidatorPass(array('required' => false)),
          'mobile_phone'  => new sfValidatorPass(array('required' => false)),
          'sms_number_confirmed' => new sfValidatorBoolean(array('required' => false)),
          'suburb'  => new sfValidatorPass(array('required' => false)),
          'address_line1'  => new sfValidatorPass(array('required' => false)),
          'address_line2' => new sfValidatorPass(array('required' => false)),
          'postcode'  => new sfValidatorPass(array('required' => false)),
          'city'  => new sfValidatorPass(array('required' => false)),
          'country'  => new sfValidatorChoice(array(
              'required' => false,
              'choices' => array_keys($countries)
            ), $messages),
          'sms_number_confirmed' => new sfValidatorBoolean(array('required' => false)),
          'relo_price_per_day' => new sfValidatorInteger(),
          'payment_cc' => new sfValidatorBoolean(array('required' => false)),
          'require_bond' => new sfValidatorBoolean(array('required' => false)),
          'bond_amount'  => new sfValidatorInteger(array('required' => false)),
          'boost_ranking' => new sfValidatorPass(array('required' => false)),
          'can_create_airport_relocs' => new sfValidatorBoolean(array('required' => false)),
        ));

        //allow extra fields with no validation
        $this->validatorSchema->setOption('allow_extra_fields', true);
        $this->validatorSchema->setOption('filter_extra_fields', false);
    }
}
