<?php

use Transfercar\Mandrill\TcMandrill;

/*
 * Form which displays single field for e-mail address
 */
class forgotPasswordForm extends BaseForm
{
    public function configure()
    {
        // Field list definition
        $this->setWidgets(array(
                'forgot_email' => new sfWidgetFormInput()
            ));

        // Labels
        $this->widgetSchema->setLabels(array(
                'forgot_email' => 'Your e-mail:'
            ));

        // Add CSRF protection
        $this->addCSRFProtection(sfConfig::get('sf_csrf_secret'));

        // Validators
        $this->setValidators(array(
            'forgot_email' => new myValidatorForgotPassword()));
        $this->validatorSchema->addMessage('invalid', 'Wrong e-mail');

        $formatter = $this->getWidgetSchema()->getFormFormatter();
        $formatter->setErrorListFormatInARow('<small>%errors%</small>');
        $formatter->setErrorRowFormatInARow('%error% ');
    }

    // Render form
    public function render($attributes = array())
    {
        sfLoader::loadHelpers(array('Form'));

        $result = '';

        $result .= label_for('forgot_email', $this->widgetSchema->getLabel('forgot_email'));
        $result .= input_tag('forgot_email', @$this->taintedValues['forgot_email']);

        $result .= $this->widgetSchema->renderField(self::$CSRFFieldName,
            $this->getCSRFToken(self::$CSRFSecret));

        return $result;
    }

    public function process($domain)
    {
        // OK, we have given correct e-mail
        // Get user object, generate new
        // password and send it to the given e-mail
        $user = Doctrine_Query::create()
            ->from('User')
            ->where('LOWER(email) = ?', strtolower($this->getValue('forgot_email')))
            ->fetchOne();

        // Generate new password
        $length = intval(sfConfig::get('app_user_new_pas_length', 16));
        $password = substr(md5(mt_rand().$user->username.time()), 0, $length);

        $user->set('password', sha1($password));
        $user->save();

        $this->sendPasswordEmail($user, $password);
    }

    public function sendPasswordEmail($user, $password)
    {
        $site = sfConfig::get('app_domain_' . $domain);
        $template_data = array(
            'name' => $user->firstname,
            'firstname' => $user->firstname,
            'lastname'  => $user->lastname,
            'when_registered' => $user->when_registered,
            'email'    => $user->email,
            'password' => $password,
        );

        $mandrill = new TcMandrill();
        $mandrill->sendTemplate(
            'New password',
            $user->getEmailToParams(),
            $template_data,
            'forgot-password'
        );
    }
}
