<?php

/**
 * Set up user data
 * @author Daniel Piechocki Planetsoft
 * @author Pawel Smolinski
 */

class joinUserDataForm extends transfercarForm
{
    public function configure()
    {
        $listVariants = sfConfig::get('mod_main_listAnswerVariants', "");
    	// Define form fields
        $this->setWidgets(array(
            'email' => new sfWidgetFormInput(),
        	'confirm_email' => new sfWidgetFormInput(),
            'username' => new sfWidgetFormInput(),
            'password' => new sfWidgetFormInputPassword(),
            'confirm' => new sfWidgetFormInputPassword(),
        	'source_information' => new sfWidgetFormSelect(array('choices' => $listVariants), array("onchange"=>"selectAnswer(); return false;")),
        	'email_newsletter' => new sfWidgetFormInputCheckbox(array(), array()),
        	'email_notify_type_freq' => new sfWidgetFormInputCheckbox(array(), array()),
        ));


        $this->widgetSchema->setNameFormat('register[%s]');

        // Add CSRF protection
        $this->addCSRFProtection(sfConfig::get('sf_csrf_secret'));

        // Set the Labels for specific fields
        $this->widgetSchema->setLabels(array(
            'email' => 'your email<sup style="color:red">*</sup>',
        	'confirm_email' => 'confirm email<sup style="color:red">*</sup>',
            'password' => 'password<sup style="color:red">*</sup>',
            'confirm' => 'confirm password<sup style="color:red">*</sup>',
            'username' => 'username<sup style="color:red">*</sup>',
        	'source_information' => 'How did you hear about us?',
        	'email_newsletter' => 'Subscribe to monthly newsletter',
        	'email_notify_type_freq' => 'Subscribe to daily update of cars listed'
        ));

        // Define validators
        // We have here two things to check:
        // a) given username has to be unique
        // b) passwords should match to each other
        $msg = array('required' => 'This field is required');
        $pass_msg = array_merge($msg, array(
                'min_length' => 'Password is too short'
            ));

        $this->setValidators(array(
            'email' => new myValidatorUniqueEmail(array(), array(
                    'required' => 'This field is required',
                    'invalid' => 'This is not valid e-mail address',
                    'unique' => 'This e-mail is already in use'
                )),
            'confirm_email' => new sfValidatorString(array(), $msg),
            'password' => new sfValidatorString(array('min_length' => '6'), $pass_msg),
            'confirm' => new sfValidatorString(array(), $msg),
            'username' => new sfValidatorDoctrineUnique(array(
                    'model' => 'User',
                    'column' => 'username',
                ), array(
                    'required' => 'This field is required',
                    'invalid' => 'This name is already in use'
                )),
            'source_information' => new sfValidatorChoice(array("choices" => array_keys($listVariants), "required" => false)),
            'email_newsletter' => new sfValidatorBoolean(array("required" => false)),
            'email_notify_type_freq' => new sfValidatorBoolean(array("required" => false))
        ));

        $formatter = $this->getWidgetSchema()->getFormFormatter();

        $formatter->setErrorListFormatInARow('<small>%errors%</small>');
        $formatter->setErrorRowFormatInARow('%error% ');

        return;
    }


    public function configureSubscribe(){
    	unset($this["confirm_email"]);
        $this->validatorSchema["email"] = new sfValidatorString(array("required" => false));
        $this->widgetSchema["email"] = new sfWidgetFormInput(array(), array("readonly" => "readonly"));
    }

}
