<?php
/**
 * Form for adding/editing listings
 *
 * @package transfercar
 * @subpackage form
 */
class listingForm extends transfercarForm
{

  /**
   * Prepare form
   */
  public function configure()
  {
    // Load required helpers (we need push_to_begin() function)
    sfLoader::loadHelpers(array('Tag', 'Array', 'Url'));

		// country list
		$countries = COUNTRY_LIST();
		unset($countries[0]);

		// Get all data we need from db

    $country = $this->getOption('country', sfContext::getInstance()->getUser()->getAttribute('country'));

    // Location names
    $locations = Locations::locationList($country);
    $locations_with_empty = $locations;
    $locations_with_empty = push_to_begin($locations, 'Please choose...');

    // Vehicle types
    $vehicles = VehicleType::typeList();
    $vehicles_with_empty = $vehicles;
    $vehicles_with_empty = push_to_begin($vehicles, 'Please choose...');

    // Options for yes/no radiobuttons
    $yes_no_opts = array('choices' => array(1 => 'Yes', 0 => 'No'));

    $licenseChoices = array('all' => 'All full licenses', 'NZ+AU' => 'NZ+AU license only');

    // Fields
    $this->setWidgets(array(
      'pickup_location_id' => new sfWidgetFormSelect(array(
        'choices' => $locations_with_empty), array("style" => "width:234px")),
      'dropoff_location_id' => new sfWidgetFormSelect(array(
        'choices' => $locations_with_empty), array("style" => "width:234px")),
      'price' => new sfWidgetFormInput(array(), array(
        'readonly' => 'readonly', 'style' => 'border: none',
        'onfocus' => 'this.blur();')),
      'date_available_from' => new myWidgetFormDateWithPicker(),
      'date_return_by' => new myWidgetFormDateWithPicker(),
      'vehicle_type_id' => new sfWidgetFormSelect(array(
          'choices' => $vehicles_with_empty)),
      'kms_allowed' => new sfWidgetFormInput(array(), array('size' => 10)),
      'days_allowed' => new sfWidgetFormInput(array(), array('size' => 10)),
      'paid_days_allowed' => new sfWidgetFormInput(array(), array('size' => 10)),
      'price_per_day' => new sfWidgetFormInput(array(), array('size' => 10)),
      'petrol_included' => new myWidgetFormRadio($yes_no_opts),
      'ferry_included' => new myWidgetFormRadio($yes_no_opts),
      'expenses_included' => new myWidgetFormRadio($yes_no_opts),
      'insurance_included' => new myWidgetFormRadio($yes_no_opts),
      'tolls_included' => new myWidgetFormRadio($yes_no_opts),
      'pickup_address' => new sfWidgetFormInput(),
      'contact_person' => new sfWidgetFormInput(),
      'contact_phone' => new sfWidgetFormInput(),
      'auto_accept' => new sfWidgetFormInputCheckbox(
        array(), array('value' => 1)),
      'comment' => new sfWidgetFormTextarea(),
      'min_age' => new sfWidgetFormInput(array(), array('size' => 10)),
      'license' => new sfWidgetFormSelectRadio(array('choices' => $licenseChoices)),
      'terms_title' => new sfWidgetFormInput(),
      'terms_content' => new sfWidgetFormRichTextarea(array(), array('rows' => 10, 'style' => 'width:100%')),
      'addons' => new sfWidgetFormAddonList(),
    ));

    // Set default values
    $this->setDefaults(array(
      'pickup_location_id' => 0,
      'dropoff_location_id' => 0,
      'vehicle_type_id' => 0,
      'date_available_from' => date('d-m-Y', time()),
      'date_return_by' => date('d-m-Y', time()),
      'license' => 'all'
    ));

    // Field names
    $this->widgetSchema->setLabels(array(
      'pickup_location_id' => 'Pickup location',
      'dropoff_location_id' => 'Dropoff location',
      'price' => 'Price',
      'date_available_from' => 'Pick-up',
      'date_return_by' => 'Drop-off',
      'vehicle_type_id' => 'Vehicle type',
      'kms_allowed' => 'Kms allowed',
      'days_allowed' => 'Free days allowed',
      'paid_days_allowed' => '# of additional paid days',
      'price_per_day' => 'Price per day ($)',
      'petrol_included' => 'Fuel included',
      'expenses_included' => 'Expenses included',
      'ferry_included' => 'Ferry included',
      'insurance_included' => 'Insurance included',
      'auto_accept' => 'Automatically accept request',
      'comment' => 'Comment',
      'min_age' => 'Minimum driver age',
      'license' => 'License requirement',
      'terms_title' => 'Title (for identification)',
      'terms_content' => 'Terms and Conditions',
    ));

    $this->widgetSchema->setNameFormat('listing[%s]');

    // CSRF protection
    $this->addCSRFProtection(sfConfig::get('sf_csrf_secret'));

	$messages = array('required' => 'This field is required', 'invalid' => 'Invalid value');

    // Allowed HTML tags (used for XSS prevention)
    $allowed_tags = array(
        'p', 'strong', 'em', 'span', 'ul', 'ol', 'li', 'hr', 'h1', 'h2', 'h3',
        'h4', 'dl', 'dt', 'dd'
    );

    // Validators
      $this->setValidators(array(
        // Pickup and drop-off location - should be on the lists generated above
        'pickup_location_id' => new sfValidatorChoice(array(
          'choices' => array_keys($locations)), array('invalid' => 'Please choose location')),
        'dropoff_location_id' => new sfValidatorChoice(
            array('choices' => array_keys($locations)),
            array('invalid' => 'Please choose location')
        ),

        // Price - non-negative integer value
        'price' => new sfValidatorPass(array('required' => false)),

        // Availability date - can't be in the past (86400sec = 1day)
        'date_available_from' => new sfValidatorDate(array(
        'min' => time() - 86400,
        'date_output' => 'd-m-Y',
        'date_format' => '~(?P<day>\d{2})-(?P<month>\d{2})-(?P<year>\d{4})~',
      ), array(
        'min' => 'Date cannot be in the past',
        'invalid' => 'Invalid value'
      )),
        // Return date - can't be in the future and return date >= availability date
        'date_return_by' => new sfValidatorAnd(array(
          new sfValidatorDate(array(
            'min' => time(),
            'date_output' => 'd-m-Y',
            'date_format' => '~(?P<day>\d{2})-(?P<month>\d{2})-(?P<year>\d{4})~',
          ), array(
            'min' => 'Date must be in the future',
            'invalid' => 'Invalid value'
          )),
            new myValidatorDatesInListing()
          )),
        // Vehicle type - should be
        'vehicle_type_id' => new sfValidatorChoice(array(
                'choices' => array_keys($vehicles)),
                array('invalid' => 'Please choose vehicle type')),
        'kms_allowed' => new sfValidatorInteger(array('min' => 1, 'required' => false),
                array('invalid' => 'Invalid value', 'min' => 'Invalid value')),
        'days_allowed' => new sfValidatorInteger(
                array('min' => 1, 'required' => true),
                array('invalid' => 'Invalid value', 'min' => 'You must allow at least one free day',
                    'max' => 'Value is too big')),
        'paid_days_allowed' => new sfValidatorInteger(
                array('min' => 1, 'required' => false),
                array('invalid' => 'Invalid value', 'min' => 'Invalid value',
                    'max' => 'Value is too big')),
        'price_per_day' => new sfValidatorNumber(array('min' => 1, 'required' => false),
                array('invalid' => 'Invalid price', 'min' => 'Price must be at least 1$')),
        'petrol_included' => new sfValidatorPass(array('required' => false)),
        'expenses_included' => new sfValidatorPass(array('required' => false)),
        'ferry_included' => new sfValidatorPass(array('required' => false)),
        'insurance_included' => new sfValidatorPass(array('required' => false)),
        'tolls_included' => new sfValidatorPass(array('required' => false)),
        'auto_accept' => new sfValidatorPass(array('required' => false)),
        'comment' => new sfValidatorPass(array('required' => false)),
        'min_age' => new sfValidatorNumber(array('required' => true, 'min' => 18, 'max' => 99), array(
          'min' => 'Age cannot be less than 18',
          'max' => 'Age cannot be greater than 99',
          'invalid' => 'Please enter a valid number'
      )),
      'license'        => new sfValidatorChoice(array('choices' => array_keys($licenseChoices), 'required' => false), $messages),
      'terms_title'    => new myValidatorXSSCleaner(
        array('required' => false, 'allowed_tags' => $allowed_tags), array()),
      'terms_content'  => new myValidatorXSSCleaner(
        array('required' => false, 'allowed_tags' => $allowed_tags), array()),
      'pickup_address' => new sfValidatorPass(),
      'contact_phone'  => new sfValidatorPass(),
      'contact_person' => new sfValidatorPass(),
      'addons'         => new sfValidatorAddonList(array('required' => false))
    ));

    $this->mergePostValidator(new myValidatorAllowedDaysInListing());
    $this->mergePostValidator(new tcValidatorPaidDays());
    $this->mergePostValidator(new sfValidatorSchemaCompare(
        'dropoff_location_id',
        sfValidatorSchemaCompare::NOT_EQUAL,
        'pickup_location_id',
        array(),
        array('invalid' => 'Dropoff location can\'t be the same as the pickup location')
    ));

		$this->widgetSchema->addFormFormatter('myFormatter', new sfWidgetFormatter($this->widgetSchema));
		$this->widgetSchema->setFormFormatterName('myFormatter');
  }
}
