<?php

class ListingImportForm extends BaseForm
{
    public function configure()
    {
        $this->widgetSchema['import'] = new sfWidgetFormInputFile();
        $this->validatorSchema['import'] = new sfValidatorFile(array(
            'required'   => true,
            //'path'       => sfConfig::get('sf_upload_dir'),
            'mime_types' => array('application/csv', 'text/comma-separated-values', 'text/csv', 'text/plain'),
            ), array('mime_types' => 'The file only accept csv format.'));

        $this->validatorSchema['import']->setOption('mime_type_guessers', [[$this->validatorSchema['import'], 'guessFromMimeContentType']]);

        $this->mergePostValidator(new sfValidatorCallback(array('callback' => [$this, 'validateImport'])));

        $this->widgetSchema->setNameFormat('import[%s]');
    }

public function detectDelimiter($csvFile)
{
    $delimiters = [";" => 0, "," => 0, "\t" => 0, "|" => 0];

    $handle = fopen($csvFile, "rb");
    rewind($handle);
    $firstLine = fgets($handle);
    fclose($handle); 
    foreach ($delimiters as $delimiter => &$count) {
        $count = count(str_getcsv($firstLine, $delimiter));
    }

    return array_search(max($delimiters), $delimiters);
}

    public function validateImport($validator, $values)
    {
    if (!$values['import']) {
       throw new sfValidatorError($validator, 'Missing file');
    }

        $delim=$this->detectDelimiter($values['import']->getTempName());;
        $h = fopen($values['import']->getTempName(), 'rb');
        rewind($h);
        $header = array_map('strtolower', fgetcsv($h, null, $delim));
        $vehicletypekey = array_search('vehicle_type', $header);
        $pickupkey = array_search('pickup', $header);
        $dropoffkey = array_search('dropoff', $header);
        if (false === $vehicletypekey || false === $pickupkey || false === $dropoffkey) {
            throw new sfValidatorError($validator, 'Missing vehicle_type, pickup or dropoff columns'.json_encode($header));
        }

        $data=[];
        while($row = fgetcsv($h, null, $delim)) {
            if ($row[$vehicletypekey] == '' || $row[$pickupkey] == '' || $row[$dropoffkey] == '') {
                throw new sfValidatorError($validator, 'Missing vehicle_type, pickup or dropoff values'.json_encode($row).' keys:'.$vehicletypekey.'/'.$pickupkey.'/'.$dropoffkey);
            }

            $data[] = array_combine($header, $row);
        }

        if (empty($data)) {
            throw new sfValidatorError($validator, 'Nothing to import');
        }

        $values['import'] = $data;

        return $values;
    }
}
