<?php

use Transfercar\Ranking\SearchScore;

class OwnerListingForm extends listingForm
{
    public function configure()
    {
        parent::configure();

        unset($this['pickup_address'], $this['contact_person'], $this['contact_phone'], $this['license']);

        $times = $this->getTimes();

        $this->widgetSchema['quantity'] = new tcWidgetFormInputNumber();
        $this->widgetSchema['date_available_from'] = new tcWidgetFormDateWithPicker(array(
          'domain' => $this->getOption('domain')
        ));
        $this->widgetSchema['date_return_by'] = new tcWidgetFormDateWithPicker(array(
          'domain' => $this->getOption('domain')
        ));
        $this->widgetSchema['pickup_time'] = new sfWidgetFormChoice(array(
          'choices' => $times
        ));
        $this->widgetSchema['latest_pickup_time'] = new sfWidgetFormChoice(array(
          'choices' => $times
        ));
        $this->widgetSchema['dropoff_time'] = new sfWidgetFormChoice(array(
          'choices' => $times
        ));
        $this->widgetSchema['latest_dropoff_time'] = new sfWidgetFormChoice(array(
          'choices' => $times
        ));

        $this->widgetSchema['kms_allowed'] = new tcWidgetFormInputNumber();
        $this->widgetSchema['kms_estimated'] = new sfWidgetFormInputHidden();
        $this->widgetSchema['days_allowed'] = new tcWidgetFormInputNumber();
        $this->widgetSchema['paid_days_allowed'] = new tcWidgetFormInputNumber();
        $this->widgetSchema['price_per_day'] = new sfWidgetFormInput();
        $this->widgetSchema['petrol_included'] = new sfWidgetFormInputCheckbox(array(), array('value' => 1));
        $this->widgetSchema['petrol_included_type'] = new sfWidgetFormChoice(array('choices' => array(
          1 => 'Free tank(s) of fuel', 
          2 => 'Partial refund'
        )));
        $this->widgetSchema['ferry_included'] = new sfWidgetFormInputCheckbox(array(), array('value' => 1));
        $this->widgetSchema['ferry_included_type'] = new sfWidgetFormChoice(array('choices' => array(
          1 => 'Vehicle only', 
          2 => 'Vehicle and driver included', 
          3 => 'Vehicle + 2 passengers', 
          4 => 'Partial refund'
        )));
        $this->widgetSchema['expenses_included'] = new sfWidgetFormInputCheckbox(array(), array('value' => 1));
        $this->widgetSchema['insurance_included'] = new sfWidgetFormInputCheckbox(array(), array('disabled' => 'disabled', 'value' => 1));
        $this->widgetSchema['insurance_policy'] = new sfWidgetFormDoctrineSelect([
          'model' => 'InsurancePolicy',
          'method' => 'getDetails',
          'criteria' => Doctrine::getTable('InsurancePolicy')->createQuery()->where('owner_id = ?', $this->getOption('user')->getOwnerId())
        ]);
        $this->validatorSchema['insurance_included']->setOption('required', false);

        $this->validatorSchema['insurance_policy'] = new sfValidatorInteger(['required' => true]);
        $this->widgetSchema['tolls_included'] = new sfWidgetFormInputCheckbox(array(), array('value' => 1));

        $this->widgetSchema['petrol_included_text'] = new sfWidgetFormInput();
        $this->widgetSchema['ferry_included_text'] = new sfWidgetFormInput();
        $this->widgetSchema['expenses_included_text'] = new sfWidgetFormInput();
        //$this->widgetSchema['insurance_included_text'] = new sfWidgetFormInput();
        $this->widgetSchema['tolls_included_text'] = new sfWidgetFormInput();

        $this->widgetSchema['terms_id'] = new sfWidgetFormInputHidden();

        $this->widgetSchema['addons'] = new sfWidgetFormOwnerAddonList(array('user_id' => $this->getOption('user')->getOwnerId()));
        $this->widgetSchema['nb_listings'] = new tcWidgetFormInputNumber();
        $this->widgetSchema['internal_reference'] = new sfWidgetFormInput();
        $this->widgetSchema['listing_contact_list'] = new tcWidgetFormOwnerContacts(array(
          'model'   => 'OwnerContact',
          'country' => $this->getOption('country'),
          'user'    => $this->getOption('user')
        ));

        $this->widgetSchema['vehicle_type_id'] = new tcWidgetFormOwnerVehicle(array(
          'user' => $this->getOption('user'),
          'country' => $this->getOption('country'),
          'tc_vehicles' => false
        ));

        $this->widgetSchema['pickup_location_id'] = new tcWidgetFormOwnerBranch(array(
          'user' => $this->getOption('user'),
          'country' => $this->getOption('country')
        ));
        $this->widgetSchema['dropoff_location_id'] = new tcWidgetFormOwnerBranch(array(
          'user' => $this->getOption('user'),
          'country' => $this->getOption('country')
        ));

        if ($this->getOption('user')->canCreateAirportRelocs())
        {
          $this->widgetSchema['is_airport_reloc'] = new sfWidgetFormInputCheckbox();
          $this->validatorSchema['is_airport_reloc'] = new sfValidatorPass();
        }

        $this->validatorSchema['listing_contact_list'] = new sfValidatorDoctrineChoiceMany(array(
          'model' => 'OwnerContact',
          'required' => false,
          'query' => Doctrine::getTable('OwnerContact')->createQuery()->addWhere('owner_id = ?', $this->getOption('user')->getOwnerId())
        ));

        $this->validatorSchema['vehicle_type_id'] = new tcValidatorVehicleType(array(
          'user' => $this->getOption('user')
        ));

        $this->validatorSchema['pickup_location_id'] = new tcValidatorBranch(array(
          'user' => $this->getOption('user')
        ));

        $this->validatorSchema['dropoff_location_id'] = new tcValidatorBranch(array(
          'user' => $this->getOption('user')
        ));

        $this->validatorSchema['pickup_time'] = new sfValidatorChoice(array('choices' => array_keys($times)));
        $this->validatorSchema['dropoff_time'] = new sfValidatorChoice(array('choices' => array_keys($times)));
        $this->validatorSchema['latest_pickup_time'] = new sfValidatorChoice(array('choices' => array_keys($times)));
        $this->validatorSchema['latest_dropoff_time'] = new sfValidatorChoice(array('choices' => array_keys($times)));
        $this->validatorSchema['ferry_included_type'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['petrol_included_type'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['petrol_included_text'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['ferry_included_text'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['expenses_included_text'] = new sfValidatorString(array('required' => false));
        //$this->validatorSchema['insurance_included_text'] = new sfValidatorString(array('required' => true));
        $this->validatorSchema['tolls_included_text'] = new sfValidatorString(array('required' => false));
        $this->validatorSchema['internal_reference'] = new sfValidatorString(array('required' => false));

        $this->validatorSchema['nb_listings'] = new sfValidatorInteger(array('min' => 1));
        $this->validatorSchema['kms_estimated'] = new sfValidatorInteger(array('required' => false));

        $this->validatorSchema['terms_title']->setOption('allowed_tags', array(
          'p', 'strong', 'em', 'span', 'ul', 'ol', 'li', 'hr', 'h1', 'h2', 'h3',
          'h4', 'dl', 'dt', 'dd'
        ));

        $this->validatorSchema['terms_content']->setOption('allowed_tags', array(
          'p', 'strong', 'em', 'span', 'ul', 'ol', 'li', 'hr', 'h1', 'h2', 'h3',
          'h4', 'dl', 'dt', 'dd'
        ));

        $dateFormat = sfConfig::get('app_date_time_pattern_'.$this->getOption('domain'), 'dd/mm/yyyy');
        $pattern = preg_replace_callback('#([dmy]{2,4})/([dmy]{2,4})/([dmy]{2,4})#i', function($match) {
          $pattern = array();

          foreach ($match as $key => $value) {
            if ($key === 0)
            {
              continue;
            }

            $pKey = '';
            switch ($value[0])
            {
              case 'd':
                $pKey = 'day';
                break;
              case 'm':
                $pKey = 'month';
                break;
              case 'y':
                $pKey = 'year';
                break;
            }

            $pattern[] = '(?P<'.$pKey.'>\d{'.strlen($value).'})';
          }

          return '~'.implode('/', $pattern).'~';
        }, strtolower($dateFormat), 1);

        $this->validatorSchema['date_available_from'] = new sfValidatorDate(array(
            'min' => time() - 86400,
            'date_output' => 'Y-m-d',
            'date_format' => $pattern,
            'date_format_error' => 'must be '.$dateFormat,
          ), array(
            'min' => 'Date cannot be in the past',
            'invalid' => 'Invalid value'
          ));

          // Return date - can't be in the future and return date >= availability date
          $this->validatorSchema['date_return_by'] = new sfValidatorDate(array(
            'min' => time(),
            'date_output' => 'Y-m-d',
            'date_format' => $pattern,
            'date_format_error' => 'must be '.$dateFormat,
          ), array(
            'min' => 'Date must be in the future',
            'invalid' => 'Invalid value'
          ));

          $this->mergePostValidator(new sfValidatorDateCompare(
            'date_return_by', 'greater_than_equal', 'date_available_from', array(), array(
              'invalid' => 'Return date cannot be earlier than availability date'
            )
          ));

          $this->mergePostValidator(new sfValidatorSchemaCompare(
            'pickup_time', 'less_than_equal', 'latest_pickup_time', array(), array(
              'invalid' => 'Latest pickup time cannot be earlier than earlies pickup time'
            )
          ));

          $this->mergePostValidator(new sfValidatorSchemaCompare(
            'dropoff_time', 'less_than_equal', 'latest_dropoff_time', array(), array(
              'invalid' => 'Latest dropoff time cannot be earlier than earliest dropoff time'
            )
          ));

          $this->mergePostValidator(
            new sfValidatorCallback(array('callback' => array($this, 'add24hrsRentalTimeValidator')))
          );

          $this->mergePostValidator(
            new sfValidatorCallback(array('callback' => array($this, 'validatePricing')), array('invalid' => 'Invalid pricing, please contact Transfercar'))
          );
    }


    public function validatePricing($validator, $values)
    {
      $price = Doctrine::getTable('Prices')->getPriceFor(
        $this->getOption('user')->getOwnerId(),
        $values['pickup_location_id'] instanceof OwnerBranch ? $values['pickup_location_id']->get('location_id') : $values['pickup_location_id']->getId(),
        $values['dropoff_location_id'] instanceof OwnerBranch ? $values['dropoff_location_id']->get('location_id') : $values['dropoff_location_id']->getId(),
        $values['vehicle_type_id'] instanceof OwnerVehicleType ? $values['vehicle_type_id']->get('vehicle_type_id') : $values['vehicle_type_id']->getId()
      );

      if (!$price) {
        throw new sfValidatorError($validator, 'invalid');
      }

      return $values;
    }

    public function add24hrsRentalTimeValidator($validator, $values)
    {
      // $vehicle_type_id
      if ($values['vehicle_type_id'] instanceof OwnerVehicleType && $values['vehicle_type_id']->get('rental_type') != 'day')
      {
        $validator = new sfValidatorSchemaCompare(
          'latest_pickup_time', 'greater_than_equal', 'dropoff_time', array(), array(
            'invalid' => 'Latest pickup time cannot be earlier than earliest dropoff time'
          )
        );

        return $validator->clean($values);
      }

      return $values;
    }

    private function getTimes()
    {
      sfLoader::loadHelpers(array('Date', 'Transfercar'));
      $time = strtotime('today');

      for ($i = 0; $i < 48; $i++)
      {
        $seconds = ($i * 1800);
        $times[$seconds] = tc_format_date($seconds, 'hours');
      }

      return $times;
    }

    public function setDefaultsFromListing($listing)
    {
      $defaults = $listing->toArray(false);
      $defaults['insurance_included'] = true;
      $contacts = array();
      foreach ($listing->OwnerContacts as $obj)
      {
        $contacts[] = current($obj->identifier());
      }

      $defaults['listing_contact_list'] = $contacts;

      if ($listing->get('Terms'))
      {
        $defaults['terms_id'] = $listing->get('Terms')->getId();
        $defaults['terms_title'] = $listing->get('Terms')->get('title');
        $defaults['terms_content'] = $listing->get('Terms')->get('content');
      }

      if ($listing->isAirportReloc())
      {
        $defaults['is_airport_reloc'] = 1;
      }

      if ($listing->get('owner_vehicle_type_id'))
      {
        $defaults['vehicle_type_id'] = 'owner_'.$listing->get('owner_vehicle_type_id');
      }
      else
      {
        $defaults['vehicle_type_id'] = 'tc_'.$listing->get('vehicle_type_id');
      }

      if ($listing->get('owner_pickup_branch_id'))
      {
        $defaults['pickup_location_id'] = 'owner_'.$listing->get('owner_pickup_branch_id');
      }
      else
      {
        $defaults['pickup_location_id'] = 'tc_'.$listing->get('pickup_location_id');
      }

      if ($listing->get('owner_dropoff_branch_id'))
      {
        $defaults['dropoff_location_id'] = 'owner_'.$listing->get('owner_dropoff_branch_id');
      }
      else
      {
        $defaults['dropoff_location_id'] = 'tc_'.$listing->get('dropoff_location_id');
      }

      $addons = array();

      foreach ($listing->get('Addons') as $addon)
      {
        $addons[] = array(
          'id'     => $addon->getId(),
          'name'   => $addon->get('name'),
          'price'  => $addon->get('price'),
          'period' => $addon->get('period')
        );
      }

      $defaults['addons'] = $addons;

      $defaults['date_available_from'] = $this->fixDate($defaults['date_available_from'], 'today');
      $defaults['date_return_by'] = $this->fixDate($defaults['date_return_by'], 'tomorrow');

      $this->setDefaults($defaults);
    }

    private function fixDate($date, $when)
    {
      $time = strtotime($date);
      $defaultTime = strtotime($when);

      $time = $time < $defaultTime ? $defaultTime : $time;

      return date('Y-m-d', $time);
    }

    public function setDefaults($defaults)
    {
      if (!isset($defaults['pickup_time'])) {
        $defaults['pickup_time'] = 28800; // 8pm
      }

      if (!isset($defaults['latest_pickup_time'])) {
        $defaults['latest_pickup_time'] = 59400; // 4.30pm
      }

      if (!isset($defaults['dropoff_time'])) {
        $defaults['dropoff_time'] = 28800; // 8pm
      }

      if (!isset($defaults['latest_dropoff_time'])) {
        $defaults['latest_dropoff_time'] = 59400; // 4.30pm
      }

      return parent::setDefaults($defaults);
    }

    public function save($listing)
    {
      $conn = Doctrine::getTable('Listing')->getConnection();
      $conn->beginTransaction();

      try
      {
        $this->doSave($listing);
        $conn->commit();
      }
      catch (Exception $e)
      {
        $conn->rollback();
        throw $e;
      }
    }

    private function doSave($listing)
    {
      $values = $this->getValues();
      $values['insurance_included'] = true;
      $object = $values['vehicle_type_id'];

      if ($object instanceof OwnerVehicleType)
      {
          $values['owner_vehicle_type_id'] = $object->getId();
          $values['vehicle_type_id'] = $object->get('vehicle_type_id');
      }
      else
      {
        $values['vehicle_type_id'] = $object->getId();
        $values['owner_vehicle_type_id'] = null;
      }

      $pickup = $values['pickup_location_id'];

      if ($pickup instanceof OwnerBranch)
      {
        $values['owner_pickup_branch_id'] = $pickup->getId();
        $values['pickup_location_id'] = $pickup->get('location_id');
        $values['pickup_address'] = $pickup->address;
        $values['contact_person'] = $pickup->get('Contact')->get('name');
        $values['contact_phone']  = $pickup->get('Contact')->get('phone');
      }
      else
      {
        $values['pickup_location_id'] = $pickup->getId();
      }

      $dropoff = $values['dropoff_location_id'];

      if ($dropoff instanceof OwnerBranch)
      {
        $values['owner_dropoff_branch_id'] = $dropoff->getId();
        $values['dropoff_location_id'] = $dropoff->get('location_id');
      }
      else
      {
        $values['dropoff_location_id'] = $dropoff->getId();
      }

      $values['insurance_policy_id'] = $values['insurance_policy'];

      if (isset($values['is_airport_reloc']))
      {
        $values['listing_type'] = $values['is_airport_reloc'] ? Listing::TYPE_AIRPORT_RELOC : Listing::TYPE_DEFAULT;
      }

      $listing->fromArray($values);

      $listing->set('date_return_by', $listing->get('date_return_by'));
      $listing->set('date_available_from', $listing->get('date_available_from'));

      $listing->price = Doctrine::getTable('Prices')->getPriceFor(
        $this->getOption('user')->getOwnerId(),
        $values['pickup_location_id'],
        $values['dropoff_location_id'],
        $values['vehicle_type_id']
      );

      if ($this->getValue('terms_content'))
      {
        $terms = false;
        if ($id = $this->getValue('terms_id'))
        {
          $terms = Doctrine::getTable('Terms')->find($id);
        }

        if (!$terms)
        {
          $terms = new Terms();
          $terms->user_id = $this->getOption('user')->getOwnerId();
        }

        $terms->set('title', $this->getValue('terms_title'));
        $terms->set('content', $this->getValue('terms_content'));
        $listing->setTerms($terms);
      }

      $listing->get('Addons')->delete();

      foreach((array) $this->getValue('addons') as $addon) {
        if(empty($addon['name']) || empty($addon['price']))
          continue;

        $cAddon = new Addon();
        $cAddon->fromArray($addon);
        $cAddon->set('Listing', $listing);
        $cAddon->set('user_id', $this->getOption('user')->getOwnerId());
        $listing->get('Addons')->add($cAddon);
      }

      $isNew = !$listing->exists();
      if ($isNew) {
        $listing->set('status', Listing::STATUS_NEW);
        $listing->set('created_at', date('Y-m-d'));
        $listing->set('listed_by', $this->getOption('user')->getId());
        $listing->set('owner_id', $this->getOption('user')->getOwnerId());
      } elseif ($listing->isStatus(array(Listing::STATUS_EXPIRED, Listing::STATUS_PAUSED, Listing::STATUS_CANCELED))) {
        $listing->set('status', Listing::STATUS_NEW);
        $listing->set('accepted_request_id', null);
      }

      $listing->set('updated_by', $this->getOption('user')->getId());
      $listing->set('updated_at', date('Y-m-d'));
      $score = new SearchScore();
      $listing->set('score', $score->getScore($listing));
      $listing->set('score_debug', $score->debug);
      $listing->save();

      $this->saveListingContactList($listing);

      if ($nb_listings = $this->getValue('nb_listings'))
      {
        $baseListing = $listing;
        $duplicate = $this->duplicate($listing);

        for ($i = 0; $i < intval($nb_listings -1); $i++)
        {
          $listing = $baseListing->get('ChildrenListings')->get($i);
          $listing->fromArray($duplicate);
          $listing->set('listing_id', $baseListing->getId());
          $listing->set('owner_id', $this->getOption('user')->getOwnerId());
          $listing->set('listed_by', $this->getOption('user')->getId());
          $listing->set('status', Listing::STATUS_STANDBY);
          $listing->set('created_at', date('Y-m-d', time()));
          $listing->set('OwnerContacts', $baseListing->get('OwnerContacts'));

          $listing->save();
        }

        for ($i = $nb_listings - 1, $count = $baseListing->get('ChildrenListings')->count(); $i < $count; $i++)
        {
          $baseListing->get('ChildrenListings')->get($i)->delete();
        }
      }
    }

    public function saveListingContactList($listing)
    {
      if (!$this->isValid())
      {
        throw $this->getErrorSchema();
      }

      if (!isset($this->widgetSchema['listing_contact_list']))
      {
        // somebody has unset this widget
        return;
      }

      $q = Doctrine_Query::create()
            ->delete()
            ->from('ListingContact r')
            ->where('r.listing_id = ?', current($listing->identifier()))
            ->execute();

      $values = $this->getValue('listing_contact_list');
      if (is_array($values))
      {
        foreach ($values as $value)
        {
          $obj = new ListingContact();
          $obj->listing_id = current($listing->identifier());
          $obj->owner_contact_id = $value;
          $obj->save();
        }
      }
    }

    public function duplicateFromListing($baseListing)
    {
      $duplicate = $this->duplicate($baseListing);

      $listing = new Listing();
      $listing->fromArray($duplicate);
      $listing->days_allowed = null;
      $listing->listing_id = $baseListing->getId();
      $listing->owner_id = $this->getOption('user')->getOwnerId();
      $listing->status = Listing::STATUS_NEW;
      $listing->created_at = date('Y-m-d', time());

      $this->setDefaultsFromListing($listing);

      if (isset($duplicate['listing_contact_list'])) {
        $this->setDefault('listing_contact_list', $duplicate['listing_contact_list']);
      }

      return $listing;
    }

    private function duplicate($listing)
    {

      $defaults = $listing->toArray(false);
      unset(
        $defaults['id'],
        $defaults['created_at'],
        $defaults['notify_instance'],
        $defaults['notify_daily'],
        $defaults['status'],
        $defaults['is_new_type_payment'],
        $defaults['listed_by']
      );

      $defaults['is_new_type_payment'] = 'yes';
      $terms = $listing->Terms;
      $addons_data = $listing->Addons->getData();
      $defaults['Addons'] = array();

      if (count($addons_data) > 0){
        foreach ($addons_data as $key => $item){
          $defaults["Addons"][$key]['title'] = $item["title"];
          $defaults["Addons"][$key]['name']  = $item["name"];
          $defaults["Addons"][$key]['price'] = $item["price"];
        }
      }

      if ($terms->exists()){
        $defaults['terms_title'] = $terms["title"];
        $defaults['terms_content'] = $terms["content"];
      }

      $contacts = $listing->get('OwnerContacts');
      $defaults['OwnerContacts'] = array();

      if (count($contacts)) {
        $listing_contact_list = array();
        foreach ($contacts as $contact) {
          $listing_contact_list[] = $contact->getId();
        }

        $defaults['listing_contact_list'] = $listing_contact_list;
      }

      return $defaults;
    }
}
