<?php

use Transfercar\Ranking\SearchScore;

class OwnerRentalForm extends BaseForm
{
    public function configure()
    {
        parent::configure();

        $this->widgetSchema['date_available_from'] = new tcWidgetFormDateWithPicker(array(
          'domain' => $this->getOption('domain')
        ));
        $this->widgetSchema['terms_id'] = new sfWidgetFormInputHidden();

        $this->widgetSchema['vehicle_type_id'] = new tcWidgetFormOwnerVehicle(array(
          'user' => $this->getOption('user'),
          'country' => $this->getOption('country'),
          'tc_vehicles' => false
        ));

        $this->widgetSchema['pickup_location_id'] = new tcWidgetFormOwnerBranch(array(
          'user' => $this->getOption('user'),
          'country' => $this->getOption('country'),
          'multiple' => true
        ));

        $this->validatorSchema['vehicle_type_id'] = new tcValidatorVehicleType(array(
          'user' => $this->getOption('user')
        ));

        $this->validatorSchema['pickup_location_id'] = new tcValidatorRentalBranch(array(
          'user' => $this->getOption('user')
        ));

        $this->widgetSchema['daily_rental_price'] = new sfWidgetFormInput();
        $this->widgetSchema['weekly_rental_price'] = new sfWidgetFormInput();
        $this->widgetSchema['monthly_rental_price'] = new sfWidgetFormInput();
        $this->widgetSchema['extra_day_rental_price'] = new sfWidgetFormInput();
        $this->widgetSchema['daily_insurance_rate'] = new sfWidgetFormInput();
        $this->widgetSchema['weekly_insurance_rate'] = new sfWidgetFormInput();
        $this->widgetSchema['monthly_insurance_rate'] = new sfWidgetFormInput();
        $this->widgetSchema['excess'] = new sfWidgetFormInput();
        $this->widgetSchema['reduced_excess'] = new sfWidgetFormInput();

        $this->widgetSchema['rates_file'] = new sfWidgetFormInputFile();


        $this->widgetSchema['tyres_included'] = new sfWidgetFormInputCheckbox([], ['value' => 1]);
        $this->widgetSchema['roadside_assistance_included'] = new sfWidgetFormInputCheckbox([], ['value' => 1]);
        $this->widgetSchema['maintenance_included'] = new sfWidgetFormInputCheckbox([], ['value' => 1]);
        $this->widgetSchema['comment'] = new sfWidgetFormTextarea();


        $this->validatorSchema['tyres_included'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['roadside_assistance_included'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['maintenance_included'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['comment'] = new sfValidatorString(['required' => false]);

        $this->validatorSchema['daily_rental_price'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['weekly_rental_price'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['monthly_rental_price'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['extra_day_rental_price'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['daily_insurance_rate'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['weekly_insurance_rate'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['monthly_insurance_rate'] = new sfValidatorString(['required' => false]);
        $this->validatorSchema['excess'] = new sfValidatorString();
        $this->validatorSchema['reduced_excess'] = new sfValidatorString();
        $this->validatorSchema['rates_file'] = new sfValidatorAnd([new sfValidatorFile(['required' => false]), new sfValidatorCallback(['required' => false, 'callback' => [$this, 'validateRates']])], ['required' => false]);

        $dateFormat = sfConfig::get('app_date_time_pattern_'.$this->getOption('domain'), 'dd/mm/yyyy');
        $pattern = preg_replace_callback('#([dmy]{2,4})/([dmy]{2,4})/([dmy]{2,4})#i', function($match) {
          $pattern = array();

          foreach ($match as $key => $value) {
            if ($key === 0)
            {
              continue;
            }

            $pKey = '';
            switch ($value[0])
            {
              case 'd':
                $pKey = 'day';
                break;
              case 'm':
                $pKey = 'month';
                break;
              case 'y':
                $pKey = 'year';
                break;
            }

            $pattern[] = '(?P<'.$pKey.'>\d{'.strlen($value).'})';
          }

          return '~'.implode('/', $pattern).'~';
        }, strtolower($dateFormat), 1);

        $this->validatorSchema['date_available_from'] = new sfValidatorDate(array(
            'min' => time() - 86400,
            'date_output' => 'Y-m-d',
            'date_format' => $pattern,
            'date_format_error' => 'must be '.$dateFormat,
          ), array(
            'min' => 'Date cannot be in the past',
            'invalid' => 'Invalid value'
          ));

        $this->widgetSchema->setNameFormat('rental[%s]');
    }


    public function setDefaultsFromRental($rental)
    {
      $defaults = $rental->toArray(false);

      if ($rental->get('owner_vehicle_type_id'))
      {
        $defaults['vehicle_type_id'] = 'owner_'.$rental->get('owner_vehicle_type_id');
      }
      else
      {
        $defaults['vehicle_type_id'] = 'tc_'.$rental->get('vehicle_type_id');
      }

      $defaults['date_available_from'] = $this->fixDate(@$defaults['date_available_from'], 'today');
      $defaults['date_return_by'] = $this->fixDate(@$defaults['date_return_by'], 'tomorrow');

      $pickups = [];
      foreach ($rental->get('RentalPickup') as $pickup) {
        $pickups[] = 'owner_'.$pickup->get('owner_pickup_branch_id');
      }
      $defaults['pickup_location_id'] = $pickups;

      $this->setDefaults($defaults);
    }

    private function fixDate($date, $when)
    {
      $time = strtotime($date);
      $defaultTime = strtotime($when);

      $time = $time < $defaultTime ? $defaultTime : $time;

      return date('Y-m-d', $time);
    }

    public function save($rental)
    {
      $conn = Doctrine::getTable('Rental')->getConnection();
      $conn->beginTransaction();

      try
      {
        $this->doSave($rental);
        $conn->commit();
      }
      catch (Exception $e)
      {
        $conn->rollback();
        throw $e;
      }
    }

    private function doSave($rental)
    {
      $values = $this->getValues();
      $object = $values['vehicle_type_id'];

      if ($object instanceof OwnerVehicleType)
      {
          $values['owner_vehicle_type_id'] = $object->getId();
          $values['vehicle_type_id'] = $object->get('vehicle_type_id');
      }
      else
      {
        $values['vehicle_type_id'] = $object->getId();
        $values['owner_vehicle_type_id'] = null;
      }

      $pickups = $values['pickup_location_id'];
      $rental->get('RentalPickup')->delete();

      foreach ($pickups as $pickup) {
        $rentalPickup = new RentalPickup();
        $rentalPickup->set('pickup_location_id', $pickup instanceof OwnerBranch ? $pickup->get('location_id') : $pickup->getId());
        $rentalPickup->set('owner_pickup_branch_id', $pickup instanceof OwnerBranch ? $pickup->getId() : null);
        $rental->get('RentalPickup')->add($rentalPickup);
      }

      if ($values['rates_file']) {
        $values['rates'] = $values['rates_file'];
      }

      $rental->fromArray($values);

      $isNew = !$rental->exists();
      if ($isNew) {
        $rental->set('status', Rental::STATUS_NEW);
        $rental->set('created_at', date('Y-m-d'));
        $rental->set('listed_by', $this->getOption('user')->getId());
        $rental->set('owner_id', $this->getOption('user')->getOwnerId());
      }

      $rental->set('updated_by', $this->getOption('user')->getId());
      $rental->set('updated_at', date('Y-m-d'));
      $rental->save();
    }

  public function duplicateFromRental($rental) 
  {
    $defaults = $rental->toArray(false);

    unset(
      $defaults['id'],
      $defaults['created_at'],
      $defaults['status'],
      $defaults['listed_by']
    );
    
    $newRental = new Rental();
    $newRental->fromArray($defaults);
    $newRental->owner_id = $this->getOption('user')->getOwnerId();
    $newRental->status = Rental::STATUS_NEW;
    $newRental->created_at = date('Y-m-d', time());


    $pickups = [];
    foreach ($rental->get('RentalPickup') as $pickup) {
      $rentalPickup = new RentalPickup();
      $rentalPickup->set('pickup_location_id', $pickup->get('pickup_location_id'));
      $rentalPickup->set('owner_pickup_branch_id', $pickup->get('owner_pickup_branch_id'));
      $newRental->get('RentalPickup')->add($rentalPickup);
      $pickups[] = 'owner_'.$pickup->get('owner_pickup_branch_id');
    }

    $this->setDefaultsFromRental($newRental);
    $this->setDefault('pickup_location_id', $pickups);
    
    return $newRental;
  }

  public function validateRates($validator, $value)
  {
    $delimiter = $this->detectDelimiter(fopen($value->getTempName(), 'rb'));

    $readcsv = function($data) use ($delimiter) {
      return str_getcsv($data, $delimiter);
    };
    $csv = array_map($readcsv, file($value->getTempName()));
    array_walk($csv, function(&$a) use ($csv) {
      $a = array_combine($csv[0], $a);
    });
    $header = array_values(array_shift($csv));
    sort($header);

    $validHeader = [
      'from_date', 
      'to_date', 
      'daily_rate', 
      'weekly_rate', 
      'monthly_rate', 
      'insurance_daily_rate', 
      'insurance_weekly_rate', 
      'insurance_monthly_rate'
    ];

    sort($validHeader);

    if ($header != $validHeader) {
      throw new sfValidatorError($validator, 'invalid', array('value' => (string) $value));
    }

    return $csv;

  }

  private function detectDelimiter($fh)
  {
      $data_1 = null;
      $data_2 = null;
      $delim_list = ["\t", ";", "|", ","];
      $delimiter = ',';
      foreach($delim_list as $key=>$value)
      {
          $data_1 = fgetcsv($fh, 4096, $value);
          $delimiter = sizeof($data_1) > sizeof($data_2) ? $value : $delimiter;
          $data_2 = $data_1;
      }

      return $delimiter;
  }

  /**
   * For custom rendered forms
   */
  public function renderCSRF()
  {
    return $this->widgetSchema->renderField(self::$CSRFFieldName,
            $this->getCSRFToken(self::$CSRFSecret));
  }
}
