<?php

class OwnerUserForm extends BaseUserForm
{
    public function configure()
    {
        $this->useFields(array(
            'company',
            'email',
            'address_line1',
            'address_line2',
            'suburb',
            'postcode',
            'city',
            'website',
            'contact_phone',
            'description',
            'driver_request_instructions',
            'driver_accepted_instructions',
            'logo',
            'closed_in_week_end',
            'week_end_closing_time',
            'week_end_opening_time',
            'timezone',
            'license_type',
            'allow_debit_card',
            'debit_card_text',
            //'data_export_format'
        ));

        $this->widgetSchema['logo'] = new sfWidgetFormInputFile();
        $this->validatorSchema['logo'] = new sfValidatorFile(array(
            'required' => false,
            'validated_file_class' => 'tcOwnerLogoValidatedFile'
        ));

        $this->widgetSchema['description'] = new sfWidgetFormTextarea();
        $this->widgetSchema['driver_request_instructions'] = new sfWidgetFormTextarea();
        $this->widgetSchema['driver_accepted_instructions'] = new sfWidgetFormTextarea();

        $this->widgetSchema['closed_in_week_end']    = new sfWidgetFormChoice(array(
            'choices' => $this->translate(array(1 => 'Yes', 0 => 'No')),
            'expanded' => true,
            'renderer_options' => array('formatter' => array($this, 'formatChoices'))
        ));
        $this->widgetSchema['week_end_closing_time'] = new sfWidgetFormChoice(array('choices' => $this->getTimes(0, 86400)));
        $this->widgetSchema['week_end_opening_time'] = new sfWidgetFormChoice(array('choices' => $this->getTimes(0, 86400)));
        $this->widgetSchema['timezone']   = new tcWidgetFormTimezone();

        $this->widgetSchema['license_type'] = new sfWidgetFormChoice(array(
            'choices'  => $this->getLicenseChoices(),
            'expanded' => true,
            'renderer_options' => array('formatter' => array($this, 'formatLicenseChoices'))
        ));

        $this->validatorSchema['license_type'] = new sfValidatorChoice(array(
            'choices' => array_keys($this->getLicenseChoices())
        ));

        $this->widgetSchema->setLabels(array(
            'address_line1' => 'Address line 1',
            'address_line2' => 'Address line 2',
        ));

        $this->widgetSchema['debit_card_text'] = new sfWidgetFormTextarea();

        if (!$this->getObject()->get('timezone'))
        {
            $this->getObject()->set('timezone', $this->getObject()->getDefaultTimeZone());
        }

        $choices=Transfercar\Export\BookingsExport::getLabels();
        $roChoices = $this->getObject()->get('data_export_format');
        $this->widgetSchema['data_export_format'] = new sfWidgetFormChoice([
            'multiple' => true,
            'choices' => array_combine($roChoices, $roChoices)
        ]);

        $this->validatorSchema['data_export_format'] = new sfValidatorChoiceMany([
		'choices' => $choices,
		'required'=>false
        ]);

        $this->widgetSchema['data_export_format_available'] = new sfWidgetFormChoice([
            'multiple' => true,
            'choices' => array_diff($choices, (array) $this->getObject()->get('data_export_format'))
        ]);
    }

    public function updateObject()
    {
        if ($logo = $this->getValue('logo')) {
            $path = $logo->saveThumbnail();
            $this->values['logo'] = $path;

            if ($this->getObject()->get('logo')) {
                $filename = str_replace('/uploads', '', $this->getObject()->get('logo'));
                unlink(sfConfig::get('sf_upload_dir').$filename);
            }
        } else {
            unset($this->values['logo']);
        }
        parent::updateObject();
    }

    private function translate($choices)
    {
        return array_map($this->widgetSchema->getFormFormatter()->getTranslationCallable(), $choices);
    }

    private function getTimes($earliest, $latest)
    {
      $time  = strtotime('today');
      $times = array('' => '');

      for ($seconds = $earliest; $seconds <= $latest; $seconds+=1800)
      {
        $ts = strtotime('+ '.$seconds.'seconds', $time);
        $times[date('H:i:s', $ts)] = date('h:i A', $ts);
      }

      return $times;
    }

    private function getLicenseChoices()
    {
        return $this->translate(array(
            'full'                   => 'A valid and full/unrestricted driving licence issued in [local country] only. No international driving licences accepted',
            'full_and_international' => 'A valid and full/unrestricted driving licence issued in [local country], -or- a valid International Driving Permit, accompanied by a valid driving licence from your country',
            'full_and_translated'    => 'A valid and full/unrestricted driving licence issued in [local country], -or- a legally confirmed English translation of your driving licence, accompanied by a valid driving licence from your country',
            'full_or_foreign'        => 'A valid and full/unrestricted driving licence issued in [local country], -or- a valid driving licence from your country',
        ));
    }

    public function formatLicenseChoices($widget, $inputs)
    {
      $rows = array();
      foreach ($inputs as $input)
      {
        $label = preg_replace(
          '/(<label for="[^"]+")>/',
          '\\1 class="checkbox"> '.$input['input'],
          $input['label']
        );
        $rows[] = $widget->renderContentTag('li', $label);
      }

      return $widget->renderContentTag('ul', implode($widget->getOption('separator'), $rows), array('class' => 'no-bullet large'));
    }


    public function formatChoices($widget, $inputs)
    {
        $rows = array();
        foreach ($inputs as $input)
        {
          $formattedInput = str_replace('">', '">'.$input['input'].' ', $input['label']);
          $rows[] = $widget->renderContentTag('div', $formattedInput, array('class' => 'six columns'));
        }

        return $widget->renderContentTag('div', implode($this->getOption('separator'), $rows), array('class' => 'row text-center'));
    }
}
