<?php

class RequestEmailOptions extends sfForm
{
    private $object;

    public function __construct($object, $options = array(), $CSRFSecret = null)
    {
        $this->object = $object;

        parent::__construct($options, $CSRFSecret);
        $this->updateDefaultsFromObject();
    }

    private function updateDefaultsFromObject()
    {
        $option = $this->object->get('request_email_option');
        $this->setDefault('option', $option);
        $this->setDefault('ro_review_notify', $this->object->get('ro_review_notify'));
        $this->setDefault('accepted_request_notify', $this->object->get('accepted_request_notify'));

        switch ($option) {
            case 'manager':
            case 'contact':
                list($type, $id) = explode(':', $this->object->get('request_email_option_value'));
                $this->setDefault('option', $this->object->get('request_email_option_value'));
                $this->setDefault($option, $id);
                break;
        }
    }

    public function configure()
    {
        $this->setWidgets(array(
            'option' => new tcWidgetFormRequestEmailOptions(array(
                'owner_id' => $this->object->getOwnerId()
            )),
            'ro_review_notify'        => new sfWidgetFormInputCheckbox(),
            'accepted_request_notify' => new sfWidgetFormInputCheckbox()
        ));

        $this->setValidators(array(
            'option' => new sfValidatorChoice(array('choices' => array(
                'listing_manager', 'branch_contact', 'manager', 'contact'
            ))),
            'contact'  => new sfValidatorPass(),
            'manager'  => new sfValidatorPass(),
            'ro_review_notify'        => new sfValidatorBoolean(),
            'accepted_request_notify' => new sfValidatorBoolean()
        ));

        $this->widgetSchema->setNameFormat('request_email[%s]');
    }

    public function save()
    {
        $this->object->set('request_email_option', $this->getValue('option'));

        switch ($this->getValue('option')) {
            case 'manager':
            case 'contact':
                $data = $this->getValue($this->getValue('option'));
                $this->object->set('request_email_option_value', $this->getValue('option').':'.$data);
                break;
        }

        $this->object->set('ro_review_notify', $this->getValue('ro_review_notify'));
        $this->object->set('accepted_request_notify', $this->getValue('accepted_request_notify'));

        $this->object->save();
    }
}