<?php
/**
 * Form for credit card data
 * @package transfercar
 * @subpackage form
 */

use Transfercar\Stripe\TcStripe;
use Transfercar\Exception\CreditCardException;

class paymentForm extends transfercarForm
{

    /**
     * Prepare the form
     */
    public function configure()
    {
      // Set fieldset
      $this->setWidgets(array(
        'type'    => new sfWidgetFormInput(),
        'holder_name' => new sfWidgetFormInput(),
        'stripeToken' => new sfWidgetFormInputHidden(),
        'save' => new sfWidgetFormInputCheckbox(),
        'amount' => new sfWidgetFormInput(array(), array(
          'size' => 10, 
          'readonly' => 'readonly'
        )),
        'receipt' => new sfWidgetFormInputCheckbox()
      ));


      // Set form name schema
      $this->widgetSchema->setNameFormat('payment[%s]');

      // Set labels
      $this->widgetSchema->setLabels(array(
          'type' => 'Card Type',
          'save' => 'Save this card for next time I credit my account',
          'amount' => 'Amount',
          'receipt' => 'Send payment receipt from payment operator to my e-mail'
      ));

      $this->setValidators(array(
        'type'    => new sfValidatorString(array('required' => false)),
        'stripeToken' => new sfValidatorString(array('required' => false)),
        'save' => new sfValidatorPass(array('required' => false)),
        'amount' => new sfValidatorInteger(array('min' => 1), array('min' => 'Invalid value')),
        'receipt' => new sfValidatorPass(array('required' => false))
      ));


      if ($this->getOption('user_id')) {
        $query = Doctrine::getTable('CreditCard')
            ->createQuery()
            ->where('owner_id = ?', $this->getOption('user_id'))
            ->addWhere('holder_name like ?', 'src_%')
        ;

        $countQuery = clone $query;

        if ($countQuery->count()) {
          $this->widgetSchema['balance_card'] = new sfWidgetFormChoice(array(
            'choices' => array('new' => 'new', 'existing' => 'existing')
          ));

          $this->validatorSchema['balance_card'] = new sfValidatorChoice(array(
            'choices' => array('new', 'existing')
          ));

          $this->widgetSchema['card_id'] = new sfWidgetFormDoctrineSelect(array(
            'model' => 'CreditCard',
            'method' => 'getNumber',
            'add_empty' => false,
            'criteria' => clone $query
          ));

          $this->validatorSchema['card_id'] = new sfValidatorDoctrineChoice(array(
            'model' => 'CreditCard',
            'required' => false,
            'query' => clone $query
          ));
        }
      }

      $this->addCSRFProtection(sfConfig::get('sf_csrf_secret'));
  }

  public function configureNoShowFee(){
    unset($this['amount']);
    unset($this['receipt']);
    unset($this['save']);
  }

  public function renderHiddenFields()
  {
    $output = '';
    foreach ($this->getFormFieldSchema() as $name => $field)
    {
      if ($field->isHidden())
      {
        $output .= $field->render();
      }
    }

    return $output;
  }

  public function configureVirtualCurrency()
  {
    unset($this['amount']);
  }

  private function addStripeCard($user, $token)
  {
    $stripe = new TcStripe();
    $user_id = $this->getOption('user_id');
    $user = Doctrine::getTable('User')->find($user_id);
    if (!$user->get('stripe_token')) {
      $customer = $stripe->createCustomer($user->get('email'), $user->full_name(), $user->get('is_owner') ? 'ro' : 'driver', $token);
      $user->set('stripe_token', $customer->id);
      $user->save();
      $card = $customer->sources->data[0];
    } else {
      $card = $stripe->addCard($user->get('stripe_token'), $token);
    }

    return $card->id;
  }

  public function checkCreditCard($validator, $payment_data)
  {
      if (sfConfig::get('sf_payment_simulate_token'))
      {
        $payment_data['token'] = sfConfig::get('sf_payment_simulate_token');
        return $payment_data;
      }

      if ($payment_data['stripeToken']) {

        $user_id = $this->getOption('user_id');
        $user = Doctrine::getTable('User')->find($user_id);
        try {
          $payment_data['token'] = $this->addStripeCard($user, $payment_data['stripeToken']);
        }
        catch (Stripe\Error\Card $error) {
          $error = new sfValidatorError($validator, 'invalid');
          throw new sfValidatorErrorSchema($validator, array('type' => $error));
        }
        catch (\Stripe\Error\Base $error) {
          $validator->setMessage('invalid', $error->getMessage());
          $error = new \sfValidatorError($validator, 'invalid');
          throw new \sfValidatorErrorSchema($validator, array('type' => $error));
        }
        catch (\Exception $error) {
          $validator->setMessage('invalid', $error->getMessage());
          $error = new \sfValidatorError($validator, 'invalid');
          throw new \sfValidatorErrorSchema($validator, array('type' => $error));
        }

        return $payment_data;
      }

      if (isset($payment_data['balance_card']) && $payment_data['balance_card'] == 'existing') {
        return $payment_data;
      } 
      elseif (!isset($payment_data['stripeToken'])) {
        $validator->setMessage('invalid', 'Invalid credit card');
        $error = new \sfValidatorError($validator, 'invalid');
        throw new \sfValidatorErrorSchema($validator, array('type' => $error));
      }

      return $payment_data;
    }

    /**
     * Save common(for token) parameters for credit card in DB
     * @param $card_data
     * @return integer - credit card id
     */
    public function saveCreditCard($card_data)
    {
      if (isset($card_data['balance_card']) && $card_data['balance_card'] == 'existing') {
        return $card_data['card_id'];
      }

      $user_id = $this->getOption('user_id');
      $user = Doctrine::getTable('User')->find($user_id);

      if (!$user->get('stripe_token') || !isset($card_data['token'])) {
        throw new CreditCardException("Invalid credit token");
      }

      $stripe = new TcStripe();
      $card = $stripe->retrieveCard($user->get('stripe_token'), $card_data['token']);

      $id = '';
      if ($card instanceof \Stripe\Source){
          $id = $card->id;
          $card = $card->card;
      } else {
          $id = $card->id;
      }
      $card_save_number = $card->brand.' - **** ' . $card->last4;

      $ccard = new CreditCard();
      $ccard->card_type = 0;
      $ccard->expiry = null;
      $ccard->holder_name = $id;
      $ccard->number = $card_save_number;
      $ccard->csv = '';
      $ccard->owner_id = $user_id;
      $ccard->country = $card->country;
      $ccard->save();

      return $ccard->id;
    }

    public function chargeCreditCard(Request $request, $amount, $username)
    {
        $listing = $request->get('Listing');

        if (!$listing->hasBond()) {
          return true;
        }

        $logger = new sfFileLogger(new sfEventDispatcher(), array(
          'file' => sfConfig::get('sf_log_dir') . '/payments_php.log',
          'format' => '%time% %message%%EOL%',
          'timeFormat' => '%g-%m-%d %H:%M:%S'
        ));
        $logger->log("stripe:process-payment started >>>");

        //Get accepted response
        $listing = $request->get('Listing');

        $payment = new Payments();
        $payment->type = 'cc';
        $payment->date= date('Y-m-d H:i:s', time());
        $payment->amount = $amount;
        $payment->user_id = $request->get('driver_id');
        $payment->status = Payments::STATUS_NEW;
        $payment->description = $this->getValue('description');
        $payment->is_new_type_payment = "yes";
        $payment->save();

        $cct = new CCTransaction();
        $cct->status = CCTransaction::STATUS_NEW;
        $cct->unique_key = implode('.', array(
           $listing->Owner->id, $listing->id, time(), mt_rand()));
        $cct->credit_card_id = $request->CreditCard->id;
        $cct->communication_log = array();
        $cct->Payment = $payment;
        $cct->save();

        $url = sfConfig::get ('sf_payment_url_new');
        $accounts = sfConfig::get('sf_payment_account');

        $requestDomain = $request->getCountryDomain();
        $ccDomain = $request->get('CreditCard')->get('country');
        $domain = $requestDomain;

        if ($requestDomain === 'au' && $ccDomain === 'NZ') {
          $domain = 'nz';
          $converter = new Transfercar\CurrencyRate\Converter();
          $amount = $converter->convert('AUD', 'NZD', $amount);
        }
        try
        {
          $stripe = new TcStripe();
          $charge = $stripe->chargeCreditCard([
            'source' => $request->CreditCard->holder_name, 
            'customer' => $request->get('Driver')->get('stripe_token'),
            'amount' => (int) $amount * 100,
            'description' => substr('BND/'.$request->getIncrementedId().'/'.$username, 0, 50),
            'statement_descriptor' => 'Transfercar: bond relocation '.$request->getIncrementedId(),
            'currency' => strtolower($domain.'d')
          ]);
        } 
        catch (\Stripe\Error\Base $ex)
        {
          $is_error = true;
        }

        $log = $cct->communication_log;
        if(!is_array($log)){
            $log = array();
        }

        //was ERROR: <SOAP FAULT> or <PHP>
        if ($is_error == true)
        {
            
            $log[] = '';
            $cct->communication_log = $log;
            $cct->status_message = $ex->getMessage();
            $this->fee_error = $cct->status_message;
        }
        else//all OK
        {
            $log[] = serialize($result);
            $cct->communication_log = $log;
            $cct->status_message = $result->transactionresult->Message;
        }

        // Check transaction status
        $status = $is_error ? 'declined' : $result->transactionresult->Status;

        if ($status != 'SUCCESSFUL')
        {
          $logger->log(print_r($log, 1));
          $logger->log("stripe:process-payment Transaction declined");

          $cct->status = CCTransaction::STATUS_REJECTED;
          $cct->save();

          $logger->log("stripe:process-payment Transaction saved");

          $payment->status = Payments::STATUS_REJECTED;
          $payment->save();

          $logger->log("stripe:process-payment Payment saved");

          $is_error = true;
          $this->fee_error = $result->transactionresult->Message;

        } else { // transaction has been accepted
          $logger->log("stripe:process-payment Transaction accepted");

          $cct->status = CCTransaction::STATUS_ACCEPTED;
          $cct->save();

          $logger->log("stripe:process-payment Transaction saved");

          $payment->status = Payments::STATUS_ACCEPTED;
          $payment->save();

          $logger->log("stripe:process-payment Payment saved: #" . $payment->get('id'));
        }

        $cct->save();
        return !$is_error;
    }
}

