<?php

class ListingFormHandler
{
    public function handle($form, $user)
    {
      $values = $form->getValues();
      // Convert date to mysql format
      $values["date_return_by"] = date('Y-m-d', strtotime($values["date_return_by"]));
      $values["date_available_from"] = date('Y-m-d', strtotime($values["date_available_from"]));

      $user_id = $user->id;

      // Calculate fee from given data
      $price = Doctrine_Query::create()
       ->from('Prices p')
       ->leftJoin('p.CategoryFrom pf')
       ->leftJoin('p.CategoryTo pt')
       ->leftJoin('pf.Locations pfl')
       ->leftJoin('pt.Locations ptl')
       ->where('pfl.id = ?', $values['pickup_location_id'])
       ->addWhere('ptl.id = ?', $values['dropoff_location_id'])
       ->fetchOne();

      if(is_object($price)){
        $values['price'] = $price->fee;
      }

      $listing = new Listing();
      $listing->fromArray($values);
      $listing->owner_id = $user_id;
      $listing->status = Listing::STATUS_NEW;
      $listing->created_at = date('Y-m-d', time());
      $listing->is_new_type_payment = "yes";


      $terms_id = 0;//intval($request->getParameter("terms_id"));
      if ($terms_id > 0){
      $old_term_info = Doctrine_Query::create()
           ->from('Terms')
           ->where('id = ?', $terms_id)
           ->fetchOne();
      }

      sfLoader::loadHelpers(array('XSS'));
      // Allowed HTML tags (used for XSS prevention)
      $allowed_tags = array(
      'p', 'strong', 'em', 'span', 'ul', 'ol', 'li', 'hr', 'h1', 'h2', 'h3',
      'h4', 'dl', 'dt', 'dd'
      );

      if(empty($values['terms_content']))
      {
        $listing->terms_id = null;
      }
      else
      {
        if ($terms_id > 0 && is_object($old_term_info) && xss_cleanup($values['terms_title'], $allowed_tags) == $old_term_info->title
        && xss_cleanup($values['terms_content'], $allowed_tags) == $old_term_info->content){
          $listing->terms_id = $terms_id;
        }
        else
        {
          $terms = new Terms();
          $terms->user_id = $user_id;
          $terms->title = xss_cleanup($values['terms_title'], $allowed_tags);
          $terms->content = xss_cleanup($values['terms_content'], $allowed_tags);
          $terms->save();
          $listing->terms_id = $terms->id;
        }
      }
      $listing->save();

      foreach($values['addons'] as $addon) {
        if(empty($addon['name']) || empty($addon['price']))
          continue;
        $cAddon = new Addon();
        $cAddon->fromArray($addon);
        $cAddon->listing_id = $listing->id;
        $cAddon->user_id    = $user_id;
        $cAddon->save();
      }

      $listing->refresh(true);

      return $listing;
    }
}