<?php

use Transfercar\Mandrill\TcMandrill;
use Transfercar\Mailchimp\MailchimpApi;

class RegistrationFormHandler
{
    public static function save($request, $sfUser, $params)
    {
        if (!isset($params["source_information"]) || $params["source_information"] == "") {
            $params["source_information"] = "no info";
        } elseif ($params["source_information"] == "other") {
            $params["source_information"] = $params["own_variant"];
        } else {
            $listVariants = sfConfig::get('mod_main_listAnswerVariants', "");
            $params["source_information"] = $listVariants[$params["source_information"]];
        }

        if(isset($params['email_newsletter']) && $params['email_newsletter'] == "on"){
            $params['email_newsletter'] = 1;
            $params['email_country_newsletter'] = $sfUser->getAttribute('country');
        }else{
            $params['email_country_newsletter'] = 'none';
        }

        if (isset($params['email_notify_type_freq']) && $params['email_notify_type_freq'] == "on") {
            $params['email_notify_type_freq'] = 'daily';
            $params['email_notify_new_reloc'] = 'all';
            $params['email_country_new_reloc'] = $sfUser->getAttribute('country');
        } else {
            $params['email_country_new_reloc'] = 'none';
            $params['email_notify_new_reloc']  = 'none';
            $params['email_notify_type_freq'] = null;
        }

        // hashed with sha1 for bc, but use bcrypt at the end, see User::setPassword()
        if (isset($params['password']) && $params['password']) {
            $params['password'] = sha1($params['password']);
            unset($params['confirm']);
        }

        return self::saveFromParams($sfUser, $request, $params);
    }

    public static function saveFromParams($sfUser, $request, $params)
    {
        $user_profile = $sfUser->getAttribute('profile');
        $activation_code = mt_rand(100000, 999999);

        if (is_object($user_profile) && $user_profile->user_status = "subscribe") {
            $params["id"] = $user_profile->get('id');
            $user = Doctrine_Query::create()
                        ->from('User')
                        ->where('id = ?', $params["id"])
                        ->fetchOne();
            $params["user_status"] = "inactive";
        } else {
            $user = Doctrine::getTable('User')->findOneByEmail($params['email']);
        }

        if (!$user instanceof User) {
            $user = new User();
        }

        $user->fromArray($params);
        $user->activation_code      = $activation_code;
        $user->country_domain_id    = $sfUser->getAttribute('country');
        $user->sms_number_confirmed = false;
        $user->user_status = 0;
        $user->is_owner = User::USER_ONLY_DRIVER;
        $user->save();

        Doctrine::getTable('NotificationUser')->createAccount($user);

        // tcDoctrineRecord::getEventDispatcher()->notify(new sfEvent($user, 'user.updated'));

        if ($user->get('email_newsletter'))
        {
            //Add user to Mailchimp Web service
            $domain = $sfUser->getUserDomain();
            $mc = new MailchimpApi();
            $mc->subscribe(sfConfig::get('app_mailchimp_'.$domain.'_list_apikey'), $params['email'], $params['firstname'], $params['lastname']);
        }

        self::sendWelcomeEmail($user);

        return $user;
    }

    public static function sendWelcomeEmail($user)
    {
        $template_data = array();
        $template_data['username'] = $user->username;
        $template_data['firstname'] = $user->firstname;
        $template_data['lastname'] = $user->lastname;
        $template_data['when_registered'] = $user->when_registered;

        $template_data['code'] = $user->activation_code;
        $url_code = $user->id.'.'.$user->activation_code; // 21.3432432

        $controller = sfContext::getInstance()->getController();
        $request = sfContext::getInstance()->getRequest();
        $sfUser = sfContext::getInstance()->getUser();

        $url = $request->getParameter('referer', $controller->genUrl('@signedUp'));
        $url = $sfUser->getAttribute('oauth2referer', $url);
        $template_data['link'] = $controller->genUrl('home/activateAccount?code=' . $url_code, true).'?redirect='.$url;
        $template_data['activation_code'] = $user->activation_code;
        $template_data['activation_link_label'] = 'Confirm your email address'. (strpos($url, 'request/confirm') !== false ? ' and complete your request' : '');
        $template_data['email_opt_link_real'] = $controller->genUrl('main/emailUnsubscribe?email='.$user->get('email'), true);
        $template_data['email_opt_link'] = $controller->genUrl('main/emailUnsubscribe?email='.$user->get('email'), true);

        $mandrill = new TcMandrill();
        $mandrill->sendTemplate(
            'Welcome to Transfercar',
            $user->getEmailToParams(),
            $template_data,
            'activation-code'
        );
    }
}