<?php

function render_row($field, $label, $fieldAttributes = array()) {
  $errorClass = array();

  if ($field->hasError()) {
    $errorClass = array('class' => 'error');

    if (isset($fieldAttributes['class'])) {
      $fieldAttributes['class'] .= ' error';
    }
  }

  $tooltip = isset($fieldAttributes['help']) ? tooltip($fieldAttributes['help']) : '';

  return ($label ? $field->renderLabel($label.$tooltip, $errorClass) : '') .
         $field->render($fieldAttributes) .
         $field->renderError();
}



function object_custom_select_tag($object, $method, $options = array(), $default_value = null)
{
  $object = $object instanceof sfOutputEscaper ? $object->getRawValue() : $object;
  $related_class = _get_option($options, 'related_class', false);
  $peer_method = _get_option($options, 'peer_method');

  if (is_object($object))
  {
    $value = _get_object_value($object, $method, $default_value);
    $value = is_object($value) ? $value->getId() : $value;
  }
  else
  {
    $value = $object;
  }

  $select_options = call_user_func(array($related_class, $peer_method));
  $option_tags = options_for_select($select_options, $value, $options);

  return select_tag(_convert_method_to_name($method, $options), $option_tags, $options);
}

function object_custom_select_location_tag($object, $method, $options = array(), $default_value = null)
{
  $object = $object instanceof sfOutputEscaper ? $object->getRawValue() : $object;
  $options = _parse_attributes($options);
  $related_class = _get_option($options, 'related_class', false);
  $peer_method = _get_option($options, 'peer_method');
  $text_method = _get_option($options, 'text_method');
  $key_method = _get_option($options, 'key_method', 'getPrimaryKey');

  if (is_object($object))
  {
    $value = _get_object_value($object, $method, $default_value);
    $value = $object->get('country_id') ? $object->get('country_id').'_'.$value->getId() : $value;
    $prefix = $object->get('country_id').'_';
  }
  else
  {
    $value = $object;
    $prefix = '';
  }

  $select_options = call_user_func(array($related_class, $peer_method), $prefix);
  $option_tags = options_for_select($select_options, $value, $options);
  $script = '(function($){
    $(function(){
      var from = "#%s",
          to = "#%s";
      $(from).change(function() {
        updateSelector(from, to);
      });

      updateSelector(from, to);
      $(to).select2();
    });
  }(jQuery));';

  $fromName = get_id_from_name('owner_branch_Country');
  $toName = _convert_method_to_name($method, $options);
  $toId = get_id_from_name($toName);

  $options['style'] = 'width:280px';
  $options['class'] = 'select2';

  return
    select_tag($toName, $option_tags, $options) .
    javascript_tag(sprintf($script, $fromName, $toId))
  ;
}