<?php
/**
 * Function which remove all unallowed tags and all "on*" attributes
 * 
 * @param string $code Initial code
 * @param array $allowed_tags set of allowed tags
 * @return string cleaned code  
 */
function xss_cleanup($code, $allowed_tags = array('i', 'b', 'u', 'strong'))
{
    // Pattern which will find all tags in code
    $tags_pattern = '/(<[^>]*>)/';
    // Patern which will find all on* attributes inside the tag
    $attr_pattern = '/\s[oO][nN][^=]*=[^\s>]*[\s>]/';
    
    $result_code = $code;
    
    // Add spaces at the end of each tag - serch engine will be more efective
    $result_code = str_replace('/>', ' />', $result_code);
    $result_code = str_replace('>', ' >', $result_code);
    
    // Find all tags
    $results = array();
    preg_match_all($tags_pattern, $result_code, $results, PREG_SET_ORDER);
    
    // Rearrange result array
    $orig_tags = array();
    foreach($results as $result){
        $orig_tags[] = $result[0];
    }
    
    // Main part
    $result_tags = array();
    foreach($orig_tags as $idx => $tag){
        // Remove unallowed attributes from the tag
        $attrs = array();
        preg_match_all($attr_pattern, $tag, $attrs, PREG_SET_ORDER);
        foreach($attrs as $attr){
            $tag = str_replace($attr[0], '', $tag);
        }
        
        // Split the tag and change tag name to lower cases
        $splitted = explode(' ', $tag);
        $splitted[0] = strtolower($splitted[0]);
        
        // Get tag name and check if it is allowed
        $tag_name = preg_replace('/[^a-z]/', '', $splitted[0]);
        $is_allowed = array_search($tag_name, $allowed_tags);
        
        // If tag is not allowed, remove it
        if($is_allowed === null || $is_allowed === false){
            $result_tags[$idx] = '';
        } else {
            $result_tags[$idx] = implode(' ', $splitted);
        }
    } // foreach tag
    
    // Replace original tags with processed one
    foreach($orig_tags as $idx => $value){
        $result_code = str_replace($value, $result_tags[$idx], $result_code);
    }
    
    // Remove initially added spaces at the end of tag names
    $result_code = str_replace(' >', '>', $result_code);
    $result_code = str_replace(' />', '/>', $result_code);
    
    if($result_code != $code){
        $result_code = xss_cleanup($result_code, $allowed_tags); // small recurency
    }
    
    // OK, we can finish now ;)
    return $result_code;
}
