<?php

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class AdListing extends BaseAdListing
{
    const LISTED = 0;
    const PAUSED = 1;
    const DELETED = 2;

    public function isStatus($status)
    {
        return $this->get('status') == $status;
    }

    public function getStatusName()
    {
        switch ($this->get('status')) {
            case self::LISTED:
                return 'Listed';

            case self::PAUSED:
                return 'Paused';

            case self::DELETED:
                return 'Deleted';

            default:
                throw new Exception("Unknow status", 1);
        }
    }

    public function getCountry()
    {
        return $this->get('Routes')->getFirst()->get('PickupLocation')->get('Category')->get('Country');
    }

    public function getImagePath()
    {
        if ($this->get('image'))
        {
            return $this->get('image');
        }
        return $this->get('VehicleType')->icon_path();
    }

    public function getUrlParameter($name, $default)
    {
        $values = $this->get('url_parameters_defaults');
        return isset($values[$name]) && $values[$name] != '' ? $values[$name] : $default;
    }

    private function getStatQueryForPeriodType($table, $type)
    {
        switch ($type) {
            case 'daily':
                $dateFormat = '%Y-%m-%d';
                break;

            case 'weekly':
                $dateFormat = '%Y-%u';
                break;

            case 'monthly':
                $dateFormat = '%Y-%m';
                break;

            default:
                throw new RuntimeException("invalid type $type");
        }

        return sprintf('SELECT DATE_FORMAT(created_at, \'%s\') as "date", count(1) as nb FROM %s '.
                  'WHERE advertiser_id = ? '.
                  'AND ad_listing_id = ? '.
                  'AND created_at between ? and ?
                  group by "date"',
                  $dateFormat, $table
            );
    }

    private function getStatsForPeriod($type, $from, $to, $periodType)
    {
        $from = new DateTime($from);
        $to   = new Datetime($to.' 23:59:59');
        $query =  $this->getStatQueryForPeriodType($type, $periodType);

        $connection = $this->getTable()->getConnection();
        $stmt = $connection->prepare($query);
        $stmt->execute(array(
            $this->get('advertiser_id'),
            $this->getId(),
            $from->format('Y-m-d'),
            $to->format('Y-m-d H:i:s')
        ));

        $data = [];
        while ($row = $stmt->fetch()) {
            $data[$row['date']] = $row['nb'];
        }

        $dates = $this->createDateRangeArray($from->format('Y-m-d'), $to->format('Y-m-d'), $periodType);

        $stats = [];
        foreach ($dates as $date) {
            $stats[$date] = isset($data[$date]) ? (int) $data[$date] : 0;
        }

        return $stats;
    }

    public function getClicksForPeriod($from, $to, $type)
    {
        return $this->getStatsForPeriod('ad_listing_click', $from, $to, $type);
    }

    public function getViewsForPeriod($from, $to, $type)
    {
        return $this->getStatsForPeriod('ad_listing_view', $from, $to, $type);
    }

    public function getSeasonalOffset($date)
    {
        $offsets = $this->get('Advertiser')->get('seasonal_offsets');

        $date = new DateTime($date);

        foreach ($offsets as $offset) {
            $from = new DateTime($this->formatOffsetDate($offset['from_date']));
            $to = new DateTime($this->formatOffsetDate($offset['to_date']));

            if ($from < $date && $to > $date) {
                return $offset['offset'];
            }
        }

        return false;
    }

    private function formatOffsetDate($date)
    {
        list($day, $month, $year) = explode('/', $date);

        return $year.'-'.$month.'-'.$day;
    }

    private function createDateRangeArray($from, $to, $periodType)
    {
        switch ($periodType) {
            case 'weekly':
                $interval = 'P1W';
                $format = 'Y-W';
                break;

            case 'monthly':
                $interval = 'P1M';
                $format = 'Y-m';
                break;

            default:
            case 'daily':
                $interval = 'P1D';
                $format = 'Y-m-d';
                break;
        }

        $to = new DateTime($to);
        $period = new DatePeriod(
             new DateTime($from),
             new DateInterval($interval),
             $to
        );

        $dates = [];
        foreach($period as $date){
            $dates[] = $date->format($format);
        }

        $dates[] = $to->format($format);

        return $dates;
    }

    public function areReservationClosed()
    {
        return false;
    }
}