<?php
/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class AdListingTable extends Doctrine_Table
{
    public static function getStatuses()
    {
        return array(
            AdListing::LISTED => 'Listed',
            AdListing::PAUSED => 'Paused',
            AdListing::DELETED => 'Deleted',
        );
    }

    public function findForFilters($country, $filters)
    {
      return $this->findForFiltersQuery($country, $filters)->execute();
    }

    public function findForFiltersQuery($country, $filters)
    {

      $filters = array_merge(array(
          'area' => false,
          'from_date' => false,
          'pickup' => false,
          'dropoff' => false,
          'vehicle_type' => false,
          'transmission' => false,
          'travellers' => false,
        ), $filters);
        $query = $this
            ->createQuery('a')
            ->innerJoin('a.Routes r')
            ->innerJoin('r.PickupLocation lp')
            ->innerJoin('r.DropoffLocation ld')
            ->innerJoin('a.VehicleType v')
            ->innerJoin('a.Advertiser ad')
            ->addWhere('a.country_id = ?', $country)
            ->addWhere('a.status = ?', AdListing::LISTED)
            ->orderBy('a.price')
            ->groupBy('a.id')
        ;

        if (isset($filters['vehicle_type']) && $filters['vehicle_type'])
        {
            $type = current($filters['vehicle_type']);

            if (in_array($type, array('Motorhome', 'Campervan'))) {
                $query->addWhere('(v.description like ? or v.description like ?)', array(
                    'Campervan%',
                    'Motorhome%'
                ));
            }
            else {
               $query->addWhere('v.description like ?', current($filters['vehicle_type']).'%');
            }

        }

        if (isset($filters['transmission_type']))
        {
            $query->whereIn('a.transmission_type', $filters['transmission_type']);
        }

        if ($filters['pickup'])
        {
          $from = str_replace(array('#'), ' ', trim($filters['pickup']));
          $ancestor = Doctrine::getTable('LocationNode')->findOneBySlug($from);

          if ($ancestor)
          {
            if (false && $ancestor->get('type') == 'location')
            {
                $query->addWhere('lp.id = ?', $ancestor->getId());
            }
            else
            {
                $query
                    ->innerJoin('lp.Descendants lpa')
                    ->addWhere('(lpa.node_id = ? OR lpa.node_ancestor_id= ?)', array($ancestor->getId(), $ancestor->getId()));
            }
          }
        }

        if ($filters['dropoff'])
        {
          $to = str_replace(array('#'), ' ', trim($filters['dropoff']));
          $ancestor = Doctrine::getTable('LocationNode')->findOneBySlug($filters['dropoff']);

          if ($ancestor)
          {
		  #error_log('ancestor'.$ancestor->getId());
            if (false &&$ancestor->get('type') == 'location')
            {
                $query->addWhere('ld.id = ?', $ancestor->getId());
            }
            else
            {
                $query
                    ->innerJoin('ld.Descendants lda')
                    ->addWhere('(lda.node_id = ? OR lda.node_ancestor_id= ?)', array($ancestor->getId(), $ancestor->getId()));
            }
          }
        }
// error_log(serialize($query->getSqlQuery()));
// error_log(serialize($query->getParams()));
        return $query;
    }

    public function findListed($id)
    {
        $query = $this
            ->createQuery('a')
            ->addWhere('a.status = ?', AdListing::LISTED)
            ->addWhere('a.id = ?', $id)
        ;

        return $query->fetchOne();
    }

    public function incrementViewCounters($ads)
    {
        $ids = array();

        foreach ($ads as $ad)
        {
            $ids[] = $ad->getId();

            // $view = new AdListingView();
            // $view->set('advertiser_id', $ad->get('advertiser_id'));
            // $view->set('ad_listing_id', $ad->get('id'));
            // $view->save();
        }

        if (count($ids)) {
            Doctrine_Query::create()
                ->update('AdListing')
                ->set('nb_views', 'nb_views + 1')
                ->whereIn('id', $ids)
                ->execute();
        }
    }

    public function incrementClickCounter($ad)
    {

        Doctrine_Query::create()
            ->update('AdListing')
            ->set('nb_clicks', 'nb_clicks + 1')
            ->where('id = ?', $ad->getId())
            ->execute();

        $click = new AdListingClick();
        $click->set('advertiser_id', $ad->get('advertiser_id'));
        $click->set('ad_listing_id', $ad->get('id'));
        $click->save();
    }
}
