<?php

use Transfercar\Email\Sender\RequestConfirmationEmail;
use Transfercar\Mandrill\TcMandrill;

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class Listing extends BaseListing
{
    /*
     * Possible states of listing record
     */
    const STATUS_NEW         = 0;  // new, waiting for requests
    const STATUS_ACCEPTED    = 1;  // accepted, waiting for close
    const STATUS_CLOSED      = 2;  // closed STATUS_ARCHIVED
    const STATUS_EXPIRED     = 3;  // expired (passed date_return_by)
    const STATUS_DELETED     = 4;  // deleted by owner
    const STATUS_STANDBY     = 5;  // standy listing
    const STATUS_CANCELED    = 6;  // status cancelled by operator
    const STATUS_PICKED_UP   = 7;  // accepted listing has reached pickup date
    const STATUS_DROPPED_OFF = 8;  // accepted listing has reached dropoff date
    const STATUS_PAUSED      = 9;  // listing paused
    const STATUS_BOOKED      = 10; // listing booked, merged with "accepted"
    const STATUS_COMPLETED   = 11; // relocation completed

    /*
     * Possible listing types
     */
    const TYPE_DEFAULT = 0;
    const TYPE_AIRPORT_RELOC = 1;

    public function expires()
    {
      $this->status = self::STATUS_EXPIRED;
      $this->save();

      // reject the pending requests if any, and notify the driver
      foreach ($this->get('Requests') as $tcRequest)
      {
        $tcRequest->expires(true);
      }
    }

    public function getManager()
    {
      if (parent::rawGet('listed_by'))
      {
        return parent::get('ListedBy');
      }

      return $this->get('Owner');
    }

    public function isAirportReloc()
    {
      return $this->get('listing_type') === self::TYPE_AIRPORT_RELOC;
    }

    public function isStatus($status)
    {
      if (!is_array($status)) {
        $status = array($status);
      }

      return in_array($this->status, $status);
    }

    public function isBooked()
    {
      // happens when using left join with Doctrine...
      if ($this->get('accepted_request_id') instanceof Request)
      {
        return $this->get('accepted_request_id')->exists();
      }

      return $this->get('accepted_request_id') ? true : false;
    }

    public function isBookedBy($user_id)
    {
      return $this->isBooked() && $this->getAcceptedRequest()->get('driver_id') == $user_id;
    }

    public function getCompoundId()
    {
      $ids = array($this->getId());
      $l = $this->getChildrenListings()->getLast();

      if ($l)
      {
        $ids[] = $l->getId();
      }

      return implode('-', $ids);
    }

    public function getTitle($pattern = '%s to %s')
    {
      $from = $this->get('PickupLocation')->get('name');
      $to = $this->get('DropoffLocation')->get('name');

      return sprintf($pattern, $from, $to);
    }

   /**
    * Returns allowed kms and days
    */
    public function getAllowance()
    {
      $result = '';
      $days = $this->days_allowed;
      $kms  = $this->kms_allowed;
      if ($days > 0) {
          $result .= $days . ' free day' . ($days > 1 ? 's' : '');
      }
      if ($kms > 0) {
        if ($days > 0) {
          $result .= ' and ';
        }
        $result .= $kms . ' free km';
      }

      return $result;
    }

    public function getEarliestPickupDate()
    {
      $ts_start = max(strtotime('today') ,strtotime($this->get('date_available_from')));

      return date('Y-m-d', $ts_start);
    }

    public function getDaysAllowedAvailable($days = null)
    {
      $days    = null === $days ? $this->days_allowed : $days;
      $today = strtotime('today');
      $maxTime = strtotime('+'.$days.' days', $today);

      if ($maxTime > strtotime($this->date_return_by))
      {
        $days = (strtotime($this->date_return_by) - $today) / 86400 + 1;

        return max(1, round($days, 0));
      }

      return $days;
    }

    public function getPaidDaysAllowedAvailable()
    {
      if (!$this->paid_days_allowed) {
        return false;
      }

      $daysAvailable = $this->getDaysAllowedAvailable($this->days_allowed + $this->paid_days_allowed);

      return max(0, $daysAvailable - $this->days_allowed);
    }


    /**
     * Returns relocation query used in seo links
     *
     * @param string $action Action URL for the link
     * @param boolean $full Should we generate full URL?
     * @return string Requested URL
     */
    public function getRelocationQuery($action = 'main/index', $full = true, $params = '')
    {
      $pattern = '/[^a-zA-Z0-9]/';
      $from = preg_replace($pattern, '_', $this->PickupLocation->name);
      $to = preg_replace($pattern, '_', $this->DropoffLocation->name);
      $params .= 'from='.$from.'&to='.$to.'&relo_id='.$this->id;

      sfLoader::loadHelpers(array('Url'));
      $result = url_for($action.'?'.$params, true);

      return $result;
    }

   /**
    * Check this listing has optional extras
    *
    * @return True if this listing has optional extras
    */
    public function hasExtras()
    {
      return $this->Addons && $this->Addons->count() > 0;
    }

   /**
    * Check this listing has terms and conditions
    *
    * @return True if this listing has terms and conditions
    */
    public function hasTerms()
    {
      return $this->Terms && $this->Terms->exists();
    }

   /**
    * Checks if current listing has any extras
    *
    * @return boolean True if current listing has any extras
    */
    public function hasIncluded()
    {
      return $this->ferry_included ||
             $this->petrol_included ||
             $this->expenses_included ||
             $this->insurance_included;
    }

    /**
     * Calculate how many extras has this listing
     *
     * @return Extras count
     */
    public function included_count()
    {
        return (int)($this->petrol_included)
             + (int)($this->ferry_included)
             + (int)($this->expenses_included)
             + (int)($this->insurance_included)
             + (int)($this->tolls_included);
    }

    /**
     * Return string which represents current listing state
     *
     * @return string Current state value
     */
    public function getStatusName()
    {
      if ($this->get('accepted_request_id') && $this->getAcceptedRequest()->get('no_show_fee'))
      {
        return 'NoShow';
      }

      switch($this->status) {
        case self::STATUS_NEW:
          if ($this->getRequestCount(array(
              Request::STATUS_PENDING))) {
            return 'Requested';
          }

          return 'Listed';
        case self::STATUS_ACCEPTED:
        case self::STATUS_BOOKED:
          return 'Booked';
        case self::STATUS_COMPLETED:
          return 'Completed';
        case self::STATUS_CLOSED:
          return 'Closed';
        case self::STATUS_EXPIRED:
          return 'Expired';
        case self::STATUS_DELETED:
          return 'Deleted';
        case self::STATUS_CANCELED:

          if ($this->getAcceptedRequest()->exists())
          {
            return $this->getAcceptedRequest()->cancelledByDriver() ? 'DriverCancelled' : 'OwnerCancelled';
          }

          return 'Cancelled';
        case self::STATUS_PICKED_UP:
          return 'PickedUp';
        case self::STATUS_DROPPED_OFF:
          return 'DroppedOff';
        case self::STATUS_PAUSED:
          return 'Paused';
        case self::STATUS_COMPLETED:
          return $this->getAcceptedRequest()->get('no_show_fee') ? 'NoShow' : 'Closed';
        default:
            return 'Unknown';
      }
    }

   /**
    * Returns request count with given status
    *
    * @param array $atatus Statuses that will be counted
    * @returns int Request count
    */
    public function getRequestCount($status = array())
    {
      $count = 0;
      foreach($this->Requests as $request) {
        $isOk = false;
        foreach($status as $s) {
          if ($request->status == $s) {
            $isOk = true;
            break;
          }
        }
        if ($isOk) {
          $count++;
        }
      }
      return $count;
    }

    /**
     * Can this listing be requested by current user?
     * Listings cannot be requested by its owners and
     * when are accepted or closed
     */
    public function can_be_requested()
    {
        // Status of this listing
        if($this->status != self::STATUS_NEW){
            return false;
        }

        // Is user logged in?
        $user = sfContext::getInstance()->getUser();
        if(!$user->isAuthenticated()){
            return true;
        }

        // Am I an owner?
        if ($user->is_owner && $this->owner_id == $user->id) {
            return false;
        }

        // Have I already requested for this listing
        foreach($this->Requests as $request){
            if($request->driver_id == $user->id &&
                in_array($request->status, array(Request::STATUS_PENDING, Request::STATUS_REJECTED, Request::STATUS_CANCELED))) {


                return false;
            }
        }

        // Now we know that request is allowed
        return true;
    }

    /**
     * Gets conflicting requests
     *
     * @param timestamp|string $pickup Pickup date
     * @param Doctrine_Collection|null $requests User accepted requests, used for optimisation if multiple pickup dates have to be validated
     * @returns true|array True if no errors found or array of conflicting Doctrine_Record's
     */
    public function getConflictingRequests($pickup, $requests = null)
    {
      $user = sfContext::getInstance()->getUser();
      if (!$user->isAuthenticated()) {
        return false;
      }

      if(!is_int($pickup)) {
        $pickup = strtotime($pickup);
      }

      if (!($requests instanceof Doctrine_Collection)) {
        $requests = Doctrine_Query::create()
          ->from("Request r")
          ->leftJoin("r.Listing l")
          ->leftJoin("l.PickupLocation")
          ->leftJoin("l.DropoffLocation")
          ->where("r.driver_id = ?", $user->id)
          ->addWhere("r.status = ?", Request::STATUS_ACCEPTED)
          ->addWhere('r.pickup_date <= ?', date('Y-m-d', $pickup))
          ->addWhere('r.drop_off_date >= ?', date('Y-m-d', $pickup))
          ->execute();
      }

      return count($requests) ? $requests : false;
    }

    /// Get an array for owner drop-down list
    /// @return array of owner usernames asc
    public static function ownerList()
    {
      $columns = Doctrine::getTable('User')->getColumnNames();
        $owners = Doctrine_Query::create()
            ->from('User u')
            ->orderBy('u.username ASC')
            ->execute(array(1), Doctrine::HYDRATE_NONE);

        foreach($owners as $owner){
            $result[$owner[array_search('id', $columns)]] = $owner[array_search('username', $columns)];
        }

        return $result;
    }

    public function getNewRequestContact()
    {
      $option = $this->get('Owner')->get('request_email_option');

      switch ($option) {
        default:
        case 'listing_manager':
          $contact = $this->get('ListedBy');
          break;

        case 'branch_contact':
          $contact = $this->get('OwnerPickupBranch')->get('Contact');

          if (!$contact || !$contact->exists())
          {
            $contact = $this->get('Owner')->getDefaultContact();
          }
          break;

        case 'manager':
        case 'contact':
          list($type, $id) = explode(':', $this->get('Owner')->get('request_email_option_value'));

          $table = 'manager' == $type ? 'OwnerUser' : 'OwnerContact';

          $contact = Doctrine::getTable($table)->find($id);
          break;
      }

      return $contact;
    }

    /**
     * Notify owner about new request for this listing
     *
     * @param integer $rid Request id
     */
    public function notifyAboutNewRequest($request)
    {
      sfLoader::loadHelpers('Partial', 'Url', 'Tag', 'Transfercar');

      $isBanned = $request->get('Driver')->isBannedFor($this->get('owner_id')) || $request->get('Driver')->get('user_status') == 'blocked';
      if (!$isBanned && $request->get('Listing')->rawGet('listed_by')) {
        RequestConfirmationEmail::notifyRo($request);
      }

      RequestConfirmationEmail::notifyDriver($request);
    }

    public function sendOldEmail($request)
    {
        sfLoader::loadHelpers(array('Partial', 'Url', 'Tag', 'Transfercar', 'Date'));
        // Two links

        $domain = $this->getCountryDomain();
        $site = sfConfig::get('app_domain_' . $domain);

        $controller = sfContext::getInstance()->getController();
        $accept_link = 'http://' .$site . '/request/emailaccept/' . $request->id.'/uid/contact/key/contact';
        $email_opt_link = 'http://' . $site . '/user/emailOptions';

        /**********************
        * Sibers version
        **********************/
        // Prepare data for e-mail template

        $template_data = array(
          'firstname' => $this->Owner->firstname,
          'lastname' => $this->Owner->lastname,
          'username' => $this->Owner->username,
          'when_registered' => $this->Owner->when_registered,
          'from' => $listing->PickupLocation->name,
          'to' => $listing->DropoffLocation->name,
          'from_date' => format_date($request->get('pickup_date'), 'dd/MM/yyyy'),
          'to_date' => format_date($request->get('drop_off_date'), 'dd/MM/yyyy'),
          'request_id' => $request->id,
          'driver' => $request->Driver->username,
          'driver_rate' => $driver_rate,
          'reloc_details' => $this->detailedInfo(true, false, $request),
          'pickup_date' => $request->pickup_date,
          'accept_link' => $accept_link,
          'email_opt_link_real' => $email_opt_link);

        $template_data = array_merge($template_data, $this->detailedInfoArray());

        if($this->Owner->account_balance <= 0 && $this->auto_accept) {
          // If owner has unsufficient funds, set no_funds template
          $template_data['balance']      = $this->Owner->account_balance;
          $template_data['balance_diff'] = ($listing->price - $this->Owner->account_balance);
          $template_name = 'new-request-no-funds';
        } else {
          $template_name = 'new-request';
        }



        $mandrill = new TcMandrill();
        $mandrill->sendTemplate(
          'Driver requested for trip',
          $this->Owner->getEmailToParams(),
          $template_data,
          $template_name
        );
    }

    /**
     * Get detailed information about this listing (used in e-mails)
     * @return string Information about this listing
     */
    public function detailedInfo($show_owner_data = true, $from_request_accept = false, $for_accepted_driver = false)
    {
        $result = array();

        $result[] = 'Listing ID: '.$this->id;
        // Owner
        if($show_owner_data){
            $result[] = 'owner: '.$this->Owner->username;
            if ($from_request_accept){
                $result[] = 'pickup address: '.$this->pickup_address;
                $result[] = 'contact person: '.$this->contact_person;
                $result[] = 'contact phone: '.$this->contact_phone;
            }
        }

        $result[] = 'Pickup: '.$this->getPickupLocation()->get('name');
        $result[] = 'Dropoff: '.$this->getDropoffLocation()->get('name');

        if ($for_accepted_driver) {
          $request = $for_accepted_driver instanceof Request ? $for_accepted_driver : $this->getAcceptedRequest();
          $result[] = 'Pickup Date: '.date('d/m/y', strtotime($request->pickup_date));
          $result[] = 'return by: '.date('d/m/y', strtotime($request->drop_off_date));
        }else {
          // Dates
          $result[] = 'available from: '.date('d/m/y', strtotime($this->date_available_from));
          $result[] = 'return by: '.date('d/m/y', strtotime($this->date_return_by));
        }

        // Allowed
        $txt = 'allowed:';
        if($this->kms_allowed){
            $txt .= ' '.$this->kms_allowed.'km';
        }
        if($this->kms_allowed && $this->days_allowed){
            $txt .= ' and';
        }
        if($this->days_allowed){
            $txt .= ' '.$this->days_allowed.' day(s)';
        }
        $result[] = $txt;

        // Vehicle type
        $result[] = 'vehicle: '.$this->VehicleType->description;

        // Included
        if($this->ferry_included || $this->petrol_included ||
            $this->expenses_included || $this->insurance_included){
            $txt = array();

            if($this->ferry_included){
                $txt[] = 'ferry';
            }

            if($this->petrol_included){
                $txt[] = 'petrol';
            }

            if($this->expenses_included){
                $txt[] = 'expenses';
            }

            if($this->insurance_included){
                $txt[] = 'insurance';
            }

            $result[] = 'included: '.implode(', ', $txt);
        }

        if($this->comment){
            $result[] = 'comment: '.$this->comment;
        }

        return implode('; ', $result);
    }

	public function detailedInfoArray()
	{
		$result = array(
			'reloc_owner'          => $this->Owner->username,
			'reloc_available_from' => date('d/m/y', strtotime($this->date_available_from)),
			'reloc_return_by'      => date('d/m/y', strtotime($this->date_return_by)),
			'reloc_allowed'        => '',
			'reloc_included'       => ''
		);

		if($this->kms_allowed){
            $result['reloc_allowed'] .= $this->kms_allowed.'km';
        }
        if($this->kms_allowed && $this->days_allowed){
            $result['reloc_allowed'] .= ' and';
        }
        if($this->days_allowed){
            $result['reloc_allowed'] .= ' '.$this->days_allowed.' day(s)';
        }

        $result['reloc_vehicle_type'] = $this->VehicleType->description;

        if($this->ferry_included || $this->petrol_included ||
            $this->expenses_included || $this->insurance_included){
            $txt = array();

            if($this->ferry_included){
                $txt[] = 'ferry';
            }

            if($this->petrol_included){
                $txt[] = 'petrol';
            }

            if($this->expenses_included){
                $txt[] = 'expenses';
            }

            if($this->insurance_included){
                $txt[] = 'insurance';
            }

            $result['reloc_included'] = 'included: '.implode(', ', $txt);
        }

        $result['reloc_comment'] = $this->comment;

        $result['reloc_pickup_address'] = $this->pickup_address;
        $result['reloc_contact_person'] = $this->contact_person;
        $result['reloc_contact_phone'] = $this->contact_phone;

        return $result;
	}

    /**
     * Calculate number of pending requests for this listing
     * @return integer Number of pending requests
     */
    public function pending_requests_count()
    {
        $count = 0;
        foreach($this->Requests as $req){
            if($req->status == Request::STATUS_PENDING){
                $count++;
            }
        }

        return $count;
    }

    public function getRequestsByDate()
    {
      return Doctrine::getTable('Request')
        ->createQuery('r')
        ->where('r.listing_id = ?', $this->getId())
        ->orderBy('r.drop_off_date asc')
        ->execute();
    }

    public function hasPendingRequests()
    {
      return $this->pending_requests_count() > 0;
    }

   /**
    * Get accepted driver username. Used by backend application.
    * @return string driver username
	*/
  public function getDriverUsername()
  {
    if ($req = $this->getAcceptedRequest()) {
      return $req->Driver->getShortName();
    }
  }

  public function getDriverEmail()
  {
    if ($req = $this->getAcceptedRequest()) {
      return $req->Driver->get('email');
    }
  }

  public function getDriverFullname()
  {
    if ($req = $this->getAcceptedRequest()) {
      return $req->Driver->full_name();
    }
  }

  public function getRequestedPickupDate()
  {
    if (($req = $this->getAcceptedRequest()) && $req->exists()) {
      return $req->get('pickup_date') ? $req->get('pickup_date') : $this->get('date_available_from');
    }

    return $this->get('date_available_from');
  }

  public function getRequestedDropOffDate()
  {
    if ($req = $this->getAcceptedRequest()) {
      return $req->get('drop_off_date') ? $req->get('drop_off_date') : $this->get('date_return_by');
    }

    return $this->get('date_return_by');
  }

  public function oldgetAcceptedRequest($includePending = true)
  {
    $statuses = array(Request::STATUS_ACCEPTED, Request::STATUS_RATED);

    // not sure why the status pending is used here, just keep it for BC
    if ($includePending)
    {
      $statuses[] = Request::STATUS_PENDING;
    }

    foreach($this->Requests as $req)
    {
      if (in_array($req->status, $statuses) &&
         $req->listing_id == $this->id)

      return $req;
    }
  }

  public function getCancelledRequest()
  {
    $statuses = array(Request::STATUS_DRIVER_CANCELED, Request::STATUS_OWNER_CANCELED, Request::STATUS_CANCELED);

    foreach($this->Requests as $req)
    {
      if (in_array($req->status, $statuses) &&
         $req->listing_id == $this->id)

      return $req;
    }
  }

  public function getRequestForUser($userId)
  {
    $request = false;
    foreach ($this->Requests as $req)
    {
      if ((int) $req->driver_id === (int) $userId) {
        $request = !$request || $request->getId() < $req->getId() ? $req : $request;
      }
    }

    return $request;
  }

  public function getRequestedFreeDays()
  {
    if (($req = $this->getAcceptedRequest()) && $req->exists()) {
      return $req->get('nb_free_days') ? $req->get('nb_free_days') : $this->get('days_allowed');
    }

    return $this->get('days_allowed');
  }

  public function getRequestedPaidDays()
  {
    if (($req = $this->getAcceptedRequest()) && $req->exists()) {
      return intval($req->get('nb_paid_days'));
    } else {
      return $this->get('paid_days_allowed');
    }
  }

  /**
   * Returns the number of days available between the pickup and drop off dates
   */
  public function getNumberOfDays()
  {
    $from = max(strtotime('today'), strtotime($this->get('date_available_from')));
    $to = strtotime($this->get('date_return_by'));

    return ceil(($to - $from) / 86400);
  }


  public function isGreatDeal()
  {
    $kms = $this->get('kms_estimated') ?: $this->getApproximateDistance();
    $days = $this->get('days_allowed');
    $distance = Settings::getDistanceByDay($this->getCountryDomain());
    $recommended_days = ceil($kms/$distance);

    if ($recommended_days > $days) {
      return false;
    }

    return $days >= ($recommended_days * 2) || $this->get('ferry_included') || $this->get('petrol_included');
  }

   /**
    * Get accepted driver username. Used by backend application.
	* @return string owner username
	*/
  public function getOwnerUsername()
  {
   return $this->Owner->username;
  }

  public function getDatesBeforeResponseTime()
  {
    $responseTime = $this->get('Owner')->get('average_response_time');

    $dates = array();

    $i = 0;
    while ($responseTime > 0) {
      $dates[] = date('Y-m-d', strtotime('+'.$i.' days'));
      $responseTime -= 1;
      $i++;
    }

    return $dates;
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string date confirmed
   */
  public function getDateConfirmed()
  {
    return $this->get('accepted_request_id') ? $this->getAcceptedRequest()->get('when_accepted') : null;
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string pick-up location
   */
  public function getPickLocation()
  {
   return $this->PickupLocation->name;
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string drop-off location
   */
  public function getDropLocation()
  {
   return $this->DropoffLocation->name;
  }

  public function getPickup()
  {
    if ($this->get('owner_pickup_branch_id')) {
      return $this->get('OwnerPickupBranch');
    }

    return $this->get('PickupLocation');
  }

  public function getDropoff()
  {
    if ($this->get('owner_dropoff_branch_id')) {
      return $this->get('OwnerDropoffBranch');
    }

    return $this->get('DropoffLocation');
  }

    /**
     * Get accepted driver username. Used by backend application.
     * @return string pick-up date
     */
  public function getPickupDate()
  {
   foreach($this->Requests as $req)
   {
    if ( ($req->status == Request::STATUS_ACCEPTED || $req->status == Request::STATUS_RATED ||
          $req->status == Request::STATUS_PENDING ) &&
	      $req->listing_id == $this->id) return $req->pickup_date;
   }
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string owner firstname
   */
  public function getOwnerFirstname()
  {
   return $this->Owner->firstname;
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string owner lastname
   */
  public function getOwnerLastname()
  {
   return $this->Owner->lastname;
  }

  /**
   * Get accepted driver username. Used by backend application.
   * @return string owner company
   */
  public function getOwnerCompany()
  {
   return $this->Owner->company;
  }

  public function getContact($type)
  {
    if ($this->get('owner_'.$type.'_branch_id'))
    {
      return $this->get('Owner'.ucfirst($type).'Branch')->get('Contact');
    }


    if ($contact = $this->get('Owner')->getDefaultContact())
    {
      return $contact;
    }

    return $this->get('Owner');
  }

  public function getContactName($type)
  {
    $contact = $this->getContact($type);

    return $contact instanceof User ? $contact->get('company') : $contact->get('name');
  }

  public function getContactPhoneNumber($type)
  {
    $contact = $this->getContact($type);

    return $contact instanceof User ? $contact->get('contact_phone') : $contact->get('phone');
  }

  public function getContactEmailAddress($type)
  {
    return $this->getContact($type)->get('email');
  }

  /**
   * Check weather listing has driver requirements
   *
   * @returns bool
   */
  public function hasDriverRequirements()
  {
    return $this->min_age || $this->license;
  }

  public function getLocationCountry()
  {
    return $this->PickupLocation->Category->Country;
  }

  public function getCountry()
  {
    return $this->PickupLocation->Category->Country->name;
  }

  public function getCountryDomain()
  {
    return $this->PickupLocation->Category->Country->domain;
  }

  /**
   * Return the total price of the current listing for the given request object
   *
   * @param Request $request
   * @return integer
   */
  public function getTotalPrice($request)
  {
    // recalculate the price
    $price = Doctrine::getTable('Prices')->getPriceFor(
      $this->get('owner_id'),
      $this->get('pickup_location_id'),
      $this->get('dropoff_location_id'),
      $this->get('vehicle_type_id')
    );

    return $price + $this->getPriceForPaidDays($request);
  }

  public function getIncludedDaysPrice()
  {
    return (int) $this->get('Owner')->get('relo_price_per_day');
  }

  public function getRevenue($request)
  {
    $price_per_allowed_day = $this->get('Owner')->get('relo_price_per_day');
    return
      $price_per_allowed_day * $request->get('nb_free_days') + // price for free days
      $this->price_per_day * $request->get('nb_paid_days'); // price for paid days
  }

  /**
   * Return the paid days price for the given request object
   *
   * @param Request $request
   * @return integer
   */
  public function getPriceForPaidDays($request)
  {
    // price for upselling days is freezed when the request is accepted.
    if (is_object($request) && $this->get('accepted_request_id') && $request->getId() == $this->get('accepted_request_id'))
    {
      return $this->get('upselling_price');
    }

    return (is_object($request) ? $this->getPriceForAdditionalDay() * $request->get('nb_paid_days') : 0);
  }

  public function getPriceForAdditionalDay()
  {
    if ($this->get('accepted_request_id') && $this->get('upselling_price'))
    {
      $nb_paid_days = $this->getAcceptedRequest()->get('nb_paid_days');

      return $this->get('upselling_price') / $nb_paid_days;
    }

    return Doctrine::getTable('Prices')->getPriceForAdditionalDays(
      $this->get('PickupLocation'),
      $this->get('DropoffLocation'),
      $this->get('vehicle_type_id'),
      $this->get('owner_id')
    );
  }

  public function remove($offset)
  {
    $this->status = Listing::STATUS_DELETED;
    $this->save();
    $this->rejectRequests();
  }

  public function pause()
  {
    $this->setStatus(Listing::STATUS_PAUSED);
    $this->save();
    $this->rejectRequests();
  }

  public function rejectRequests()
  {
    // reject the pending requests if any, and notify the driver
    foreach ($this->get('Requests') as $tcRequest)
    {
      if ($tcRequest->status == Request::STATUS_PENDING)
      {
        $tcRequest->reject();
      }
    }
  }
  public function getVehicle()
  {
    return $this->get('owner_vehicle_type_id') ? $this->get('OwnerVehicleType') : $this->get('VehicleType');
  }

  public function confirmBooking($internal_reference)
  {
    $this->internal_reference = $internal_reference;

    $is_late_confirm = $this->is_confirmed == -1;
    if ($this->isStatus(Listing::STATUS_NEW)) {
      $this->status = Listing::STATUS_ACCEPTED;
    }

    $this->is_confirmed = 1;
    $this->save();

    self::getEventDispatcher()->notify(new sfEvent($this, 'relocation.confirmed', ['template_name' => $is_late_confirm ? 'booking-number-confirmed': 'request-update']));
  }

  public function refund()
  {
    $payment = new Payments();
    $payment->type = 'manual';
    $payment->date = date('Y-m-d', time());
    $payment->amount = -$fee;
    $payment->user_id = $this->Owner->id;
    $payment->status = Payments::STATUS_ACCEPTED;
    $payment->assigned_record_id = $this->id;
    $payment->country_id = $this->PickupLocation->Category->Country->id;
    $payment->is_new_type_payment = "yes";
    $payment->save();
  }

  public function getCurrency()
  {
    return strtoupper($this->getCountryDomain()).'D';
  }

  public function getChecksum()
  {
    $data = $this->toArray(false);

    return sha1(serialize($data));
  }

  public function getApproximateDistance()
  {
      $pickup  = $this->get('PickupLocation');
      $dropoff = $this->get('DropoffLocation');

      $lat1 = floatval($dropoff->get('lat'));
      $lat2 = floatval($pickup->get('lat'));

      $lon1 = floatval($dropoff->get('lon'));
      $lon2 = floatval($pickup->get('lon'));

      return 1.2 * round(sqrt(pow(110.56*($lat2 - $lat1), 2) + pow(84.8*($lon2 - $lon1), 2)), 0);
  }

  public  function areReservationClosed()
  {
    if (!$this->get('Owner')->get('closed_in_week_end'))
    {
      return false;
    }

    $tz = $this->get('Owner')->get('timezone');
    $tzObject = new DateTimeZone($tz ? $tz : $this->get('Owner')->getDefaultTimezone());
    $current = new DateTime('now', $tzObject);
    $monday  = new DateTime('last monday', $tzObject);

    list($hours, $minutes, $seconds) = explode(':', $this->get('Owner')->get('week_end_closing_time'));
    $closing = clone $monday;
    $closing->add(DateInterval::createFromDateString('friday'));
    $closing->add(DateInterval::createFromDateString(sprintf('+%s hours %s minutes', $hours, $minutes)));

    list($hours, $minutes, $seconds) = explode(':', $this->get('Owner')->get('week_end_opening_time'));
    $opening = clone $monday;
    $opening->add(DateInterval::createFromDateString('sunday'));
    $opening->add(DateInterval::createFromDateString(sprintf('+%s hours %s minutes', $hours, $minutes)));

    return $current > $closing && $current < $opening;
  }

  public function getDaysFor($pickup_date_ts, $dropoff_date_ts)
  {
    if ($this->get('Vehicle')->get('rental_type') != 'day')
    {
      return $this->getDaysForNightRentals($pickup_date_ts, $dropoff_date_ts);
    }

    return $this->getDaysForDayRentals($pickup_date_ts, $dropoff_date_ts);
  }

  public function getDaysForDayRentals($pickup_date_ts, $dropoff_date_ts)
  {
    $free_days = $paid_days = 0;
    $night = $this->get('Vehicle')->get('rental_type') == 'day' ? 0 : 1;
    $i = 0;

    $date1 = new DateTime(date('Y-m-d', $pickup_date_ts));
    $date2 = new DateTime(date('Y-m-d', $dropoff_date_ts));
    $date2 = $date2->modify( '+1 day' );

    $daterange = new DatePeriod($date1, new DateInterval('P1D') ,$date2);

    foreach ($daterange as $date)
    {
        if ($i++ < ($this->days_allowed))
        {
            $free_days++;
        }
        else
        {
            $paid_days++;
        }
    }

    return array(max(1, $free_days), $paid_days);
  }

  public function getDaysForNightRentals($pickup_date_ts, $dropoff_date_ts)
  {
    $i = $free_days = $paid_days = 0;
    $night = 1;

    $date1 = new DateTime(date('Y-m-d H:i:s', $pickup_date_ts));
    $date2 = new DateTime(date('Y-m-d H:i:s', $dropoff_date_ts));

    $diff = ceil(floatval($date2->diff($date1)->format("%d.%h%i%s")));
    $diff = min($diff, $this->getDaysAllowedAvailable($this->get('days_allowed')+$this->get('paid_days_allowed')));

    while ($diff-- >= 0)
    {
        if ($i++ < ($this->days_allowed + $night))
        {
            $free_days++;
        }
        else
        {
            $paid_days++;
        }
    }

    return array(max(1, $free_days - $night), $paid_days);
  }

  public function hasBond()
  {
    return $this->get('Owner')->get('require_bond') && $this->getBondAmount();
  }


  public function getBondAmount($days = 0)
  {
    $bonds = $this->get('Owner')->get('bond_amounts');
    $domain = $this->getCountryDomain();

    $bond = isset($bonds[$domain]) ? $bonds[$domain] : 0;

    if ($this->get('accepted_request_id')) {
      $days = $this->getAcceptedRequest()->get('nb_paid_days');
    }

    $cost = $this->get('owner_id') == 17410 ? 15:0;
    $bond += ($cost*$days);

    return $bond;
  }

  public function getRankingBoost()
  {
    $domain = $this->getCountryDomain();
    $boosts = (array)$this->get('Owner')->get('boost_ranking');

    return array_key_exists($domain, $boosts) ? $boosts[$domain] : 0;
  }

  public function getMinimumPickupTimestamp()
  {
    $max_start = max(
        strtotime('today'), 
        strtotime($this->get('date_available_from'))
    ) + $this->get('latest_pickup_time') - 7200;

    $min_start = $max_start > time() ? 'today' : 'tomorrow';
    return max(strtotime($min_start), strtotime($this->get('date_available_from')));
  }

  public function postSave($event)
  {
      $listing = $event->getInvoker();

      self::getEventDispatcher()->notify(new sfEvent($listing, 'listing.updated'));
  }

  public function postUpdate($event)
  {
    $listing = $event->getInvoker();
    self::getEventDispatcher()->notify(new sfEvent($listing, 'ro.listing.updated'));
  }
}
