<?php
/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class ListingTable extends Doctrine_Table
{
    public static function getStatuses()
    {
      return array(
        ''                          => 'All but deleted',
        'all'                       => 'Any',
        Listing::STATUS_NEW         => 'Listed',
        'requested'                 => 'Requested',
        Listing::STATUS_ACCEPTED    => 'Booked',
        Listing::STATUS_PICKED_UP   => 'Picked Up',
        Listing::STATUS_DROPPED_OFF => 'Dropped Off',
        Listing::STATUS_CLOSED      => 'Completed',
        Listing::STATUS_EXPIRED     => 'Expired',
        Listing::STATUS_DELETED     => 'Deleted',
        Listing::STATUS_CANCELED    => 'Cancelled',
        Listing::STATUS_PAUSED      => 'Paused',
        'refunding'                 => 'Refund in progress',
      );
    }

    public static function getAdminStatuses()
    {
      return array(
        ''                          => 'All but deleted',
        'all'                       => 'Any',
        Listing::STATUS_NEW         => 'Listed',
        'requested'                 => 'Requested',
        'accepted'                  => 'Booked',
        Listing::STATUS_PICKED_UP   => 'Picked Up',
        Listing::STATUS_DROPPED_OFF => 'Dropped Off',
        Listing::STATUS_CLOSED      => 'Completed',
        Listing::STATUS_EXPIRED     => 'Expired',
        Listing::STATUS_DELETED     => 'Deleted',
        Listing::STATUS_CANCELED    => 'Cancelled',
        Listing::STATUS_PAUSED      => 'Paused',
        'refunding'                 => 'Refund in progress'
      );
    }

    public function findLatest($limit, $country)
    {
      return Doctrine_Query::create()
        ->from('Listing l, l.PickupLocation p, l.DropoffLocation d, l.VehicleType v')
        ->innerJoin('p.Category pC')
        ->where('l.status = ?', Listing::STATUS_NEW)
        ->addWhere("l.date_return_by >= ?", date('Y-m-d'))
        ->addWhere("pC.country_id = ?", $country)
        ->addSelect('l.*, p.*, d.*, v.*')
        ->addSelect('(
            COALESCE(l.expenses_included, 0) +
            COALESCE(l.insurance_included, 0) +
            COALESCE(l.ferry_included, 0) +
            COALESCE(l.petrol_included, 0)
          ) as extras_sort')
        ->addOrderBy('extras_sort DESC')
        ->addOrderBy('nb_listings DESC')
        ->addOrderBy('l.date_available_from ASC')
        ->limit($limit)
        ->execute();
    }

    public static function getManagers($loggedUser)
    {
      if ($loggedUser instanceof sfOutputEscaper)
      {
        $loggedUser = $loggedUser->getRawValue();
      }

      $users = Doctrine::getTable('OwnerUser')->findByOwnerId($loggedUser->getOwnerId());
      $options = array();

      $options[''] = 'Everyone';
      $options[$loggedUser->getId()] = 'Myself';


      foreach ($users as $user)
      {
        if ($loggedUser instanceof OwnerUser && $user->getId() === $loggedUser->getId())
        {
          continue;
        }

        $options[$user->getId()] = $user->getFullName();
      }

      return $options;
    }

    public function findNonCompletedRelos($countryId)
    {
      return Doctrine_Query::create()
          ->from('Listing l')
          ->innerJoin('l.Requests r')
          ->innerJoin('l.Owner o')
          ->whereIn('l.status', array(Listing::STATUS_ACCEPTED, Listing::STATUS_BOOKED, Listing::STATUS_PICKED_UP, Listing::STATUS_DROPPED_OFF))
          ->addWhere('r.status = ?', Request::STATUS_ACCEPTED)
          ->addWhere('(l.status != ? or o.payment_cc = ?)', array(Listing::STATUS_DROPPED_OFF, 1))
          ->innerJoin('l.VehicleType v')
          ->addWhere('v.country_id = ?', $countryId)
          ->execute();
    }

    public function findForAddRequest($id, $country_id)
    {
        // Get listing object
        $listing = Doctrine_Query::create()
          ->from('Listing l')
          ->innerJoin('l.PickupLocation p')
          ->innerJoin('p.Category')
          ->innerJoin('l.DropoffLocation d')
          ->innerJoin('d.Category c')
          ->innerJoin('l.VehicleType v')
          ->leftJoin('l.Requests')
          ->leftJoin('l.Terms')
          ->where('l.id = ?', $id)
          ->addWhere('c.country_id = ?', $country_id)
          ->fetchOne();

        if ($listing && !$listing->can_be_requested())
        {
          $l = $this->findSimilar($listing);
          $listing = $l ? $l : false;
        }

        return $listing;
    }

    public function findSimilar($listing)
    {
      $data = $listing->toArray(false);
      unset(
        $data['id'],
        $data['listing_id'],
        $data['status'],
        $data['created_at'],
        $data['updated_at'],
        $data['notify_instance'],
        $data['notify_daily'],
        $data['nb_listings'],
        $data['accepted_request_id'],
        $data['is_new_type_payment']
      );

      $query = Doctrine_Query::create()
          ->from('Listing l')
          ->where('l.status = ?', Listing::STATUS_NEW)
          ->limit(1);

      foreach ($data as $key => $value)
      {
        if (null === $value)
        {
          $query->addWhere('l.'.$key.' is null');
        }
        else
        {
          $query->addWhere('l.'.$key.' = ?', $value);
        }
      }
      $listing = $query->fetchOne();

      return $query->fetchOne();
    }

    public function findForFrontend($id, $country_id)
    {
        return Doctrine_Query::create()
          ->from('Listing l')
          ->innerJoin('l.PickupLocation lp')
          ->innerJoin('l.DropoffLocation ld')
          ->innerJoin('l.VehicleType')
          ->innerJoin('ld.Category lc')
          ->leftJoin('l.Addons')
          ->leftJoin('l.Requests')
          ->where('l.id = ?', $id)
          ->addWhere('lc.country_id = ?', $country_id)
          ->fetchOne();
    }

    /**
     * Get listing by given parameter, this is written originally forv landing pages
     * @param <type> $country
     * @param <type> $fromLocation
     * @param <type> $toLocation
     * @param <type> $vehicleType
     * @param <type> $owner
     * @param <type> $sortBy
     * @param <type> $limit
     * @return <type>
     */
    public function getListingsByParams(
            $country,
            $fromLocation,
            $toLocation,
            $vehicleType,
            $owner,
            $sortBy,
            $limit = 7,
            $showAllOfLess = null,
            $bothDirections = false
            ){

        $q = Doctrine_Query::create()//->select('l.id , p.name')
                ->from('Listing l')
                ->innerJoin('l.PickupLocation p')
                ->innerJoin('l.DropoffLocation d')
                ->innerJoin('l.VehicleType vt')
                ->innerJoin('p.Category pC')
                ->innerJoin('l.Owner u')
                ->where('l.status = ?', Listing::STATUS_NEW)
                ->addWhere("l.date_return_by >= ?", date('Y-m-d'))
                ->addWhere("pC.country_id = ?", $country);

        $fromIds = array();
        $fromNames = array();
        $toIds = array();
        $toNames = array();

        if(isset($fromLocation) && !empty ($fromLocation)){
            $ids = array_filter($fromLocation, 'is_numeric');
            $names = array_diff($fromLocation, $ids);

            $fromIds = array_merge($fromIds, $ids);
            $fromNames = array_merge($fromNames, $names);

            if ($bothDirections) {
                $toIds = array_merge($toIds, $ids);
                $toNames = array_merge($toNames, $names);
            }

        }

        if(isset ($toLocation) && !empty ($toLocation)){
            $ids = array_filter($toLocation, 'is_numeric');
            $names = array_diff($toLocation, $ids);

            $toIds = array_merge($toIds, $ids);
            $toNames = array_merge($toNames, $names);

            if ($bothDirections) {
                $fromIds = array_merge($fromIds, $ids);
                $fromNames = array_merge($fromNames, $names);
            }
        }

        // The current doctrine version does not support OR operations with the IN operator
        // So we have to build the query and params manually
        $pQuery = array('1=1');
        $params = array();
        if (!empty($fromNames)) {
            $pQuery = array();
            $pQuery[] = sprintf('p.name IN (%s)', implode(', ', array_fill(0, count($fromNames), '?')));
            $params = array_merge($params, $fromNames);
        }

        if (!empty($fromIds)) {
            $pQuery = array();
            $pQuery[] = sprintf('p.id IN (%s)', implode(', ', array_fill(0, count($fromIds), '?')));
            $params = array_merge($params, $fromIds);
        }

        $pQuery = '('.implode(' OR ', $pQuery).')';


        $dQuery = array('1=1');
        if (!empty($toNames)) {
            $dQuery = array();
            $dQuery[] = sprintf('d.name IN (%s)', implode(', ', array_fill(0, count($toNames), '?')));
            $params = array_merge($params, $toNames);
        }

        if (!empty($toIds)) {
            $dQuery = array();
            $dQuery[] = sprintf('d.id IN (%s)', implode(', ', array_fill(0, count($toIds), '?')));
            $params = array_merge($params, $toIds);
        }

        $dQuery = '('.implode(' OR ', $dQuery).')';

        $operator = $bothDirections ? ' OR ' : ' AND ';

        $q->addWhere(sprintf('(%s %s %s)', $pQuery, $operator, $dQuery), $params);

        if(isset ( $vehicleType ) && !empty ($vehicleType)){
            $q->whereIn('vt.description', $vehicleType  );
        }

        if(isset ( $owner ) && !empty ($owner)){
            $q->addWhere( 'u.id = ?' , $owner );
        }

        if (isset ($sortBy) && is_array($sortBy)){
            $sortCondition = '';

                if( isset($sortBy['auto'])  && $sortBy['auto']){
                    $q->orderBy('(l.expenses_included + l.insurance_included + l.ferry_included + l.petrol_included) DESC');
                    $q->addOrderBy('l.date_available_from ASC');
                }

                if( isset($sortBy['from_location']) && !empty ($sortBy['from_location']) ){
                    $q->addOrderBy('p.name '. $sortBy['from_location']);
                }

                if( isset($sortBy['to_location']) && !empty ($sortBy['to_location'])){
                    $q->addOrderBy('d.name '. $sortBy['to_location']);
                }

                if( isset($sortBy['vehicle_type']) && !empty ($sortBy['vehicle_type']) ){
                    $q->addOrderBy('vt.description '. $sortBy['vehicle_type']);
                }

                if( isset($sortBy['pickup_date']) && !empty ($sortBy['pickup_date']) ){
                    $q->addOrderBy('l.date_available_from '. $sortBy['pickup_date']);
                }

                if( isset($sortBy['droppoff_date']) && !empty ($sortBy['droppoff_date'])  ){
                    $q->addOrderBy('l.date_return_by '. $sortBy['droppoff_date']);
                }

                if( isset($sortBy['number_of_extras']) && !empty ($sortBy['number_of_extras']) ){
                  $q->addSelect('l.*, lp.*, ld.*, vt.*, ovt.*');
                  $q->addSelect('(
                    COALESCE(l.expenses_included, 0) +
                    COALESCE(l.insurance_included, 0) +
                    COALESCE(l.ferry_included, 0) +
                    COALESCE(l.petrol_included, 0)
                  ) as extras_sort');
                  $q->addOrderBy('extras_sort DESC')
                        ->addOrderBy('nb_listings DESC')
                        ->addOrderBy('l.date_available_from ASC');

                    // $q->addOrderBy('(l.expenses_included + l.insurance_included + l.ferry_included + l.petrol_included) '. $sortBy['number_of_extras']);
                }
        }


        if(isset ( $limit ) && !empty ($limit)){
            $q->limit($limit);
        }

        $tmp = $q->execute(array(), Doctrine::HYDRATE_RECORD);

        if (!is_null($showAllOfLess) && is_numeric($showAllOfLess)){
          if ((count($tmp)) <= $showAllOfLess)
          {
            $q->where('l.status = ?', Listing::STATUS_NEW)
              ->addWhere("l.date_return_by >= ?", date('Y-m-d'))
              ->addWhere("pC.country_id = ?", $country);
          }
        }

        //        exit(1);
        return $q->execute(array(), Doctrine::HYDRATE_RECORD);
    }

    public function countPendingRequestFor($listing)
    {
      return Doctrine::getTable('Request')
        ->createQuery('r')
        ->where('r.listing_id = ?', $listing->getId())
        ->addWhere('r.status = ?', Request::STATUS_PENDING)
        ->count();
    }

}