<?php

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class Locations extends BaseLocations
{
    /**
     * Get location name extended to category name
     *
     * @return string Name of the location with category
     */
    public function getNameWithCategory()
    {
        return $this->get('name').', '.$this->Category->get('description');
    }

    public function getNameWithCountry()
    {
        if (in_array($this->get('Category')->get('description'), ['Quebec', 'British Columbia', 'Alberta', 'Ontario/Quebec'])) {
            return str_replace('Quebec', 'Quebec City', $this->name).', Canada';
        }

        return $this->name.', '.$this->get('Category')->get('Country')->get('name');
    }

    public function getInternalName()
    {
        return $this->name;
    }

    public function getOpeningHours()
    {
        return 'N/A';
    }

    public function getAddress()
    {
        return 'N/A';
    }

    public function getDescription()
    {
        return '';
    }

    /**
     * Get the of location list
     *
     * @param bollean $with_category Add category name
     * @param bollean $show_empty If we should show "Please choose..." value
     * @param bollean $show_other If we should show "Not in NZ" value
     * @return array List of locations
     */
    public static function locationList($id_country, $with_category = true, $show_empty = false, $show_other = false)
    {
		if($with_category){
			$query_rule = 'l.category_id IS NOT NULL';
		} else {
			$query_rule = 'l.category_id IS NULL';
		}
        $locations = Doctrine_Query::create()
            ->from('Locations l')
            ->leftJoin('l.Category c')
            ->where($query_rule)
            ->addWhere('l.departure_id IS NOT NULL')
            ->orderBy('c.description ASC, l.name ASC')
            ->where('c.country_id=?', $id_country)
            ->execute();

        $result = array();

        if($show_empty){
            $result[''] = 'Please choose...';
        }

        if($show_other){
            $result[-1] = 'Currently not in New Zealand';
        }

        foreach($locations as $location){
            if($with_category){
                $result[$location->id] = $location->Category->description.' - '.$location->name;
            } else {
                $result[$location->id] = $location->name;
            }
        }
		return $result;
    }

public static function locationListFull($with_category = true, $sort_by_name = false)
{
    $columnsLocations = Doctrine::getTable('Locations')->getColumnNames();
    $columnsCategory  = Doctrine::getTable('LocationCategory')->getColumnNames();
    $query = Doctrine_Query::create()
       ->from('Locations l')
       ->innerJoin('l.Category c')
       ->orderBy('c.country_id ASC');

    if ($sort_by_name) {
        $query->orderBy('l.name');
    }
    else {
       $query->orderBy('c.description ASC');
    }

    $locations = $query->execute(array(1), Doctrine::HYDRATE_NONE);

    $result = array();
    foreach($locations as $location){
        if($with_category){
            $result[$location[array_search('id', $columnsLocations)]] = $location[array_search('description', $columnsCategory) + count($columnsLocations)] . " - " . $location[array_search('name', $columnsLocations)];
        } else {
            $result[$location[array_search('id', $columnsLocations)]] = $location[array_search('name', $columnsLocations)];
        }
    }

    return $result;
}

	public static function getAllByDeparture($id_country = null)
	{
		if(!is_null($id_country)){
			$query = Doctrine_Query::create()
				->from("Locations l")
				->innerJoin('l.Category c')
				->orderBy("c.id ASC")
				->where('c.country_id=?', $id_country);
		}else{
			$query = Doctrine_Query::create()
				->from("Locations l")
				->innerJoin('l.Category c')
				->orderBy("c.id ASC");
		}

		return $query->execute()->toArray();
	}

    public function getCountryAndPk()
    {
        return $this->get('Category')->get('country_id').'_'.$this->getId();
    }

    public function getCountry()
    {
        return $this->get('Category')->get('Country');
    }

    public function getLatitude()
    {
        return $this->get('lat');
    }

    public function getLongitude()
    {
        return $this->get('lon');
    }

    public function getLatLong()
    {
        return $this->get('lat').','.$this->get('lon');
    }

    public function getDepartureNames()
    {
        return array_map(function($d){ return $d->get('name'); }, $this->get('Departures')->getData());
    }


    public function postInsert($event)
    {
        $node = Doctrine::getTable('LocationNode')->createNode('location', $event->getInvoker());

        $category = $event->getInvoker()->get('Category');
        $ancestor = Doctrine::getTable('LocationNode')->findByTypeAndId('category', $category->get('id'));
        Doctrine::getTable('LocationNodeAncestor')->createNode($node, $ancestor);

        $ancestor = Doctrine::getTable('LocationNode')->findByTypeAndId('country', $category->get('country_id'));
        Doctrine::getTable('LocationNodeAncestor')->createNode($node, $ancestor);
    }

    public function postUpdate($event)
    {
        Doctrine::getTable('LocationNode')->updateNode('location', $event->getInvoker());
    }

    public function preDelete($event)
    {
        $node = Doctrine::getTable('LocationNode')->findByTypeAndId('location', $this->get('id'));
        $node && $node->delete();
    }

    public function getLocationNode()
    {
        return Doctrine::getTable('LocationNode')->findByTypeAndId('location', $this->get('id'));
    }
}
