<?php

use Transfercar\Geoloc\GoogleMapsDistancesMatrix;

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class PricesTable extends Doctrine_Table
{
    public function getPriceFor($owner_id, $pickup_location_id, $dropoff_location_id, $vehicle_type_id)
    {
        $pickup  = Doctrine::getTable('Locations')->find($pickup_location_id);
        $dropoff = Doctrine::getTable('Locations')->find($dropoff_location_id);

        $price = $this->findPricePerDistanceFor($pickup, $dropoff, $vehicle_type_id, $owner_id);

        return $price ? $price->get('price') : 0;
    }

    public function getPriceForAdditionalDays(Locations $pickup, Locations $dropoff, $vehicle_type_id, $owner_id)
    {
        $price = $this->findPricePerDistanceFor($pickup, $dropoff, $vehicle_type_id, $owner_id);


        if ($price && $price->get('upselling_price')) {
            return $price->get('upselling_price');
        }

        return $price ? $price->get('PriceGroup')->get('upselling_price') : 0;
    }

    public function findPricePerDistanceFor(Locations $pickup, Locations $dropoff, $vehicle_type_id, $owner_id)
    {
        $api  = new GoogleMapsDistancesMatrix();


        $distance = $api->getDistances(
            $pickup->getNameWithCountry('name'),
            $dropoff->getNameWithCountry('name'),
            $pickup->getCountry()->get('domain') == 'us' ? 'imperial' : 'metric'
        );

        if (!$distance)
        {
            $distance = $this->getDistance($pickup, $dropoff);
        }

        $price = Doctrine::getTable('PricePerDistance')
            ->createQuery('p')
            ->innerJoin('p.PriceGroup g')
            ->innerJoin('g.VehiclePriceGroup vpg')
            ->innerJoin('vpg.VehicleType v')
            ->innerJoin('p.ROPricingGroup pg')
            ->where('v.id = ?', $vehicle_type_id)
            ->addWhere('pg.owner_id = ?', $owner_id)
            ->addWhere('p.min_distance <= ?', $distance)
            ->addWhere('p.max_distance > ?', $distance)
            ->orderBy('p.max_distance asc')
            ->limit(1)
            ->fetchOne();

        return $price;
    }

    private function getDistance($dropoff, $pickup)
    {
      $lat1 = floatval($dropoff->get('lat'));
      $lat2 = floatval($pickup->get('lat'));

      $lon1 = floatval($dropoff->get('lon'));
      $lon2 = floatval($pickup->get('lon'));

      return sqrt(pow(110.56*($lat2 - $lat1), 2) + pow(84.8*($lon2 - $lon1), 2));
    }

}
