<?php


use Transfercar\Exception\CreditCardException;
use Transfercar\Stripe\TcStripeForm;

/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class Request extends BaseRequest
{
    /*
     * Request possible states
     */
    const STATUS_PENDING      = 0; // Waiting for action from listing owner
    const STATUS_ACCEPTED     = 1; // Accepted by listing owner
    const STATUS_REJECTED     = 2; // Rejected by listing owner
    const STATUS_OTHER_CHOOSE = 3; // Listing owner chosed other request for the specific listing
    const STATUS_RATED        = 4; // Accepted & rated or no-show
    const STATUS_EXPIRED      = 5; // Request has expired
    const STATUS_CANCELED     = 6; // Canceled by system (i.e. overlapping request)
    const STATUS_DELETED      = 7; // Deleted by user
    const STATUS_DRIVER_CANCELED = 8; // Ro: Cancelled by driver
    const STATUS_OWNER_CANCELED  = 9; // Cancelled by owner
    const STATUS_CANCELED_BY_DRIVER  = 10; // Cancelled by driver

    // Internal request statuses
    const INTERNAL_STATUS_CHARGED           = 1;
    const INTERNAL_STATUS_NOT_CHARGED       = 2;
    const INTERNAL_STATUS_REFUNDED          = 4;
    const INTERNAL_STATUS_NOT_REFUNDED      = 8;
    const INTERNAL_STATUS_CLOSED            = 16;
    const INTERNAL_STATUS_CC_FAILED         = 32;
    const INTERNAL_STATUS_LATE_CANCELLATION = 64;
    const INTERNAL_STATUS_NO_SHOW           = 128;

    const TURN_OFF_SHOW = 3; //
    const NO_SHOW_RATED = 2; // Driver no show up and charge money
    const NO_SHOW       = 1; // Driver no show up
    const YES_SHOW      = 0; // Driver show

    public function expires($force = false)
    {
      // stop if the request has already expired or being cancelled
      if ($this->status != self::STATUS_PENDING)
      {
        return;
      }

      // set status to expired if pickup date is in the past
      if ($force || ($this->status == self::STATUS_PENDING &&
          $this->pickup_date < date("Y-m-d")))
      {
          $this->status = Request::STATUS_EXPIRED;
          $this->when_accepted = date('Y-m-d H:i:s'); // Will be used in statistics
          self::getEventDispatcher()->notify(new sfEvent($this, 'request.expired'));
          $this->save();
      }
    }

    public function getIncrementedId()
    {
      $rows = $this->getTable()->createQuery('r')
        ->select('id')
        ->where('r.listing_id = ?', $this->get('listing_id'))
        ->orderBy('r.id DESC')
        ->execute(array(), Doctrine::HYDRATE_NONE);

      $key = 0;
      foreach ($rows as $row)
      {
        $key++;
        if ($row[0] == $this->get('id'))
        {
          break;
        }
      }

      return $this->get('listing_id') .'-'. ($key);
    }

    public function generateEmailKey($user)
    {
        return sha1(sprintf('#%s#%s#%s#%s#',
          $this->getId(),
          $this->get('listing_id'),
          $this->get('status'),
          $user->get('password')
        ));
    }

    public function getEmailKey($user)
    {
      $salt = sprintf('$2a$%02d$', 12);
      $salt .= hash('whirlpool', $this->getId().'//'.$this->get('status'));

      $key = $this->generateEmailKey($user);

      return base64_encode(crypt($key, $salt));
    }

    public function checkKey($key, $user)
    {
        $requestKey = $this->parseKey($this->getEmailKey($user));
        $key =  $this->parseKey($key);

        return crypt($this->generateEmailKey($user), $key) === $requestKey;
    }

    private function parseKey($key)
    {
      return base64_decode($key);
    }

    public function isAccepted()
    {
      return $this->get('id') === $this->get('Listing')->get('accepted_request_id');
    }

    public function canBeReviewed()
    {
      return !$this->get('no_show_fee') && $this->isAccepted() && $this->get('Listing')->isStatus(array(Listing::STATUS_DROPPED_OFF, Listing::STATUS_CLOSED)) && !$this->get('Review')->exists();
    }

    public function isReviewed()
    {
      return $this->get('Review')->exists();
    }

    public function isDriverId($id)
    {
      return $this->get('driver_id') === $id;
    }

    /**
     * Returns string representing status name
     */
    public function getStatusName()
    {
      switch ($this->status) {
        case Request::STATUS_REJECTED:
          return 'Rejected';
        case Request::STATUS_EXPIRED:
          return 'Expired';
        case Request::STATUS_ACCEPTED:
          return 'Accepted';
        case Request::STATUS_PENDING:
          return 'Pending';
        case Request::STATUS_OTHER_CHOOSE:
          return 'Other choose';
        case Request::STATUS_CANCELED:
          return 'Canceled';
        case Request::STATUS_RATED:
          return $this->get('no_show_fee') ? 'No show' : 'Rated';
        case Request::STATUS_DRIVER_CANCELED:
          $status = 'RO: Driver Cancelled ';
          $status .= $this->isLateCancellation() ? '(+'.round($this->getResponseTime()*24, 0).'h)' : '';

          return $status;
        case Request::STATUS_OWNER_CANCELED:
          $status = 'RO: Owner Cancelled ';
          $status .= $this->isLateCancellation() ? '(+'.round($this->getResponseTime()*24, 0).'h)' : '';

          return $status;
        case Request::STATUS_CANCELED_BY_DRIVER:
          $status = 'Driver Cancelled ';
          $status .= $this->isLateCancellation() ? '(+'.round($this->getResponseTime()*24, 0).'h)' : '';

          return $status;
        default:
          return 'Unknown';
      }
    }

    // Notification constants
    const NOTIFY_ALL           = 1; // notify everyone
    const NOTIFY_EXCEPT_DRIVER = 2; // notify everyone except driver
    const NOTIFY_EXCEPT_OWNER  = 3; // notify everyone except owner

    public function buildRelocLocationDetails($groupedRequest)
    {
      $tpl =<<<EOF
Relocation from ###from### to ###to###
###reloc_details###

EOF;
      $details = array();
      foreach ($groupedRequest as $request)
      {
        $listing = $request->Listing;
        $details[] = str_replace(
          array('###from###', '###to###', '###reloc_details###'),
          array(
            $listing->PickupLocation->name,
            $listing->DropoffLocation->name,
            $listing->detailedInfo()
          ),
          $tpl
        );
      }

      return implode("\n", $details);
    }

  /**
   * Get accepted driver username. Used by backend application.
   *
   * @return string driver username
   */
  public function getDriverUsername()
  {
   return $this->Driver->getShortName();
  }

  /**
   * Get accepted driver email. Used by backend application.
   *
   * @return string driver email
   */
  public function getDriverEmail()
  {
   return $this->Driver->get('email');
  }

  /**
   * Get accepted driver username. Used by backend application.
   *
   * @return string owner username
   */
  public function getOwner()
  {
   return $this->Listing->Owner->username;
  }

  /**
   * Get accepted driver username. Used by backend application.
   *
   * @return string pick-up location
   */
  public function getPickLocation()
  {
   return $this->Listing->PickupLocation->name;
  }

  /**
   * Get accepted driver username. Used by backend application.
   *
   * @return string drop-off location
   */
  public function getDropLocation()
  {
   return $this->Listing->DropoffLocation->name;
  }

  public function getDropoff()
  {
    return $this->getDropLocation();
  }

  public function getCountry()
  {
  	return $this->Listing->PickupLocation->Category->Country->name;
  }

  public function getCountryDomain()
  {
    return $this->Listing->PickupLocation->Category->Country->domain;
  }

  public function getCountryid()
  {
  	return $this->Listing->PickupLocation->Category->Country->id;
  }

  public function getNbDays()
  {
    return $this->nb_free_days + $this->nb_paid_days;
  }

  private function getDistance($dropoff, $pickup)
  {
      $lat1 = floatval($dropoff->get('lat'));
      $lat2 = floatval($pickup->get('lat'));

      $lon1 = floatval($dropoff->get('lon'));
      $lon2 = floatval($pickup->get('lon'));

      return sqrt(pow(110.56*($lat2 - $lat1), 2) + pow(84.8*($lon2 - $lon1), 2));
  }

  public function overlaps($listing, $pickup_date, $drop_off_date)
  {
    $overlaps = (
      strtotime($this->get('pickup_date')) <= strtotime($drop_off_date) &&
      strtotime($this->get('drop_off_date')) >= strtotime($pickup_date)
    );

    // if requests have the same dropoff or pickup location on the same day, let's calculate the distance between them
    $distance = 999;

    // dates overlaps but pickup date is same as the other request dropoff date
    if ($overlaps && $this->get('pickup_date') === $drop_off_date)
    {
      $distance = $this->getDistance($listing->get('DropoffLocation'), $this->Listing->get('PickupLocation'));
    }
    // dates overlaps but dropoff date is same as the other request pickup date
    elseif ($overlaps && $pickup_date === $this->get('drop_off_date'))
    {
      $distance = $this->getDistance($listing->get('PickupLocation'), $this->Listing->get('DropoffLocation'));
    }

    // if dates overlaps and distance > 200km, the request is rejected
    return $overlaps && $distance > 200;
  }

  private function cancelOverlappingRequests()
  {
    // Find other user requests
    $other = Doctrine_Query::create()
      ->from("Request r")
      ->leftJoin("r.Listing")
      ->where("r.listing_id != ?", $this->listing_id)
      ->addWhere("r.driver_id = ?", $this->driver_id)
      ->addWhere("r.id != ?", $this->id)
      ->whereIn("r.status", array(Request::STATUS_PENDING))
      ->execute();

    $groupedRequests = array();
    // check if other user requests conflicts with this one
    foreach($other as $r) {
      // check if requests overlaps in time
      $overlaps = $this->overlaps($r->get('Listing'), $r->get('pickup_date'), $r->get('drop_off_date'));

      // Send mail if this relocation overlaps with
      // other requests and cancel it
      if ($overlaps) {
        $r->status = Request::STATUS_CANCELED;
        $r->save();

        if ($r->Listing->owner_id != $this->Listing->owner_id)
        {
          if (!isset($groupedRequests[$r->Listing->owner_id]))
          {
            $groupedRequests[$r->Listing->owner_id] = array();
          }

          $groupedRequests[$r->Listing->owner_id][] = $r;

          // trigger the cancel event only for overlapping requests from a different RO
          // see https://basecamp.com/1780292/projects/446799/todos/6295424
          self::getEventDispatcher()->notify(new sfEvent($r, 'request.cancelled'));
        }
      }
    }

    // Notify cancellation of multiple requests at once to avoid a lot of unnecessary emails
    // see https://basecamp.com/1780292/projects/446799/todos/6295424
    foreach ($groupedRequests as $groupedRequest)
    {
      self::getEventDispatcher()->notify(new sfEvent($groupedRequest, 'requests.someone_else_accepted'));
    }
  }

  private function rejectOrReassignSiblingRequests()
  {
    // Mark all other requests for given listing as not chosen
    $reqs = Doctrine_Query::create()
      ->from('Request r')
      ->leftJoin('r.Driver')
      ->where('r.listing_id = ?', $this->Listing->id)
      ->addWhere('r.id != ?', $this->id)
      ->addWhere('r.status = ?', Request::STATUS_PENDING)
      ->execute();

    $children = $this->Listing->get('ChildrenListings');
    if (count($children))
    {
      $l = $children->getFirst();
      $l->status = Listing::STATUS_NEW;
      $l->listing_id = null;

      foreach ($reqs as $req) {
        $req->set('Listing', $l);
      }

      for ($i = 1; $i < count($children); $i++) {
        $children[$i]->set('ParentListing', $l);
      }
      $l->set('nb_listings', $this->Listing->get('nb_listings') - 1);

      $this->Listing->set('nb_listings', 1);
      $this->Listing->save();
      $reqs->save();
      $children->save();
      $l->save();
    }
    else
    {
      foreach($reqs as $r)
      {
        $r->status = Request::STATUS_OTHER_CHOOSE;
        $r->save();
        // other choose
        self::getEventDispatcher()->notify(new sfEvent($r, 'request.rejected'));
      }
    }
  }

  public function accept($username)
  {
    $conn = Doctrine_Manager::getInstance()->getCurrentConnection();

    $conn->beginTransaction();

    try
    {
      $this->status = Request::STATUS_ACCEPTED;
      $this->when_accepted = date('Y-m-d H:i:s');
      $this->save();

      // cancel overlapping requests from the same driver
      $this->cancelOverlappingRequests();

      $owner = $this->Listing->Owner;
      // Mark request and listing objects as accepted
      $this->Listing->set('status', Listing::STATUS_ACCEPTED);
      $this->Listing->set('upselling_price', $this->get('nb_paid_days') * $this->Listing->getPriceForAdditionalDay());
      $this->Listing->set('AcceptedRequest', $this);
      $this->Listing->save();

      if ($this->Listing->hasBond())
      {
        $form = new TcStripeForm(
          ['amount' => $this->Listing->getBondAmount()], 
          [
            'user_id' => $this->get('driver_id'),
            'currency' => strtolower($this->Listing->getCountryDomain().'d')
          ], false
        );

        $particular = 'Transfercar bond';
        $description = substr('BND/'.$this->getIncrementedId().'/'.$username, 0, 50);
        $values = [
          'amount' => $this->Listing->getBondAmount(),
          'stripeToken' => $this->get('CreditCard')->get('holder_name')
        ];

        if (!$cct = $form->processPayment($description, $particular, $values, $this->get('CreditCard')))
        {
          throw new CreditCardException("Error Processing Credit card: ".$form->paymentError);
        }
      }

      self::getEventDispatcher()->notify(new sfEvent($this, 'relocation.accepted'));

      // listing may have multiple requests, they need to be cancelled or reassigned to the next available listing
      $this->rejectOrReassignSiblingRequests();

      // Get fee for this operation and update user information
      $fee = $this->Listing->getTotalPrice($this);

      // process payment using RO credits
      $payment = new Payments();
      $payment->type = 'manual';
      $payment->date = date('Y-m-d', time());
      $payment->amount = -$fee;
      $payment->user_id = $this->Listing->Owner->id;
      $payment->status = Payments::STATUS_ACCEPTED;
      $payment->assigned_record_id = -$this->Listing->id;
      $payment->country_id = $this->Listing->PickupLocation->Category->Country->id;
      $payment->is_new_type_payment = "yes";
      $payment->save();
      // Owner account balance will be updated by SQL trigger

      $conn->commit();
    } catch (CreditCardException $ex) {
      $conn->rollback();
      error_log(__METHOD__.': '.$ex->getMessage());
      return false;

    } catch(Exception $ex) {
      $conn->rollback();
      error_log(__METHOD__.': '.$ex->getMessage());
      return false;
    }

    return true;
  }

  public function rateAndClose($rate, $comment, $no_show_fee = 0)
  {
    $this->close();
    $this->rate($rate, $comment, '', $no_show_fee);
  }

  public function rate($rate, $comment, $internal_comment, $no_show_fee = 0)
  {
    $no_show_fee = isset($no_show_fee) && $no_show_fee == Request::NO_SHOW ? Request::NO_SHOW : Request::YES_SHOW;

    $this->status = Request::STATUS_RATED;
    $this->rating = $rate;
    $this->rating_comment = $comment;
    $this->internal_rating_comment = $internal_comment;
    $this->no_show_fee = $no_show_fee;
    $this->save();
    self::getEventDispatcher()->notify(new sfEvent($this, 'request.rated'));

    // 3. Update information about driver average rating
    $this->get('Driver')->recalculate_avg_rating();
  }

  public function close()
  {
    // 2. Close request and listing records
    $listing = $this->Listing;
    $listing->status = Listing::STATUS_CLOSED;
    $listing->save();
  }

  public function noShowAndClose($comment)
  {
    $this->close();
    $this->noshow($comment);
  }

  public function noshow($comment)
  {
    $this->status         = Request::STATUS_RATED;
    $this->no_show_fee    = 1;
    $this->no_show_fee_amount = Settings::getNoShowFeeCost();

    $this->rating_comment = $comment;
    $this->cancelled_at   = date('Y-m-d H:i:s');
    $this->addInternalStatus(self::INTERNAL_STATUS_NO_SHOW);
    $this->save();

    self::getEventDispatcher()->notify(new sfEvent($this, 'request.rated'));

    // 3. Update information about driver average rating
    $this->Driver->recalculate_avg_rating();
  }

  public function driverCancel($comment, $cancellation_reason_id = null, $filename = null)
  {
    $this->driver_cancel_comment = $comment;
    $this->cancellation_reason_id = $cancellation_reason_id;
    $this->cancellation_document = $filename;
    $this->is_withdrawn = Doctrine::getTable('CancellationReason')->isWithdrawn($cancellation_reason_id);
    $this->cancel(
      $cancellation_reason_id ? Doctrine::getTable('CancellationReason')->find($cancellation_reason_id)->get('cancelled_by') : Request::STATUS_CANCELED_BY_DRIVER,
      !$this->isStatus(Request::STATUS_PENDING) && $this->pickupInLessThan('+2 days'),
      '', '' // no internal comments yet
    );
  }

  public function cancel($status, $is_late_cancel, $comment, $internal_comment)
  {
    $this->status         = $status;
    $this->cancelled_at   = date('Y-m-d H:i:s');
    $this->is_late_cancel = $is_late_cancel;
    $this->rating_comment = $comment;
    $this->internal_rating_comment = $internal_comment;
    $this->no_show_fee_amount = $is_late_cancel ? Settings::getLateCancellationFee() : Settings::getEarlyCancellationFee();
    

    if (!$this->Listing->isStatus(Listing::STATUS_NEW) && $this->isAccepted())
    {
      $this->Listing->status = Listing::STATUS_CANCELED;
    }

    if ($this->isAccepted() && $is_late_cancel)
    {
      $this->is_late_cancel = 1;
      $this->addInternalStatus(self::INTERNAL_STATUS_LATE_CANCELLATION);
    }

    $this->save();
    $this->refundDeposit();

    self::getEventDispatcher()->notify(new sfEvent($this, 'request.cancelled'));
  }

  public function refundDeposit()
  {
    if ($this->get('Listing')->hasBond() && $this->isAccepted() && $this->isStatus(Request::STATUS_OWNER_CANCELED))
    {
      Flo2CashRefundHandler::processRefund($this->get('Driver'));
    }
  }

  public function pickupInLessThan($time)
  {
    $maxTime = new DateTime($time);
    $pickupTime = new DateTime($this->get('pickup_date'));
    $pickupTime->add(new DateInterval('PT'.$this->get('pickup_time').'S'));

    return $pickupTime < $maxTime;
  }

  public function isPickupDate($time)
  {
    return strtotime($this->get('pickup_date')) === strtotime($time);
  }

  public function isCompletedSince($time)
  {
    return date('Y-m-d', strtotime($time, strtotime($this->get('drop_off_date')))) === date('Y-m-d');
  }

  public function isLateCancellation()
  {
    return $this->is_late_cancel;
  }

  public function reject()
  {
    try {
      $conn = Doctrine_Manager::getInstance()->getCurrentConnection();

      $conn->beginTransaction();

      $this->status = Request::STATUS_REJECTED;
      $this->when_accepted = date('Y-m-d H:i:s'); // Will be used in statistics
      $this->save();

      self::getEventDispatcher()->notify(new sfEvent($this, 'request.rejected'));

      $conn->commit();
    } catch (Exception $e) {
      $conn->rollback();
      return false;
    }

    return true;
  }

  public function setDataFromListing($listing)
  {
    $this->set('listing_data', array(
      'listing' => $listing->toArray(false),
      'vehicle' => $listing->get('VehicleType')->toArray(false),
      'owner_vehicle' => $listing->get('owner_vehicle_type_id') ? $listing->get('OwnerVehicleType')->toArray(false) : null,
      'owner_pickup' => $listing->get('owner_pickup_branch_id') ? $listing->get('OwnerPickupBranch')->toArray(false) : null,
      'owner_dropoff' => $listing->get('owner_dropoff_branch_id') ? $listing->get('OwnerDropoffBranch')->toArray(false) : null,
    ));
  }

  public function getResponseTime()
  {
    $createdAtTs = strtotime($this->get('created_at'));
    $acceptedAtTs = strtotime($this->get('when_accepted'));

    return ($acceptedAtTs - $createdAtTs) / 86400;
  }

  public function isStatus($status)
  {
    if (!is_array($status)) {
      $status = array($status);
    }

    return in_array($this->status, $status);
  }

  public function addInternalComment($status, $comment)
  {
    $comments = $this->get('internal_comments');
    $comments[$status] = $comment;

    $this->set('internal_comments', $comments);
  }

  public function removeInternalComment($status)
  {
    $comments = $this->get('internal_comments');
    unset($comments[$status]);

    $this->set('internal_comments', $comments);
  }

  public function getInternalComment($status)
  {
    $comments = $this->get('internal_comments');

    return isset($comments[$status]) ? $comments[$status] : '';
  }

  public function addInternalStatus($status, $comment='')
  {
    $internal_status = $this->get('internal_status');

    $this->set('internal_status', $internal_status | $status);
    $this->addInternalComment($status, $comment);
  }

  public function removeInternalStatus($status)
  {
    $internal_status = $this->get('internal_status');

    $this->set('internal_status', $internal_status ^ $status);
    $this->removeInternalComment($status);
  }

  public function isInternalStatus($status)
  {
    return (bool) ($this->get('internal_status') & $status);
  }

  public function cancelledByDriver()
  {
    return ( $this->isStatus(array(self::STATUS_DRIVER_CANCELED, self::STATUS_CANCELED_BY_DRIVER)) ||
                 $this->get('no_show_fee') == 1
              );
  }

  public function isCancelled()
  {
    return in_array($this->get('status'), array(
      self::STATUS_CANCELED,
      self::STATUS_DRIVER_CANCELED,
      self::STATUS_OWNER_CANCELED,
      self::STATUS_CANCELED_BY_DRIVER
    ));
  }


  public function canBeCharged()
  {
    return !$this->isInternalStatus(self::INTERNAL_STATUS_CHARGED |
                                    self::INTERNAL_STATUS_NOT_CHARGED |
                                    self::INTERNAL_STATUS_CLOSED
                  )  //&&
            //$this->isInternalStatus(self::INTERNAL_STATUS_NO_SHOW|self::INTERNAL_STATUS_LATE_CANCELLATION)
            //&&
            //$this->cancelledByDriver()
            && ($this->get('cancellation_reason_id') === null or $this->get('CancellationReason')->get('driver_fee_action') != 'waive')
;
  }
  public function canBeWaived(){

    return !$this->isInternalStatus(self::INTERNAL_STATUS_CHARGED |
                                    self::INTERNAL_STATUS_NOT_CHARGED |
                                    self::INTERNAL_STATUS_CLOSED
                  )  //&&
            //$this->isInternalStatus(self::INTERNAL_STATUS_NO_SHOW|self::INTERNAL_STATUS_LATE_CANCELLATION)
            //&&
            //$this->cancelledByDriver()
            && ($this->get('cancellation_reason_id') === null or $this->get('CancellationReason')->get('driver_fee_action') != 'charge');
  }

  public function canBeRefunded()
  {
    return !$this->isInternalStatus(self::INTERNAL_STATUS_REFUNDED |
                                    self::INTERNAL_STATUS_NOT_REFUNDED |
                                    self::INTERNAL_STATUS_CLOSED
                  )
            // && $this->cancelledByDriver()
            && $this->isAccepted()
            && ($this->get('cancellation_reason_id') === null or $this->get('CancellationReason')->get('ro_refund_action') != 'no_refund')
          ;
  }

  public function getTripParams()
  {
    $pattern = '/[^a-zA-Z0-9]/';
    $from = preg_replace($pattern, '_', $this->Listing->PickupLocation->name);
    $to = preg_replace($pattern, '_', $this->Listing->DropoffLocation->name);
    $id = $this->getId();

    return sprintf('from=%s&to=%s&id=%d', $from, $to, $id);
  }

  public function postSave( $event)
  {
    $request = $event->getInvoker();
    // fetch again to refresh the listing data
    $listing = Doctrine::getTable('Listing')->find($request->get('listing_id'));
    $listing->refresh();
    tcElasticaListing::listenToListingUpdated(new sfEvent($listing, '.'));

    if ($request->isStatus(Request::STATUS_PENDING) && ($request->get('Driver')->get('internal_status') == 1 || $request->get('Driver')->isBannedFor($listing->get('owner_id')) || $request->get('Driver')->get('user_status') == 'blocked')) {
      $request->reject();
    }
  }

  public function hasOverlappingRequest()
  {
      // Find other user requests
      $other = Doctrine_Query::create()
        ->from("Request r")
        ->leftJoin("r.Listing")
        ->where("r.listing_id != ?", $this->listing_id)
        ->addWhere("r.driver_id = ?", $this->driver_id)
        ->addWhere("r.id != ?", $this->id)
        ->whereIn("r.status", array(Request::STATUS_PENDING))
        ->execute();

      $groupedRequests = array();
      // check if other user requests conflicts with this one
      foreach($other as $r) {
        // check if requests overlaps in time
        if ($this->overlaps($r->get('Listing'), $r->get('pickup_date'), $r->get('drop_off_date'))) {
          return true;
        }
      }

      return false;
  }

}
