<?php
/**
 * This class has been auto-generated by the Doctrine ORM Framework
 */
class UserTable extends Doctrine_Table
{

    /**
     * This is a query for TC admin pager
     * @return Doctrine_Query
     */
    public function getWithExtraFieldsQuery(){

        $q = Doctrine_Query::create()->select('u.*, MAX(l.created_at) as LastRelocation')
                ->from('User u')
                ->leftJoin('u.Listings l WITH (l.accepted_request_id > 0)')
                ->where('u.is_owner = ?' , array(1))
                ->groupBy('u.id');

        return $q;
    }

    public function getAuthenticatedUser($username, $password)
    {
        $user = Doctrine_Query::create()
           ->from('User')
	   ->where('(username = ? or email = ?)', [$username, $username])
           ->addWhere('user_status = ? OR user_status = ?', array('active', 'subscribe'))
           ->fetchOne();

        if ($user && $user->checkPassword(sha1($password)))
        {
            return $user;
        }

        return false;
    }

    public function getOrCreateFromOAuth(League\OAuth2\Client\Provider\ResourceOwnerInterface $OAuthUser)
    {
        if (!$OAuthUser->getEmail())
        {
            throw new RuntimeException("Email not provided");
        }

        if (! $tcUser = $this->findOneByEmail( $OAuthUser->getEmail()))
        {
            $sfUser  = sfContext::getInstance()->getUser();
            $request = sfContext::getInstance()->getRequest();

            $tcUser = RegistrationFormHandler::saveFromParams($sfUser, $request, array(
                'email'     =>  $OAuthUser->getEmail(),
                'firstname' =>  $OAuthUser->getFirstName(),
                'lastname'  =>  $OAuthUser->getLastName(),
                // this will create an invalid password (for legacy reasons, it needs to be hashed before being saved)
                // This way users will only be able to log using their oauth account
                'password'  =>  $OAuthUser->getId(),
                'source_information' => 'oauth'
            ));
        }

        return $tcUser;
    }

    private function cleanName($name, $default = null)
    {
        if (preg_match('#^[\p{Latin}[A-Za-z\s-]+$#', $name)) {
            return $default;
        }

        return $name;
    }

    public function findOwners()
    {
        return $this->createQuery()
            ->where('is_owner = ?', true)
            ->orderBy('company, username')
            ->execute();
    }

    public function findOwnersByUsername()
    {
        return $this->createQuery()
            ->where('is_owner = ?', true)
            ->orderBy('username')
            ->execute();
    }

    public function findROsByRating()
    {
        return $this->createQuery()
            ->where('is_owner = ?', true)
            ->orderBy('average_rating desc')
            ->execute();
    }

    public function findByActivationCode($code)
    {
        return Doctrine_Query::create()
           ->from('User')
           ->where('id = ?', intval($code[0]))
           ->addWhere('activation_code = ?', intval($code[1]))
           ->addWhere('user_status = ?', 'inactive')
           ->fetchOne();
    }

    public function incrementSoftBounce($email)
    {
        $query = Doctrine_Query::create()
            ->update('User u')
            ->set('emails_softbounced', 'emails_softbounced + 1')
            ->where('u.email = ?', $email);


        $query->execute();
        $query->free(true);
    }

    public function flagHardBounce($email)
    {
        $query = Doctrine_Query::create()
            ->update('User u')
            ->set('is_hardbounced', '1')
            ->where('u.email = ?', $email);

        $query->execute();
        $query->free(true);
    }

    public function unflagHardBounce($email)
    {
        $query = Doctrine_Query::create()
            ->update('User u')
            ->set('is_hardbounced', '0')
            ->where('u.email = ?', $email);

        $query->execute();
        $query->free(true);
    }

    public function unsubscribe($email)
    {
        $notif_user = Doctrine_Query::create()
            ->from('NotificationUser nu')
            ->leftJoin('nu.User u')
            ->where('(nu.email = ? or (u.id is not null and u.email = ?))', array($email, $email))
            ->fetchOne();

        $user = Doctrine::getTable('User')->findOneByEmail($email);

        if ($user)
        {
            $user->set('email_notify_new_reloc', 'none');
            $user->set('email_country_new_reloc', 'none');
            $user->set('email_notify_new_req', false);
            $user->set('email_notify_update_req', false);
            $user->save();
        }

        if ($notif_user)
        {
            $notif_user->get('NotifyRoutes')->delete();
            $notif_user->delete();
        }
    }

    public function resetBounceStats($email)
    {
        $query = Doctrine_Query::create()
            ->update('User u')
            ->set('is_hardbounced', '0')
            ->set('emails_softbounced', '0')
            ->where('u.email = ?', $email)
            // add criteria to reduce the number of writes
            ->addWhere('(is_hardbounced != 0 OR emails_softbounced > 0)');

        $query->execute();
        $query->free(true);
    }

    public function isEmailBounced($email)
    {
        $query = Doctrine_Query::create()
            ->from('User u')
            ->where('u.email = ?', $email)
            ->addWhere('(is_hardbounced != 0 OR emails_softbounced > 10)')
            ->limit(1);

        $res = $query->count();
        $query->free(true);
        $isBounced = (bool) $res;

        return (bool) $isBounced;
    }

    public function isSuspect($user)
    {

        // already suspect
        if ($user->get('internal_status') == 1) {
            return true;
        }

        // approved
        if ($user->get('internal_status') == 2) {
            return false;
        }	

        $query = $this->createQuery('u')
            ->leftJoin('u.ROBans b')
            ->where('(
                (
                    (u.firstname = ? and u.lastname=?)
                ) 
                and 
                (u.user_status=? or b.driver_id is not null)
            )', [
                $user->get('firstname'), 
                $user->get('lastname'), 
                2 // blocked
            ]);

        if ($user->get('date_of_birth')) {
            $query->addWhere('u.date_of_birth=?', $user->get('date_of_birth'));
        }

        if ($user->get('mobile_phone')) {
            $query->addWhere('u.mobile_phone=?', $user->get('mobile_phone'));
        }

        $nb =  $query->count();

        return $nb > 0;
    }
}
