<?php

use Transfercar\Mandrill\TcMandrill;

class Flo2CashPaymentHandler
{
	public static function handle($user_id)
	{
        $logger = new sfFileLogger(new sfEventDispatcher(), array(
          'file' => sfConfig::get('sf_log_dir') . '/payments_php.log',
          'format' => '%time% %message%%EOL%',
          'timeFormat' => '%g-%m-%d %H:%M:%S'
        ));
        $logger->log("flo2cash:process-payment started >>>");

        /*
         * Possible results from payment gate
         */
        define('TRANSATION_ACCEPTED', 'SUCCESSFUL');
        define('TRANSACTION_DECLINED', 'FAILED');

        //$user_id = $argv[1];

        /*
        *   Additional code. Sibers version.
        *   we have strange issue with double and triple transactions with an interval of several seconds
        *   try to create lock file for one user. Lock file will be deleted in the end of this script
        */

        $lock_filename = sys_get_temp_dir() . '/.payment_locker_id_'.$user_id;

        /*
        * This part of code was created in case there was a failure of the server and the lock file is not deleted. Here, I delete it manually
        */
        if (file_exists($lock_filename)) {
            $time_sec = time();
            $time_file = filemtime($lock_filename);
            $time = $time_sec-$time_file;
            if ($time > 30){ // if the file was created greater than 30 seconds ago - delete it
                unlink($lock_filename);
                $subject = '[Transfercar] Forced removal of the lock file was detected';
                $body = 'Forced removal of the lock file was detected for user with id '.$user_id.'. The lifetime of the lock file - '.$time.' seconds.';
                $receiver = array('email' => 'noel@transfercar.co.nz','name' => 'Noel','format' => 'plain'); // developer

                $mandrill = new TcMandrill();
                $mandrill->sendTemplate(
                  $subject,
                  $user->getEmailToParams(),
                  array('content' => $body)
                );
            }else{ // it means server trying to execute double or tripple transactions, preventing it
                echo 'Another script is already running. Finishing...';
                $logger->log("flo2cash:process-payment lock file exists for user : " . $lock_filename);
                $logger->log("flo2cash:process-payment finished >>>\n\n\n");
                $logger->shutdown();
                exit(0);
            }
        }
        // Additional code

        // Create lock file
        $fp = fopen($lock_filename, 'w');
        fwrite($fp, 'timestamp');
        fclose($fp);

        $logger->log("flo2cash:process-payment lock file created: " . $lock_filename);


        //while (true) {

        // Get pending transaction
        $transaction = Doctrine_Query::create()
          ->from('CCTransaction c')
          ->leftJoin('c.Payment cp')
          ->leftJoin('c.CreditCard')
          ->addWhere('c.status = ?', CCTransaction::STATUS_NEW)
          ->addWhere('c.payment_id IS NOT NULL')
          ->addWhere('cp.user_id = ?', $user_id)
          ->orderBy('c.id desc')
          ->limit(1)
          ->fetchOne();

          $logger->log("payer id: ".$user_id);

        /**
         * Get array of accounts for all countries
         */
        $accounts = sfConfig::get('sf_payment_account');
        $card_types = sfConfig::get('sf_payment_card_types_codes');

        if (!is_object($transaction)) { // No another work, finish
          echo 'No other work. Finishing...' . "\n";
          $logger->log("flo2cash:process-payment pending transaction not found");

          //Delete locking file
          if (unlink($lock_filename)) {
            $logger->log("flo2cash:process-payment lock file removed");
          }

          $logger->log("flo2cash:process-payment finished >>>\n\n\n");
          $logger->shutdown();
          exit(0);
        }
        $logger->log("flo2cash:process-payment pending transaction found\n");
        // Prepare data for payment gate
        $gate_url = sfConfig::get('sf_payment_url_new');
        //Correct country account!!!

        $requestDomain = $transaction->Payment->Country->domain;
        $ccDomain = $transaction->CreditCard->country;
        $account_id = $accounts[$requestDomain];
        $domain = $requestDomain;

        //Amount in NATIONAL CURRENCY!!!
        $amount = $transaction->Payment->national_currency_amount; // Flo2Cash accepts only full-dollar amount

        if ($requestDomain === 'au' && $ccDomain === 'NZ') {
          $account_id = $accounts['nz'];
          $converter = new Transfercar\CurrencyRate\Converter();
          $amount = $converter->convert('AUD', 'NZD', $amount);
        }

        $txn_data['amount'] = $amount;

        // Send e-mail with receipt from gate operator?
        $assigned_record_id = $transaction->Payment->assigned_record_id;

        $user_email = '';
        $company = '';
        if ($assigned_record_id > 0) {
          $user = Doctrine_Query::create()
          ->from('User')
          ->where('id = ?', $assigned_record_id)
          ->fetchOne();
          if (is_object($user)) {
            $user_email = $user->email;
            $company = $user->get('company');
            $logger->log("flo2cash:process-payment users email fetched: " . $user_email);
          }
        }

        // We can have here 3 possible situations:
        // a) we have new card and we want to save it for future use
        // b) we have new card and we won't to save it
        // c) we have already saved card
        $should_save_card = false;
        $card_already_saved = false;
        $is_error = false;

        $card = $transaction->CreditCard;

        //Base parameters initialization(when we used in all operation)
        $InputVariables = array(
          "Username" => sfConfig::get('sf_payment_username'),
          "Password" => sfConfig::get('sf_payment_password'),
          "AccountId" => $account_id,
          "Amount" => $amount,
          "Reference" => sprintf('Transfercar: %s credit purchase', $transaction->Payment->amount),
          "Particular" => substr('CRD/'.$company, 0, 50),
          "Email" => $user_email
        );


        if ($card->card_type < 0) { // card already stored
          // We have card token in "card holder name" field
          $card_already_saved = true;
          $card_token_id = $card->holder_name;
          $card_csc = $card->csv;

          $InputVariables["CardToken"] = $card_token_id;
          $PaymentService = new SoapClient($gate_url);

          $logger->log("flo2cash:process-payment card already stored: token id: #" . $card_token_id);

          try {
            $logger->log("flo2cash:process-payment SOAP Call with: parameters: " . print_r($InputVariables, 1));
            $result = $PaymentService->ProcessPurchaseByToken($InputVariables);
          } catch (SoapFault $fault) {
            $logger->log("flo2cash:process-payment SOAP Call ERROR: SoapFault: " . $fault->detail->error->errormessage);
            $is_error = true;
          } catch (Exception $exp) {
            $logger->log("flo2cash:process-payment SOAP Call Exception: " . $exp->getMessage());
            $is_error = true;
          }
        } else {

          $logger->log("flo2cash:process-payment card not already stored");

          $card_number = $card->number;
          $stored_card = false;
          if (substr($card_number, 0, 1) != '#') {
            $logger->log("flo2cash:process-payment card NOT marked for delete");
            $should_save_card = true;
            $stored_card = true;
          }
          $card_number = preg_replace('/[^0-9]/', '', $card_number);

          $expiry = date('my', strtotime($card->expiry));


          $InputVariables["CardNumber"] = $card_number;
          $InputVariables["CardType"] = $card_types[$card->card_type];
          $InputVariables["CardExpiry"] = $expiry;
          $InputVariables["CardHolderName"] = $card->holder_name;
          $InputVariables["CardCSC"] = $card->csv;
          $InputVariables["StoreCard"] = $stored_card;

          $PaymentService = new SoapClient($gate_url);

          // Clear $InputVariables to NOT PRINT CREDIT CARD NUMBER into the log file!
          $logInputVariables = $InputVariables;
          $logInputVariables['CardNumber'] = preg_replace('/(\d+)(\d{4})/', '***********\\2', $logInputVariables['CardNumber']);
          $logInputVariables['CardCSC'] = '***';

          try {
            $logger->log("flo2cash:process-payment SOAP Call with: parameters: " . print_r($logInputVariables, 1));
            $result = $PaymentService->ProcessPurchase($InputVariables);
          } catch (SoapFault $fault) {
            $logger->log("flo2cash:process-payment SOAP Call ERROR: SoapFault: " . $fault->detail->error->errormessage);
            $is_error = true;
          } catch (Exception $phpExp) {
            $logger->log("flo2cash:process-payment SOAP Call Exception: " . $phpExp->getMessage());
            $is_error = true;
            // Handle non SOAP:Fault error
          }
        }

        $log = $transaction->communication_log;
        if (!is_array($log)) {
          $log = array();
        }

        //was ERROR: <SOAP FAULT> or <PHP>
        if ($is_error == true) {
          if (is_object($fault)) {
            $log[] = serialize($fault);
            $transaction->communication_log = $log;
            $transaction->status_message = $fault->detail->error->errormessage;
          } else {
            $log[] = serialize($phpExp);
            $transaction->communication_log = $log;
            $transaction->status_message = $phpExp->getMessage();
          }
        } else {//all OK
          $log[] = serialize($result);
          $transaction->communication_log = $log;
          $transaction->status_message = $result->transactionresult->Message;

          $logger->log("flo2cash:process-payment SOAP Call Successfully");
          $logger->log("flo2cash:process-payment SOAP Call Result: " . var_export($result, 1));
        }

        // Check transaction status
        $status = $result->transactionresult->Status;
        if ($status != TRANSATION_ACCEPTED) {
          $status = TRANSACTION_DECLINED;
        }

        if ($status == TRANSACTION_DECLINED) {
          $logger->log("flo2cash:process-payment Transaction declined");

          $transaction->status = CCTransaction::STATUS_REJECTED;
          $transaction->save();

          $logger->log("flo2cash:process-payment Transaction saved");

          $payment = $transaction->Payment;
          $payment->status = Payments::STATUS_REJECTED;
          $payment->save();

          $logger->log("flo2cash:process-payment Payment saved");
          // User account balance will be updated by database trigger
          // Remove card if it is not already stored
          if (!$card_already_saved) {
            $transaction->credit_card_id = null;
            $transaction->save();
            $card->delete();

            $logger->log("flo2cash:process-payment Card deleted");
          }
        } else { // transaction has been accepted
          $logger->log("flo2cash:process-payment Transaction accepted");

          $transaction->status = CCTransaction::STATUS_ACCEPTED;
          $transaction->save();

          $logger->log("flo2cash:process-payment Transaction saved");

          $transaction->Payment->status = Payments::STATUS_ACCEPTED;
          $transaction->Payment->save();

          $logger->log("flo2cash:process-payment Payment saved: #" . $payment->Id);

          // Should save card?
          if ($should_save_card) {
            $logger->log("flo2cash:process-payment Card saved: #" . $result->transactionresult->CardToken);
            $card->card_type = -$card->card_type - 1; // -1, -2, ... instead 0, 1, ...
            $card->expiry = null;
            $card->holder_name = $result->transactionresult->CardToken;
            $card->number = '**** ' . substr($card->number, -4);
            $card->save();

            $logger->log("flo2cash:process-payment Card saved with token: ".$card->holder_name.", #".$card->number);
          } else if (!$card_already_saved) { // remove card
            $logger->log("flo2cash:process-payment Card NOT saved");
            $transaction->credit_card_id = null;
            $transaction->save();
            $card->delete();

            $logger->log("flo2cash:process-payment Card deleted");
          }

        }

        // if transaction has been accepted/declined
        $logger->log("flo2cash:process-payment Transaction end");
        $little_log = '/var/tmp/payment.log';
        if (is_file($little_log)) {
          $lg = fopen($little_log, 'a');
          $time = date("d-m-y (h:i:s)");
          $log_string = "\n" . $time . " || acc_id: " . $account_id . " || amount: " . $amount . " || status: ".$status."\n";
          fwrite($lg, $log_string);
          fclose($lg);
        }
        unlink($lock_filename); // delete lock file
	}
}