<?php

class checkstatsTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'check-stats';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [check-stats|INFO] task does things.
Call it with:

  [php symfony check-stats|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }
  private $conn;

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    $this->conn = $databaseManager->getDatabase('doctrine')->getConnection();
    $finder = new TcStats($this->conn);

    foreach (array(1, 2) as $country)
    {
      $owners = $finder->findTop5($country, 'monthly');

      foreach ($owners as $owner)
      {
        $listings = $this->getListings($owner->user_id);
        $requests = $this->getRequests($owner->user_id);
        $relos    = $this->getRelos($owner->user_id);

        $stats = array(
          'listings' => array(),
          'requests' => array(),
          'relos' => array(),
        );

        $this->logSection('listings', 'computing stats for '.$owner->user_id);

        foreach ($listings as $i => $listing)
        {
          $this->addToStats($stats, 'listings', $listing->created_at, 1);
          $listing->free(true);
        }

        foreach ($requests as $i => $request)
        {
          $this->addToStats($stats, 'requests', $request->created_at, 1);
          $request->free(true);
        }

        foreach ($relos as $i => $relo)
        {
          $this->addToStats($stats, 'relos', substr($relo->when_accepted, 0, 10), 1);
          $relo->free();
        }

        $this->checkStats($owner, $stats);
      }
    }

  }

  protected function checkStats($tcStat, $stats)
  {

    $query = 'SELECT * from tc_listing_stats
          where user_id = ?
          and week = ?
          and day = ?
          and month = ?
          and year = ?';

    $query = $this->conn->prepare($query);

    foreach ($stats as $type => $stat) {
      foreach ($stat as $key => $nb) {
        $date = new DateTime($key);

        $query->execute(array(
          $tcStat->user_id,
          $date->format('W'),
          $date->format('j'),
          $date->format('n'),
          $date->format('Y')
        ));

        $result = $query->fetch();

        if ($result['nb_'.$type] != $nb) {
          $this->logSection($type,
            sprintf("Invalid stat for day %s. Expected: %s, got %s", $key, $nb, $result['nb_'.$type]),
            null, 'ERROR'
          );
        }
      }
    }
  }

  protected function addToStats(&$stats, $key, $date, $nb)
  {
    if (!isset($stats[$key][$date])) {
      $stats[$key][$date] = 0;
    }

    $stats[$key][$date] += $nb;
  }

  protected function getListings($ownerId)
  {
    $this->logSection('listings', 'getting listings for '.$ownerId);

    return Doctrine::getTable('Listing')->createQuery('l')
      ->where('l.owner_id = ?', $ownerId)
      ->addWhere('l.created_at = ?', '2013-01-01')
      ->execute();
  }

  protected function getRequests($ownerId)
  {
    $this->logSection('requests', 'getting requests for '.$ownerId);

    return Doctrine::getTable('Request')->createQuery('r')
      ->innerJoin('r.Listing l')
      ->where('l.owner_id = ?', $ownerId)
      ->addWhere('r.created_at > ?', '2013-01-01')
      ->execute();
  }

  protected function getRelos($ownerId)
  {
    $this->logSection('relos', 'getting relos for '.$ownerId);

    return Doctrine::getTable('Request')->createQuery('r')
      ->innerJoin('r.Listing l')
      ->where('l.owner_id = ?', $ownerId)
      ->whereIn('r.status', array(1, 4))
      ->addWhere('r.when_accepted = ?', '2013-01-01')
      ->execute();
  }
}