<?php

class listingGenerateTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'listing';
    $this->name             = 'generate';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [listing:generate|INFO] task does things.
Call it with:

  [php symfony listing:generate|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::REQUIRED, 'The application name');
    $this->addArgument('owner', sfCommandArgument::REQUIRED, 'The owner username');
    $this->addArgument('driver', sfCommandArgument::REQUIRED, 'The driver username');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    ini_set('error_reporting', E_ALL|E_STRICT);
    ini_set('display_errors', true);
    $this->cleanupListings();
    $listings = $this->generateListings($arguments['owner']);
    $requests = $this->generateRequests($listings, $arguments['driver']);
  }

  private function generateListings($owner)
  {
    $ownerInstance = Doctrine::getTable('User')->findOneByUsername($owner);
    if (!($ownerInstance->exists())) {
      throw new InvalidArgumentException("Owner $owner not found");
    }

    $listings = array();
    $vehicleType = Doctrine::getTable('VehicleType')->createQuery()->limit(1)->fetchOne();

    for ($i = 0; $i < 10; $i++) {
      $pickup_date  = date('Y-m-d', strtotime('+4 days') + (rand(0, 3) * 86400));
      $dropoff_date = date('Y-m-d', strtotime('+14 days') + (rand(0, 3) * 86400));
      $pickup = $this->getLocation();
      $dropoff = $this->getLocation($pickup->getId());

      $listing = new Listing();
      $listing->set('date_available_from', $pickup_date);
      $listing->set('date_return_by', $dropoff_date);
      $listing->set('PickupLocation', $pickup);
      $listing->set('DropoffLocation', $dropoff);
      $listing->set('days_allowed', 6);
      $listing->set('paid_days_allowed', 3);
      $listing->set('price_per_day', 42);
      $listing->price = Doctrine::getTable('Prices')->getPriceFor(
        $listing->get('pickup_location_id'),
        $listing->get('dropoff_location_id'),
        $listing->get('vehicle_type_id')
      );
      $listing->set('VehicleType', $vehicleType);
      $listing->set('Owner', $ownerInstance);
      $listing->set('ListedBy', $ownerInstance);
      $listing->set('comment', '@GENERATED@');
      $listing->set('status', Listing::STATUS_NEW);
      $listing->set('created_at', date('Y-m-d'));
      $listing->set('updated_at', date('Y-m-d'));
      $listing->save();

      $listings[] = $listing;
    }

    return $listings;
  }

  private function generateRequests($listings)
  {
    $drivers = array();
    for ($i = 0; $i < 5; $i++) {
      $driver = $this->getDriver(array_keys($drivers));
      $drivers[$driver->getId()] = $driver;
    }

    $drivers = array_values($drivers);

    for ($i = 0; $i < 10; $i++) {
      $driver = $drivers[mt_rand(0, count($drivers) - 1)];
      $listing = $listings[mt_rand(0, count($listings) - 1)];
      $request = new Request();
      $request->set('Driver', $driver);
      $request->set('Listing', $listing);
      $request->set('status', Request::STATUS_PENDING);
      $request->set('pickup_date', $listing->get('date_available_from'));
      $nb_paid_days = mt_rand(0, 3);
      $nb_days = 3 + $nb_paid_days;
      $dropoff_time =  strtotime('+'.$nb_days.' days');
      $request->set('drop_off_date', date('Y-m-d', strtotime('+'.$nb_days.' days', strtotime($listing->get('date_available_from')))));
      $request->set('nb_free_days', 3);
      $request->set('nb_paid_days', $nb_paid_days);
      $request->set('created_at', date('Y-m-d'));
      $request->save();
    }
  }

  private function getDriver($ids = null)
  {
    $query = Doctrine::getTable('User')
      ->createQuery('u')
      ->where('u.country_domain_id = 1')
      ->addWhere('u.is_owner <> 1');

    if ($ids) {
      $query->whereNotIn('u.id', $ids);
    }

    $countQuery = clone $query;
    $count = $countQuery->count();

    return $query
      ->limit(1)
      ->offset(rand(0, $count - 1))
      ->fetchOne();
  }


  private function getLocation($id = null)
  {
    $query = Doctrine::getTable('Locations')
      ->createQuery('l')
      ->innerJoin('l.Category c')
      ->where('c.country_id = 1');

    if ($id) {
      $query->addWhere('l.id <> ?', $id);
    }

    $countQuery = clone $query;
    $count = $countQuery->count();

    return $query
      ->limit(1)
      ->offset(rand(0, $count - 1))
      ->fetchOne();
  }

  private function cleanupListings()
  {
    Doctrine_Query::create()
      ->delete()
      ->from('Listing')
      ->where('comment = ?', '@GENERATED@')
      ->execute()
    ;
  }
}