<?php

class transfercarAbglocationsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'abg-locations';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:abg-locations|INFO] task does things.
Call it with:

  [php symfony transfercar:abg-locations|INFO]
EOF;
    $this->addArgument('file', sfCommandArgument::REQUIRED, 'The abg file');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    error_reporting(E_ALL);
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    // add code here
    $ros = [
      'NQLD' => 232255, // north queensland
      'NSW' => 232256, // new south wales
      'SQLD' => 212056, // sout queensland
      'MEL' => 232259, // victoria
      'WA' => 232332, // western australia + Karratha
      'Karratha' => 232332, // 
      //'NT' => false, // nothern territories
      'SA' => 232257, // south australia
      'TAS' => 232258, // tasmania
    ];

    foreach ($ros as $name => $id) {
      $this->log(str_repeat('-', 80));
      $this->logSection('import', $name);
      $this->log(str_repeat('-', 80));
      $this->import($name, $id, $arguments['file']);
    }  
  }

  private function import($name, $id, $file) 
  {
    $file = fopen(sfConfig::get('sf_root_dir').'/../'.$file, 'rb');

    while ($row = fgetcsv($file, 2048,',')) {
      if ($row[1] == 'Address') {
        continue;
      }

      $this->logSection('import', $row[0]);

      $branch = new OwnerBranch();
      $branch->set('owner_id', $id);
      $branch->set('internal_name', $row[0]);
      $branch->set('name', $row[0]);
      $branch->set('description', $row[0]);
      $branch->set('address', $row[1]);
      $branch->set('opening_hours', $row[2]);
      $branch->set('country_id', 2);
      $location = $this->getLocationId(trim($row[0]));
      $branch->set('latitude', $location->get('lat'));
      $branch->set('longitude', $location->get('lon'));
      $times = $this->parseTimes($row[2]);
      $branch->set('default_pickup_time', $times[0]);
      $branch->set('latest_pickup_time', $times[1]);
      $branch->set('default_dropoff_time', $times[0]);
      $branch->set('latest_dropoff_time', $times[1]);
      $branch->set('location_id', $location->getId());
      $contact = Doctrine::getTable('OwnerContact')->createQuery()->where('owner_id = ?', $id)->fetchOne();
      $branch->set('contact_id', $contact->getId());
      $branch->save();
    }
  }

  private function parseTimes($opening_hours)
  {
    if (!preg_match_all('#[0-9]{1,2}:[0-9]{1,2} [AP]M#', $opening_hours, $matches))
    {
      throw new Exception("Error parsing times $opening_hours", 1);
    }
    $times = [];
    foreach ($matches[0] as $value) {
      $times[] = strtotime($value) - strtotime(date('Y-m-d'));
    }
    sort($times);

    return [$times[0], $times[count($times)-1]];
    
  }

  private function getLocationId($name) 
  {
    $nearest = [
      'Kings Cross' => 'Sydney',
      'Melbourne International Airport' => 'Melbourne Airport',
      'Melbourne City' => 'Melbourne',
      'Adelaide City' => 'Adelaide',
      'Bunderberg Airport' => 'Hervey Bay',
      'Ayers rock Connellan Airport' => 'Ayers Rock (Uluru) Airport (AYQ)',
      'Ayers Rock Resort' => 'Ayers Rock (Uluru)',
      'Wyndyard Airport' => 'Burnie Wynyard Airport',
      'Launceston Hotel Grand Chancellor' => 'Launceston',
      'Kalgoorlie Airport' => 'Kalgoorlie-Boulder Airport',
    ];

    $location = Doctrine::getTable('Locations')
      ->createQuery('l')
      ->innerJoin('l.Category c')
      ->where('c.country_id = ?', 2)
      ->addWhere('l.name = ?', isset($nearest[$name])?$nearest[$name]:$name)
      ->limit(1)
      ->fetchOne();


    if (!$location) {

      if (stripos($name, 'downtown') !== false || stripos($name, 'city') !== false) {
        return $this->getLocationId(trim(str_ireplace([' city', ' downtown'], '', $name)));
      }
      throw new Exception("Invalid location ".(isset($nearest[$name])?$nearest[$name]:$name), 1);
    }

    return $location;
    
  }  
}
