<?php

use Transfercar\Alert\AlertEvents;
use PhpAmqpLib\Connection\AMQPConnection;
use PhpAmqpLib\Message\AMQPMessage;
use Transfercar\PhpAmqpLib\tcRabbitmq;

class transfercarCreateAlertsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'create-alerts';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:create-alerts|INFO] task does things.
Call it with:

  [php symfony transfercar:create-alerts|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    $this->addOption('timeout', null, sfCommandOption::PARAMETER_REQUIRED, 'Timeout', 300);
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    error_reporting(E_ALL);
    ini_set('display_errors', 1);
    $_SERVER['TRANSFER_CAR_COUNTRY'] = 'nz';

    // // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);


    $this->logSection('queue', 'Connection to AMQP server');
    $conn = new tcRabbitmq(
      sfConfig::get('app_amqp_host'),
      sfConfig::get('app_amqp_port'),
      sfConfig::get('app_amqp_username'),
      sfConfig::get('app_amqp_password'),
      sfConfig::get('app_amqp_path')
    );

    $channel = $conn->getChannel('listing_created');
    $channel
        ->basic_consume(
          'listing_created',
          $this->getFullName(),
          false,
          false,
          false,
          false,
          array($this, 'process_message')
    );

    $this->logSection('queue', 'Reading listing_created queue with timeout '.$options['timeout']);
    try
    {
      while (count($channel->callbacks)) {
        $channel->wait(null, false, $options['timeout']);
      }
    } catch (PhpAmqpLib\Exception\AMQPTimeoutException $ex)
    {
      // timeout exceeds, closing task to avoid memory leaks
    }
  }

  public function process_message($msg)
  {
    $this->logSection('memory start', $this->convert(memory_get_usage()));
    $listing = Doctrine::getTable('Listing')->find($msg->body);

    if ($listing && $listing->getStatus() == Listing::STATUS_NEW)
    {
      $this->log('notify '.$listing->getId());
    $this->logSection('memory before create', $this->convert(memory_get_usage()));
      AlertEvents::createNotification($listing);
    $this->logSection('memory after create', $this->convert(memory_get_usage()));
    }

    $listing->free(true);
    unset($listing);


    $this->logSection('memory free', $this->convert(memory_get_usage()));
    // ack message to remove it from the queue
    $msg
      ->delivery_info['channel']
      ->basic_ack($msg->delivery_info['delivery_tag']);


    $this->logSection('memory end', $this->convert(memory_get_usage()));
  }

  private function convert($size)
  {
     $unit=array('b','kb','mb','gb','tb','pb');
     return @round($size/pow(1024,($i=floor(log($size,1024)))),2).' '.$unit[$i];
  }
}



















