<?php

class transfercarExpireslistingsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'expire-listings';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:expires-listings|INFO] task does things.
Call it with:

  [php symfony transfercar:expire-listings|INFO]
EOF;
    $this->addArgument('country_id', sfCommandArgument::REQUIRED, 'The country Id');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');

    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);

    $listings = Doctrine::getTable('Listing')
      ->createQuery('l')
      ->where('l.status = ?', Listing::STATUS_NEW)
      ->addWhere('l.date_return_by < ?', date('Y-m-d'))
      ->execute();

    foreach ($listings as $listing)
    {
      $listing->expires();
    }

    $this->logSection('transfercar', count($listings).' listings expired');

    $expiration = 24;
    $force = true;

    if ($arguments['country_id'] == 3) // longer expiration in the US
    {
      switch (date('N')) {
        case '1': // friday
        case '6': // friday
        case '7': // friday
          $force = false;
          break;
      }
    }

    $requests = Doctrine::getTable('Request')
      ->createQuery('r')
      ->where('r.status = ?', Request::STATUS_PENDING)
      ->innerJoin('r.Listing l')
      ->innerJoin('l.PickupLocation pl')
      ->innerJoin('pl.Category lpC')
      ->addWhere('(hour(timediff(r.created_at, now())) >= ? or DATE_ADD(r.pickup_date, \'INTERVAL r.pickup_time SECOND\') <= now())', $expiration)
      ->addWhere('lpC.country_id = ?', $arguments['country_id'])
      ->execute();

    foreach ($requests as $request)
    {
      $request->expires($force);
    }

    $this->logSection('transfercar', count($requests).' requests expired');

  }
}