<?php

class transfercarGeneratesitemapTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'generate-sitemap';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:generate-sitemap|INFO] task does things.
Call it with:

  [php symfony transfercar:generate-sitemap|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::REQUIRED, 'The application name');
    $this->addArgument('domain', sfCommandArgument::REQUIRED, 'The domain name');
    $this->addArgument('host', sfCommandArgument::REQUIRED, 'The host name');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    sfContext::createInstance($this->configuration);
    sfLoader::loadHelpers(array('Url'));
    $_SERVER['HTTP_HOST']   = $arguments['host'];
    $_SERVER['HTTPS']       = 'on';
    $_SERVER['SCRIPT_NAME'] = '';
    sfConfig::set('sf_no_script_name', true);

    file_put_contents(sfConfig::get('sf_app_cache_dir').'/'.$arguments['domain'].'.sitemap.xml', $this->generateSitemap($arguments['domain']));
  }

  protected function generateSitemap($dom)
  {
    $links = array(
      'listing/browse',
      'user/myTransfer'
    );

    $listings = Doctrine_Query::create()
      ->from("Listing l")
      ->innerJoin("l.DropoffLocation dl")
      ->innerJoin("dl.Category dlc")
      ->innerJoin("dlc.Country dlcc")
      ->where("l.status = ?", Listing::STATUS_NEW)
      ->addWhere("dlcc.domain = ?", $dom);
    //var_dump ($listings->getSqlQuery());
    $listings = $listings->execute();

    foreach($listings as $listing) {
      $links[] = $listing->getRelocationQuery('listing/view');
      if ($listing->hasTerms()) {
        $links[] = "user/terms?id={$listing->terms_id}";
      }
      if ($listing->hasExtras()) {
        //$links[] = "listing/addons?id={$listing->id}";
      }
    }

    $links2 = array();

    $pages = Doctrine::getTable('CMSPage')
      ->createQuery('p')
      ->where('p.domain = ?', $dom)
      ->addWhere('p.is_published = ?', true)
      ->execute();

    foreach ($pages as $page)
    {
      $links2[] = 'https://'.$_SERVER['HTTP_HOST'].$page->getFullUrl();
    }

    $links2[] = 'https://'.$_SERVER['HTTP_HOST'].'/blog';
    $links2[] = 'https://'.$_SERVER['HTTP_HOST'].'/help';

    return $this->buildSitemap($links, $links2);
  }

  protected function buildSitemap($links, $links2)
  {
    $tpl = <<<EOF
<url>
    <loc>%s</loc>
    <priority>0.5</priority>
    <changefreq>daily</changefreq>
</url>
EOF;

    $nodes = array();

    foreach ($links as $link)
    {
      $nodes[] = sprintf($tpl, urldecode(url_for($link, true)));
    }

    foreach ($links2 as $link)
    {
      $nodes[] = sprintf($tpl, urldecode($link));
    }

    $layout = <<<EOF
<?xml version="1.0" encoding="utf-8" standalone="no" ?>
<urlset
      xmlns="http://www.sitemaps.org/schemas/sitemap/0.9"
      xmlns:xhtml="http://www.w3.org/1999/xhtml"
      xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9
             http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd">

      %s
</urlset>
EOF;

    return sprintf($layout, implode("\n", $nodes));
  }
}