<?php

class transfercarImportcartrawleradsTask extends sfPropelBaseTask
{
  private $importing = 0, $imported = 0, $advertiser, $start_time, $start_ts, $ip;
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'import-cartrawler-ads';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:import-cartrawler-ads|INFO] task does things.
Call it with:

  [php symfony transfercar:import-cartrawler-ads|INFO]
EOF;
    $this->addArgument('advertiser_username', sfCommandArgument::REQUIRED, 'Advertiser username');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }


  protected function execute($arguments = array(), $options = array())
  {
    $record = dns_get_record(sfConfig::get('app_domain_nz'), DNS_A);
    $this->ip = $record[0]['ip'];

    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    $locations = CarTrawlerLocation::getNzRoutes();

    if (!$advertiser = Doctrine::getTable('Advertiser')->findOneByUsername($arguments['advertiser_username']))
    {
      throw new InvalidArgumentException("Advertiser account not found");
    }

    $this->advertiser = $advertiser;

    $gateway = new CarTrawlerGateway($this);
    error_reporting(E_ALL);
    ini_set('display_errors', 1);

    $requests = [];

    foreach ($locations as $key => $dropoffs)
    {
      foreach ($dropoffs as $key2)
      {
          $requests[] = $this->importAds($gateway, $key, $key2, date('Y-m-d\T10:00:00', strtotime('+2 days')), date('Y-m-d\T10:00:00', strtotime('+9 days')));
      }
    }

    $this->importing = count($requests);
    $this->logSection('import', sprintf('%s API calls pending', $this->importing));

    $pool = $gateway->build_pool($requests, array($this, 'importVehicles'));

    $this->logSection('waiting', "...");

    $this->conn = Doctrine::getTable('AdListing')->getConnection();
    $this->conn->prepare('SET AUTOCOMMIT=0')->execute();
    $this->start_time = date('Y-m-d H:i:s');
    $this->start_ts = time();
    $pool->wait();


    $this->logSection('import', sprintf('Imported %s ads', $this->imported));
    $this->logSection('cleanup', "Deleting not updated listings");
    $this->cleanupAds();
  }

  private function importAds($gateway, $pickup, $dropoff, $pickup_datetime, $dropoff_datetime)
  {

    return $gateway->find_vehicles($pickup, $pickup_datetime, $dropoff, $dropoff_datetime, 30, 'NZ', $this->ip, 'NZD', 1, array($this, 'importVehicles'));
  }

  public function importVehicles($vehicles)
  {
    $this->importing--;
    $this->logSection('import', sprintf('%s API calls pending', $this->importing));
    $this->logSection('import-callback', 'importing '.count($vehicles));

    $this->conn->beginTransaction();

    foreach ($vehicles as $vehicle){
      $adListing = $this->getAdListingOrCreate($vehicle);
      $adListing->set('advertiser_id', $this->advertiser->getId());
      $adListing->set('status', 0);
      $adListing->set('vehicle_type_id', $this->mapVehicleType($vehicle['vehicle_type']));
      $adListing->set('transmission_type', $vehicle['transmission']);
      $this->logSection('travellers', intval($vehicle['passengers_quantity']));
      $adListing->set('travellers', intval($vehicle['passengers_quantity']));
      $adListing->set('price', $vehicle['total_charge']);
      $adListing->set('destination_url', $vehicle['reference']['url']);
      $adListing->set('image', $vehicle['picture_url']);
      $adListing->set('country_id', $this->advertiser->get('country_id'));
      $adListing->save($this->conn);

      $this->imported++;
    }

    $this->conn->commit();
    $ts = time() - $this->start_ts;
    $this->logSection('import', 60 - $ts + 1);
    // CT firewalls start blocking IPs from 3 requests per 90s. By security, let's do only 1 per minute
    sleep(60 - $ts + 1);
    $this->start_ts = time();
  }

  private function getAdListingOrCreate($vehicle)
  {
    if (!$adListing = Doctrine::getTable('AdListing')->findOneByReference('cartrawler_import_'.$vehicle['reference']['uniqid']))
    {
      $adListing = new AdListing();
      $adListing->set('reference', 'cartrawler_import_'.$vehicle['reference']['uniqid']);

      $route = $this->getOrCreateRoute($vehicle['pickup_location_id'], $vehicle['return_location_id']);

      $adListingRoutes = new AdListingRoutes();
      $adListingRoutes->set('AdListingRoute', $route);
      $adListing->get('AdListingRoutes')->add($adListingRoutes);
    }

    return $adListing;
  }

  private function getOrCreateRoute($pickup, $dropoff)
  {
    $pickup = $this->map($pickup);
    $dropoff = $this->map($dropoff);

    $route = Doctrine::getTable('AdListingRoute')->createQuery()
      ->where('pickup_location_id = ?',  $pickup)
      ->addWhere('dropoff_location_id = ?', $dropoff)
      ->limit(1)
      ->fetchOne();

    if (!$route) {
      $route = new AdListingRoute();
      $route->set('pickup_location_id', $pickup);
      $route->set('dropoff_location_id', $dropoff);
    }

    return $route;
  }

  private function map($locationId)
  {
    $map = Doctrine::getTable('AdvertiserLocationMap')
      ->createQuery('m')
      ->where('advertiser_id = ?', $this->advertiser->getId())
      ->addWhere('advertiser_location_id = ?', $locationId)
      ->limit(1)
      ->fetchOne();

    if (!$map) {
      throw new RuntimeException("Unable to map $locationId", 1);
    }

    return $map->get('tc_location_node_id');
  }

  private function mapVehicleType($id)
  {
    // $ctTypes = array(
    //   1  => 'Mini',
    //   3  => 'Economy ',
    //   4  => 'Compact ',
    //   6  => 'Intermediate ',
    //   7  => 'Standard ',
    //   8  => 'Fullsize',
    //   9  => 'Luxury ',
    //   10 => 'Premium ',
    //   11 => 'Minivan ',
    //   12 => 'passenger van ',
    //   24 => 'Exotic   ',
    // );

    $tcTypes = array(
      1 => 8,   // hatchback 2 seats
      3 => 10,  // sedan
      4 => 11,  // hatchback 4 seats
      6 => 10,  // sedan 4 seats
      8 => 13,  // sedan 5 seats
      9 => 30,  // other
      10 => 30, // other
      11 => 28, // van 3 seats
      12 => 29, // van 5+ seats,
      24 => 30, // other
    );

    return isset($tcTypes[$id]) ? $tcTypes[$id] : 30;
  }


  private function cleanupAds()
  {
    $this->conn->beginTransaction();
    $nb = Doctrine_Query::create($this->conn)
      ->delete()
      ->from('AdListing')
      // ->set('status', AdListing::DELETED)
      ->where('updated_at < ?', $this->start_time)
      ->addWhere('advertiser_id = ?', $this->advertiser->getId())
      ->addWhere('status != ?', AdListing::LISTED)
      ->execute();

    $this->conn->commit();

    $this->logSection('cleanup', 'Removed '.$nb.' listings');
  }
}




