<?php

use Symfony\Component\DomCrawler\Crawler;

class transfercarImportrentalcarscomadsTask extends sfPropelBaseTask
{
  private $start_time, $advertiser, $imported, $conn;

  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'import-rentalcarscom-ads';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:import-rentalcarscom-ads|INFO] task does things.
Call it with:

  [php symfony transfercar:import-rentalcarscom-ads|INFO]
EOF;
    $this->addArgument('advertiser_username', sfCommandArgument::REQUIRED, 'Advertiser username');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'propel');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    $gateway = new RentalcarsComGateway($this);

    error_reporting(E_ALL);
    ini_set('display_errors', 1);

    if (!$advertiser = Doctrine::getTable('Advertiser')->findOneByUsername($arguments['advertiser_username']))
    {
      throw new InvalidArgumentException("Advertiser account not found");
    }

    $this->conn = Doctrine::getTable('AdListing')->getConnection();
    $this->conn->prepare('SET AUTOCOMMIT=0')->execute();

    $this->advertiser = $advertiser;

    $locations = RentalcarsComLocation::getNzLocations($this->advertiser->getId());
    $this->ip = '127.0.0.1';
    $requests = [];
    foreach ($locations as $pickup)
    {
      foreach ($locations as $dropoff)
      {
        if ($pickup != $dropoff)
        {
          // $requests[] = $gateway->find_vehicles($pickup, date('Y-m-d 10:00:00', strtotime('+2 days')), $dropoff, date('Y-m-d\T10:00:00', strtotime('+9 days')), 30, '198.132.90.122', 'NZD', array($this, 'importVehicles'));
          $requests[] = $gateway->find_vehicles($pickup, date('Y-m-d\T10:00:00', strtotime('+2 days')), $dropoff, date('Y-m-d\T10:00:00', strtotime('+9 days')), 30, 'NZ', $this->ip, 'NZD', 1, array($this, 'importVehicles'));
          // $requests[] = $this->importAds($gateway, $key, $key2, date('Y-m-d\T10:00:00', strtotime('+2 days')), date('Y-m-d\T10:00:00', strtotime('+9 days')));
        }
      }
    }

    $this->logSection('import', sprintf('%s API calls pending', count($requests)));

    $pool = $gateway->build_pool($requests, array($this, 'importVehicles'));

    $this->imported = 0;
    $this->start_time = date('Y-m-d H:i:s');
    $this->logSection('importing', "...");
    $pool->wait();

    $this->logSection('import', sprintf('Imported %s ads', $this->imported));
    $this->logSection('cleanup', "Deleting not updated listings");
    $this->cleanupAds();
  }


  public function importVehicles($vehicles)
  {
    $this->logSection('import-callback', 'importing '.count($vehicles));

    $this->imported += count($vehicles);

    $this->conn->beginTransaction();

    foreach ($vehicles as $vehicle){
      try {
        $adListing = $this->getAdListingOrCreate($vehicle);

        $adListing->set('advertiser_id', $this->advertiser->getId());
        $adListing->set('status', 0);
        $adListing->set('vehicle_type_id', $this->mapVehicleType($vehicle['passengers_quantity']));
        $adListing->set('transmission_type', $vehicle['transmission']);
        $adListing->set('travellers', intval($vehicle['passengers_quantity']));
        $adListing->set('price', $vehicle['total_charge']);
        $adListing->set('destination_url', $vehicle['reference']['url']);
        $adListing->set('image', $vehicle['picture_url']);
        $adListing->set('country_id', $this->advertiser->get('country_id'));

        $adListing->save($this->conn);
      } catch (RuntimeException $e){
        $this->logSection('error', 'Unable to import vehicle '.$vehicle['reference']['id'].': '.$e->getMessage(), 1024, 'ERROR');
      } catch (Exception $e) {
        $this->logSection('error', 'Unable to import vehicle '.$vehicle['reference']['id'].': '.$e->getMessage(), 1024, 'ERROR');
      }
    }

    $this->conn->commit();
  }

  private function getAdListingOrCreate($vehicle)
  {
    if (!$adListing = Doctrine::getTable('AdListing')->findOneByReference('rental_cars_'.$vehicle['reference']['id']))
    {
      $adListing = new AdListing();
      $adListing->set('reference', 'rental_cars_'.$vehicle['reference']['id']);
      $route = $this->getOrCreateRoute($vehicle['pickup_location_id'], $vehicle['return_location_id']);
      $adListingRoutes = new AdListingRoutes();
      $adListingRoutes->set('AdListingRoute', $route);
      $adListing->get('AdListingRoutes')->add($adListingRoutes);
    }

    return $adListing;
  }

  private function getOrCreateRoute($pickup, $dropoff)
  {
    $pickup = $this->map($pickup);
    $dropoff = $this->map($dropoff);

    $route = Doctrine::getTable('AdListingRoute')->createQuery()
      ->where('pickup_location_id = ?',  $pickup)
      ->addWhere('dropoff_location_id = ?', $dropoff)
      ->limit(1)
      ->fetchOne();

    if (!$route) {
      $route = new AdListingRoute();
      $route->set('pickup_location_id', $pickup);
      $route->set('dropoff_location_id', $dropoff);
    }

    return $route;
  }

  private function map($locationId)
  {
    $map = Doctrine::getTable('AdvertiserLocationMap')
      ->createQuery('m')
      ->where('advertiser_id = ?', $this->advertiser->getId())
      ->addWhere('advertiser_location_id = ?', $locationId)
      ->limit(1)
      ->fetchOne();

    if (!$map) {
      throw new RuntimeException("Unable to map $locationId ".$this->advertiser->getId(), 1);
    }

    return $map->get('tc_location_node_id');
  }

  private function mapVehicleType($id)
  {
    static $vehicles;

    if (!$vehicles){
      $vehicles = Doctrine::getTable('VehicleType')
        ->createQuery()
        ->where('description like ?', 'Motorhome%')
        ->addWhere('country_id = ?', 1)
        ->execute();
    }

    foreach ($vehicles as $vehicle) {
      if ($vehicle->get('travellers') == $id) {
        return $vehicle->getId();
      }
    }

    return $this->mapVehicleType(5);
  }

  private function cleanupAds()
  {
    $this->conn->beginTransaction();
    $nb = Doctrine_Query::create($this->conn)
      ->update('AdListing')
      ->set('status', AdListing::DELETED)
      ->where('updated_at < ?', $this->start_time)
      ->addWhere('advertiser_id = ?', $this->advertiser->getId())
      ->addWhere('status = ?', AdListing::LISTED)
      ->execute();

    $this->conn->commit();

    $this->logSection('cleanup', 'Removed '.$nb.' listings');
  }
}