<?php


use Symfony\Component\DomCrawler\Crawler;

class transfercarImportrentalcarscomlocationsTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'import-rentalcarscom-locations';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:import-rentalcarscom-locations|INFO] task does things.
Call it with:

  [php symfony transfercar:import-rentalcarscom-locations|INFO]
EOF;

    $this->addArgument('advertiser_username', sfCommandArgument::REQUIRED, 'Advertiser username');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'propel');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    error_reporting(E_ALL);

    if (!$advertiser = Doctrine::getTable('Advertiser')->findOneByUsername($arguments['advertiser_username']))
    {
      throw new InvalidArgumentException("Advertiser account not found");
    }
    $this->advertiser = $advertiser;

    $gateway   = new RentalcarsComGateway($this);

    $xml = $gateway->findLocations();
    $crawler = new Crawler((string) $xml);

    $this->loadCountry($crawler, 'NZ');
    $this->loadCountry($crawler, 'US');
    $this->loadCountry($crawler, 'AU');
  }

  private function loadCountry($crawler, $countryCode)
  {
    $cities = $crawler->filterXPath('//Location[descendant::CountryName[@Code="'.$countryCode.'"]]');

    $this->logSection('cities', count($cities));
    foreach ($cities as $city) {
      $cityName = $city->getAttribute('Name');

      $this->logSection('cities', $cityName);
      if ($locs = $this->findLocations(trim($cityName)))
      {
        $this->logSection('count', count($locs));
        foreach ($locs as $loc)
        {
          $node = Doctrine::getTable('LocationNode')->findByTypeAndId('location', $loc->getId());
          $this->createLocationMap($this->advertiser->getId(), $city->getAttribute('Code'), $loc->getId());
          $this->logSection('location', $cityName);
        }
      }
    }
  }


  private function createLocationMap($advertiser_id, $advertiser_location_id, $location_id)
  {
    try {
      $map = new AdvertiserLocationMap();
      $map->set('advertiser_id', $advertiser_id);
      $map->set('advertiser_location_id', $advertiser_location_id);
      $map->set('tc_location_node_id', $location_id);
      $map->save();
      $this->logSection('added', 'locations added '.$location_id);
    } catch (Exception $e) {
      $this->logSection('error', 'locations exists '.$location_id);
    }
  }

  private function findLocations($value)
  {
    $this->logSection('location', 'looking for '.$value);
    $value = str_replace(' Downtown', '', $value);

    $locs = Doctrine::getTable('LocationNode')->findByName(str_replace(' - ', ' ', $value));

    if (!$locs->count()) {
      // try without the "- XXX" suffix
      $name = preg_replace('/ - .*+$/i', '', $value);
      $locs = Doctrine::getTable('LocationNode')->findByName($name);

      // try by append " City"
      if (!$locs->count()) {
        $locs = Doctrine::getTable('LocationNode')->findByName($name.' City');


        if (strpos($value, 'Airport') !== false) {
          return $this->findLocations(str_replace(' Airport', '', $value));
        }
        if (!$locs->count()) {
          $this->logSection('not found', $value.' not found in our db: ');
        }
      }
    }

    return $locs;
  }
}