<?php

class transfercarInitlocationnodesTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'init-location-nodes';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:init-location-nodes|INFO] task does things.
Call it with:

  [php symfony transfercar:init-location-nodes|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    $conn = Doctrine_Manager::getInstance()->getCurrentConnection();

    $conn->beginTransaction();
    //
    $countries = Doctrine::getTable('LocationCountry')->findAll();
    $departureNodes = [];

    foreach ($countries as $country) {
      $countryNode = $this->createNode($conn, 'country', $country);

      foreach ($country->get('Departures') as $departure) {
        $departureNode = $this->createNode($conn, 'departure', $departure);
        $departureNodes[$departure->getId()] = $departureNode;
        $this->createAncestorNode($conn, $countryNode, $departureNode);
      }

      foreach ($country->get('Categories') as $category) {
        $categoryNode = $this->createNode($conn, 'category', $category);
        $this->createAncestorNode($conn, $countryNode, $categoryNode);

        foreach ($category->get('Locations') as $location) {
          $locationNode = $this->createNode($conn, 'location', $location);
          $this->createAncestorNode($conn, $categoryNode, $locationNode);
          $this->createAncestorNode($conn, $countryNode, $locationNode);

          foreach ($location->get('Departures') as $departure) {
            $departureNode = $departureNodes[$departure->getId()];
            $this->createAncestorNode($conn, $departureNode, $locationNode);
          }
        }
      }
    }

    $conn->commit();
  }

  private function createNode($conn, $type, $object)
  {
    $node = new LocationNode();
    $node->set('type', $type);
    $node->set('type_id', $object->getId());
    $node->set('name', (string) $object);
    $node->save($conn);

    return $node;
  }

  private function createAncestorNode($conn, $ancestorNode, $node)
  {
    $ancestor = new LocationNodeAncestor();
    $ancestor->set('node_id', $node->getId());
    $ancestor->set('node_ancestor_id', $ancestorNode->getId());
    $ancestor->save($conn);

    return $ancestor;
  }
}