<?php

class transfercarListingCompleteTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'complete-listing';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [listing:complete|INFO] task completes listing.
Call it with:

  [php symfony listing:complete|INFO]
EOF;
    $this->addArgument('country_id', sfCommandArgument::REQUIRED, 'The country_id');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    ini_set('error_reporting', E_ALL);
    ini_set('display_errors', 1);
    $listings = Doctrine::getTable('Listing')->findNonCompletedRelos($arguments['country_id']);
    $this->logSection('updating', count($listings).' listings found');
    foreach ($listings as $listing)
    {
      $request = $listing->getAcceptedRequest();

      if ($request->isPickupDate('today'))
      {
        $this->logSection('updating', $listing->getId().' is picked up today');
        $listing->set('status', Listing::STATUS_PICKED_UP);
        $listing->save();
      }
      elseif ($request->isCompletedSince('+1 day'))
      {
        $this->logSection('updating', $listing->getId().' is completed since 1 day');
        $listing->set('status', Listing::STATUS_DROPPED_OFF);
        $listing->save();
      }
      elseif ($request->isCompletedSince('+30 days') && $listing->get('Owner')->get('payment_cc'))
      {
        $request->rate(1, 'Positive rating', 'Completed automatically with a neutral rating');
        $request->close();
      }
    }
    $this->logSection('updated', 'done');
  }
}
