<?php

use Transfercar\Mandrill\TcMandrill;

class transfercarNotifyCovid19Task extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'notify-covid19';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:notify-new-listings|INFO] task does things.
Call it with:

  [php symfony transfercar:notify-new-listings|INFO]
EOF;

    $this->addArgument('country', sfCommandArgument::REQUIRED, 'The country code (nz, za, au, us)');
    $this->addArgument('template', sfCommandArgument::OPTIONAL, 'The application name', 'covid19_');
    $this->addArgument('user_file', sfCommandArgument::OPTIONAL, 'The application name', 'users_');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
  }

  protected function execute($arguments = array(), $options = array())
  {
    ini_set('memory_limit', '512M');
    set_time_limit(0);
    ini_set('display_errors', true);
    ini_set('error_reporting', (E_ALL|E_STRICT) & ~E_DEPRECATED);
    $_SERVER['TRANSFER_CAR_COUNTRY'] = $arguments['country'];

    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);
    sfConfig::set('app_domain_nz', 'www.transfercar.co.nz');
    sfConfig::set('app_domain_au', 'www.transfercar.com.au');
    sfConfig::set('app_domain_us', 'www.transfercarus.com');
    sfConfig::set('app_domain_za', 'www.transfercar.co.za');
    sfConfig::set('app_tc_env', 'prod');

    $mandrill = new TcMandrill($arguments['country']);
    $users = $this->getUsers($arguments['user_file'], $arguments['country']);
    foreach($users as $receivers) {
      $this->sendBatch($mandrill, $arguments['template'], $receivers, $arguments['country']);
    }
  }

  private function sendBatch($mandrill, $template, $receivers, $domain) {
    $this->logSection('send batch', count($receivers));
    $mergeVars = [];

    foreach ($receivers as $receiver) {
      # code...
      $mergeVars[] = [
        'rcpt' => $receiver['email'],
        'vars' => ['name' => isset($receiver['name']) ? $receiver['name'] : null]
      ];
    }

    $mandrill->sendTemplate(
      'COVID-19 update', 
      $receivers, 
      ['merge_vars' => $mergeVars], 
      $template.$domain);
  }

  private function getCountry($domain)
  {
        switch ($domain) {
            case 'nz':
                return 'New Zealand';


            case 'au':
                return 'Australia';

            case 'us':
                return 'North America';

            case 'za':
                return 'South Africa';
        }
    }


  private function getUsers($file, $domain)
  {
    $h = fopen(sfConfig::get('sf_root_dir').'/'.$file.$domain.'.csv', 'rb');
    $users = [];
    while ($row=fgetcsv($h, null, ',', '"')) {
        if (count($row) == 1) {
          $users[] = [
            'email' => $row[0]
          ];

        } else {
          $users[] = [
            'name' => $row[0],
            'email' => $row[1]
          ];
        }
    }
    fclose($h);

    $users[] = ['name' => 'Noel', 'email' => 'noel@transfercar.co.nz'];
#    $users[] = ['name' => 'Pascal', 'email' => 'pascal@transfercar.co.nz'];
    // $users[] = ['name' => 'Sharon', 'email' => 'sharon@transfercar.co.nz'];
    // $users[] = ['name' => 'Joana', 'email' => 'joana@transfercar.co.nz'];
    $users[] = ['name' => 'Brian', 'email' => 'brian@transfercar.co.nz'];

    print_r(count($users));

    return array_chunk($users, 100);
    return array_chunk([
      ['name' => 'Noel NZ', 'email' => 'noel@transfercar.co.nz'],
      ['name' => 'Pascal', 'email' => 'pascal@transfercar.co.nz'],
      // ['name' => 'Noel AU', 'email' => 'noel@transfercar.com.au'],
      // ['name' => 'Noel US', 'email' => 'noel@transfercarus.com'],
      // ['name' => 'Noel Perso', 'email' => 'noelguilbert@gmail.com'],
      // ['name' => 'Brian', 'email' => 'brian@transfercar.co.nz'],

    ], 100);
  }
}
