<?php

use PhpAmqpLib\Connection\AMQPConnection;
use PhpAmqpLib\Message\AMQPMessage;
use Transfercar\PhpAmqpLib\tcRabbitmq;
use Transfercar\Email\Sender\RequestNotificationEmail;

class transfercarNotifyExpiringRequestsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'notify-expiring-requests';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:send-accepted-emails|INFO] expires requests
Call it with:

  [php symfony transfercar:expire-requests|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    $this->addOption('timeout', null, sfCommandOption::PARAMETER_REQUIRED, 'Timeout', 300);
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    error_reporting(E_ALL);
    ini_set('display_errors', 1);
    $_SERVER['TRANSFER_CAR_COUNTRY'] = 'nz';
        $_SERVER['HTTP_HOST'] = 'www.transfercar.co.nz';
        $_SERVER['SERVER_NAME'] = 'www.transfercar.co.nz';

    // // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);


    $this->logSection('queue', 'Connection to AMQP server');
    $conn = new tcRabbitmq(
      sfConfig::get('app_amqp_host'),
      sfConfig::get('app_amqp_port'),
      sfConfig::get('app_amqp_username'),
      sfConfig::get('app_amqp_password'),
      sfConfig::get('app_amqp_path')
    );

    $channel = $conn->getChannel('expiring_requests');
    $channel
        ->basic_consume(
          'expiring_requests',
          $this->getFullName(),
          false,
          false,
          false,
          false,
          array($this, 'process_message')
    );

    $this->logSection('queue', 'Reading expiring_requests queue with timeout '.$options['timeout']);
    try
    {
      while (count($channel->callbacks)) {
        $channel->wait(null, false, $options['timeout']);
      }
    } catch (PhpAmqpLib\Exception\AMQPTimeoutException $ex)
    {
      // timeout exceeds, closing task to avoid memory leaks
    }
  }

  public function process_message($msg)
  {
    $request = Doctrine::getTable('Request')->find($msg->body);

    if ($request && $request->isStatus(Request::STATUS_PENDING))
    {
      $listing  = $request->get('Listing');
      $receiver = $listing->getNewRequestContact()->getEmailToParams();


      try
      {
        $notification = new RequestNotificationEmail($request);
        $notification->send('expiring-request', $receiver);
        $notification->send('driver-expiration-reminder', $request->get('Driver'));
      } catch (Exception $e) {
        $this->log('unable to send reminder for listing %s (%s)', $listing->getId(), $e->getMessage());
      }
    }

    // ack message to remove it from the queue
    $msg
      ->delivery_info['channel']
      ->basic_ack($msg->delivery_info['delivery_tag']);
  }
}



















