<?php

use PhpAmqpLib\Connection\AMQPConnection;
use PhpAmqpLib\Message\AMQPMessage;
use Transfercar\PhpAmqpLib\tcRabbitmq;
use Transfercar\Email\Sender\RelocationAcceptedEmail;
use Transfercar\Mandrill\TcMandrill;
class transfercarSendacceptedemailsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'send-accepted-emails';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:send-accepted-emails|INFO] task does things.
Call it with:

  [php symfony transfercar:send-accepted-emails|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    $this->addOption('timeout', null, sfCommandOption::PARAMETER_REQUIRED, 'Timeout', 300);
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    error_reporting(E_ALL);
    ini_set('display_errors', 1);
    $_SERVER['TRANSFER_CAR_COUNTRY'] = 'nz';

    // // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);


    $this->logSection('queue', 'Connection to AMQP server');
    $conn = new tcRabbitmq(
      sfConfig::get('app_amqp_host'),
      sfConfig::get('app_amqp_port'),
      sfConfig::get('app_amqp_username'),
      sfConfig::get('app_amqp_password'),
      sfConfig::get('app_amqp_path')
    );

    $channel = $conn->getChannel('accepted_requests');
    $channel
        ->basic_consume(
          'accepted_requests',
          $this->getFullName(),
          false,
          false,
          false,
          false,
          array($this, 'process_message')
    );

    $this->logSection('queue', 'Reading accepted_requests queue with timeout '.$options['timeout']);
    try
    {
      while (count($channel->callbacks)) {
$this->logSection('wait', '.');
        $channel->wait(null, false, $options['timeout']);
      }
    } catch (PhpAmqpLib\Exception\AMQPTimeoutException $ex)
    {
      // timeout exceeds, closing task to avoid memory leaks
    }
  }

  public function process_message($msg)
  {
$this->logSection('processing', $msg->body);
    $request = Doctrine::getTable('Request')->find($msg->body);

    if ($request && $request->isStatus(Request::STATUS_ACCEPTED) && !$request->get('Listing')->get('is_confirmed'))
    {

      $driver = $request->get('Driver');
$this->logSection('processing', $driver->get('email'));
      $request->get('Listing')->set('is_confirmed', -1);
      $request->get('Listing')->save();
      RelocationAcceptedEmail::notify($request);
$this->logSection('processed', $driver->get('email'));
      if ($driver->get('sms_number_confirmed'))
      {
        tcRequestSmsEvents::sendRequestNotification(
            $request,
            'request_accepted',
            array(
              'contact_phone' => $request->get('Listing')->getNewRequestContact()->get('phone'),
              'contact_name'  => $request->get('Listing')->getNewRequestContact()->getFullName(),
            )
        );
      }
    }


    // ack message to remove it from the queue
    $msg
      ->delivery_info['channel']
      ->basic_ack($msg->delivery_info['delivery_tag']);
  }
}



















