<?php

use Transfercar\Mandrill\TcMandrill;
use GuzzleHttp\Client as HttpClient;

class transfercarSendreviewnotificationsTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'send-review-notifications';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:send-review-notifications|INFO] task does things.
Call it with:

  [php symfony transfercar:send-review-notifications|INFO]
EOF;

    $this->addArgument('domain', sfCommandArgument::REQUIRED, 'The country domain code');
    $this->addArgument('days', sfCommandArgument::OPTIONAL, 'Number of days since dropoff', 3);
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);

    $listings = Doctrine::getTable('Listing')
      ->createQuery('l')
      ->innerJoin('l.AcceptedRequest r')
      ->innerJoin('l.VehicleType v')
      ->innerJoin('v.Country c')
      ->leftJoin('r.Review rv')
      ->where('c.domain = ?', $arguments['domain'])
      ->addWhere('rv.id is null')
      ->addWhere('l.status = ?', Listing::STATUS_DROPPED_OFF)
      ->addWhere('datediff(now(), r.drop_off_date) = ?', $arguments['days'])
      ->execute();

    $_SERVER['TRANSFER_CAR_COUNTRY']= $arguments['domain'];
    sfLoader::loadHelpers(array('Partial', 'Url', 'Tag', 'Transfercar'));
    $count = 0;

    foreach ($listings as $i => $listing)
    {
      $this->sendNPS($this->buildPayload($listing, $arguments['domain']));
      $count++;
    }

    $this->logSection('emails', sprintf('%s review emails have been sent', $count));
  }

  private function buildPayload($listing, $domain)
  {
    $acceptedRequest = $listing->getAcceptedRequest();
    $driver = $acceptedRequest->get('Driver');
    return [
      'email' => $driver->get('email'),
      'name' => $driver->full_name(),
      'segment' => 'Driver '.strtoupper($domain),
      'tcid'      => $listing->getId(),
      'from'      => $listing->get('PickupLocation')->get('name'),
      'to'        => $listing->get('DropoffLocation')->get('name'),
      'operator'  => $listing->get('Owner')->get('company'),
      'vehicle'   => $listing->get('VehicleType')->get('description'),
      'free_days' => $acceptedRequest->get('nb_free_days')
    ];
  }

  private function test()
  {
    $this->log(__METHOD__);
    $this->sendNPS([
      'email' => 'noel@transfercar.co.nz',
      'name' => 'Noel Guilbert',
      'segment' => 'Driver ZA',
      'triggeremail' => true,
      'from'      => 'Location X',
      'to'        => 'Location Y',
      'operator'  => 'Fresh Campers',
      'vehicle'   => 'Motorhome',
      'free_days' => 2
    ]);
  }

  private function sendNPS($payload)
  {
    $client = new HttpClient();

    $this->log(__METHOD__);
    $response = $client->request('POST', 'https://nextransfer.asknice.ly/api/v1/contact/trigger', [
      'form_params' => $payload,
      'headers'     => ['X-apikey' => 'AgwHFbZgf5v53KFhWCzUf0ledciNMM1oOzuu5jnWF4c']
    ]);
    
    $this->log(__METHOD__);
  }

  private function sendEmail($listing)
  {
    $request = $listing->get('AcceptedRequest');
    $driver  = $request->get('Driver');
    $domain = $listing->getCountryDomain();


    $site = sfConfig::get('app_domain_' . $domain);
    $email_opt_link = 'http://' . $site . '/user/emailOptions';

    $template_data = array(
      'from'                => $listing->PickupLocation->name,
      'to'                  => $listing->DropoffLocation->name,
      'from_date'           => tc_format_date($request->get('pickup_date'), 'short'),
      'to_date'             => tc_format_date($request->get('drop_off_date'), 'short'),
      'firstname'           => $driver->firstname,
      'email_opt_link_real' => $email_opt_link,
      'company_name'        => $listing->get('Owner')->get('company'),
      'request_id'          => $request->getIncrementedId(),
      'link'                => 'https://'. $site . '/review/auth/id/'.$request->getId().'/key/'.$request->getEmailKey($driver).'/uid/'.$driver->getId()
    );

    $mandrill = new TcMandrill();
    $mandrill->sendTemplate(
      'Rate your experience',
      $driver->getEmailToParams(),
      $template_data,
      'request-review'
    );
  }
}
