<?php

use Goutte\Client;

class transfercarSyncapolloTask extends sfBaseTask
{
  private static $owner_id = 199447;
  private $manager_id;

  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'sync-apollo';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:scrap-apollo|INFO] task does things.
Call it with:

  [php symfony transfercar:sync-apollo|INFO]
EOF;
    $this->addArgument('owner_id', sfCommandArgument::REQUIRED, 'Owner Id');
    $this->addArgument('manager_id', sfCommandArgument::REQUIRED, 'Manager Id');
    $this->addArgument('country', sfCommandArgument::OPTIONAL, 'Country', 'au');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    ini_set('display_errors', 1);
    ini_set('error_reporting', E_ALL);
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    self::$owner_id = $arguments['owner_id'];
    $this->manager_id = $arguments['manager_id'];

    $scapper = '/usr/bin/python /home/admin/apollo/scraper.py';

    $vehicleAU = json_decode(exec($scapper.' https://b2c.apollocamper.com/vendor-service/v2/relocations/'.$arguments['country']), true);
    $vehicleNZ = [];// json_decode(exec($scapper.' https://b2c.apollocamper.com/vendor-service/v2/relocations/nz'), true);
    $ids=[];
    foreach (array_merge($vehicleAU, $vehicleNZ) as $relocation) {


      $data = [
	'product' => $relocation['product'],
        'additionalInfo' => $relocation['additionalInfo'],
        'pickup' => $relocation['defaultFromDepot'],
        'dropoff' => $relocation['defaultToDepot'],
        'date_from' => $relocation['earliestStartDateTime'],
        'date_to' => $relocation['latestEndDateTime'],
        'details' => $relocation['detail'],
        'price' => $relocation['detail']['price_per_day'],
        'id' => $relocation['inventoryKey'].':'.$relocation['inventoryControlNumber'],
        'url' => $relocation['continuationUrl']
      ];

      $ids[] = $data['id'];
      try {
          $this->createOrUpdateListing($data);
      } catch (Exception $e) {
	  $this->logSection('error', $e->getMessage());
      }
    }
    $this->cleanUpListings($ids);
  }

  private function cleanUpListings($ids)
  {
    $listings = Doctrine::getTable('Listing')->createQuery('l')
      ->where('l.owner_id = ?', self::$owner_id)
      ->whereNotIn('l.internal_reference', $ids)
      ->addWhere('(l.internal_reference like ? or l.internal_reference like ?)', ['au:%', 'nz:%'])
      ->addWhere('l.status = ?', 0)
      ->execute();

    foreach ($listings as $listing)
    {
      if (!$listing->hasPendingRequests()) {
        $listing->expires();
        $this->logSection('expired', 'expires listing '.$listing->get('internal_reference'));
      }
    }
  }

  private function findListing($data)
  {

    if (!$listing = Doctrine::getTable('Listing')->findOneByInternalReference($data['id'])) {
      $pickup_date = date('Y-m-d', strtotime('-1 day', strtotime($data['date_from'])));
      $dropoff_date = date('Y-m-d', strtotime('-1 day', strtotime($data['date_to'])));

      $listing = Doctrine::getTable('Listing')->createQuery('l')
        ->where('l.owner_id = ?', self::$owner_id)
        ->whereIn('l.date_available_from', [$pickup_date, date('Y-m-d', strtotime($data['date_from']))])
        ->whereIn('l.date_return_by', [$dropoff_date, date('Y-m-d', strtotime($data['date_to']))])
        ->addWhere('l.status = ?', 0)
        ->addWhere('l.owner_vehicle_type_id = ?', $this->findVehicle($data['product']['code'])->getId())
        ->addWhere('l.owner_pickup_branch_id = ?', $this->findBranch($data['pickup']['code'])->getId())
        ->addWhere('l.owner_dropoff_branch_id = ?', $this->findBranch($data['dropoff']['code'])->getId())
        ->limit(1)
        ->fetchOne();
    }

      return $listing ?: new Listing();
  }

  private function createOrUpdateListing($data)
  {

    $id = $data['id'];
    $ad = $ad = $this->findListing($data);
    $this->logSection($ad->exists() ? 'update':'new', 'listing '.$id);
    $ad = $this->findListing($data);
    $ad->set('internal_reference', $id);
    $ad->set('owner_id', self::$owner_id);
    $ad->set('nb_listings', 1);
    $ad->set('status', 0);
    
    $ad->set('owner_pickup_branch_id', $pickup = $this->findBranch($data['pickup']['code']));
    $ad->set('pickup_location_id', $pickup->get('location_id'));

    $ad->set('owner_dropoff_branch_id', $dropoff = $this->findBranch($data['dropoff']['code']));
    $ad->set('dropoff_location_id', $dropoff->get('location_id'));

    $ad->set('pickup_address', $pickup->address);
    $ad->set('contact_person', $pickup->get('Contact')->get('name'));
    $ad->set('contact_phone', $pickup->get('Contact')->get('phone'));

    $ad->set('owner_vehicle_type_id', $vehicle = $this->findVehicle($data['product']['code']));
    $ad->set('vehicle_type_id', $vehicle->get('vehicle_type_id'));

    $ad->price = Doctrine::getTable('Prices')->getPriceFor(
      self::$owner_id,
      $ad->get('pickup_location_id'),
      $ad->get('dropoff_location_id'),
      $ad->get('vehicle_type_id')
    );

        $ad->set('petrol_included', (isset($data['details']['fuel_allowance']) || isset($data['details']['free_fuel'])));
        $ad->set('petrol_included_text', isset($data['details']['fuel_allowance']) ? '$'.$data['details']['fuel_allowance'].' against receipts'  : null);
    $ad->set('expenses_included', isset($data['details']['general_allowance']));
    $ad->set('expenses_included_text', isset($data['details']['general_allowance']) ? '$'.$data['details']['general_allowance'].' against receipts'  : null);

    $ad->set('ferry_included', isset($data['details']['includes_ferry']));
    $ad->set('ferry_included_text', isset($data['additionalInfo']) ? $data['additionalInfo'] : null);

    $ad->set('date_available_from', date('Y-m-d', strtotime($data['date_from'])));
    $ad->set('date_return_by', date('Y-m-d', strtotime($data['date_to'])));

    $ad->set('listed_by', $this->manager_id);
    $ad->set('insurance_policy_id', $this->getInsurancePolicy());
    $ad->set('pickup_time', 3600*9);
    $ad->set('dropoff_time', 3600*9);
    $ad->set('latest_pickup_time', 3600*16);
    $ad->set('latest_dropoff_time', 3600*14.5);

    $getDaysAndDistance = $this->getDaysAndDistance(
      $data['pickup']['code'], 
      $data['dropoff']['code']
    );


    $date1 = new DateTime($ad->get('date_available_from'));
    $date2 = new DateTime($ad->get('date_return_by'));
    $diff = $date2->diff($date1)->d+1;

    $ad->set('days_allowed', min($getDaysAndDistance['days'], $diff));
    $ad->set('kms_allowed', $getDaysAndDistance['distance']);

    $api      = new Transfercar\Geoloc\GoogleMapsDistancesMatrix();
    $distance = $api->getDistances(
      $pickup->getCountry()->get('domain') == 'us' ? $pickup->get('NearestLocation')->getNameWithCategory('name') : $pickup->get('NearestLocation')->getNameWithCountry('name'),
      $dropoff->getCountry()->get('domain') == 'us' ? $dropoff->get('NearestLocation')->getNameWithCategory('name') : $dropoff->get('NearestLocation')->getNameWithCountry('name'),
      $dropoff->getCountry()->get('domain') == 'us' ? 'imperial' : 'metric'
    );

    $ad->set('kms_estimated', $distance);

    if ($paid_days = $this->getPaidDays($ad)) {
      $ad->set('paid_days_allowed', $paid_days);
      $ad->set('price_per_day', 100);
    } else {
      $ad->set('paid_days_allowed', 0);
      $ad->set('price_per_day', 0);
    }

    $ad->set('updated_by', $this->manager_id);
    $ad->set('updated_at', date('Y-m-d'));

    $ad->set('created_at', date('Y-m-d'));

    if (preg_match('/(Motorhome|Campervan)/i', $vehicle->get('VehicleType')->get('description'))) {
      $cAddon = new Addon();
      $cAddon->fromArray([
        'title' => 'Kitchen and Personal Kits',
        'name'  => 'Kitchen and Personal Kits',
        'price' => 'Free'
      ]);
      $cAddon->set('Listing', $ad);
      $cAddon->set('user_id', self::$owner_id);
      $ad->get('Addons')->delete();
      $ad->get('Addons')->add($cAddon);
    }

    $ad->save();
  }

  private function getPaidDays($listing)
  {
    $pickup_date = new DateTime($listing->get('date_available_from'));
    $dropoff_date = new DateTime($listing->get('date_return_by'));

    $diff = $pickup_date->diff($dropoff_date);
    $day = (int) $listing->get('OwnerVehicleType')->get('rental_type') == 'day';

    return max(0, ((int) $diff->days) + $day - $listing->get('days_allowed'));
  }

  private $insurance_policy;
  private function getInsurancePolicy()
  {
    if (!$this->insurance_policy) {
      $this->insurance_policy = Doctrine::getTable('InsurancePolicy')
        ->createQuery('i')
        ->where('i.owner_id = ?', self::$owner_id)
        ->addWhere('i.name = ?', 'Relocations Policy')
        ->limit(1)
        ->fetchOne();
    }

    if (!$this->insurance_policy) {
      throw new Exception("Unable to find the insurance Policy", 1); 
    }

    return $this->insurance_policy;
  }

  private function findBranch($code)
  {
    $branch = Doctrine::getTable('OwnerBranch')
      ->createQuery('b')
      ->where('owner_id = ?', self::$owner_id)
      ->addWhere('internal_name = ?', $code)
      ->limit(1)
      ->fetchOne();

    if (!$branch) {
      throw new Exception("Can't find branch $code", 1);
      
    }

    return $branch;
  }
  private function findVehicle($code)
  {
    $code = str_replace('NZC', 'NZ', $code);
    $vehicle = Doctrine::getTable('OwnerVehicleType')
      ->createQuery('b')
      ->where('owner_id = ?', self::$owner_id)
      ->addWhere('internal_name = ?', $code)
      ->limit(1)
      ->fetchOne();

    if (!$vehicle) {
      throw new Exception("Can't find vehicle $code", 1);
      
    }

    return $vehicle;
  }

  private function getDaysAndDistance($pickup, $dropoff)
  {
    if (in_array($pickup, ['CHC', 'AKL'])) {
      return ['days' => 5, 'distance' => 1150];
    }

    $idx = [
      "ADL",
      "ASP",
      "BNE",
      "BME",
      "CNS",
      "DRW",
      "MEL",
      "PER",
      "SYD",
      "HBT"
    ];
    $pIdx = array_search($pickup, $idx);
    $dIdx = array_search($dropoff, $idx);

    $matrix = [
        ["ADL", 4,   5,   7,  7,   6,  3,    6,    4,  5],
        [1700, "ASP",5,   5,  5,   4,  5,    6,    5,  7],
        [2350,3250,"BNE", 7,  4,   6,  4,    8,    3,  6],
        [4550,2900,4900,"BME",6,   4,  8,    5,    9,  10],
        [4025,2475,1950,4120,"CNS",5,  8,    9,    7,  8], #CNS-MEL days #CNS-SYD
        [3200,1650,3650,2100,2950,"DRW",6,   6,    6,  8],
        [900,2450,1950,5250,3600,3950,"MEL", 6,    3,  4],
        [2950,3850,4950,2500,5450,4250,3700,"PER", 6,  8],
        [1600,3050,1150,5450,3200,4150,1020,4350,"SYD",5],
        [1230,2780,2280,5580,3930,4280,400,4030,1350, "HBT"],
      ];

     $distance = $matrix[$pIdx >= $dIdx ? $pIdx : $dIdx][$pIdx >= $dIdx ? $dIdx : $pIdx];
     $days = $matrix[$pIdx <= $dIdx ? $pIdx : $dIdx][$pIdx <= $dIdx ? $dIdx : $pIdx];

     return ['days' => $days, 'distance' => $distance];
  }
}
