<?php

class transfercarSynclandingpagesTask extends sfPropelBaseTask
{
  private $prodConn;

  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'sync-landing-pages';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:sync-landing-pages|INFO] synchronizes landing pages from staging to prod.
Call it with:

  [php symfony transfercar:sync-landing-pages|INFO]
EOF;
    $this->addArgument('file', sfCommandArgument::REQUIRED, 'A file with the urls to synchronize');
    $this->addArgument('domain', sfCommandArgument::REQUIRED, 'The domain (nz or au)');
    $this->addArgument('prod_dir', sfCommandArgument::REQUIRED, 'The upload dir to sync files to');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    ini_set('error_reporting', E_ALL|E_STRICT);
    ini_set('display_errors', true);
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    if (!file_exists($arguments['file'])) {
      throw new InvalidArgumentException("File ".$arguments['file']." does not exists or is not readable", 1);
    }

    $urls = array_map('trim', file($arguments['file']));
    $dsn = 'mysql:dbname=transfercar_live_db;host=localhost';
    $this->prodConn = new PDO($dsn, 'root', 'tc_live_u1', 'bdyhFLQhMUKcE77B');

    foreach ($urls as $url) {
      if (!$this->exists($url, $arguments['domain'])) {
        $page = Doctrine::getTable('CMSPage')->findPage($arguments['domain'], $url);

        if (!$page) {
          $this->logSection('sync', 'Skipped page '.$url, null, 'ERROR');
          continue;
        }

        $this->logSection('sync', 'Process page '.$url);

        list($query, $values) = $this->getInsertQuery($page);

        $stmt = $this->prodConn->prepare($query);

        if (!$options['dry-run']) {
          $result = $stmt->execute($values);

          if(!$result) {
            throw new Exception(var_export($this->prodConn->errorInfo()));
          }

          $this->syncFiles($page, $argument['prod_dir']);
        }
      }
    }
  }

  private function syncFiles($page, $sf_prod_upload_dir)
  {
    foreach ($page->get('carousel_images') as $path)
    {
      copy(sfConfig::get('sf_upload_dir').'/../'.$path, $sf_prod_upload_dir.'/../'.$path);
    }
  }

  private function exists($url, $domain)
  {
    $query = 'SELECT COUNT(*) as nb from cms_page where url = ? and domain = ?';
    $stmt = $this->prodConn->prepare($query);

    $stmt->execute(array($url, $domain));
    $count = $stmt->fetch();

    return $count['nb'] > 0;
  }

  private function getInsertQuery($page)
  {
    $query = 'INSERT INTO `cms_page` VALUES (' .
      'NULL,' . // --  `id` bigint(20) NOT NULL AUTO_INCREMENT,
      '?,' . //    --  `domain` varchar(128) NOT NULL,
      '?,' . //    --  `url` varchar(128) NOT NULL,
      '?,' . //    --  `title` varchar(255) NOT NULL,
      '?,' . //    --  `subtitle` varchar(255) NOT NULL,
      '?,' . //    --  `block_1_title` varchar(255) NOT NULL,
      '?,' . //    --  `block_1_body` text NOT NULL,
      '?,' . //    --  `block_2_title` varchar(255) NOT NULL,
      '?,' . //    --  `block_2_body` text NOT NULL,
      '?,' . //    --  `carousel_images` text,
      '?,' . //    --  `block_vehicles_title` varchar(255) DEFAULT NULL,
      '?,' . //    --  `block_vehicles_options` text,
      '?,' . //    --  `block_3_title` varchar(255) DEFAULT NULL,
      '?,' . //    --  `block_3_body` text,
      '?,' . //    --  `block_map_title` varchar(255) DEFAULT NULL,
      '?,' . //    --  `block_map_body` text,
      '?,' . //    --  `block_4_title` varchar(255) DEFAULT NULL,
      '?,' . //    --  `block_4_body` text,
      '?,' . //    --  `block_5_title` varchar(255) DEFAULT NULL,
      '?,' . //    --  `block_5_body` text,
      '?,' . //    --  `allowed_parameters` text,
      '?,' . //    --  `meta_title` text,
      '?,' . //    --  `og_title` text,
      '?,' . //    --  `og_description` text,
      '?,' . //    --  `og_sitename` text,
      '?,' . //    --  `og_image` text,
      '?'  . //    --  `meta_description` varchar(512) DEFAULT NULL,
    ')';

    $values = $page->toArray();
    unset($values['id']);
    $values['carousel_images'] = serialize($values['carousel_images']);
    $values['block_vehicles_options'] = serialize($values['block_vehicles_options']);

    return array($query, array_values($values));
  }
}