<?php

use PhpAmqpLib\Connection\AMQPConnection;
use PhpAmqpLib\Message\AMQPMessage;
use Transfercar\PhpAmqpLib\tcRabbitmq;

use Transfercar\Alert\AlertEvents;

class transfercarUpdateAlertsTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'update-alerts';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:update-alerts|INFO] task does things.
Call it with:

  [php symfony transfercar:update-alerts|INFO]
EOF;
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    $this->addOption('timeout', null, sfCommandOption::PARAMETER_REQUIRED, 'Timeout', 300);
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    error_reporting(E_ALL);
    ini_set('display_errors', 1);
    $_SERVER['TRANSFER_CAR_COUNTRY'] = 'nz';

    // // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);
    sfContext::createInstance($this->configuration);


    $this->logSection('queue', 'Connection to AMQP server');
    $conn = new tcRabbitmq(
      sfConfig::get('app_amqp_host'),
      sfConfig::get('app_amqp_port'),
      sfConfig::get('app_amqp_username'),
      sfConfig::get('app_amqp_password'),
      sfConfig::get('app_amqp_path')
    );

    $channel = $conn->getChannel('listing_updated');
    $channel
        ->basic_consume(
          'listing_updated',
          $this->getFullName(),
          false,
          false,
          false,
          false,
          array($this, 'process_message')
    );

    $this->logSection('queue', 'Reading listing_updated queue with timeout '.$options['timeout']);
    try
    {
      while (count($channel->callbacks)) {
        $channel->wait(null, false, $options['timeout']);
      }
    } catch (PhpAmqpLib\Exception\AMQPTimeoutException $ex)
    {
      // timeout exceeds, closing task to avoid memory leaks
    }
  }

  public function process_message($msg)
  {
    $listing = Doctrine::getTable('Listing')->find($msg->body);

    AlertEvents::removeNotification($listing);

    // ack message to remove it from the queue
    $msg
      ->delivery_info['channel']
      ->basic_ack($msg->delivery_info['delivery_tag']);
  }
}



















