<?php

class transfercarUpdateSearchIndexTask extends sfPropelBaseTask
{
  protected function configure()
  {
    $this->namespace        = 'transfercar';
    $this->name             = 'update-index';
    $this->briefDescription = '';
    $this->detailedDescription = <<<EOF
The [transfercar:remove-weekend-listings|INFO] task does things.
Call it with:

  [php symfony transfercar:remove-weekend-listings|INFO]
EOF;
    $this->addArgument('country', sfCommandArgument::REQUIRED, 'The country id');
    $this->addArgument('application', sfCommandArgument::OPTIONAL, 'The application name', 'frontend');
    // add other arguments here
    $this->addOption('env', null, sfCommandOption::PARAMETER_REQUIRED, 'The environment', 'dev');
    $this->addOption('connection', null, sfCommandOption::PARAMETER_REQUIRED, 'The connection name', 'doctrine');
    // add other options here
  }

  protected function execute($arguments = array(), $options = array())
  {
    // Database initialization
    $databaseManager = new sfDatabaseManager($this->configuration);

    $q = Doctrine_Query::create()//->select('l.id , p.name')
                ->from('Listing l')
                ->innerJoin('l.PickupLocation p')
                ->innerJoin('l.DropoffLocation d')
                ->innerJoin('l.VehicleType vt')
                ->innerJoin('p.Category pC')
                ->innerJoin('l.Owner u')
                ->where('l.status = ?', Listing::STATUS_NEW)
                ->addWhere("pC.country_id = ?", $arguments['country'])
                ;

    $listings = $q->execute();

    if (!count($listings)) {
      $this->logSection('end', 'No listing found');
      return;
    }

    $this->logSection('listings', count($listings));
    $documentsToAdd = [];
    $domain = '';
    foreach ($listings as $listing) {
      $domain = $listing->getCountryDomain();
      $this->logSection('add', $listing->getId());
      $listingsToAdd[] = $listing;
    }

    $this->logSection('Elastica', 'Connecting to elasticsearch');
    $client       = new tcElasticaClient();
    $elasticaType = $client->getIndex($domain)->getType('listing');

    $this->logSection('Elastica', 'updating documents');

    if (count($listingsToAdd)) {
      $this->addListingDocument($elasticaType, $listingsToAdd);
    }

    $this->logSection('Save', 'Refreshing index');
    $elasticaType->getIndex()->refresh();

    $this->logSection('added', count($listingsToAdd));
  }

  private function addListingDocument($elasticaType, $listings)
  {
      foreach ($listings as $listing) {
        // Create document data
        $listingData = tcElasticaListing::buildListingDocument($listing);
        // First parameter is the id of document.
        $listingDocument = new \Elastica\Document($listing->getId(), $listingData);
        // Add listing to type
        $elasticaType->addDocument($listingDocument);
      }
  }
}