<?php
/**
 * Implements check if given "days allowed" value is less or equal the difference
 * between the availability and return by dates
 *
 * @package transfercar
 * @subpackage validator
 */
class myValidatorAllowedDaysInListing extends sfValidatorBase
{
    protected function configure($options = array(), $messages = array())
    {
        $this->addMessage('max', 'The number of free and/or paid days are more than the number of days in selected date range');
    }

    public function doClean($values)
    {
        $from = strtotime($values['date_available_from']);
        // $from = mktime(0, 0, 0, $from['month'], $from['day'], $from['year']);

        $to = strtotime($values['date_return_by']);

        // Calculate today date
        $year = date('Y', time());
        $month = date('m', time());
        $day = date('d', time());
        $today = mktime(0, 0, 0, $month, $day, $year);

        // If it is something wrong with the dates, exit
        if($from < $today || $from > $to){
            return $values;
        }

        $nb_days = (intval($values['days_allowed']) + intval($values['paid_days_allowed'])) - 1;
        $interval = strtotime('+ '.$nb_days.'day', $from);

        if ($interval > $to)
        {
            $error = new sfValidatorError($this, 'max', array('value' => $nb_days));
            throw new sfValidatorErrorSchema($this, array('days_allowed' => $error));
        }

        return $values;

    }
}