<?php
/*
 * Validate value entered in join form.
 * We can have 4 possible cases
 * a) given e-mail belongs to registered user
 * b) given e-mail has been already entered (it's OK - activation code will be resent)
 * c) given e-mail did not exists yet - it's OK
 * d) given e-mail address is invalid
 */
class myValidatorJoinEmail extends sfValidatorEmail
{
    public function configure($options = array(), $messages = array())
    {
        // Add specific messages
        $this->addMessage('already_exists', 'E-mail already registered');
        $this->addMessage('invalid', 'Wrong e-mail');

        return parent::configure($options, $messages);
    }

    // Validate
    public function doClean($value)
    {
        $value = strtolower($value);
        $value = parent::doClean($value); // d)

        // Check if user is present in database, and is
        // active (username is not null)
        $count = Doctrine_Query::create()
            ->from('User')
            ->where('LOWER(email) = ?', $value)
            ->addWhere('password IS NOT NULL')
            ->count();

        if($count > 0){ // a)
            throw new sfValidatorError($this, 'already_exists',
                array('value' => $value));
        }

        return $value; // b) or c)
    }
}
?>