<?php
/**
 * Check if given e-mail address is valid and unique in the database
 */
class myValidatorUniqueEmail extends sfValidatorEmail
{
    /**
     * Configure validator
     * 
     * @param array $options Validation options
     * @param array $messages Validation messages
     */
    public function configure($options = array(), $messages = array())
    {
        $this->addMessage('unique', 'This e-mail is already in use');
        parent::configure($options, $messages);
    }

    /**
     * Validate and clean value given in the form
     *
     * @param mixed $value Value entered by user
     * @return mixed Validated and cleaned value
     */
    public function doClean($value)
    {
        $email = parent::doClean($value);

        // Check if we have already such e-mail
        $user = Doctrine_Query::create()
            ->from('User')
            ->where('LOWER(email) = ?', strtolower($email))
            ->fetchOne();

        // allow drivers to be able to  complete their registration again
        // if they did not completed it (they don't have firstname in that case)
        if($user && $user->get('firstname')) {
            throw new sfValidatorError($this, 'unique', array('value' => $value));
        }

        return $email;
    }
}