<?php

class tcRequestedTimesValidator extends sfValidatorBase
{
    public function configure($options = array(), $messages = array())
    {
        $this->addRequiredOption('listing');
        $this->addMessage('invalid', 'Requested times are invalid (maximum dropoff time is %maxdropoff%)');
    }

    protected function doClean($values)
    {
        $listing = $this->getOption('listing');

        if ($listing->getVehicle()->getRentalType())
        {
            $this->doCleanRelocationTimes($values);
        }

        return $values;
    }

    private function parseTime($time)
    {
        $hours = floor($time / 3600);
        $minutes = ($time % 3600)/60;

        return array($hours, $minutes);
    }

    private function doCleanRelocationTimes($values)
    {
        $listing         = $this->getOption('listing');

        list($pickup_hour, $pickup_minutes) = $this->parseTime($values['pickup_time']);
        list($dropoff_hour, $dropoff_minutes) = $this->parseTime($values['dropoff_time']);
        $pickupDateTime  = new DateTime($values['pickup_date']);
        $pickupDateTime->setTime($pickup_hour, $pickup_minutes, 0);
        $dropoffDateTime = new DateTime($values['dropoff_date']);
        $dropoffDateTime->setTime($dropoff_hour, $dropoff_minutes, 0);
        $nb_days         = $values['free_days'] + $values['paid_days'];
        $nb_days         = $listing->getDaysAllowedAvailable($listing->get('days_allowed') + $listing->get('paid_days_allowed'));
        $day             = $listing->getVehicle()->get('rental_type') == 'day' ? 1 : 0;
        $maxDropoffTime  = clone $pickupDateTime;
        $maxDropoffTime->add(new DateInterval(sprintf('P%uD', $nb_days-$day)));

        if ($day) {
            list($dropoff_hour, $dropoff_minutes) = $this->parseTime($listing->get('latest_dropoff_time'));
            $maxDropoffTime->setTime($dropoff_hour, $dropoff_minutes, 0);
        }

        if ($dropoffDateTime > $maxDropoffTime) {
            $error = new sfValidatorError($this, 'invalid', array(
                'maxdropoff' => $maxDropoffTime->format('Y-m-d H:i:s'),
                'dropoff' => $dropoffDateTime->format('Y-m-d H:i:s')
            ));

            throw new sfValidatorErrorSchema($this, ['dropoff_time' => $error]);
        }
    }
}