<?php
/**
 * myWidgetFormNZPhone represents phone number for New Zealand area
 * 
 * @package transfercar
 * @subpackage form
 */
class myWidgetFormNZPhone extends sfWidgetForm
{
    protected static $area_codes = array('03', '04', '06', '07', '09', '020', '021', '024', '0240', '025', '027', '028', '029');
    
    
    /**
     * Configures the current widget
     * 
     * @param array $options an array of options
     * @param array $attributes an array of default HTML attributes
     * 
     * @see sfWidgetForm
     */
    protected function configure($options = array(), $attributes = array())
    {
        $this->addOption('area_codes', self::$area_codes);
    }
    
    /**
     * Renders the field
     * 
     * @param string $name Field name
     * @param string $value Field value
     * @param array $attributes An array of HTML attributes
     * @param array $errors An array of errors for the field
     * 
     * @return string A HTML tag string
     */
    public function render($name, $value = null, $attributes = array(), $errors = array())
    {
        $areas = $this->getOption('area_codes');
        
        // Try to find area code inside the value
        if(!is_array($value)){
            $new_value = array('area' => null, 'number' => $value);
            $splitted_number = explode(' ', $value);
            foreach($areas as $area_code){
                if(isset($splitted_number[0]) && $splitted_number[0] == $area_code){
                    $new_value['area'] = $area_code;
                    unset($splitted_number[0]);
                    $new_value['number'] = implode(' ', $splitted_number);
                }
            } // for each area code
            $value = $new_value;
        } // if value is not array
        
        // Prepare array with area codes + empty place
        $codes = array('' => '');
        foreach($areas as $area_code){
            $codes[$area_code] = $area_code;
        }
        
        // HTML attributes for tags
        $dropdown_attrs = array_merge(array('class' => 'area_code_select'), $attributes);
        $number_attrs = array_merge(array('class' => 'phone_number_input'), $attributes);
        
        // Render the tags
        $result = '';
        
        // Drop-down list
        $widget = new sfWidgetFormSelect(array('choices' => $codes));
        $result .= $widget->render($name.'[area]', $value['area'], $dropdown_attrs, array());
        
        // Input tag with phone number
        $widget = new sfWidgetFormInput();
        $result .= $widget->render($name.'[number]', $value['number'], $number_attrs, $errors);
        
        return $result;
    }
}
?>