<?php

/**
 * myWidgetFormInput represents an HTML radio buttons tag.
 *
 * @package    transfercar
 * @subpackage widget
 * @author     Pawel Smolinski <pawel.smolinski@planetsoft.pl>
 */
class myWidgetFormRadio extends sfWidgetForm
{
  /**
   * Constructor.
   *
   * Available options:
   *
   *  * type: The widget type (radio by default)
   *  * choices: list of radio button values in format:
   *      value => text
   *  * left_label: put labels on the left side of buttons
   *
   * @param array $options     An array of options
   * @param array $attributes  An array of default HTML attributes
   *
   * @see sfWidgetForm
   */
  protected function configure($options = array(), $attributes = array())
  {
    $this->addOption('type', 'radio');
    $this->setOption('is_hidden', false);
    
    $this->addOption('choices', $options['choices']);
    
    if(isset($options['left_label'])){
        $this->addOption('position', 'left');
    } else {
        $this->addOption('position', 'right');
    }
    
  }

  /**
   * @param  string $name        The element name
   * @param  string $value       The value displayed in this widget
   * @param  array  $attributes  An array of HTML attributes to be merged with the default HTML attributes
   * @param  array  $errors      An array of errors for the field
   *
   * @return string An HTML tags string
   *
   * @see sfWidgetForm
   */
  public function render($name, $value = null, $attributes = array(), $errors = array())
  {
    $result = '';
    
    foreach($this->getOption('choices') as $v => $title){
        $result .= '<div class="radio_container">';
        if($this->getOption('position') == 'left'){
            $result .= $title;
        }
        
        $options = array_merge(array('type' => $this->getOption('type'),
            'name' => $name, 'value' => $v), $attributes);
        
        // Check if we should check this button
        if($v == $value){
            $options['checked'] = 'checked';
        }
        
        $result .= $this->renderTag('input', $options);
        
        if($this->getOption('position') == 'right'){
            $result .= $title;
        }
        
        $result .= '</div>';
        
    } // foreach choice
    
    return $result;
  }
}
