<?php

/**
 * sfWidgetFormAddonList represents a list of addons
 *
 * @package    transfercar
 * @subpackage widget
 * @author     Bartosz Pasinski <bartosz.pasinski@planetsoft.pl>
 */

class sfWidgetFormAddonList extends sfWidgetForm
{
  protected $_addons = array();
  protected static $_recent;

  protected function configure($options = array(), $attributes = array())
  {
    $this->addOption("count", 1);
  }

  public function render($name, $values = array(), $attributes = array(), $errors = array())
  {
  	$request = sfContext::getInstance()->getRequest();
  	$similar = $request->hasParameter("similar") ? (int)$request->getParameter("similar") : 0;

  	$count = $this->getOption('count');
    if(count($values) > 1) {
      $count = count($values);
    }
    $recent     = $this->getRecent();
    $newAddon   = new sfWidgetFormAddon();
    $addonJs    = "var addons = Array();\n";
    $attributes['name'] = $name;
    $attributes = $this->fixFormId($attributes);
    $id         = $attributes['id'];
    foreach ($recent as $listingId => $addons) {
      $addonJs .= "addons['{$listingId}'] = new Array();\n";
      foreach($addons as $addon) {
        $addonJs .= "addons['{$listingId}'].push({name: '{$addon->name}', price: {$addon->price}});\n";
      }
    }
    $result  = <<<EOS
<script type="text/javascript">
{$addonJs}
var newAddonId = {$count};
function addAddon() {
  $('addon_list').insert({bottom: '<tr id="{$id}_' + newAddonId + '">{$newAddon->render($name . "[' + (newAddonId) + ']")}</tr>'});
  return (newAddonId++);
}
function setAddon(id, name, price) {
  $('{$id}_' + id + '_name').setValue(name);
  $('{$id}_' + id + '_price').setValue(price);
}
function resetAddons()
{
  for (var i = 0; i < newAddonId; i++) {
    if ($('{$id}_' + i)) {
      $('{$id}_' + i).remove();
    }
  }
  newAddonId = 0;
}
function updateAddons()
{
  var id = $('recent_addons').getValue();
  resetAddons();
  if(id == 0) {
    addAddon();
  } else {
    addons[id].each(function(e){
      setAddon(addAddon(), e.name, e.price);
    });
  }
}
</script>
EOS;
;
    $result = str_replace('&#039;', "'", $result);
    $result .= '<table class="form" style="width:400px">';
    if (count($recent) > 0) {
      $options = array('New list');
      foreach ($recent as $listingId => $addons) {
        $options[$listingId] = $addons[0]['title'];
      }
      $result .= '<tr><td>Your extras:</td><td>' . select_tag('recent_addons', options_for_select($options, $similar), array('onchange' => 'updateAddons()')) . '</td></tr>';
    }
    $title = (isset($values[0]['title'])
           ? $title = $values[0]['title']
           : (isset($values['title'])
             ? $values['title']
             : ''));

    $result .= '<tr><td>Title (for identification):</td><td>' . input_tag($name . '[title]', $title) . '</td></tr>';
    $result .= '</table>';
    if($errors) {
      $result .= '<div style="clear:both" class="error"><label class="error">' . $errors . '</label></div>';
    }
    $result .= '<table id="addon_list" class="form">';
    $result .= '<tr><td style="width:35%">Description</td><td style="width:25%">Price</td><td style="width:40%">&nbsp;</td></tr>';

    for($i = 0, $j = 0; $j < $count; $j++) {
      $addon = new sfWidgetFormAddon();
      if (!isset($values[$i])) $values[$i] = null;
      if (!isset($attributes[$i])) $attributes[$i] = array();
      $result .= '<tr id="' . $id . '_' . $j . '">' . $addon->render($name . "[$i]", $values[$i], $attributes[$i], array()) . '</tr>';
      $i++;
    }
    $result .= '</table>';
    $result .= '<div style="clear:both"><a href="#" onclick="addAddon(); return false;">Add item</a></div>';
    return $result;
  }

 /**
  * Retrieve user recent addon list
  */
  public function getRecent()
  {
    if (!isset(self::$_recent)) {
      $user = sfContext::getInstance()->getUser();
      $addons = Doctrine_Query::create()
        ->from("Addon")
        ->where("user_id = ?", $user->id)
        ->addWhere("title != ''")
        ->orderBy('listing_id ASC')
        ->execute();

      self::$_recent = array();
      foreach ($addons as $addon) {
        self::$_recent[$addon->listing_id][] = $addon;
      }
    }
    return self::$_recent;
  }
}

?>