<?php

/**
 * sfWidgetFormLocation allows to select location
 *
 * @package    transfercar
 * @subpackage widget
 * @author     Bartosz Pasinski <bartosz.pasinski@planetsoft.pl>
 */

class sfWidgetFormLocation extends sfWidgetForm
{
  private static $_jsRendered = false;
  private static $_categories = array();
  private static $_locations  = array();
  
  public static function getLocations()
  {
    if(empty(self::$_locations)) {
      self::$_locations  = Locations::getAllByDeparture();
    }
    return self::$_locations;
  }
  
  public static function getCategories()
  {
    if(empty(self::$_categories)) {
      self::$_categories  = LocationDeparture::getAll();
    }
    return self::$_categories;
  }
  
  protected function configure($options = array(), $attributes = array())
  {
    $this->addOption('sublocation', true);
    $this->addOption('displayLocation', true);
    $this->addOption('empty', array(' ', ' '));
    
    if(empty(self::$_categories)) {
      self::$_categories = LocationDeparture::getAll();
      self::$_locations  = Locations::getAllByDeparture();
    }
  }
  
  protected function renderJs()
  {
    $result = '<script type="text/javascript">';
    $result .= 'var locations = new Array();';
    $last = null; 
    foreach(self::$_locations as $location) {
      if ($last != $location['departure_id']) {
        $result .= "locations[{$location['departure_id']}] = new Array();\n";
      }
      $last = $location['departure_id'];
      $result .= "locations[{$location['departure_id']}].push({id:{$location['id']}, name:'{$location['name']}'});\n";
    }
    $result .= <<<EOS
function update_select(elem, update){
  $(update).innerHTML = '';
  id = $(elem).getValue();
  select_push($(update), ' ');
  if(typeof locations[id] == 'undefined') {
    $(update).hide();
    $(elem).style.width = '100%';
    $(update).disable();
  } else {
    $(update).show();
    locations[id].sort(function cmp(a,b){ if(a.name < b.name) return -1; else return 1; });
    $(elem).style.width = '43%';
    $(update).enable();
    select_push($(update), locations[id]);
  }
}
</script>
EOS;
    self::$_jsRendered = true;
    return $result;
  }
  
  public function render($name, $value = null, $attributes = array(), $errors = array())
  {

    $attributes['category']['name'] = $name . '[category]';
    $attributes['location']['name'] = $name . '[location]';
    $attributes['category'] = $this->fixFormId($attributes['category']);
    $attributes['location'] = $this->fixFormId($attributes['location']);

    if(!isset($value['category'])) {
      $value['category'] = 0;
    } else {
      $value['category'] = intval($value['category']);
    }
    if(!isset($value['location'])) {
      $value['location'] = 0;
    } else {
      $value['location'] = intval($value['location']);
    }
    if ($value['category'] == 0 && $value['location'] != 0) {
      $value['category'] = $this->findCategoryForLocation($value['location']);
    }

    $attributes['location']['style'] = 'width: 55%;' . ($value['category'] == 0 ? 'display:none;' : '');
    $attributes['category']['style'] = ($value['category'] == 0 ? 'width:100%;' : 'width:43%;');
    if ($this->getOption('displayLocation')) {
      $attributes['category']['onchange'] = "update_select('{$attributes['category']['id']}', '{$attributes['location']['id']}')";
    }
    $result = '';
    
    // if script wasn't rendered already
    if (!self::$_jsRendered && $this->getOption('displayLocation')) {
      $result .= $this->renderJs();
    }
    $empty = $this->getOption('empty');
    
    $categoriesById = array($empty[0]);
    foreach(self::$_categories as $category) { 
      $categoriesById[$category['id']] = $category['name']; 
    }
    
    $options = options_for_select($categoriesById, $value['category']);
    $result .= select_tag($name . '[category]', $options, $attributes['category']) . ' ';
    
    $options = array($empty[1]);
    if ($value['category'] > 0) {
      foreach(self::$_locations as $location) {
        if ($value['category'] == $location['departure_id']) {
          $options[$location['id']] = $location['name'];
        }
      }
    }
    if ($this->getOption('displayLocation')) {
      $options = options_for_select($options, $value['location']);
      $result .= select_tag($name . '[location]', $options, $attributes['location']);
    }
    return $result;
  }
  
  public function findCategoryForLocation($location)
  {
    foreach(self::$_locations as $l) {
      if ($location == $l['id']) {
        return $l['departure_id'];
      }
    }
    return 0;
  }
}

?>