<?php

/**
 * sfWidgetFormAddonList represents a list of addons
 *
 * @package    transfercar
 * @subpackage widget
 * @author     Bartosz Pasinski <bartosz.pasinski@planetsoft.pl>
 */

class sfWidgetFormOwnerAddonList extends sfWidgetForm
{
  protected $_addons = array();
  protected static $_recent;

  protected function configure($options = array(), $attributes = array())
  {
    $this->addOption("count", 1);
    $this->addRequiredOption('user_id');
  }

  public function render($name, $values = array(), $attributes = array(), $errors = array())
  {
  	$request = sfContext::getInstance()->getRequest();
  	$similar = $request->hasParameter("similar") ? (int)$request->getParameter("similar") : 0;

  	$count = $this->getOption('count');
    if(count($values) > 1) {
      $count = count($values);
    }
    $recent     = $this->getRecent();
    $newAddon   = new sfWidgetFormOwnerAddon();
    $addonJs    = "var addons = Array();\n";
    $attributes['name'] = $name;
    $attributes = $this->fixFormId($attributes);
    $id         = $attributes['id'];
    foreach ($recent as $listingId => $addons) {
      $addonJs .= "addons['{$listingId}'] = new Array();\n";
      foreach($addons as $addon) {
        $addonJs .= "addons['{$listingId}'].push({name: '{$addon->name}', price: {$addon->price}});\n";
      }
    }
    $widget = str_replace(array("\r", "\n"), '', $newAddon->render($name . "[' + (newAddonId) + ']"));
    $result  = <<<EOS
<script type="text/javascript">
{$addonJs}
var newAddonId = {$count};
function addAddon() {
  jQuery('#addon_list')
    .append('<div class="row" id="{$id}_' + newAddonId + '">{$widget}</div>');

  jQuery('#addon_list .row:last input[name*="[price]"]').rules('add', {
    number: true
  })
  return (newAddonId++);
}
function setAddon(id, name, price) {
  jQuery('#{$id}_' + id + '_name').val(name);
  jQuery('#{$id}_' + id + '_price').val(price);
}
function resetAddons()
{
  for (var i = 0; i < newAddonId; i++) {
    if (jQuery('#{$id}_' + i)) {
      jQuery('#{$id}_' + i).remove();
    }
  }
  newAddonId = 0;
}
function updateAddons()
{
  var id = jQuery('#recent_addons').val();
  resetAddons();
  if(id == 0) {
    addAddon();
  } else {
    jQuery(addons[id]).each(function(){
      setAddon(addAddon(), this.name, this.price);
    });
  }
}
</script>
EOS;
;
    $result = str_replace('&#039;', "'", $result);
    $result .= '<div class="row">';
    if (count($recent) > 0) {
      $options = array('New list');
      foreach ($recent as $listingId => $addons) {
        $options[$listingId] = $addons[0]['title'];
      }
      $result .= '<div class="four columns"><label>Your extras</label>' . select_tag('recent_addons', options_for_select($options, $similar), array('onchange' => 'updateAddons()')) . '</div>';
    }
    $title = (isset($values[0]['title'])
           ? $title = $values[0]['title']
           : (isset($values['title'])
             ? $values['title']
             : ''));

    $result .= '<div class="four columns end"><label>Title (for identification)</label>' . input_tag($name . '[title]', $title) . '</div>';
    $result .= '</div>';
    if($errors) {
      $result .= '<small class="row error">' . $errors . '</small>';
    }
    $result .= '<div id="addon_list">';
    $result .= '<div class="row">';
    $result .= '<div class="six columns"><label>Description</label></div><div class="two columns"><label>Price</label></div><div class="two columns end"><label>Period</label></div>';
    $result .= '</div>';

    for($i = 0, $j = 0; $j < $count; $j++) {
      $addon = new sfWidgetFormOwnerAddon();
      if (!isset($values[$i])) $values[$i] = null;
      if (!isset($attributes[$i])) $attributes[$i] = array();
      $result .= '<div class="row" id="' . $id . '_' . $j . '">' . $addon->render($name . "[$i]", $values[$i], $attributes[$i], array()) . '</div>';
      $i++;
    }

    $result .= '</div>';
    $result .= '<p><a href="" onclick="addAddon(); return false">Add another item</a></p>';
    return $result;
  }

 /**
  * Retrieve user recent addon list
  */
  public function getRecent()
  {
    if (!isset(self::$_recent)) {
      $addons = Doctrine_Query::create()
        ->from("Addon")
        ->where("user_id = ?", $this->getOption('user_id'))
        ->addWhere("title != ''")
        ->orderBy('listing_id ASC')
        ->execute();

      self::$_recent = array();
      foreach ($addons as $addon) {
        self::$_recent[$addon->listing_id][] = $addon;
      }
    }
    return self::$_recent;
  }
}
