<?php
/**
 * Implements date choose with calendar picker
 *
 * @package transfercar
 * @subpackage form
 */
class tcWidgetFormDateWithPicker extends sfWidgetForm
{
    /**
     * Constructor
     *
     * @param array $options An array of options
     * @param array $attributes An array of default HTML attributes
     */
    protected function configure($options = array(), $attributes = array())
    {
        $this->setOption('is_hidden', false);
        $this->addOption('picker_name', 'Calendar');
        $this->addRequiredOption('domain');
    }

    /**
     * Return widget HTML representation
     *
     * @param string $namen Element name
     * @param string $value Displayed value
     * @param array $attributes Additional HTML attributes
     * @param array $errors Element errors
     *
     * @return string Widget HTML representation
     */
    public function render($name, $value = null, $attributes = array(), $errors = array())
    {
        $result = '';

        if ($value === null)
        {
            $value = date('Y-m-d');
        }

        sfLoader::loadHelpers(array('Date', 'Transfercar'));
        $old = $value;
        /* when the form is submitted and an error occured with the date, the validator does
           not sent back a standardized formatted date, but rather the user input, which
           we will reuse as is to avoid formatting errors.
        */
        $value = preg_match('/\d{4}-\d{2}-\d{2}/', $value) ? tc_format_date($value) : $value;

        // First, display input box
        $result .= $this->renderTag('input', array_merge(
            array(
                'type' => 'text',
                'value' => $value,
                'name' => $name
            ),
            $attributes
        ));

        $result .= ' ';

        // Generate ID from given name
        $id = $name;
        $id = str_replace('[', '_', $id);
        $id = str_replace(']', '', $id);

        // Generate button ID
        $btn_id = $id.'_btn';

        // Add button
        //$result .= '<button class="calendar_button" type="button" id="'.$btn_id.'" onclick="return false;"></button>';
        $year = date('Y');
        $endYear = $year + 5;

        $domain     = $this->getOption('domain');
        $dateFormat = strtoupper(sfConfig::get('app_date_time_pattern_'.$domain, 'dd/mm/yyyy'));

        // Init calendar picker
        $result .= '<script type="text/javascript">';
        $result .= <<<EOF
(function($) {
    jQuery(document).ready(function(){
        jQuery("#$id").daterangepicker({
            format: '$dateFormat',
            minDate: moment().subtract('days', 1),
            singleDatePicker: true
        });
    });
})(jQuery);
EOF;

        $result .= '</script>';

        return $result;
    }
}