<?php

class tcWidgetFormRequestEmailOptions extends sfWidgetFormChoice
{
    public function __construct($options = array(), $attributes = array())
    {
        $options['choices'] = array(

            'listing_manager' => 'To the manager who created the listing',
            'branch_contact'  => 'The the listing\'s pickup branch contact',
            'manager'         => 'A specific manager',
            'contact'         => 'A specific contact'
        );

        $options['expanded'] = true;
        $options['renderer_class'] = 'tcWidgetFormSelectRadio';
        $options['renderer_options'] = array('owner_id' => $options['owner_id']);

        parent::__construct($options, $attributes);
    }

    protected function configure($options = array(), $attributes = array())
    {
      $this->addRequiredOption('owner_id');
      parent::configure($options, $attributes);
    }
}

class tcWidgetFormSelectRadio extends sfWidgetFormSelectRadio
{
    protected function configure($options = array(), $attributes = array())
    {
      $this->addRequiredOption('owner_id');
      parent::configure($options, $attributes);
    }

    public function render($name, $value = null, $attributes = array(), $errors = array())
    {
      $selectValue = null;
      if (strpos($value, ':') !== false)
      {
        list($value, $selectValue) = explode(':', $value);
      }

      if ('[]' != substr($name, -2))
      {
        $name .= '[]';
      }

      $choices = $this->getOption('choices');
      if ($choices instanceof sfCallable)
      {
        $choices = $choices->call();
      }

      $inputs = array();
      foreach ($choices as $key => $option)
      {
        $baseAttributes = array(
          'name'  => substr($name, 0, -2),
          'type'  => 'radio',
          'value' => self::escapeOnce($key),
          'id'    => $id = $this->generateId($name, self::escapeOnce($key)),
        );

        if (strval($key) == strval($value === false ? 0 : $value))
        {
          $baseAttributes['checked'] = 'checked';
        }

        switch ($key) {
          case 'manager':
            $widget = new sfWidgetFormDoctrineSelect(array(
              'model' => 'OwnerUser',
              'criteria' => Doctrine::getTable('OwnerContact')->createQuery()->where('owner_id = ?', $this->getOption('owner_id'))
            ), array(
              'class' => 'two'
            ));

            $label = $this->renderContentTag('label', $option.tooltip('email_options_specific_manager'), array('for' => $id)).' '.$widget->render('request_email[manager]', $selectValue);
            break;

          case 'contact':
            $widget = new sfWidgetFormDoctrineSelect(array(
              'model' => 'OwnerContact',
              'criteria' => Doctrine::getTable('OwnerContact')->createQuery()->where('owner_id = ?', $this->getOption('owner_id'))
            ), array(
              'class' => 'two'
            ));

            $label = $this->renderContentTag('label', $option.tooltip('email_options_specific_contact'), array('for' => $id)).' '.$widget->render('request_email[contact]', $selectValue);
            break;

          default:
            $label = $this->renderContentTag('label', $option.tooltip('email_options_'.$key), array('for' => $id));
        }

        $inputs[] = array(
          'input' => $this->renderTag('input', array_merge($baseAttributes, $attributes)),
          'label' => $label,
        );
      }

      return call_user_func($this->getOption('formatter'), $this, $inputs);
    }
}