<?php

class tcElasticaClient
{
    private $elasticaClients;

    public function __construct()
    {
        $this->elasticaClients = array();
    }

    public function getElasticaClient($domain)
    {
        if (!isset($this->elasticaClients[$domain])) {
            $this->elasticaClients[$domain] = new \Elastica\Client(array(
                'servers' => array(
                    array('host' => 'elastica'.$domain, 'port' => 9200)
                )
            ));
        }

        return $this->elasticaClients[$domain];
    }

    public function getIndex($domain)
    {
        return $this->getElasticaClient($domain)->getIndex('listings_'.$domain);
    }

    public function getSearch($domain)
    {
        $search = new Elastica\Search($this->getElasticaClient($domain));
        $search
            ->addIndex('listings_'.$domain)
            ->addType('listing')
        ;

        return $search;
    }

    public function createIndex($domain)
    {
        $elasticaIndex = $this->getElasticaClient($domain)->getIndex('listings_'.$domain);
        // Create the index new
        $elasticaIndex->create(
            array(
                'number_of_shards' => 4,
                'number_of_replicas' => 1,
                'analysis' => array(
                    'analyzer' => array(
                        'standard' => array(
                            'tokenizer' => 'keyword',
                            'filter' => array('lowercase'),
                            'type' => 'standard'
                        )
                    ),
                    array(
                        'filter' => array(
                            'search' => array(
                                'type' => 'keyword',
                                'filter' => array('standard')
                            )
                        )
                    )
                )
            ),
            true
        );

        //Create a type
        $elasticaType = $elasticaIndex->getType('listing');

        // Define mapping
        $mapping = new \Elastica\Type\Mapping();
        $mapping->setType($elasticaType);

        // Set mapping
        $mapping->setProperties(array(
            'id'              => array('type' => 'integer', 'include_in_all' => false),
            'from_date'       => array('type' => 'date', 'include_in_all' => false),
            'to_date'         => array('type' => 'date', 'include_in_all' => false),
            'nb_listings'     => array('type' => 'integer', 'include_in_all' => false),
            'nb_free_days'    => array('type' => 'integer', 'include_in_all' => false),
            'nb_paid_days'    => array('type' => 'integer', 'include_in_all' => false),
            'price_per_free_day' => array('type' => 'integer', 'include_in_all' => false, 'index' => 'no'),
            'price_per_paid_day' => array('type' => 'float', 'include_in_all' => false, 'index' => 'no'),
            'ferry_included'     => array('type' => 'boolean', 'include_in_all' => false),
            'expenses_included'  => array('type' => 'boolean', 'include_in_all' => false),
            'insurance_included' => array('type' => 'boolean', 'include_in_all' => false),
            'petrol_included'    => array('type' => 'boolean', 'include_in_all' => false),
            'tolls_included'     => array('type' => 'boolean', 'include_in_all' => false),
            'ferry_included_text'     => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'expenses_included_text'  => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'insurance_included_text' => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'petrol_included_text'    => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'tolls_included_text'     => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'timezone'                => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'week_end_closing_time'   => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'week_end_opening_time'   => array('type' => 'string', 'include_in_all' => false, 'index' => 'no'),
            'pickupLocation'    => array(
                'type' => 'object',
                'properties' => array(
                    'id'         => array('type' => 'integer', 'include_in_all' => false),
                    'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    'location'   => array('type' => 'geo_point', 'include_in_all' => false),
                    'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    'state'      => array('type' => 'object', 'properties' => array(
                        'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                        'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    )),
                    'custom_departures'    => array('type' => 'nested', 'include_in_parent' => true, 'properties' => array(
                        'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                        'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    )),
                ),
            ),
            'dropoffLocation'    => array(
                'type' => 'object',
                'properties' => array(
                    'id'         => array('type' => 'integer', 'include_in_all' => false),
                    'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    'location'   => array('type' => 'geo_point', 'include_in_all' => false),
                    'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    'state'      => array('type' => 'object', 'properties' => array(
                        'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                        'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    )),
                    'custom_departures'    => array('type' => 'nested', 'properties' => array(
                        'name'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                        'slug'       => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
                    )),
                ),
            ),
            'vehicleType'    => array(
                'type' => 'object',
                'properties' => array(
                    'id'           => array('type' => 'integer', 'include_in_all' => false),
                    'type'         => array('type' => 'string', 'index' => 'not_analyzed', 'include_in_all' => false),
                    'filter_type'  => array('type' => 'string', 'index' => 'not_analyzed', 'include_in_all' => false),
                    'travellers'   => array('type' => 'integer', 'include_in_all' => false),
                    'transmission' => array('type' => 'string', 'include_in_all' => false),
                    'image'         => array('type' => 'string', 'include_in_all' => false),
                ),
            ),
            'tstamp'  => array('type' => 'date', 'include_in_all' => false),
            'nb_included'  => array('type' => 'integer', 'include_in_all' => false),
            'nb_requests'  => array('type' => 'integer', 'include_in_all' => false),
            'boost_ranking' => array('type' => 'integer', 'include_in_all' => false),
            'listing_score' => array('type' => 'float', 'include_in_all' => false),
            'listing_score_debug' => array('type' => 'string', 'include_in_all' => false),
            'country' => array('type' => 'string', 'include_in_all' => false),
            'is_airport_reloc' => array('type' => 'boolean', 'include_in_all' => false),
            'is_great_deal' => array('type' => 'boolean', 'include_in_all' => false),
            'min_age' => array('type' => 'integer', 'include_in_all' => false),
	    'min_age_filter' => array('type' => 'string', 'include_in_all' => false, 'index' => 'not_analyzed'),
            'closed_in_week_end' => array('type' => 'boolean', 'include_in_all' => false),
        ));

        // Send mapping to type
        $mapping->send();

        return $elasticaIndex;
    }
}
/* for debugging purpose
class tcElasticaSearch extends \Elastica\Search
{
    public function search($query = '', $options = null)
    {
        $cache = new sfFileCache(array('cache_dir' => '/tmp'));
        $key = sha1(serialize($query).serialize($options));


        if ($data = $cache->get($key)) {
            $this->setOptionsAndQuery($options, $query);
            $response = new \Elastica\Response(unserialize($data));
            return new \Elastica\ResultSet($response, $this->getQuery());
        }

        // throw new Exception("Error Processing Request ".$key, 1);

        $resultSet = parent::search($query, $options);

        $cache->set($key, serialize($resultSet->getResponse()->getData()));

        return $resultSet;
    }
}*/
