<?php 

class tcElasticaGhostListing
{
    public static function buildGhostListingDocument(GhostListing $listing)
    {
        // Create a document
        $pickupNode = $listing->get('PickupLocation')->getLocationNode();
        $dropoffNode = $listing->get('DropoffLocation')->getLocationNode();

        return array(
            'id'                 => $listing->getId(),
            'nb_listings'        => 1,
            'pickupLocation' => array(
                'id'       => $listing->get('PickupLocation')->getId(),
                'name'     => $listing->get('PickupLocation')->get('name'),
                'location' => $listing->get('PickupLocation')->getLatLong(),
                'slug'     => $pickupNode->get('slug'),
                'state'    => $pickupNode->getParentOfType('category')->toElasticaArray(),
                'custom_departures' => self::getDepartures($pickupNode)
            ),
            'dropoffLocation' => array(
                'id'       => $listing->get('DropoffLocation')->getId(),
                'name'     => $listing->get('DropoffLocation')->get('name'),
                'location' => $listing->get('DropoffLocation')->getLatLong(),
                'slug'     => $dropoffNode->get('slug'),
                'state'    => $dropoffNode->getParentOfType('category')->toElasticaArray(),
                'custom_departures' => self::getDepartures($dropoffNode)
            ),
            'vehicleType' => array(
                'id'           => $listing->get('vehicle_type_id'),
                'type'         => $listing->get('VehicleType')->getCleanName(),
                'image'        => $listing->get('image')
            ),
            'tstamp'       => time(),
            'boost_ranking' => (int) 0,
            'listing_score' => 0,
            'country' => strtolower(str_replace(' ', '', $listing->get('PickupLocation')->get('Country')->get('name'))),
            'available_text' => $listing->get('available_text')
        );
    }


    private static function getDepartures(LocationNode $location)
    {
        $departures = array();

        foreach ($location->getParentsOfType('departure') as $departure)
        {
            $departures[] = $departure->toElasticaArray();
        }

        // location has no departure
        if (empty($departures)) {
            $departures[] = $location->toElasticaArray();
        }

        return $departures;
    }

    public static function listenToGhostListingUpdated(sfEvent $event)
    {
        ini_set('error_reporting', E_ALL);
        ini_set('display_errors', true);
        $listing = $event->getSubject();

        self::addGhostListingDocument($listing);
    }

    public static function listenToGhostListingDeleted(sfEvent $event)
    {
        ini_set('error_reporting', E_ALL);
        ini_set('display_errors', true);
        $listing = $event->getSubject();

        self::removeGhostListingDocument($listing);
    }

    private static function removeGhostListingDocument(GhostListing $listing)
    {
        $client       = new tcElasticaClient();
        $elasticaType = $client->getIndex($listing->get('PickupLocation')->getCountry()->get('domain'))->getType('listing');

        $elasticaType->deleteIds(array('ghost_'.$listing->getId()));
        $elasticaType->getIndex()->refresh();
    }


    private static function addGhostListingDocument(GhostListing $listing)
    {
        // Create document data
        $listingData = self::buildGhostListingDocument($listing);
        // First parameter is the id of document.
        $listingDocument = new \Elastica\Document('ghost_'.$listing->getId(), $listingData);

        $client       = new tcElasticaClient();
        $elasticaType = $client->getIndex($listing->get('PickupLocation')->get('Country')->get('domain'))->getType('listing');
        // Add listing to type
        try {
            $elasticaType->updateDocument($listingDocument);
        } catch(\Elastica\Exception\ResponseException $e) {
            $elasticaType->addDocument($listingDocument);
        }

        $elasticaType->getIndex()->refresh();
    }
}