<?php

use Transfercar\Ranking\SearchScore;

class tcElasticaListing
{
    public static function listenToListingUpdated(sfEvent $event)
    {
        $listing = $event->getSubject();

        if ($listing->isStatus(Listing::STATUS_NEW))
        {
            self::addListingDocument($listing);
        }
        else
        {
           self::removeListingDocument($listing);
        }
    }

    public static function buildListingDocument(Listing $listing)
    {
        // Create a document
        $pickupNode = $listing->get('PickupLocation')->getLocationNode();
        $dropoffNode = $listing->get('DropoffLocation')->getLocationNode();

        return array(
            'id'                 => $listing->getId(),
            'from_date'          => $listing->get('date_available_from'),
            'to_date'            => $listing->get('date_return_by'),
            'nb_listings'        => $listing->get('nb_listings'),
            'nb_free_days'       => $listing->getDaysAllowedAvailable(),
            'nb_paid_days'       => (int) $listing->getPaidDaysAllowedAvailable(),
            'price_per_free_day' => $listing->getIncludedDaysPrice(),
            'price_per_paid_day' => $listing->get('price_per_day'),
            'ferry_included'     => (int) $listing->get('ferry_included'),
            'ferry_included_text'  => $listing->get('ferry_included_text'),
            'ferry_included_type'  => (string) ($listing->get('ferry_included') ? $listing->get('ferry_included_type') : 0),
            'expenses_included'  => (int) $listing->get('expenses_included'),
            'expenses_included_text'  => $listing->get('expenses_included_text'),
            'insurance_included'      => (int) $listing->get('insurance_included'),
            'insurance_included_text' => $listing->get('insurance_included_text'),
            'petrol_included'         => (int) $listing->get('petrol_included'),
            'petrol_included_text'    => $listing->get('petrol_included_text'),
            'petrol_included_type'    => (string) ($listing->get('petrol_included') ? $listing->get('petrol_included_type') : 0),
            'tolls_included'          => (int) $listing->get('tolls_included'),
            'tolls_included_text'     => $listing->get('tolls_included_text'),
            'closed_in_week_end'      => $listing->get('Owner')->get('closed_in_week_end'),
            'week_end_closing_time'   => $listing->get('Owner')->get('week_end_closing_time'),
            'week_end_opening_time'   => $listing->get('Owner')->get('week_end_opening_time'),
            'timezone'                => $listing->get('Owner')->get('timezone') ?: $listing->get('Owner')->getDefaultTimezone(),
            'pickupLocation' => array(
                'id'       => $listing->get('PickupLocation')->getId(),
                'name'     => $listing->get('PickupLocation')->get('name'),
                'location' => $listing->get('PickupLocation')->getLatLong(),
                'slug'     => $pickupNode->get('slug'),
                'state'    => $pickupNode->getParentOfType('category')->toElasticaArray(),
                'custom_departures' => self::getDepartures($pickupNode)
            ),
            'dropoffLocation' => array(
                'id'       => $listing->get('DropoffLocation')->getId(),
                'name'     => $listing->get('DropoffLocation')->get('name'),
                'location' => $listing->get('DropoffLocation')->getLatLong(),
                'slug'     => $dropoffNode->get('slug'),
                'state'    => $dropoffNode->getParentOfType('category')->toElasticaArray(),
                'custom_departures' => self::getDepartures($dropoffNode)
            ),
            'vehicleType' => array(
                'id'           => $listing->get('vehicle_type_id'),
                'type'         => $listing->get('VehicleType')->getCleanName(),
                'filter_type'  => in_array(strtolower($listing->get('VehicleType')->getCleanName()), ['motorhome', 'campervan', 'van']) ? 'Motorhome' : 'Car',
                'transmission' => $listing->getVehicle()->getTransmissionType(),
                'travellers'   => 4,
                'image'        => $listing->isAirportReloc() ? '/images/vehicles/airport_transfer.gif' : $listing->getVehicle()->icon_path()
            ),
            'tstamp'       => time(),
            'nb_included'  => $listing->included_count(),
            'nb_requests'  => Doctrine::getTable('Listing')->countPendingRequestFor($listing),
            'boost_ranking' => (int) $listing->getRankingBoost(),
            'listing_score' => self::getScore($listing),
            'listing_score_debug' => $listing->getScoreDebug(),
            'country' => strtolower(str_replace(' ', '', $listing->get('Owner')->get('Country')->get('name'))),
            'is_airport_reloc' => $listing->isAirportReloc(),
            'is_great_deal' => $listing->isGreatDeal(),
            'min_age' => $listing->get('min_age'),
            'min_age_filter' => $listing->get('min_age') >= 21 ? '21 or over' : 'Under 21',
        );
    }

    private static function getScore(Listing $listing)
    {
        $score = new SearchScore();

        return $score->getScore($listing);
    }

    private static function addListingDocument(Listing $listing)
    {
        // Create document data
        $listingData = self::buildListingDocument($listing);
        // First parameter is the id of document.
        $listingDocument = new \Elastica\Document($listing->getId(), $listingData);

        $client       = new tcElasticaClient();
        $elasticaType = $client->getIndex($listing->getCountryDomain())->getType('listing');
        // Add listing to type
        try {
            $elasticaType->updateDocument($listingDocument);
        } catch(\Elastica\Exception\ResponseException $e) {
            $elasticaType->addDocument($listingDocument);
        }

        $elasticaType->getIndex()->refresh();
    }

    private static function removeListingDocument(Listing $listing)
    {
        $client       = new tcElasticaClient();
        $elasticaType = $client->getIndex($listing->getCountryDomain())->getType('listing');

        $elasticaType->deleteIds(array($listing->getId()));
        $elasticaType->getIndex()->refresh();
    }

    private static function getDepartures(LocationNode $location)
    {
        $departures = array();

        foreach ($location->getParentsOfType('departure') as $departure)
        {
            $departures[] = $departure->toElasticaArray();
        }

        // location has no departure
        if (empty($departures)) {
            $departures[] = $location->toElasticaArray();
        }

        return $departures;
    }
}
