<?php

class tcElasticaListingResult extends Elastica\Result
{
    public function get($key)
    {
        return $this->$key;
    }

    public function isGhostListing()
    {
        return strpos($this->getId(), 'ghost_') === 0;
    }

    public function getNbRequests()
    {
      return (int) $this->get('nb_requests');
    }

    public function getRelocationQuery($action = 'main/index', $full = true, $params = '')
    {
      $pattern = '/[^a-zA-Z0-9]+/';
      $from = preg_replace($pattern, '_', $this->pickupLocation['name']);
      $to = preg_replace($pattern, '_', $this->dropoffLocation['name']);
      $params .= 'from='.$from.'&to='.$to.'&relo_id='.$this->id;

      sfLoader::loadHelpers(array('Url'));
      $result = url_for($action.'?'.$params, true);

      return $result;
    }

    private function getDaysAvailable($days)
    {
      $today = strtotime('today');
      $maxTime = strtotime('+'.$days.' days', $today);

      if ($maxTime > strtotime($this->get('to_date')))
      {
        $days = (strtotime($this->get('to_date')) - $today) / 86400 + 1;

        return max(1, round($days, 0));
      }

      return $days;
    }

    public function getDaysAllowedAvailable()
    {
        return $this->getDaysAvailable($this->get('nb_free_days'));
    }

    public function getPaidDaysAllowedAvailable()
    {
        return $this->getDaysAvailable($this->get('nb_paid_days'));
    }

    public function hasIncluded()
    {
        return $this->ferry_included || $this->tolls_included || $this->insurance_included || $this->expenses_included || $this->petrol_included;
    }

    public function isGreatDeal()
    {
      return $this->get('is_great_deal');
    }

    public function isAirportReloc()
    {
      return $this->get('is_airport_reloc');
    }

    public function areReservationClosed()
    {
      if (!$this->get('closed_in_week_end'))
      {
        return false;
      }

      $tz = $this->get('timezone');
      $tzObject = new DateTimeZone($tz);
      $current = new DateTime('now', $tzObject);
      $monday  = new DateTime('last monday', $tzObject);

      list($hours, $minutes, $seconds) = explode(':', $this->get('week_end_closing_time'));
      $closing = clone $monday;
      $closing->add(DateInterval::createFromDateString('friday'));
      $closing->add(DateInterval::createFromDateString(sprintf('+%s hours %s minutes', $hours, $minutes)));

      list($hours, $minutes, $seconds) = explode(':', $this->get('week_end_opening_time'));
      $opening = clone $monday;
      $opening->add(DateInterval::createFromDateString('sunday'));
      $opening->add(DateInterval::createFromDateString(sprintf('+%s hours %s minutes', $hours, $minutes)));

      return $current > $closing && $current < $opening;
    }

}