<?php

use Transfercar\Email\Sender\RequestNotificationEmail;

class tcEmailNotificationEvents
{

    /**
     * Connect callbacks to request and relocation events to the symfony event dispatcher
     *
     * @param sfEventDispatcher $event
     */
    public static function register(sfEventDispatcher $dispatcher)
    {
        $dispatcher->connect('relocation.confirmed', array('Transfercar\Email\Sender\RelocationConfirmedEmail', 'notify'));
        $dispatcher->connect('relocation.accepted', array('Transfercar\Email\Sender\RelocationAcceptedEmail', 'queue'));
        $dispatcher->connect('relocation.accepted', array('Transfercar\Email\Sender\RelocationAcceptedEmail', 'notifyRo'));
        $dispatcher->connect('request.sent', array('Transfercar\Event\RequestEvents', 'listenToRequestSent'));
        $dispatcher->connect('requests.someone_else_accepted', array('tcEmailNotificationEvents', 'listenToSomeElseAccepted'));
        // Cancellations triggered by users (RO or Driver)
        $dispatcher->connect('request.cancelled', array('Transfercar\Email\Sender\RequestCancelledEmails', 'notify'));
        // Automatic cancellations: overlapping or rejected requests
        $dispatcher->connect('request.cancelled', array('tcEmailNotificationEvents', 'listenToRequestCancelled'));
        $dispatcher->connect('request.rejected', array('tcEmailNotificationEvents', 'listenToRequestRejected'));
        $dispatcher->connect('request.expired', array('tcEmailNotificationEvents', 'listenToRequestExpired'));
        $dispatcher->connect('request.rated', array('tcEmailNotificationEvents', 'listenToRequestRated'));
    }

    /**
     * Notify ROs that another RO accepted a driver before them
     *
     * If the RO has multiple overlapping request from the same driver,
     * only one email containing all the cancelled requests must be sent
     * out to prevent a lot of unnecessary emails
     *
     * Event subject must be an array of requests
     *
     * @param sfEvent $event
     */
    public static function listenToSomeElseAccepted(sfEvent $event)
    {
        $requests = $event->getSubject();

        if (!is_array($requests) || empty($requests))
        {
            return;
        }

        $request  = $requests[0];
        $ro       = $request->get('Listing')->get('Owner');

        $notification = new RequestNotificationEmail($request, $requests);
        $notification->send('someone-else-accepted', $ro);
    }

    /**
     * Listen to the request.cancelled event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestCancelled(sfEvent $event)
    {
        // handle only automatic cancellation for overlapping request or not accepted
        if (!$event->getSubject()->isStatus(array(Request::STATUS_CANCELED))) {
            return;
        }

        self::sendDriverNotification($event, 'request-canceled');
    }

    /**
     * Listen to the request.rejected event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestRejected(sfEvent $event)
    {
        self::sendDriverNotification($event, 'request-rejected');
    }

    /**
     * Listen to the request.expired event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestExpired(sfEvent $event)
    {
        # self::sendRONotification($event, 'driver-request-expired');
        self::sendDriverNotification($event, 'request-expired');
    }

    /**
     * Listen to the request.rated event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestRated(sfEvent $event)
    {
        self::sendDriverNotification($event, 'relocation-closed');
        self::sendTcNotification($event, 'request-rated');
    }

    /**
     * Calls self::sendNotification with the driver as the email recipient
     *
     * @param sfEvent $event the original sfEvent
     * @param string $template The email template name without the domain
     */
    private static function sendDriverNotification(sfEvent $event, $template)
    {
        self::sendNotification($event, $template, $event->getSubject()->get('Driver'));
    }

    /**
     * Calls self::sendNotification with the RO as the email recipient
     *
     * @param sfEvent $event the original sfEvent
     * @param string $template The email template name without the domain
     */
    private static function sendRONotification(sfEvent $event, $template)
    {
        self::sendNotification($event, $template, $event->getSubject()->get('Listing')->get('Owner'));
    }

    /**
     * Sends the email notification defined by the template name to the recipient
     *
     * @param sfEvent $event the original sfEvent
     * @param string $template The email template name without the domain
     */
    private static function sendNotification(sfEvent $event, $template, $to)
    {
        $notification = new RequestNotificationEmail($event->getSubject());
        $notification->send($template, $to);
    }

    private static function sendTcNotification(sfEvent $event, $template)
    {
        $request = $event->getSubject();
        if ($request->get('rating') < 0) { 
          $driver = $request->get('Driver');
          Transfercar\Slack\SlackNotification::sendMessage('Driver '.$driver->full_name().' had a negative review', [
            'Rental Operator' => $request->get('Listing')->get('Owner')->get('company'),
            'Driver name' => $driver->full_name(),
            'Driver email' => $driver->get('email'),
            'TCID' => $request->get('listing_id')
          ], 'driver-reviews');
        }
    }
}
