<?php

class tcRequestSmsEvents extends tcSmsEvent
{

    /**
     * Listen to the request.confirmed event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRelocationConfirmed(sfEvent $event)
    {
        $listing = $event->getSubject();
        $request = $listing->getAcceptedRequest();

        self::sendRequestNotification(
            $request,
            'request_accepted',
            array(
                'contact_phone' => $listing->getNewRequestContact()->get('phone'),
                'contact_name'  => $listing->getNewRequestContact()->getFullName(),
            )
        );
    }

    /**
     * Listen to the request.cancelled event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestCancelled(sfEvent $event)
    {
        $request = $event->getSubject();

        if (!$request->isStatus(array(Request::STATUS_CANCELED, Request::STATUS_DRIVER_CANCELED, Request::STATUS_OWNER_CANCELED))) {
            return;
        }

        self::sendRequestNotification(
            $request,
            'request_cancelled',
            array(
                'reason' => self::getCancellationReason($request)
            )
        );
    }

    private static function getCancellationReason(Request $request)
    {
        switch ($request->get('status'))
        {
            case Request::STATUS_CANCELED:
                return 'overlaps with another request';
            case Request::STATUS_DRIVER_CANCELED:
                return 'rental operator cancelled the request on your behalf';
            default:
            case Request::STATUS_OWNER_CANCELED:
                return 'rental operator cancelled the relocation';
        }
    }

    /**
     * Listen to the request.rejected event and send a notification to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToRequestRejected(sfEvent $event)
    {
        self::sendRequestNotification(
            $event->getSubject(),
            'request_rejected'
        );
    }

    public static function sendRequestNotification($request, $template_name, $template_data = array())
    {
        // don't send sms to user without a verified phone number, or with sms notifications disabled
        if (!$request->get('Driver')->hasPhoneVerified() || !$request->get('Driver')->get('sms_notify_update_req')) {
            return;
        }

        sfLoader::loadHelpers(array('Date'));
        $domain = $request->getCountryDomain();
        self::send(
            $template_name,
            array_merge(
                array(
                    'tcid'      => $request->getIncrementedId(),
                    'from'      => $request->get('Listing')->get('PickupLocation')->get('name'),
                    'from_date' => format_date($request->get('pickup_date'), 'd', 'en_'.strtoupper($domain)),
                    'to'        => $request->get('Listing')->DropoffLocation->name,
                    'to_date'   => format_date($request->get('drop_off_date'), 'd', 'en_'.strtoupper($domain)),
                ),
                $template_data
            ),
           $request->get('Driver')->get('mobile_phone')
        );
    }
}