<?php

class CreditsUsageStats
{
    private $conn;

    public function __construct($conn)
    {
        $this->conn = $conn;
    }

    public function getStats($type, $from, $to, $country)
    {
        switch ($type) {
            case 'daily':
                $query = $this->getDailyQuery();
                break;

            case 'weekly':
                $query = $this->getWeeklyQuery();
                break;

            case 'monthly':
                $query = $this->getMonthlyQuery();
                break;
            default:
                throw new Exception("Unknow query type: ".$type);
                break;
        }

        $from = $this->getDate($from);
        $to   = $this->getDate($to);


        $params = array($from, $to);

        if ($country) {
            $query = str_replace('WHERE', 'WHERE u.country_domain_id = ? AND', $query);
            array_unshift($params, $country);
        }

        $query = $this->conn->prepare($query);
        $query->execute($params);

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    public function getDailyQuery()
    {
        return 'SELECT day(when_accepted) as day, month(when_accepted) as month, year(when_accepted) as year,
                  sum(price) as total_credits,
                  sum(CASE WHEN u.country_domain_id = 1 THEN v.credits_per_additional_day_nz * r.nb_paid_days ELSE v.credits_per_additional_day_au * r.nb_paid_days END) as paid_days_credits
                  FROM listing l
                  INNER JOIN user u on l.owner_id=u.id
                  INNER JOIN request r on r.listing_id=l.id and r.status in (1, 4)
                  INNER JOIN vehicle_type v on l.vehicle_type_id=v.id
                  WHERE when_accepted >= ? AND when_accepted <= ?
                  GROUP BY day, month, year
                  ORDER BY year, month, day';
    }

    public function getWeeklyQuery()
    {
        return 'SELECT week(when_accepted) as week, month(when_accepted) as month, year(when_accepted) as year,
                  sum(price) as total_credits,
                  sum(CASE WHEN u.country_domain_id = 1 THEN v.credits_per_additional_day_nz * r.nb_paid_days ELSE v.credits_per_additional_day_au * r.nb_paid_days END) as paid_days_credits
                  FROM listing l
                  INNER JOIN user u on l.owner_id=u.id
                  INNER JOIN request r on r.listing_id=l.id and r.status in (1, 4)
                  INNER JOIN vehicle_type v on l.vehicle_type_id=v.id
                  WHERE when_accepted >= ? AND when_accepted <= ?
                  GROUP BY week, year
                  ORDER BY year, month, week';
    }

    public function getMonthlyQuery()
    {
        return 'SELECT month(when_accepted) as month, year(when_accepted) as year,
                  sum(price) as total_credits,
                  sum(CASE WHEN u.country_domain_id = 1 THEN v.credits_per_additional_day_nz * r.nb_paid_days ELSE v.credits_per_additional_day_au * r.nb_paid_days END) as paid_days_credits
                  FROM listing l
                  INNER JOIN user u on l.owner_id=u.id
                  INNER JOIN request r on r.listing_id=l.id and r.status in (1, 4)
                  INNER JOIN vehicle_type v on l.vehicle_type_id=v.id
                  WHERE when_accepted >= ? AND when_accepted <= ?
                  GROUP BY month, year
                  ORDER BY year, month';
    }

    private function getDate($date)
    {
      if (!is_numeric($date)) {
        $date = strtotime($date);
      }

      return date('Y-m-d', $date);
    }
}