<?php

class PaymentStats
{
	private $conn;

	public function __construct($conn)
	{
		$this->conn = $conn;
	}

	public function getStats($type, $from, $to, $country)
	{
		switch ($type) {
			case 'daily':
				$query = $this->getDailyQuery();
				break;

			case 'weekly':
				$query = $this->getWeeklyQuery();
				break;

			case 'monthly':
				$query = $this->getMonthlyQuery();
				break;
			default:
				throw new Exception("Unknow query type: ".$type);
				break;
		}

        $from = $this->getDate($from);
        $to   = $this->getDate($to);

        $params = array($from, $to);

        if ($country) {
        	$query = str_replace('WHERE', 'WHERE country_id = ? AND', $query);
        	array_unshift($params, $country);
        }

        $query = $this->conn->prepare($query);
        $query->execute($params);

        return $query->fetchAll(PDO::FETCH_OBJ);
	}

	public function getDailyQuery()
	{
		return 'SELECT day, month, year,
				  sum(nb_payments) as nb_payments, sum(amount_nz) as amount_nz, sum(amount_au) as amount_au
				  FROM tc_payments_stats
				  WHERE date >= ? AND date <= ?
				  GROUP BY day, month, year
				  ORDER BY year, month, day';
	}

	public function getWeeklyQuery()
	{
		return 'SELECT week, month, year,
				  sum(nb_payments) as nb_payments, sum(amount_nz) as amount_nz, sum(amount_au) as amount_au
				  FROM tc_payments_stats
				  WHERE date >= ? AND date <= ?
				  GROUP BY week, year
				  ORDER BY year, month, week';
	}

	public function getMonthlyQuery()
	{
		return 'SELECT month, year,
				  sum(nb_payments) as nb_payments, sum(amount_nz) as amount_nz, sum(amount_au) as amount_au
				  FROM tc_payments_stats
				  WHERE date >= ? AND date <= ?
				  GROUP BY month, year
				  ORDER BY year, month';
	}


    private function getDate($date)
    {
      if (!is_numeric($date)) {
        $date = strtotime($date);
      }

      return date('Y-m-d', $date);
    }
}