<?php

class TcChart
{
	public static function build($stats, $type)
	{
		$series = array(
            'nb_new_listings' => array('name' => 'Listings', 'data' => array()),
            'nb_new_requests' => array('name' => 'Requests', 'data' => array()),
            'nb_relos' => array('name' => 'Relocations', 'data' => array()),
            'nb_no_show' => array('name' => 'No-shows', 'data' => array()),
            'nb_rejected' => array('name' => 'Rejected requests', 'data' => array()),
		);

		$categories = array();

		foreach ($stats as $stat)
		{
			switch ($type) {
				case 'daily':
					$categories[] = $stat->day .'/'.$stat->month;
					break;

				case 'weekly':
					$categories[] = $stat->week;
					break;

				case 'monthly':
					$categories[] = $stat->month.'/'.$stat->year;
					break;

				default:
					throw new Exception("unknown type ".$type, 1);

			}

            $series['nb_new_listings']['data'][] = (int) $stat->nb_new_listings;
            $series['nb_new_requests']['data'][] = (int) $stat->nb_new_requests;
            $series['nb_relos']['data'][] = (int) $stat->nb_relos;
            $series['nb_no_show']['data'][] = (int) $stat->nb_no_show;
            $series['nb_rejected']['data'][] = (int) $stat->nb_rejected;
		}

		return array($series, $categories);
	}

	public static function buildDonut($stats)
	{
		$data = array();
		$owners = array();

		foreach ($stats as $i => $stat) {
			$owners[] = $stat->username;
			$data[] = array(
				'y' => (int) $stat->total,
                'drilldown' => array(
                    'name' => $stat->username.' stats',
                    'categories' => array('Listings', 'requests', 'Relos', 'No show', 'Rejected'),
                    'data' =>  array(
                    	(int) $stat->nb_listings,
                    	(int) $stat->nb_requests,
                    	(int) $stat->nb_relos,
                    	(int) $stat->nb_no_show,
                    	(int) $stat->nb_rejected
                    )
                )
            );
		}

		return array($data, $owners);
	}

	public static function buildUserChart($stats, $type)
	{
		list($series, $categories) = self::buildPaymentChart($stats, array(), $type);
		$series['payment']['name'] = 'Users';

		return array($series, $categories);
	}

	public static function buildPaymentChart($paymentStats, $creditStats, $type)
	{
		$series = array(
            'nb_payments' => array('name' => 'Transactions', 'data' => array()),
            'amount_nz' => array('name' => '$NZ amount', 'data' => array()),
            'amount_au' => array('name' => '$AU amount', 'data' => array()),
            'credits_used' => array('name' => 'Total credits', 'data' => array()),
            'listing_credits_used' => array('name' => 'Listing credits', 'data' => array()),
            'upselling_credits_used' => array('name' => 'Upselling credits', 'data' => array())
		);

		$categories = self::getCategories($type, array($paymentStats, $creditStats));

        foreach (array($paymentStats, $creditStats) as $stats)
        {
    		foreach ($stats as $stat)
    		{
    			switch ($type) {
    				case 'daily':
    					$category = $stat->day .'/'.$stat->month;
    					break;

    				case 'weekly':
    					$category = $stat->week;
    					break;

    				case 'monthly':
    					$category = $stat->month.'/'.substr($stat->year, -2);
    					break;

    				default:
    					throw new Exception("unknown type ".$type, 1);

    			}

                $idx = (int) array_search($category, $categories);

                if (!isset($series['nb_payments']['data'][$idx])) {
                    $series['nb_payments']['data'][$idx] = 0;
                    $series['amount_au']['data'][$idx] = 0;
                    $series['amount_nz']['data'][$idx] = 0;
                    $series['credits_used']['data'][$idx] = 0;
                    $series['listing_credits_used']['data'][$idx] = 0;
                    $series['upselling_credits_used']['data'][$idx] = 0;
                }

                if (property_exists($stat, 'nb_payments')) {
                    $series['nb_payments']['data'][$idx] = (int) $stat->nb_payments;
                    $series['amount_au']['data'][$idx] = (int) $stat->amount_au;
                    $series['amount_nz']['data'][$idx] = (int) $stat->amount_nz;
                } else {
                    $series['credits_used']['data'][$idx] = (int) $stat->total_credits;
                    $series['listing_credits_used']['data'][$idx] = (int) $stat->total_credits - (int) $stat->paid_days_credits;
                    $series['upselling_credits_used']['data'][$idx] = (int) $stat->paid_days_credits;
                }
    		}
        }

        foreach ($series as $key => $serie) {
            ksort($serie['data'], SORT_NATURAL);
            $series[$key]['data'] = array_values($serie['data']);
        }

		return array($series, $categories);
	}

    private static function getCategories($type, $aggregatedStats)
    {
        $categories = array();

        foreach ($aggregatedStats as $stats)
        {
            foreach ($stats as $stat)
            {
                switch ($type) {
                    case 'daily':
                        $categories[] = $stat->day .'/'.$stat->month;
                        break;

                    case 'weekly':
                        $categories[] = $stat->week;
                        break;

                    case 'monthly':
                        $categories[] = $stat->month.'/'.substr($stat->year, -2);
                        break;

                    default:
                        throw new Exception("unknown type ".$type, 1);

                }
            }
        }

        $categories = array_unique($categories);

        sort($categories, SORT_NATURAL);

        return $categories;
    }
}