<?php

class TcUserStats
{
	private $conn;

	public function __construct($conn)
	{
		$this->conn = $conn;
	}

	public function getStats($type, $from, $to, $country = 1)
	{
        $from = $this->getDate($from);
        $to   = $this->getDate($to);

        switch ($type) {
          case TcStats::DAILY_QUERY:
            $query = $this->getDailyQuery();
            break;
          case TcStats::WEEKLY_QUERY:
            $query = $this->getWeeklyQuery();
            break;
          case TcStats::MONTHLY_QUERY:
            $query = $this->getMonthlyQuery();
            break;
          default:
            throw new Exception("unknown type ".$searchType);
        }

        $query = $this->conn->prepare($query);
        $query->execute(array($from, $to, $country));

        return $query->fetchAll(PDO::FETCH_OBJ);
	}

	public function getDailyQuery()
	{
		return 'SELECT day(when_registered) as day, month(when_registered) as month, year(when_registered) as year, count(*) as total
				  FROM user
				  WHERE when_registered >= ?
                  AND when_registered <= ?
				  AND country_domain_id = ?
				  GROUP BY day, month, year
				  ORDER BY year, month, day';
	}

    public function getWeeklyQuery()
    {
            return 'SELECT week(when_registered) as week, month(when_registered) as month, year(when_registered) as year, count(*) as total
                              FROM user
                              WHERE when_registered >= ?
                              AND when_registered <= ?
                              AND country_domain_id = ?
                              GROUP BY week, year
                              ORDER BY year, month, week';
    }

    public function getMonthlyQuery()
    {
            return 'SELECT month(when_registered) as month, year(when_registered) as year, count(*) as total
                              FROM user
                              WHERE when_registered >= ?
                              AND when_registered <= ?
                              AND country_domain_id = ?
                              GROUP BY month, year
                              ORDER BY year, month';
    }

    private function getDate($date)
    {
      if (!is_numeric($date)) {
        $date = strtotime($date);
      }

      return date('Y-m-d', $date);
    }
}