<?php

class UpdatePaymentStats
{
    private $conn;

    public function __construct($conn)
    {
        $this->conn = $conn;
    }

    public function update($lastUpdateTs)
    {
        $lastUpdateTs = strtotime($lastUpdateTs);
        $query = 'SELECT country_id, day(date) as day, month(date) as month, year(date) as year, week(date) as week,
                 count(*) as nb_payments,
                 sum(CASE when country_id = 1 THEN national_currency_amount ELSE 0 END) as amount_nz,
                 sum(CASE when country_id = 2 THEN national_currency_amount ELSE 0 END) as amount_au,
                 date
                 FROM payments
                 INNER JOIN user u on payments.user_id=u.id
                 WHERE date = ?
                 AND status = ?
                 AND type = ?
                 AND country_id = ?
                 GROUP BY day, month, year
                 ORDER BY year, month, day';

        $inserted = 0;
        for ($ts = $lastUpdateTs; $ts <= time(); $ts = strtotime('+1 day', $ts))
        {
            $date = date('Y-m-d', $ts);
            foreach(array(1, 2) as $country_id) {
                $params = array($date, Payments::STATUS_ACCEPTED, 0, $country_id);
                $pQuery = $this->conn->prepare($query);
                $pQuery->execute($params);
                $inserted += $this->updateStat($pQuery->fetch(PDO::FETCH_ASSOC), $ts, $country_id);
            }
        }

        return array('name' => 'payment', 'inserted' => $inserted);
    }

    public function updateStat($stat, $ts, $country_id)
    {
        $insertQuery = 'REPLACE into tc_payments_stats
                (country_id, day, month, year, week, nb_payments, amount_nz, amount_au, `date`)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)'
        ;

        if (!$stat)
        {
            $stat = array(
                'country_id' => $country_id,
                'day' => date('d', $ts),
                'month' => date('m', $ts),
                'year' => date('Y', $ts),
                'week' => date('W', $ts),
                'nb_payments' => 0,
                'amount_nz' => 0,
                'amount_au' => 0,
                'date' => date('Y-m-d', $ts)
            );
        }

        $query = $this->conn->prepare($insertQuery);
        $query->execute(array_values($stat));

        return $query->rowCount();
    }
}