<?php

use Transfercar\Mailchimp\MailchimpApi;

class BaseUserSettingsActions extends sfActions
{
   /**
    * Show current user account balance history.
    * Also show links for GST receipt for credit
    * card income transactions.
    *
    * @param sfWebRequest $request HTTP request object
    */
   protected function doExecuteAccountStatement($request, $user)
   {
       $this->user = $user;

       // Get pager info
       $items_per_page = $request->getParameter('nb_per_page', sfConfig::get('sf_payment_acc_stmnt_items_per_page'));
       $page = intval($request->getParameter('page', 1));

       $userId = $user->getOwnerId();
       // get information about payments
       $query = Doctrine_Query::create()
         ->from('Payments p')
         ->innerJoin('p.User u')
         ->where('user_id = ?', $userId)
         ->orderBy('id DESC');

       $this->addAccountStatementFilters($query, $request, $userId);
       $balanceQuery = $query->copy();

       $this->pager = new Doctrine_Pager($query, $page, $items_per_page);
       $this->pager->execute();

       // There is one more thing to do: we have to get all earlier
       // payment records in order to calculate opening account balance

       // Find the highest ID from fetched array
       $id = 0;
       $starting_balance = 0;

       foreach($this->pager->execute() as $item){
           $id = max($id, $item->id);
       }

       // get rest of payment records
       $rest = Doctrine_Query::create()
         ->from('Payments')
         ->where('user_id = ?', $userId)
         ->addWhere('status != ?', Payments::STATUS_REJECTED)
         ->addWhere('id <= ?', $id)
         ->execute();

       foreach($rest as $item){
          if ($item->status == 1) {
           $starting_balance += $item->amount;
          }
       }



       $debits = $balanceQuery->copy()->select('SUM(amount) as amount')
        ->addWhere('amount < 0')
        ->execute(array(), Doctrine::HYDRATE_NONE);

       $this->debits = $debits[0][0];
       $credits = $balanceQuery->select('SUM(amount) as amount')
        ->addWhere('amount > 0')
        ->execute(array(), Doctrine::HYDRATE_NONE);

        $this->credits = $credits[0][0];

       $this->starting_balance = $starting_balance;

       return sfView::SUCCESS;
  }

  protected function addAccountStatementFilters($query, $request, $userId)
  {
    if ($date = $request->getParameter('statement_date')) {
      if (strpos($date, ',') !== false)
      {
        $query->addWhere('date between ? and ?', explode(',', $date));
      }
      else
      {
        $query->addWhere('date = ?', $date);
      }
    }

    if ($country = $request->getParameter('statement_country')) {
      $query->addWhere('country_id = ?', $country);
    }

    if ($manager = $request->getParameter('statement_manager')) {
      $q = Doctrine_Query::create()
        ->select('l.id')
        ->from('Listing l')
        ->addWhere('l.owner_id = ?', $userId);

      if ('null' == $manager)
      {
        $q->addWhere('listed_by is null');
      }
      else
      {
        $q->addWhere('listed_by = ?', $manager);
      }

      $ids = $q->execute(array(), Doctrine::HYDRATE_ARRAY);

      $listingIds = array(-1);
      foreach ($ids as $id) {
        $listingIds[] = -$id['id'];
      }

      $query->whereIn('assigned_record_id', $listingIds);
    }

  }

  protected function doExecuteEmailOptions(sfWebRequest $request, $user, $updateUserAttributes = false)
  {
    $this->getUser()->setAttribute('route_added', false);
    $this->user = $user;
    $account = Doctrine::getTable('NotificationUser')->findAccountByEmail($user->get('email'));
    $this->uid = $this->user->id;
    $this->routes = Doctrine_Query::create()
        ->from('NotifyRoute nr')
        ->leftJoin('nr.LocationFrom lf')
        ->leftJoin('nr.LocationTo lt')
        ->where('nr.user_id = ?', $this->uid)
        ->addWhere('nr.alert_type = ?', 'email')
        ->orderBy('nr.id DESC')
        ->execute();

    if(!$request->isMethod('POST'))
    {
      return sfView::SUCCESS;
    }

    $email_options = $request->getParameter('email_options', array());

    //Get all countries(for newsletter option)
    $countries = LocationCountry::getAll();

    $this->forward404Unless(is_array($email_options));

    // Save the new values with simple validation
    $user = $this->user;
    $user->email_notify_format =
       @$email_options['format'] == 'plain' ? 'plain' : 'html';
    $user->email_newsletter = isset($email_options['newsletter']);

    $user_email = trim($this->user->email) == '' ? $this->user->new_email : $this->user->email;

    if(isset($email_options['newsletter'])) {
        if($user->email_country_newsletter != $email_options['country_newsletter']) {

          $mc = new MailchimpApi();

          foreach ($countries as $item){
              $domain = $item["domain"];
              $list_id = sfConfig::get('app_mailchimp_' . $item['domain'] . '_list_apikey');
              if ($this->getValue('newsletter') && in_array($this->getValue('newsletter_country'), array('all', $country['id'])))
              {
                  $mc->subscribe($list_id, $user_email, $this->user->get('firstname'), $this->user->get('lastname'));
              }
              else
              {
                  $mc->unsubscribe($list_id, $user_email);
              }
          }

          $user->email_country_newsletter = $email_options['country_newsletter'];
        }
    }
    else
    {
        if($user->email_country_newsletter != 'none')
        {
            foreach ($countries as $item)
            {
                $domain = $item["domain"];
                $mc = new MailchimpApi();
                $list_id = sfConfig::get('app_mailchimp_' . $item['domain'] . '_list_apikey');
                $mc->unsubscribe($list_id, $user_email);
            }
        }

        $user->email_country_newsletter = 'none';
    }

    $user->email_notify_new_req = isset($email_options['new_request']);
    $user->email_notify_update_req = isset($email_options['update_req']);

    // Notifications about new relocations
    // If user choosen "none" or "all", remove all NotifyRoute records
    // associated with user
    if (isset($email_options['new_reloc']))
    {
      if (@$email_options['type_freq'] == 'instance')
      {
        $user->email_notify_type_freq = 'instance';
        $user->email_notify_new_reloc = 'routes';
        $user->email_country_new_reloc = $email_options['country_instance_new_reloc'];
        $remove_routes = false;
      }
      else
      {
        $user->email_notify_type_freq = 'daily';
        $user->email_notify_new_reloc = 'all';
        $user->email_country_new_reloc = $email_options['country_daily_new_reloc'];
        $remove_routes = true;
      }
    }
    else
    {
      $user->email_notify_new_reloc = 'none';
      $user->email_country_new_reloc = 'none';
      $remove_routes = true;
    }

    if($remove_routes)
    {
      Doctrine_Query::create()
         ->delete('NotifyRoute')
         ->from('NotifyRoute')
         ->where('user_id = ?', $user->id)
         ->addWhere('alert_type = ?', 'email')
         ->execute();
      $this->routes = array();
    }

    $user->save();

    if ($updateUserAttributes) {
        $this->getUser()->setAttribute('profile', $user);
        $this->getUser()->setAttribute('flash_notice', 'E-mail options have been updated');
    }
    // Display updated page
    $request->getParameterHolder()->clear();
    $request->setParameter('id', $user->id);
  }


   /**
    * Add new observed route (for notifications)
    * Support watching for e-mails and text alerts.
    * Redirect to email/text alert options when finish
    *
    * @param sfWebRequest $request HTTP request object
    */
   protected function doExecuteAddWatchRoute($request, $user)
   {
     $route = $request->getParameter('add_route');
     $this->forward404Unless(is_array($route));

     // Check if we have valid alert type
     $alert_type = 'email';// $route['type'];

     // Get valid redirect address

     $redirect_url = 'userSettings/emailOptions?layout=0&id='.$user->id;

     // Make simple validation - check if user has already
     // chosen requested route
     // $user = $this->getUser()->getAttribute('profile');
     $r['from']      = Doctrine::getTable('LocationNode')->find($route['from']);
     $r['to']        = Doctrine::getTable('LocationNode')->find($route['to']);


    $account = Doctrine::getTable('NotificationUser')->findAccountByEmail($user->get('email'));

    if (!$account->exists()) {
      $account->set('country_id', $user->get('country_domain_id'));
      $account->set('frequence', 'daily');
    } else {

      $count = Doctrine_Query::create()
        ->from('NotifyRoute')
        ->where('user_id = ?', $account->getId())
        ->addWhere('location_from_id = ?', $r['from'])
        ->addWhere('location_to_id = ?', $r['to'])
        ->addWhere('alert_type = ?', $alert_type)
        ->count();

      $this->redirectIf($count > 0, $redirect_url);
    }

    // OK, all data are valid, we can make new notify route object
    $route = new NotifyRoute();
    $route->User = $account;
    $route->location_from_id = $r['from'];
    $route->location_to_id = $r['to'];
    $route->alert_type = $alert_type;
    $route->save();

    $this->getUser()->setAttribute('route_added', true);
    //$this->redirect('userSettings/emailOptions?id='.$user->id);
    $this->forward('userSettings', 'emailOptions');
    // $this->redirect($redirect_url . "?layout=0");
  }


  /**
   * Delete specified route(s) from observed list
   *
   * @param sfWebRequest $request HTTP request object
   */
    protected function doExecuteDeleteWatchRoute($request, $user)
    {
        $routes = $request->getParameter('del_route');
        $account = Doctrine::getTable('NotificationUser')->findAccountByEmail($user->get('email'));

        $this->forward404Unless($account);
        $this->forward404If(!is_array($routes));

        // $user = $this->getUser()->getAttribute('profile');

        // Routes id to delete are saved as array keys
        $alert_type = 'email';
        $ids2del = array_keys($routes);

        $redirect_url = 'user/emailOptions';

        $ids2del[] = -1; // We don't like empty arrays

        // Filter IDs
        $filtered = array();
        foreach($ids2del as $id){
            $filtered[] = intval($id);
        }



        // Delete all records from DB
        Doctrine_Query::create()
            ->delete('NotifyRoute')
            ->from('NotifyRoute')
            ->where('user_id = ?', $account->id)
            ->addWhere('alert_type = ?', $alert_type)
            ->addWhere('id IN ('.implode(',', $filtered).')') // id in (2,3,54,-1)
            ->execute();

        $this->getUser()->setAttribute('route_added', true); // It's funny, but in this way we will recover all
                                                             // required data on the template
        $this->forward('userSettings', 'emailOptions');
        //$this->redirect($redirect_url . '?layout=0');

  }

  protected function doExecuteCreditCards($request, $user)
  {
    $this->credit_cards = $user->getCreditCards();
    $this->user = $user;
  }

  protected function doExecuteDeleteCreditCard($request, $creditCard, $user)
  {
    $this->forward404Unless($creditCard && $creditCard->get('owner_id') == $user->get('id'));

    $creditCard->delete();

    $this->redirect('userSettings/creditCards?id='.$user->get('id'));
  }

}
