<?php

use Transfercar\Mailchimp\MailchimpApi;

class EmailOptionsForm extends BaseForm
{
    public function configure()
    {
        $countries = array('none' => 'Please choose...', 'all' => 'Any country');
        $countries += LocationCountry::getAllIndexedByIdAndName();

        $this->setWidgets(array(
            'newsletter'         => new sfWidgetFormInputCheckbox(),
            'newsletter_country' => new sfWidgetFormChoice(array('choices' => $countries)),
            'notify_new_reloc'   => new sfWidgetFormInputCheckbox(),
            'notify_frequence'   => new sfWidgetFormChoice(array('choices' => array('instant', 'daily'), 'expanded' => true)),
            'notify_country_instant' => new sfWidgetFormChoice(array('choices' => $countries)),
            'notify_country_daily'   => new sfWidgetFormChoice(array('choices' => $countries)),
        ));

        $this->setValidators(array(
            'newsletter'             => new sfValidatorBoolean(array('required' => false)),
            'newsletter_country'     => new sfValidatorChoice(array('required' => false, 'choices' => array_keys($countries))),
            'notify_new_reloc'       => new sfValidatorBoolean(array('required' => false)),
            'notify_frequence'       => new sfValidatorChoice(array('required' => false, 'choices' => array('instant', 'daily'))),
            'notify_country_instant' => new sfValidatorChoice(array('required' => false, 'choices' => array_keys($countries))),
            'notify_country_daily'   => new sfValidatorChoice(array('required' => false, 'choices' => array_keys($countries))),
        ));

        $this->setDefaultFromUser($this->getOption('user'));

        $this->widgetSchema->setNameFormat('email_options[%s]');
    }

    public function setDefaultFromUser($user)
    {
        $this->setDefaults(array(
            'newsletter'             => $user->get('email_newsletter'),
            'newsletter_country'     => $user->get('email_country_newsletter'),
            'notify_frequence'       => $user->get('email_notify_type_freq'),
            'notify_new_reloc'       => $user->get('email_notify_new_reloc') == 'none' ? null : $user->get('email_notify_new_reloc'),
            'notify_country_daily'   => $user->get('email_notify_type_freq') == 'daily' ? $user->get('email_country_new_reloc') : '',
            'notify_country_instant' => $user->get('email_notify_type_freq') == 'instant' ? $user->get('email_country_new_reloc') : '',
        ));
    }

    public function save($user)
    {
        $this->saveNewsletterSettings($user);
        $this->saveNotificationSettings($user);

        $user->save();
    }

    private function saveNewsletterSettings($user)
    {
        $countries = LocationCountry::getAll();
        $mc = new MailchimpApi();

        foreach ($countries as $country)
        {
            $list_id = sfConfig::get('app_mailchimp_' . $country['domain'] . '_list_apikey');
            if ($this->getValue('newsletter') && in_array($this->getValue('newsletter_country'), array('all', $country['id'])))
            {
                $mc->subscribe($list_id, $user->get('email'), $user->get('firstname'), $user->get('lastname'));
            }
            else
            {
                $mc->unsubscribe($list_id, $user->get('email'));
            }
        }

        $user->set('email_country_newsletter', $this->getValue('newsletter_country'));
        $user->set('email_newsletter', $this->getValue('newsletter'));
    }

    private function saveNotificationSettings($user)
    {
        $notify_new_reloc = 'none';
        if ($this->getValue('notify_new_reloc'))
        {
            $notify_new_reloc = $this->getValue('notify_frequence') == 'daily' ? 'all' : 'routes';
        }
        $user->set('email_notify_type_freq', $this->getValue('notify_frequence'));
        $user->set('email_notify_new_reloc', $notify_new_reloc);
        $user->set('email_country_new_reloc', $this->getValue('notify_country_daily'));

        if ($notify_new_reloc == 'none' && $user->get('NotificationAccount')->count())
        {
            $user->get('NotificationAccount')->delete();
        }
        elseif($notify_new_reloc != 'none' && !$user->get('NotificationAccount')->count())
        {
            $account = Doctrine::getTable('NotificationUser')->findAccountByEmail($user->get('email'));
            $account->set('country_id', $this->getValue('notify_country_daily'));
            $account->set('frequence', $this->getValue('notify_frequence'));
            $account->save();
        }
    }
}




