<?php

use Transfercar\Email\ExternLib;
use Transfercar\Stripe\TcStripeForm;

/**
 * payment actions.
 *
 * @package    sf_sandbox
 * @subpackage payment
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 9301 2008-05-27 01:08:46Z dwhittle $
 *
 *
 * Payment DB operation logging in log/payment.log file. We use sfFileLogger
 * for logging. In file we can see date and time and operation detail information
 */
class paymentActions extends BasePaymentAction
{
    /**
	 * Create new credit card transaction
	 * Only store information to database (set transaction as NEW)
	 * It will be processed by cron script.
	 * Ask user if he want to receive transaction receipt from
	 * payment gate operator.
	 * This action supports only new card operations,
	 * operations with stored cards are supported by
	 * payWithStoredCard
	 *
	 * @param sfWebRequest $request HTTP request object
	 */
	public function executePayWithCard($request)
	{
		$this->initPayment();
		$this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

		if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
			$this->redirect("payment/selectCredits");
		}
		$this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);

	    // If we have request for pay with stored
	    // card, redirect to the proper action
	    if($request->getParameter('balance_card', 'new') == 'existing'){
	        $this->forward('payment', 'payWithStoredCard');
	    }
	    $user = $this->getUser()->getAttribute('profile');

	    $this->form = new TcStripeForm(
	    	['amount' => $this->credit_info["price"]], 
	    	[
	    		'user_id'  => $user->getOwnerId(),
	    		'credits'  => $this->credit_info["credit"],
                'currency' => strtolower(sfConfig::get('app_currency_'.$user->get('Country')->get('domain'))),
                'country_id' => $this->getUser()->getAttribute('profile')->getCountryDomainId()
	    	]
	    );
        $this->amount = $this->credit_info["price"];

        if (!$request->hasParameter('payment')) {
        	return sfView::SUCCESS;
        }

	    $this->form->bind($request->getParameter('payment'));

        if(!$this->form->isValid()){
	        return sfView::SUCCESS;
	    }

		$particular = sprintf('Transfercar credits', $this->credit_info["price"]);
        $description = substr('CRD/'.$user->getCompany(), 0, 50);
	    if (!$cct = $this->form->processPayment($description, $particular, $this->form->getValues())){
	        //redirect to page with error info
	        return sfView::SUCCESS;
	    }

	    $this->transaction = $cct;
	    $this->setTemplate('checkTransactionStatus');
	    return sfView::SUCCESS;
	}

	/**
	 * Pay with stored card
	 *
	 * @param sfWebRequest $request HTTP request object
	 */
	public function executePayWithStoredCard($request)
	{
		$this->initPayment();
		$this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

		if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
			$this->redirect("payment/selectCredits");
		}

		$this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);
		$user = $this->getUser()->getAttribute('profile');
		// If we have request for pay with new
        // card, redirect to the proper action
        if($request->getParameter('balance_card', 'new') != 'existing'){
            $this->forward('payment', 'payWithCard');
        }

        if(!$request->hasParameter('card_id')){
            // Should we put default value for amount
            $this->amount = ($this->credit_info["price"]);
            if($this->amount == 0){
                $this->amount = '';
            }

            $this->cards = CreditCard::user_cards_list();
            return sfView::SUCCESS;
        }

        $card_id = intval($request->getParameter('card_id'));

        // Check if this card belongs to current user
        $card = Doctrine_Query::create()
            ->from('CreditCard')
            ->where('id = ?', $card_id)
            ->addWhere('owner_id = ?', $user->owner_id)
            ->addWhere('holder_name LIKE ?', 'src_%')
            ->fetchOne();

        if(!is_object($card)){
            $this->renderText('Invalid card id. Please contact with system administrator');
            return sfView::NONE;
        }

        // OK, form is valid, prepare CC transaction and
        // create payment record. By default
        // add requested amount to user balance
        // It will be decreased by cron script
        // if card transactions will fail
        $amount = ($request->getParameter('amount'));
        $receipt = intval($request->getParameter('receipt', 0));

        $user->refresh(); // we are updating 'account balance' field, which
                          // can be modified by cron script, therefore we
                          // have to have sure, that we have fresh data
        $this->form = new TcStripeForm(
	    	['amount' => $this->credit_info["price"]], 
	    	[
                'is_new_token' => false,
	    		'user_id' => $user->getOwnerId(),
	    		'credits' => $this->credit_info["credit"],
                'currency' => strtolower($user->get('Country')->get('domain').'d'),
                'country_id' => $this->getUser()->getAttribute('profile')->getCountryDomainId()
	    	], false
	    );

	$particular = sprintf('Transfercar credits', $this->credit_info["price"]);
        $description = substr('CRD/'.$user->getCompany(), 0, 50);
        $values = [
        	'amount' => $this->credit_info["price"],
        	'stripeToken' => $card->get('holder_name')
        ];
        $this->form->bind($values);
        if (!$this->form->isValid() || (!$cct = $this->form->processPayment($description, $particular, $values, $card))) {
        	$this->cards = CreditCard::user_cards_list();
        	return sfView::SUCCESS;
        }

	    $this->transaction = $cct;
	    $this->setTemplate('checkTransactionStatus');
	    return sfView::SUCCESS;
	}

    /**
	 * Executes index action
	 *
	 * @param sfRequest $request A request object
	 */
	public function executeIndex(sfWebRequest $request)
	{
	    $this->getUser()->setAttribute("payment_info", array());
		$this->forward("payment", "selectCredits");
	}

	/**
	 * Enter description here...
	 *
	 * @param sfWebRequest $request
	 */
	public function executeSelectCredits(sfWebRequest $request){
		$this->initPayment();
        $credit = $request->getParameter("credits");
		$this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId(), $credit);
		if ($request->isMethod("post") && !$request->hasParameter("support")){

			if (empty($credit) || !$this->checkValidCredit($credit, $this->credits)){
				$this->getUser()->setFlash("payment_error", "You didn't select the credit!");
			}else {
				$this->getUser()->setAttribute("payment_info", $credit);
				$this->redirect("payment/chargeAccount");
			}
		}

		$uid = $this->getUser()->getAttribute('profile')->id;
        $this->user = Doctrine_Query::create()
            ->from('User u')
            ->leftJoin('u.CreditCards')
            ->where('u.id = ?', $uid)
            ->fetchOne();
	}


	/*
	* Charge account - get required information and redirect
	* to payWithCard action
	*/
	public function executeChargeAccount()
	{
		$this->initPayment();
		$this->credits = VirtualCurrency::retreiveCreditsByCountryId($this->getUser()->getAttribute('profile')->getCountryDomainId());

		if (empty($this->payment_info) || !$this->checkValidCredit($this->payment_info, $this->credits)){
			$this->redirect("payment/selectCredits");
		}

		$this->credit_info = VirtualCurrency::retreiveCreditInfo($this->payment_info);

		$uid = $this->getUser()->getAttribute('profile')->id;

		// Get user info with informations about its credit cards
        $this->user = Doctrine_Query::create()
            ->from('User u')
            ->leftJoin('u.CreditCards')
            ->where('u.id = ?', $uid)
            ->fetchOne();

        return sfView::SUCCESS;
	}



	/*
	* Add new credit card into user list
	*/
	public function executeAddCard()
	{
		$this->forward404(); // TODO
	}

	/*
	* Delete credit card from list
	*/
	public function executeDeleteCard()
	{
		$this->forward404(); // TODO
	}

	/**
	* Show receipt for specified payment
	*
	* @param sfWebRequest $request HTTP request object
	*/
	public function executeReceipt($request)
	{

	    $user = $this->getUser()->getAttribute('profile');
		$id = intval($request->getParameter('id'));

		$payment = Doctrine_Query::create()
		  ->from('Payments')
		  ->where('id = ?', $id)
		  ->addWhere('user_id = ?', $user->owner_id)
		  ->addWhere('amount > ?', 0)
		  ->addWhere('type = ?', 'cc')
		  ->fetchOne();

        $this->forward404Unless(is_object($payment));

        // Calculate tax value and amount without tax
        $tax_value = sfConfig::get('app_gst_receipt_tax_value');
        $tax_value = 1 + ($tax_value / 100);
        $value_without_tax = $payment->national_currency_amount / $tax_value;
        // Round this value to full cents
        $value_without_tax = intval($value_without_tax * 100) / 100;

        // Add data for template
        $currency = " " . strtoupper($payment->Country->domain)."D";
        $this->value_without_tax = $value_without_tax . $currency;
        $this->value_with_tax = $payment->national_currency_amount . $currency;
        $this->tax_value = ($payment->national_currency_amount - $value_without_tax) . $currency;

        $this->payment_date = strtotime($payment->date);
        $this->payment_id = $payment->id;
        $this->user = $payment->get('User');

        $this->setLayout('layout_receipt');

        if (trim($currency) == 'NZD') {
        	$this->setTemplate('receiptWithGST');
        }

        return sfView::SUCCESS;
	}

	/**
	 * Call payment authorization script as background server process
	 * and wait until authorisation will finish
	 *
	 * @param sfWebRequest $request HTTP request object
	 */
	public function executeAuthorizeTransaction($request)
	{
	    $this->forward404Unless($request->hasParameter('id'));
        $this->forward404Unless($request->hasParameter('user_id'));
	    $this->id = intval($request->getParameter('id'));
        $user_id = intval($request->getParameter('user_id'));

	    // OK, run payment script as background process
	    $batch_dir = sfConfig::get('sf_root_dir').'/batch/';
	    $runner_name = $batch_dir.'process_payment.sh';
	    $script_name = 'flo2cash:process-payment frontend --env='.sfConfig::get('sf_environment');

	    system($runner_name.' '.$script_name.' '.$user_id);

	    $this->setLayout(false);
	    return sfView::SUCCESS;
	}

	/**
	 * Check transaction status
	 *
	 * @param sfWebRequest $request HTTP request object
	 */
	public function executeCheckTransactionStatus($request)
	{
	    $user = $this->getUser()->getAttribute('profile');
	    $id = $request->getParameter('id');

	    $this->setLayout(false);

	    $transaction = Doctrine_Query::create()
	       ->from('CCTransaction c')
	       ->leftJoin('c.Payment cp')
	       ->where('c.id = ?', $id)
	       ->addWhere('cp.user_id = ?', $user->getOwnerId())
	       ->fetchOne();

	    $this->forward404Unless(is_object($transaction));

	    // OK, if transaction has not been finished yet, wait...
	    if($transaction->status == CCTransaction::STATUS_NEW){
	        sleep(5);
	        $this->id = $id;
	        return 'Wait';
	    }

	    // OK, wait few seconds to make sure that all data has been
	    // saved by payment process
	    sleep(3);
	    $transaction->refresh();

	    $this->transaction = $transaction;

        // Update user object
        $user->refresh();
        $this->getUser()->setAttribute('profile', $user);

	    if($transaction->status == CCTransaction::STATUS_ACCEPTED){
	        return sfView::SUCCESS;
	    } else {
	        return sfView::ERROR;
	    }
	}

	/**
	* This is complementary command. The main aim - redirect to page with error message.
	* So, transaction was rollback, we can't use command executeCheckTransactionStatus
	*/
	public function executeErrorTransaction($request){
		$this->setLayout(false);
		return sfView::SUCCESS;
	}

}
