<?php

namespace Transfercar\Alert;

use Transfercar\PhpAmqpLib\tcRequestProducer;
use sfEventDispatcher;
use sfEvent;
use Doctrine;
use Listing;
use Alert;

class AlertEvents
{
    private static $dispatcher;

    public static function connect(sfEventDispatcher $dispatcher)
    {
        self::$dispatcher = $dispatcher;
        $dispatcher->connect('ro.listing.created', array(__CLASS__, 'listenToListingCreated'));
        $dispatcher->connect('ro.listing.updated', array(__CLASS__, 'listenToListingUpdated'));
    }

    public static function listenToListingCreated(sfEvent $event)
    {
        tcRequestProducer::publish($event->getSubject(), 'listing_created');
    }

    public static function createNotification(Listing $listing)
    {
        $users = Doctrine::getTable('NotificationUser')->findSubscribers(
            $listing->get('PickupLocation'),
            $listing->get('DropoffLocation'),
            $listing->get('date_available_from'),
            $listing->get('date_return_by')
        );

        $conn = Doctrine::getTable('Alert')->getConnection();
        $conn->beginTransaction();

        try
        {
            foreach ($users as $user)
            {
                $alert = Alert::create(Alert::NEW_LISTING, $user->getId(), $listing->getId(), time());
                $alert->save($conn);
                self::$dispatcher->notify(new sfEvent($alert, 'alert.create'));
                $alert->free(true);
            }

            $conn->commit();
        }
        catch (\Exception $e)
        {
            $conn->rollback();
            throw $e;
        }

        $users->free(true);
    }

    /**
     * Remove activities if listing status is not pending anymore (accepted, expired, etc)
     */
    public static function listenToListingUpdated(sfEvent $event)
    {
        tcRequestProducer::publish($event->getSubject(), 'listing_updated');
    }

    public static function removeNotification(Listing $listing)
    {
        if ($listing->isStatus(Listing::STATUS_NEW))
        {
            return;
        }

        $activitiesQuery = Doctrine::getTable('Alert')->removeActivitiesFor(Alert::NEW_LISTING, $listing->getId());
    }
}