<?php


require '../application/lib/3rdparty/symfony/event/sfEvent.class.php';
require '../application/lib/3rdparty/symfony/event/sfEventDispatcher.class.php';

require 'User.php';
require 'Listing.php';
require 'Activity.php';

function findSubcribers(Listing $listing)
{
    $users = User::$users;

    return array_filter(User::$users, function($user) use ($listing){
        return $user->isSubscribed($listing->from, $listing->to);
    });
}



$dispatcher = new sfEventDispatcher();

$dispatcher->connect('listing.create', function (sfEvent $event) use ($dispatcher) {
    $listing = $event->getSubject();
    $users = findSubcribers($listing);

    foreach ($users as $user) {
        $activity = Activity::create($event['activity_type'], $user->id, $listing->id, time());
        $dispatcher->notify(new sfEvent($activity, 'activity.create'));
    }
});

$dispatcher->connect('listing.accept', function (sfEvent $event) use ($dispatcher) {
    $listing = $event->getSubject();

    $activities = array_filter(Activity::$activities, function($activity) use ($listing) {
        return $listing->id == $activity->source_id;
    });

    foreach ($activities as $activity) {
        $activity->setBooked();
    }
});

$dispatcher->connect('activity.create', function (sfEvent $event) {
    $activity = $event->getSubject();

    printf("User %s is subscribed to %s\n", $activity->user_id, $activity->source_id);
});


$user = User::create('noel');
$user->subscribe('AKL', 'CHC');
$user->subscribe('', 'CHC');


$user = User::create('paul');
$user->subscribe('QST', 'CHC');
$user->subscribe('AKL', '');
$user->subscribe('', 'DUD');

foreach (['AKL', 'DUD', 'QST'] as $value) {
    $listing = Listing::create($value.'-CHC', $value, 'CHC');
    $dispatcher->notify(new sfEvent($listing, 'listing.create', array('activity_type' => Activity::NEW_LISTING)));
}

function sendEmails()
{
    print("send emails\n");
    foreach (User::$users as $user) {
        $activities = array_filter(Activity::$activities, function($activity) use ($user) {
            return $user->id == $activity->user_id && !$activity->isEmailed() && !$activity->isBooked();
        });

        foreach ($activities as $activity) {
            $activity->setEmailed();
            printf("activity %s for user %s is emailed\n", $activity->source_id, $user->id);
        }
    }
}

$dispatcher->notify(new sfEvent(Listing::$listings[0], 'listing.accept'));

sendEmails();

$listing = Listing::create('CHC-DUD', 'CHC', 'DUD');
$dispatcher->notify(new sfEvent($listing, 'listing.create', array('activity_type' => Activity::NEW_LISTING)));

sendEmails();
