<?php

namespace Transfercar\ElasticSearch;

use Doctrine;

class BrowseQuery
{
    /**
     * Manage all the business code originally in the browse action, should be refactored
     * to make it more maintainable.
     */
    public static function buildElasticSearch($action, $values)
    {
      $values = array_merge(array(
          'area' => false,
          'from_date' => false,
          'to_date' => false,
          'pickup' => false,
          'dropoff' => false,
          'vehicle_type' => false,
          'vehicle_type_filter' => false,
          'transmission' => false,
          'travellers' => false,
          'country' => false,
          'min_age' => false,
          'min_age_filter' => false,
          'fuel' => false,
          'ferry' => false,
          'other_extras' => false
        ), $values);
        $metaTitle = $action->getResponse()->getTitle();
        $metaAppend = '';
        $metas = $action->getResponse()->getMetas();
        @list($metaDescription1, $metaDescription2) = explode('.', $metas['description'], 2);

        $esQuery = [];
        $filters = [];

        if ($values['area'] && is_array($values['area']) && count($values['area']) === 4)
        {
          $values['area'] =  array_map('floatval', $values['area']);
          $filters[] = array('geo_bounding_box' => array(
            'pickupLocation.location' => array(
              'top_left' => array(
                'lat' => max($values['area'][0], $values['area'][2]),
                'lon' => min($values['area'][1], $values['area'][3])
              ),
              "bottom_right" => array(
                'lat' => min($values['area'][2], $values['area'][0]),
                'lon' => max($values['area'][3], $values['area'][1])
              )
            )
          ));
        }

        if ($values['from_date'])
        {
          // '10-01';
          // 'pickup <= 10-01 and dropoff >= 10-01'
          $filter_date = date('Y-m-d', strtotime($values['from_date']));
          $filters[] = array('range' => array(
            'from_date' => array(
              'gte' => $filter_date
            )
          ));
          $filters[] = array('range' => array(
            'to_date' => array(
              'gte' => $filter_date
            ),
          ));
        }

        if ($values['to_date'])
        {
          // '10-01';
          // 'dropoff <= 10-01 and dropoff >= 10-01'
          $filter_date = date('Y-m-d', strtotime($values['to_date']));
          $filters[] = array('range' => array(
            'from_date' => array(
              'lte' => $filter_date
            )
          ));
          // $filters[] = array('range' => array(
          //   'to_date' => array(
          //     'gte' => $filter_date
          //   ),
          // ));
        }

        if ($values['pickup'])
        {
          $from = trim($values['pickup']);
          $node = Doctrine::getTable('LocationNode')->findOneBySlug($values['pickup']);

          if ($node)
          {
            $metaAppend .= ' from '.$node->get('name');
            switch ($node->get('type'))
            {
              case 'departure':
                $filters[] = array(
                  'nested' => array(
                    'path' => 'pickupLocation.custom_departures',
                    'filter' => array(
                      'term' => array('pickupLocation.custom_departures.slug' => array('value' => $values['pickup'])))
                    )
                  )
                ;
                break;
              case 'category':
                $filters[] = array('term' => array('pickupLocation.state.slug' => array('value' => $values['pickup'])));
                break;
              case 'location':
                $filters[] = array('term' => array('pickupLocation.slug' => array('value' => $values['pickup'])));
                break;
            }
          } elseif (!empty($from) && strpos($from, ':any') === false) {
            $filters[] = array('term' => array('pickupLocation.name' => $from));
          }
        }

        if ($values['dropoff'])
        {
          $to = str_replace(array('#', '-'), ' ', trim($values['dropoff']));
          $node = Doctrine::getTable('LocationNode')->findOneBySlug($values['dropoff']);

          if ($node)
          {
            $metaAppend .= ' to '.$node->get('name');
            switch ($node->get('type'))
            {
              case 'departure':
                $filters[] = array(
                  'nested' => array(
                    'path' => 'dropoffLocation.custom_departures',
                    'filter' => array(
                      'term' => array('dropoffLocation.custom_departures.slug' => array('value' => $values['dropoff'])))
                    )
                  )
                ;
                break;
              case 'category':
                $filters[] = array('term' => array('dropoffLocation.state.slug' => array('value' => $values['dropoff'])));
                break;
              case 'location':
                $filters[] = array('term' => array('dropoffLocation.slug' => array('value' => $values['dropoff'])));
                break;
            }
          } elseif (!empty($to) && strpos($to, ':any') === false) {
            $filters[] = array('term' => array('dropoffLocation.name' => $values['dropoff']));
          }
        }

        // Vehicle type
        if ($values['vehicle_type'])
        {
            $filters[] = array('terms' =>  array('vehicleType.type' => $values['vehicle_type']));
        }

        // Vehicle type
        if ($values['vehicle_type_filter'])
        {
            $filters[] = array('terms' =>  array('vehicleType.filter_type' => $values['vehicle_type_filter']));
        }
        // Vehicle type
        if ($values['min_age_filter'])
        {
            $filters[] = array('terms' =>  array('min_age_filter' => $values['min_age_filter']));
        }

        if ($values['transmission'])
        {
            $transmission = is_array($values['transmission']) ? $values['transmission'] : array($values['transmission']);
            $filters[] = array('terms' => array('vehicleType.transmission' => $values['transmission']));
        }

        if ($values['travellers'])
        {
          $filters[] = array('range' => array(
            'vehicleType.travellers' => array(
              'gte' => intval($values['travellers'])
            )
          ));
        }

        // min age
        if ($values['min_age'] && $values['min_age'] == ['18'])
        {
          $filters[] = array('range' => array(
            'min_age' => array(
              'lt' => 21
            ),
          ));
        }


        // ferry
        if ($values['ferry'])
        {
            $filters[] = array('terms' =>  array('ferry_included' => !!$values['ferry']));
        }

        // fuel
        if ($values['fuel'])
        {
            $filters[] = array('terms' =>  array('petrol_included' => !!$values['fuel']));
        }

        // other_extras
        if ($values['other_extras'])
        {
          foreach ($values['other_extras'] as $key => $value)
          {
            $filters[] = array('terms' =>  array($value.'_included' => [true]));
          }

        }

        // Vehicle type
        if ($values['country'])
        {
            $filters[] = array('terms' =>  array('country' => $values['country']));
        }

        $sort_col = $action->getRequestParameter('sort_col', 'dates');
        $sort = [];
        switch($sort_col){
          default:
          case 'dates':
            $sort = ['from_date' => 'asc'];
            break;

          case 'recommended':
            $sort = ['listing_score' => 'desc', 'from_date' => 'asc'];
            break;
          case 'days':
            $sort = ['nb_free_days' => 'desc', 'nb_paid_days' => 'desc'];
            break;
          case 'extras':
            $sort = ['nb_included' => 'desc'];
            break;
          case 'alpha':
            $sort = ['pickupLocation.name' => 'asc', 'dropoffLocation.name' => 'asc'];
            break;
          case 'pickup':
            $sort = ['pickupLocation.name' => 'asc', 'dropoffLocation.name' => 'asc'];
            break;
          case 'dropoff':
            $sort = ['dropoffLocation.name' => 'asc', 'pickupLocation.name' => 'asc'];
            break;
        }

        $action->sort_col = $sort_col;

        $action->getResponse()->setTitle($metaTitle.$metaAppend);
        $action->getResponse()->addMeta('description', $metaDescription1.$metaAppend.'.'.$metaDescription2);

        if (count($filters))
        {
          $esQuery['query'] = array(
            'filtered' => array(
              'filter' => array(
                'bool' => array(
                  'must' => $filters
                )
              )
            )
          );
        }

        $esQueryObj = new \Elastica\Query($esQuery);
        $esQueryObj->addSort($sort);

        return $esQueryObj;
    }
}
