<?php

namespace Transfercar\ElasticSearch;

class SuggestedListings
{
    public function find($params, $country)
    {
        $both_directions = isset($params['both_directions']) ? $params['both_directions'] : false;

        $filters = [];

        if (isset($params['from_latlong'])) {
          $filters[] = [
            'geo_distance' => [
              'distance' => '50km',
              'pickupLocation.location' => [
                'lat' => ['from_latlong'][0],
                'long' => ['from_latlong'][1]
              ]
            ]
          ];
        }

        if (isset($params['to_latlong'])) {
          $filters[] = [
            'geo_distance' => [
              'distance' => '50km',
              'dropoffLocation.location' => [
                'lat' => ['from_latlong'][0],
                'long' => ['from_latlong'][1]
              ]
            ]
          ];
        }

        if (isset($params['fromLocations']) && count($params['fromLocations'])) {
          $filters[] = array(
            'terms' => array(
              'pickupLocation.id' => (array) $params['fromLocations']
            )
          );
        }

        if (isset($params['toLocations']) && count($params['toLocations'])) {
          $filters[] = array(
            'terms' => array(
              'dropoffLocation.id' => (array) $params['toLocations']
            )
          );
        }


        if (isset($params['fromDepartures']) && $params['fromDepartures']!='') {
          $filters[] = array(
            'nested' => array(
              'path' => 'pickupLocation.custom_departures',
              'filter' => array(
                'terms' => array('pickupLocation.custom_departures.slug' => (array) $params['fromDepartures']))
              )
            )
          ;
        }

        if (isset($params['toDepartures']) && $params['toDepartures']!='') {
          $filters[] = array(
            'nested' => array(
              'path' => 'dropoffLocation.custom_departures',
              'filter' => array(
                'terms' => array('dropoffLocation.custom_departures.slug' => (array) $params['toDepartures']))
              )
            )
          ;
        }

        if (isset($params['vehicleType']) && $params['vehicleType']) {
          $filters[] = array(
            'terms' => array(
              'vehicleType.type' => (array) $params['vehicleType']
            )
          );
        }

        if (isset($params['country']) && $params['country']) {
          $filters[] = array(
            'term' => array(
              'country' => $params['country'],
            )
          );
        }

        if (isset($params['exclude']) && $params['exclude']) {
          $filters[] = array(
            'not' => [
              'filter' => [
                'terms' => [
                  'id' => $params['exclude']
                ]
              ]
            ]
          );
        }

        if (isset($params['airport_reloc'])) {
          $filters[] = array(
            'term' => array(
              'is_airport_reloc' => $params['airport_reloc'],
            )
          );
        }

        $must_filters = [];
        if (isset($params['from_date']) && $params['from_date'])
        {
          // '10-01';
          // 'pickup <= 10-01 and dropoff >= 10-01'
          $filter_date = date('Y-m-d', strtotime('-7 days', strtotime($params['from_date'])));
          // $filters[] = array('range' => array(
          //   'from_date' => array(
          //     'gte' => $filter_date
          //   )
          // ));
          $filters[] = array('range' => array(
            'to_date' => array(
              'gte' => $filter_date
            ),
          ));
        }

        if (isset($params['to_date']) && $params['to_date'])
        {
          // '10-01';
          // 'dropoff <= 10-01 and dropoff >= 10-01'
          $filter_date = date('Y-m-d', strtotime('+7 days', strtotime($params['from_date'])));
          $filters[] = array('range' => array(
            'to_date' => array(
              'gte' => $filter_date
            )
          ));
        }

        $query = empty($filters) ? [] : array(
          'query' => array(
            'filtered' => array(
              'filter' => array(
                'bool' => array(
                  'must' => array(
                    $filters
                  )
                )
              )
            )
          )
        );

        $esQueryObj = new \Elastica\Query($query);
        $esQueryObj->addSort(['boost_ranking' => 'desc', 'listing_score' => 'desc']);
        $esQueryObj->setSize($params['limit']);

        $search = (new \tcElasticaClient())->getSearch($country);
        $search->setQuery($esQueryObj);

        $resultSet = $search->search();

        return array_map(
            function($v){
                return new \tcElasticaListingResult($v->getHit());
            },
            $resultSet->getResults()
        );
    }
}