<?php

namespace Transfercar\Email\Sender;

use sfLoader;

abstract class RelocationEmail
{
    protected static function getTemplateData($request, $listing, $driver)
    {
        sfLoader::loadHelpers(array('Helper', 'Partial', 'Url', 'Asset', 'Tag', 'Transfercar', 'I18N'));
        return array(
            'from'                  => $listing->PickupLocation->name,
            'to'                    => $listing->DropoffLocation->name,
            'driver_firstname'      => $driver->get('firstname'),
            'driver_lastname'       => $driver->get('lastname'),
            'driver_fullname'       => $driver->getFullName(),
            'driver_email'          => $driver->get('email'),
            'driver_mobile_phone'   => $driver->get('mobile_phone'),
            'driver_license'        => $driver->get('license') ? format_country($driver->get('license')) : 'N/A',
            'driver_dob'            => tc_format_date($driver->get('date_of_birth')),
            'driver_avatar'         => default_text($request->get('Driver')->get('logo'), '/images/user-icon-silhouette.png'),
            'rental_operator'       => $listing->get('Owner')->get('company'),
            'rental_operator_logo'  => $listing->get('Owner')->get('logo'),
            'driver_accepted_instructions' => $listing->get('Owner')->get('driver_accepted_instructions'),
            'pickup_date'           => tc_format_date($request->get('pickup_date')),
            'pickup_time'           => tc_format_date($request->get('pickup_date'), 'short').' '.tc_format_date($request->get('pickup_time'), 'hours'),
            'pickup'                => $listing->getPickup()->get('name'),
            'pickup_address'        => $listing->getPickup()->get('address'),
            'pickup_description'    => $listing->getPickup()->get('description'),
            'pickup_contact_name'   => $listing->getContactName('pickup'),
            'pickup_contact_email'  => $listing->getContactEmailAddress('pickup'),
            'pickup_contact_phone'  => $listing->getContactPhoneNumber('pickup'),
            'pickup_opening_hours'  => $listing->getPickup()->getOpeningHours(),
            'dropoff_date'          => tc_format_date($request->get('drop_off_date')),
            'dropoff_time'          => tc_format_date($request->get('drop_off_date'), 'short').' '.tc_format_date($request->get('dropoff_time'), 'hours'),
            'dropoff'               => $listing->getDropoff()->get('name'),
            'dropoff_address'       => $listing->getDropoff()->get('address'),
            'dropoff_description'   => $listing->getDropoff()->get('description'),
            'dropoff_contact_name'  => $listing->getContactName('dropoff'),
            'dropoff_contact_email' => $listing->getContactEmailAddress('dropoff'),
            'dropoff_contact_phone' => $listing->getContactPhoneNumber('dropoff'),
            'dropoff_opening_hours' => $listing->getDropoff()->getOpeningHours(),
            'vehicle_type'          => $listing->getVehicle()->get('name'),
            'vehicle_fuel'          => $listing->getVehicle()->getFuelName(),
            'vehicle_transmission'  => $listing->getVehicle()->getTransmissionName(),
            'vehicle_image'         => tc_thumbnail_path($listing->getVehicle()->icon_path(), 'vehicle_email'),
            'fuel_included'         => $listing->get('petrol_included') ? default_text($listing->get('petrol_included_text'), 'Yes') : 'No',
            'ferry_included'        => self::getFerryText($listing),
            'expenses_included'     => $listing->get('expenses_included') ? default_text($listing->get('expenses_included_text'), 'Yes') : 'No',
            'insurance_included'    => self::getInsuranceText($listing),
            'tolls_included'        => $listing->get('tolls_included') ? default_text($listing->get('tolls_included_text'), 'Yes') : 'No',
            'road_tolls_included'   => $listing->get('tolls_included') ? default_text($listing->get('tolls_included_text'), 'Yes') : 'No',
            'kms_included'          => default_text($listing->get('kms_allowed'), 'Unlimited'),
            'listing_comment'       => $listing->get('comment'),
            'request_date'          => tc_format_date($request->get('created_at')),
            'request_status'        => $request->getStatusName(),
            'request_free_days'     => $listing->getIncludedDaysPrice() ? sprintf('%s x $%s per day', $request->get('nb_free_days'), $listing->getIncludedDaysPrice()) : sprintf('%s free days', $request->get('nb_free_days')),
            'request_paid_days'     => $request->get('nb_paid_days'),
            'request_price_per_paid_day' => $listing->get('price_per_day'),
            'request_id'            => $request->getIncrementedId(),
            'tcid'                  => $listing->getId(),
            'listed_date'           => tc_format_date($listing->get('created_at')),
            'booking_reference'     => $listing->get('internal_reference'),
            'manager'               => $listing->get('ListedBy')->get('firstname'),
            'is_agent'              => $listing->hasBond(),
            'deposit'               => $listing->hasBond() ? $listing->getBondAmount() : 0,
            'no_show_fee'           => $request->rawGet('credit_card_id') ? 1 : 0,
            'no_show_fee_amount'    => $request->get('no_show_fee_amount')
        );
    }

    private function getInsuranceText($listing)
    {
      if ($listing->get('insurance_policy_id')) {
            return sprintf('Bond: %s, excess: %s, waiver: %s', 
                  $listing->get('InsurancePolicy')->get('bond'),
                  $listing->get('InsurancePolicy')->get('excess'),
                  $listing->get('InsurancePolicy')->get('waiver')
            );
      }
      
      return $listing->get('insurance_included') ? default_text($listing->get('insurance_included_text'), 'Yes') : 'No';
    }

    private function getFerryText($listing)
    {
      if ($listing->get('ferry_included_type')) {
            $types = array(
                1 => 'Vehicle only', 
                2 => 'Vehicle and driver included', 
                3 => 'Vehicle + 2 passengers', 
                4 => 'Partial refund'
            );

            return $types[$listing->get('ferry_included_type')];
      }
      
      return $listing->get('ferry_included') ? default_text($listing->get('ferry_included_text'), 'Yes') : 'No';
    }
}