<?php

namespace Transfercar\Email\Sender;

use Transfercar\Mandrill\TcMandrill;
use \sfLoader;

class RentalNotificationEmail
{
	// 
// long_term_rental_request_details
	public static function send($request)
	{
		$email_data = self::getTemplateData($request, $request->get('Rental'));

		$mandrill = new TcMandrill($request->getCountryDomain());
        // $mandrill->sendTemplate(
        //     'Transfercar New RENTAL Enquiry',
        //     self::getROEmail($request->get('Rental')),
        //     $email_data,
        //     'long-term-rental-request-details'
        // );

        $mandrill->sendTemplate(
            'Transfercar New RENTAL Enquiry',
            [
                ['name' => 'Noel', 'email' => 'noel@transfercar.co.nz'],
                ['name' => 'Brian', 'email' => 'brian@transfercar.co.nz'],
                ['name' => 'Business', 'email' => 'business@transfercar.co.nz'],
                ['name' => 'Sharon', 'email' => 'sharon@transfercar.co.nz']
            ],
            $email_data,
            'long-term-rental-request-details'
        );


        $mandrill->sendTemplate(
            'Transfercar Long Term Rental Enquiry confirmation',
            [
            'email'  => $request->get('driver_email'),
            'name'   => $request->get('driver_firstname'),
            'format' => 'html',
            'id'     => '',
            'lang'   => 'en'
            ],
            $email_data,
            'driver-long-term-rental-request-details'
        );
	}

    private static function getROEmail($rental) 
    {
        switch ((int) $rental->get('owner_id')) {
            // aerodrive
            case 110755:
                return [
                    ['email' => 'bookings@aerodrivecarrental.com', 'name' => 'Aerodrive'], 
                    ['email' => 'chris@aerodrivecarrental.com', 'name' => 'Chris Speirs']
                ];
            // Jucy
            case 2:
                return ['email' => 'kirsty.malone@jucyworld.com', 'name' => 'Kirsty Malone'];
            case 62579:
                return ['email' => 'roy.ohara@gorentals.co.nz', 'name' => 'Roy Ohara'];
            default:
                return $rental->getNewRequestContact()->getEmailToParams();
        }
    }

	private static function getTemplateData($request, $rental)
    {
        sfLoader::loadHelpers(array('Helper', 'Partial', 'Url', 'Asset', 'Tag', 'Transfercar', 'I18N'));
        return array(
            'from'                  => $request->OwnerBranch->NearestLocation->name,
            'driver_firstname'      => $request->get('driver_firstname'),
            'driver_lastname'       => $request->get('driver_lastname'),
            'driver_fullname'       => $request->get('driver_firstname'). ' ' . $request->get('driver_lastname'),
            'driver_email'          => $request->get('driver_email'),
            'driver_mobile_phone'   => $request->get('driver_mobile_phone'),
            'driver_comment'        => $request->get('driver_comment'),
            'driver_license'        => 'N/A',// $driver->get('license') ? format_country($driver->get('license')) : 'N/A',
            'driver_dob'            => 'N/A',// tc_format_date($driver->get('date_of_birth')),
            'driver_avatar'         => '/images/user-icon-silhouette.png',
            'rental_operator'       => $rental->get('Owner')->get('company'),
            'rental_operator_logo'  => $rental->get('Owner')->get('logo'),
            'pickup_date'           => tc_format_date($request->get('pickup_date')),
            'dropoff_date'          => tc_format_date($request->get('dropoff_date')),
            'pickup_time'           => tc_format_date($request->get('pickup_time'), 'hours'),
            'dropoff_time'          => tc_format_date($request->get('dropoff_time'), 'hours'),
            'pickup'                => $request->get('OwnerBranch')->get('name'),
            'pickup_address'        => $request->get('OwnerBranch')->get('address'),
            'pickup_description'    => $request->get('OwnerBranch')->get('description'),
            'vehicle_type'          => $rental->get('OwnerVehicleType')->get('name'),
            'vehicle_fuel'          => $rental->get('OwnerVehicleType')->getFuelName(),
            'vehicle_transmission'  => $rental->get('OwnerVehicleType')->getTransmissionName(),
            'vehicle_image'         => tc_thumbnail_path($rental->get('OwnerVehicleType')->icon_path(), 'vehicle_email'),
            'request_date'          => tc_format_date($request->get('created_at')),
            'request_nb_days'       => $request->getNbDays(),
            'request_insurance'	    => $request->get('insurance') ? 'Yes' : 'No',
            'total_price'			=> __('$').$request->get('price'),
            'tcid'                  => $rental->getId(),
            'request_id'            => $request->getId(),
            'listed_date'           => tc_format_date($rental->get('created_at')),
            'manager'               => $rental->get('ListedBy')->get('firstname'),
        );
    }
}
