<?php

namespace Transfercar\Email\Sender;

use Request;
use sfEvent;
use User;

class RequestCancelledEmails
{
    public static function notify(sfEvent $event)
    {
      $request = $event->getSubject();

      // handle only manual cancellation, not automatic cancellation for overlapping request or not accepted
      if (!$request->isStatus(array(Request::STATUS_CANCELED_BY_DRIVER, Request::STATUS_DRIVER_CANCELED, Request::STATUS_OWNER_CANCELED))) {
        return;
      }

      self::notifyRO($request);
      self::notifyDriver($request);
    }

    private static function notifyRO($request)
    {
      // Don't notify ROs if they've cancelled the request
      if ($request->isStatus(array(Request::STATUS_DRIVER_CANCELED, Request::STATUS_OWNER_CANCELED)))
      {
        return;
      }

      $notification = new RequestNotificationEmail($request);
      $template_name = $request->isAccepted() ? 'booking-cancelled-by-driver' : 'request-cancelled-by-driver';
      $notification->send($template_name, $request->get('Listing')->get('Owner'));
    }

    private static function notifyDriver($request)
    {
      $template_name = $request->isStatus(Request::STATUS_CANCELED_BY_DRIVER) ? 'driver-request-canceled' : 'booking-cancelled-by-ro';

      switch ($request->get('status')) {
        case Request::STATUS_CANCELED_BY_DRIVER:
          $template_name = $request->isAccepted() ? 'driver-cancelled-booking' : 'driver-request-canceled';
          break;


        case Request::STATUS_DRIVER_CANCELED:
          $template_name = 'driver-request-cancelled-by-ro';
          break;


        case Request::STATUS_OWNER_CANCELED:
          $template_name = 'booking-cancelled-by-ro';
          break;
      }

      $notification = new RequestNotificationEmail($request);
      $notification->send($template_name, $request->get('Driver'));
    }
}
