<?php

namespace Transfercar\Email\Sender;

use Transfercar\Mandrill\TcMandrill;

use Doctrine;
use Request;
use sfLoader;

class RequestConfirmationEmail extends RelocationEmail
{
    public static function notifyDriver($request)
    {
        $template_data = self::getTemplateData($request, $request->get('Listing'), $request->get('Driver'));
        $template_data['firstname'] = $template_data['driver_firstname'];

        $mandrill = new TcMandrill($request->getCountryDomain());
        $mandrill->sendTemplate(
            'Your relocation request',
            $request->Driver->getEmailToParams(),
            $template_data,
            'driver-request-confirmation'
        );
    }

    public static function notifyRo($request)
    {
        sfLoader::loadHelpers(array('Url', 'Asset'));
        $listing = $request->get('Listing');
        $template_data = self::getTemplateData($request, $listing, $request->get('Driver'));
        $requests = Doctrine::getTable('Request')
            ->createQuery('r')
            ->where('r.listing_id = ?', $listing->getId())
            ->addWhere('r.status = ?', Request::STATUS_PENDING)
            ->orderBy('r.created_at desc')
            ->execute();

        $template_data['requests'] = array();
        foreach ($requests as $request)
        {
            $rating = floatval($request->get('Driver')->get('average_rating'));
            $template_data['requests'][] = array(
                'driver'        => $request->get('Driver')->getShortName(). '('.$request->get('driver_id').')',
                'nb_relocs'     => (int) $request->get('Driver')->reloc_count(),
                'pickup_date'   => tc_format_date($request->get('pickup_date'), 'short'),
                'dropoff_date'  => tc_format_date($request->get('drop_off_date')),
                'nb_free_days'  => $request->get('nb_free_days'),
                'nb_paid_days'  => $request->get('nb_paid_days'),
                'feedback'      => format_number_choice('[-Inf, 0) %r%% negative feedback |[0] no feedback |(0,+Inf] %r%% positive feedback', array('%r%' => sprintf('%0.0f', $rating * 100)), $rating),
                'member_since'  => format_date($request->get('Driver')->get('when_registered'), 'MMM yyyy'),
                'driver_avatar' => default_text($request->get('Driver')->get('logo'), '/images/user-icon-silhouette.png'),
                'accept_link'   => url_for('request/emailaccept?key=contact&id='.$request->getId().'&uid=contact', true),
                'reject_link'   => url_for('request/emailreject?key=contact&id='.$request->getId().'&uid=contact', true),
            );
        }

        $receivers = [$listing->getNewRequestContact()->getEmailToParams()];
        if ($request->getCountryDomain() == 'us' && in_array($listing->get('owner_id'), [64201, 77706])) {
            $receivers[] = ['email' => 'usrequests@transfercarus.com', 'name' => 'Transfercar', 'format' => 'html', 'id' => -1];
        }
        // silvercar
        if ($request->getCountryDomain() == 'us' && in_array($listing->get('owner_id'), [232246])) {
            $receivers[] = ['email' => 'micah.piller@silvercar.com', 'name' => 'Micah Piller', 'format' => 'html', 'id' => -1];
        }


        $mandrill = new TcMandrill($request->getCountryDomain());
        $mandrill->sendTemplate(
            'New relocation requests',
            $receivers,
            $template_data,
            'new-requests'
        );

        if ($request->getCountryDomain() == 'us' && !\sfConfig::get('sf_debug')) {
            $twilio_sid          = \sfConfig::get('app_twilio_sid');
            $twilio_token        = \sfConfig::get('app_twilio_token');
            $twilio_phone_number = \sfConfig::get('app_twilio_phone_number');

            try {
                $client = new \Twilio\Rest\Client($twilio_sid, $twilio_token);
                $message = $client->messages->create('+16154775399', [ // send text message to Dennis
                    'from' => $twilio_phone_number, // From a valid Twilio number
                    'body' => 'New relocation request for listing TCID #'.$request->get('listing_id')
                ]);
            } catch (\Exception $e) {
                error_log($e->getMessage());
            }
        }
    }
}
