<?php

namespace Transfercar\Event;

use sfEvent;
use sfLoader;
use LocationCountry;
use sfConfig;

use Transfercar\Mandrill\TcMandrill;

class RelocationReviewEvent
{
    /**
     * Listen to the review.created event and send a notification email to the RO
     *
     * @param sfEvent $event
     */
    public static function listenToReviewCreated(sfEvent $event)
    {
        $review = $event->getSubject();
        $request = $review->get('Request');
        $listing = $request->get('Listing');

        if (!$listing->get('Owner')->get('ro_review_notify')) {
            return;
        }

        $listing->get('Owner')->recalculateRORating();

        $listing = $request->get('Listing');
        $domain = LocationCountry::retreiveDomainById($listing->Owner->country_domain_id);
        $site = sfConfig::get('app_domain_' . $domain);
        sfLoader::loadHelpers(array('Date', 'Transfercar'));

        $email_opt_link = 'http://' . $site . '/userSettings/emailOptions';
        $template_data = array(
            'company_name'    => $listing->get('Owner')->get('company'),
            'manager'         => $listing->get('ListedBy')->get('firstname'),
            'driver'          => $review->getDriver()->username,
            'tcid'            => $request->get('listing_id'),
            'from'            => $listing->PickupLocation->name,
            'from_date'       => format_date($request->get('pickup_date'), 'd', 'en_'.strtoupper($domain)),
            'to'              => $listing->DropoffLocation->name,
            'to_date'         => format_date($request->get('drop_off_date'), 'd', 'en_'.strtoupper($domain)),
            'review'          => $review->get('ro_review'),
            'score'           => $review->get('ro_score'),
            'link'            => 'http://'.str_replace('www', 'ro', $site).'/owner.php/reviews/view/id/'.$review->getId(),
            'email_opt_link'  => $email_opt_link
        );

        $template_data = array_merge($template_data, $listing->detailedInfoArray());

        $to = array();
        if (count($listing->get('OwnerContacts')))
        {
            foreach ($listing->get('OwnerContacts') as $contact)
            {
                $to[] = $contact->getEmailToParams();
            }
        }
        else
        {
            $to[] = $listing->get('ListedBy')->getEmailToParams();
        }

        foreach ($to as $receiver)
        {
            $mandrill = new TcMandrill($domain);
            $mandrill->sendTemplate(
                'Driver rated relocation {{TCID}}',
                $to,
                $template_data,
                'booking-reviewed-by-driver'
            );
        }
    }

    /**
     * Listen to the review.replied event and send a notification email to the driver
     *
     * @param sfEvent $event
     */
    public static function listenToReviewReplied(sfEvent $event)
    {
        $review = $event->getSubject();
        $request = $review->get('Request');

        $listing = $request->get('Listing');
        $driver  = $request->get('Driver');
        $domain = LocationCountry::retreiveDomainById($request->get('Driver')->get('country_domain_id'));
        $site = sfConfig::get('app_domain_' . $domain);
        sfLoader::loadHelpers(array('Date', 'Transfercar'));

        $email_opt_link = 'http://' . $site . '/userSettings/emailOptions';
        $template_data = array(
            'firstname'       => $driver->firstname,
            'lastname'        => $driver->lastname,
            'when_registered' => $driver->when_registered,
            'username'        => $driver->username,
            'from'            => $listing->PickupLocation->name,
            'from_date'       => format_date($request->get('pickup_date'), 'd', 'en_'.strtoupper($domain)),
            'to'              => $listing->DropoffLocation->name,
            'to_date'         => format_date($request->get('drop_off_date'), 'd', 'en_'.strtoupper($domain)),
            'request_id'      => $request->getIncrementedId(),
            'reloc_details'   => $listing->detailedInfo(true, true),
            'email_opt_link_real' => $email_opt_link,
            'tcid'            => $request->get('listing_id'),
            'vehicle_type'    => $listing->get('VehicleType')->get('description'),
            'reply'           => $review->get('ro_reply')
        );

        $template_data = array_merge($template_data, $listing->detailedInfoArray());
        $mandrill = new TcMandrill($domain);
        $mandrill->sendTemplate(
            'Rental operator replied to your review',
            $driver->getEmailToParams(),
            $template_data,
            'review-replied-by-ro'
        );
    }

    /**
     * Listen to the review.reported event and send a notification email to the TC support center
     *
     * @param sfEvent $event
     */
    public static function listenToReviewReported(sfEvent $event)
    {
        $review = $event->getSubject();
        $request = $review->get('Request');

        $body = <<<EOF
Hi support,

Rental operator ###rental_operator### reported this review from relocation ###tcid### as inappropriate:

###review###

RO message:

###message###

EOF;

        $msg = nl2br(strtr($body, array(
            '###tcid###' => $request->getIncrementedId(),
            '###review###' => $review->get('ro_review'),
            '###rental_operator###' => $review->getListing()->get('Owner')->get('company'),
            '###message###' => $event['message']
        )));

        $mandrill = new TcMandrill($request->getCountryDomain());
        $mandrill->sendTemplate(
            'Rental operator reported this review',
            array('email' => sfConfig::get('app_support_email'), 'name'  => sfConfig::get('app_support_name')),
            array(
                'content' => $msg
            )
        );
    }
}