<?php

namespace Transfercar\Event;

use Transfercar\PhpAmqpLib\tcRequestProducer;

use sfContext;
use sfEvent;
use Request;

class RequestEvents
{
    /**
     * Listen to the request.sent event and send a tracking events to Google Analytics
     *
     * @param sfEvent $event
     */
    public static function listenToRequestSent(sfEvent $event)
    {
        self::triggerGAEvents($event);
        self::publishRabbitMqMessages($event);
    }

    private static function publishRabbitMqMessages(sfEvent $event)
    {
        $request = $event->getSubject();
        // expire_requests consumer will expire the request
        tcRequestProducer::publish($request, 'expire_requests_delayed');
        // expiring_requests consumer will send a notification to the RO when a request is about to expire
        tcRequestProducer::publish($request, 'expiring_requests_delayed');
    }

    private static function triggerGAEvents(sfEvent $event)
    {
        if (!sfContext::hasInstance()) {
            return;
        }

        $request = $event->getSubject();
        $listing = $request->get('Listing');
        $user    = sfContext::getInstance()->getUser();

        $user->pushFlash('ga_event', array(
            'event'           => 'gaTriggerEvent',
            'gaEventCategory' => 'Request',
            'gaEventAction'   => $listing->getTitle(),
            'gaEventLabel'    => (string) $request->getId()
        ));

        if ($request->isStatus(Request::STATUS_ACCEPTED)) {
          $user->pushFlash('ga_event', array(
            'event' => 'GAecommerce',
            'transactionId' => $request->getId(),
            'transactionTotal' => 0.0,
            'transactionProducts' => [[
                'sku'      => (string) $listing->getId(), //The unique product SKU goes here
                'name'     => $listing->getTitle(),
                'category' => $listing->get('VehicleType')->get('description'), //The car category/type
                'price'    => 0.0,
                'quantity' => 1
            ]]
          ));
        }
    }
}