<?php

namespace Transfercar\Export;

class BookingsExport
{
    public static function getLabels()
    {
        $header = [
            'Account',
            'TCID',
            'Pick Up',
            'Pick Up Date',
            'Pick Up Time',
            'Dropoff',
            'Dropoff Date',
            'Dropoff Time',
            'Vehicle',
            'First Name',
            'Last Name',
            'Phone',
            'Email',
            'Free days',
            'Paid days',
            'Fuel included',
            'Cost',
            'Reference'
        ];

        // combine labels so that we can match them with RO keys using array_intersect_key
        return array_combine($header, $header);
    }

    public static function getRequestColumns($ro, $requests)
    {
        $keys = $ro->get('data_export_format') ?: self::getLabels();
        $keys = array_flip($keys);
        $rows = [];

        foreach ($requests as $request) {
            $listing = $request->get('Listing');
            $driver = $request->get('Driver');
            $row = [
                'Account'       => $listing->get('Owner')->get('company'),
                'TCID'          => $request->getIncrementedId(),
                'Pick Up'       => $listing->getPickup()->get('internal_name') ?: $listing->getPickup()->get('name'),
                'Pick Up Date'  => $request->get('pickup_date'),
                'Pick Up Time'  => date('Hi', strtotime('+'.$request->get('pickup_time').' seconds', strtotime('today'))),
                'Dropoff'       => $listing->getDropoff()->get('internal_name'),
                'Dropoff Date'  => $request->get('drop_off_date'),
                'Dropoff Time'  => date('Hi', strtotime('+'.$request->get('dropoff_time').' seconds', strtotime('today'))),
                'Vehicle'       => $listing->getVehicle()->get('internal_name'),
                'First Name'    => $driver->get('firstname'),
                'Last Name'     => $driver->get('lastname'),
                'Phone'         => $driver->get('mobile_phone'),
                'Email'         => $driver->get('email'),
                'Free days'     => $request->get('nb_free_days'),
                'Paid days'     => $request->get('nb_paid_days'),
                'Fuel included' => $listing->get('petrol_included') ? 'Y':'N',
                'Cost'          => $listing->getTotalPrice($request),
                'Reference'     => $listing->get('internal_reference')
            ];
            // only use columns configured by ROs
            $rows[] = array_intersect_key(array_replace($keys, $row), $keys);
        }

        return $rows;
    }

    public static function exportCSV($ro, $requests)
    {
        $csv = fopen('php://temp/maxmemory:'. (5*1024*1024), 'r+');
        $keys = $ro->get('data_export_format');

        // only use columns headers configured by ROs
        fputcsv($csv, $keys);
        $keys = array_flip($keys);

        foreach (self::getRequestColumns($ro, $requests) as $row) {
            fputcsv($csv, $row);
        }

        rewind($csv);

        return stream_get_contents($csv);
    }
}