<?php

namespace Transfercar\Geoloc;

class GoogleMapsDistancesMatrix
{
    private static $cache = [];

    public function getDistances($from, $to, $unit = 'metric')
    {
        $distances = $this->call($from, $to);

        if ($distances['status'] != 'OK' || (isset($distances['error_message']) && $distances['error_message']) || $distances['rows'][0]['elements'][0]['status'] != 'OK')
        {
            return false;
        }

        $distance = ceil($distances['rows'][0]['elements'][0]['distance']['value']/1000);

        if ('imperial' === $unit)
        {
            // convert to miles
            $distance *= 0.621371;
        }

        return ceil($distance);
    }

    private function call($from, $to)
    {
        if ($data = $this->getCache($from, $to)) {
            return json_decode($data, true);
        }
        $url = 'https://maps.googleapis.com/maps/api/distancematrix/json';

        $parameters = array(
            'origins'      => $from,
            'destinations' => $to,
            'sensor'       => 'false',
            'key'          => 'AIzaSyBNjcR79PcLiZHUvuYV5Qhzy8OKlz6ktiE'
        );

        $transport = new CurlTransport();
        $data = $transport->call($url.'?'.http_build_query($parameters));

        $this->setCache($from, $to, $data);

        return json_decode($data, true);
    }

    public function getCache($from, $to)
    {
        $cache = new \sfFileCache(array(
            'cache_dir' => \sfConfig::get('sf_app_cache_dir').'/distances',
            'lifetime' => 2592000 // 30 days
        ));
        $key = $this->getCacheKey($from, $to);

        if ($cache->has($key)) {
            return $cache->get($key);
        }

        return false;
    }

    private function setCache($from, $to, $data)
    {
        $cache = new \sfFileCache(array(
            'cache_dir' => \sfConfig::get('sf_app_cache_dir').'/distances',
            'lifetime' => 2592000 // 30 days
        ));

        $key = $this->getCacheKey($from, $to);
        $cache->set($key, $data);
    }

    private function getCacheKey($from, $to)
    {
        return sha1($from.'-'.$to);
    }

}
