<?php

namespace Transfercar\Import;

class ListingImport
{
    public function import($ro, $data)
    {
        $f = fopen($file, 'rb');
        
        $this->form = new \OwnerListingForm(array(), array(
          'country' => 1,
          'domain'  => 'nz',
          'user'    => $ro
        ));

        foreach ($data as $row) {
	    $pickup = $this->getLocation($ro->getOwnerId(), $row['pickup']);
            $dropoff = $this->getLocation($ro->getOwnerId(), $row['dropoff']);
            $values = [
                'pickup_location_id' => $pickup,
                'dropoff_location_id' => $dropoff,
                'vehicle_type_id' => $this->getVehicle($ro->getOwnerId(), $row['vehicle_type']),
                'date_available_from' => date('Y-m-d', strtotime(preg_replace('@(\d\d)/(\d\d)/(\d\d\d\d)@', '$3-$2-$1', $row['date_from']))),
                'date_return_by' => date('Y-m-d', strtotime(preg_replace('@(\d\d)/(\d\d)/(\d\d\d\d)@', '$3-$2-$1', $row['date_to']))),
		'pickup_time' => $pickup->get('default_pickup_time'),
                'latest_pickup_time' => $pickup->get('latest_pickup_time'),
                'dropoff_time' => $dropoff->get('default_dropoff_time'),
                'latest_dropoff_time' => $dropoff->get('latest_dropoff_time'),
                'days_allowed' => (int) $row['free_days'],
                'paid_days_allowed' => (int) $row['paid_days'],
                'price_per_day' => $row['price_per_paid_day'],
                'ferry_included' => !!$row['ferry_included'],
                'ferry_included_text' => $row['ferry_included'],
	        'expenses_included' => !!$row['expenses_included'],
	        'expenses_included_text' => '$'. $row['expenses_included']. ' upon receipts',
	        'petrol_included' => !!$row['petrol_included'],
	        'petrol_included_text' => '$'.$row['petrol_included'].' upon receipts',
		'nb_listings' => isset($row['nb_listings']) ? $row['nb_listings'] : 1,
                'kms_allowed' => $row['kms_allowed']

	    ];
            $this->save($ro, $values);
        }
    }

    private function getLocation($roId, $internal_name)
    {
        $location = \Doctrine::getTable('OwnerBranch')
            ->createQuery()
            ->where('owner_id = ?', $roId)
            ->addWhere('internal_name = ?', $internal_name)
            ->limit(1)
            ->fetchOne();

        if (!$location) {
            throw new \Exception("Location $internal_name not found");
        }

        return $location;
    }


    private function getVehicle($roId, $internal_name)
    {
        $vehicle = \Doctrine::getTable('OwnerVehicleType')
            ->createQuery()
            ->where('owner_id = ?', $roId)
            ->addWhere('internal_name = ?', $internal_name)
            ->limit(1)
            ->fetchOne();

        if (!$vehicle) {
            throw new \Exception("Vehicle $internal_name not found");
        }

        return $vehicle;
    }

    private function save($ro, $values)
    {
        $listing = new \Listing();

        $values['insurance_included'] = true;
        $object = $values['vehicle_type_id'];

        if ($object instanceof \OwnerVehicleType)
        {
            $values['owner_vehicle_type_id'] = $object->getId();
            $values['vehicle_type_id'] = $object->get('vehicle_type_id');
        }
        else
        {
            $values['vehicle_type_id'] = $object->getId();
            $values['owner_vehicle_type_id'] = null;
        }

        $pickup = $values['pickup_location_id'];

        if ($pickup instanceof \OwnerBranch)
        {
            $values['owner_pickup_branch_id'] = $pickup->getId();
            $values['pickup_location_id'] = $pickup->get('location_id');
            $values['pickup_address'] = $pickup->address;
            $values['contact_person'] = $pickup->get('Contact')->get('name');
            $values['contact_phone']  = $pickup->get('Contact')->get('phone');
        }
        else
        {
            $values['pickup_location_id'] = $pickup->getId();
        }

        $dropoff = $values['dropoff_location_id'];

        if ($dropoff instanceof \OwnerBranch)
        {
            $values['owner_dropoff_branch_id'] = $dropoff->getId();
            $values['dropoff_location_id'] = $dropoff->get('location_id');
        }
        else
        {
            $values['dropoff_location_id'] = $dropoff->getId();
        }

        $values['insurance_policy_id'] = $values['insurance_policy'];

        if (isset($values['is_airport_reloc']))
        {
            $values['listing_type'] = $values['is_airport_reloc'] ? \Listing::TYPE_AIRPORT_RELOC : \Listing::TYPE_DEFAULT;
        }

        $listing->fromArray($values);

        $listing->set('date_return_by', $listing->get('date_return_by'));
        $listing->set('date_available_from', $listing->get('date_available_from'));

        $listing->price = \Doctrine::getTable('Prices')->getPriceFor(
            $ro->getOwnerId(),
            $values['pickup_location_id'],
            $values['dropoff_location_id'],
            $values['vehicle_type_id']
        );

        $listing->set('status', \Listing::STATUS_NEW);
        $listing->set('created_at', date('Y-m-d'));
        $listing->set('listed_by', $ro->getId());
        $listing->set('owner_id', $ro->getOwnerId());

        $listing->set('updated_by', $ro->getId());
        $listing->set('updated_at', date('Y-m-d'));

        $score = new \Transfercar\Ranking\SearchScore();
        $listing->set('score', $score->getScore($listing));
        $listing->set('score_debug', $score->debug);
        $listing->save();


      if ($nb_listings = $values['nb_listings'])
      {
        $baseListing = $listing;
        $duplicate = $this->duplicate($listing);

        for ($i = 0; $i < intval($nb_listings -1); $i++)
        {
          $listing = $baseListing->get('ChildrenListings')->get($i);
          $listing->fromArray($duplicate);
          $listing->set('listing_id', $baseListing->getId());
          $listing->set('owner_id', $ro->getOwnerId());
          $listing->set('listed_by', $ro->getId());
          $listing->set('status', \Listing::STATUS_STANDBY);
          $listing->set('created_at', date('Y-m-d', time()));
          $listing->set('OwnerContacts', $baseListing->get('OwnerContacts'));

          $listing->save();
        }

        for ($i = $nb_listings - 1, $count = $baseListing->get('ChildrenListings')->count(); $i < $count; $i++)
        {
          $baseListing->get('ChildrenListings')->get($i)->delete();
        }
      }
    }

    
    private function duplicate($listing)
    {

      $defaults = $listing->toArray(false);
      unset(
        $defaults['id'],
        $defaults['created_at'],
        $defaults['notify_instance'],
        $defaults['notify_daily'],
        $defaults['status'],
        $defaults['is_new_type_payment'],
        $defaults['listed_by']
      );

      $defaults['is_new_type_payment'] = 'yes';
      $terms = $listing->Terms;
      $addons_data = $listing->Addons->getData();
      $defaults['Addons'] = array();

      if (count($addons_data) > 0){
        foreach ($addons_data as $key => $item){
          $defaults["Addons"][$key]['title'] = $item["title"];
          $defaults["Addons"][$key]['name']  = $item["name"];
          $defaults["Addons"][$key]['price'] = $item["price"];
        }
      }

      if ($terms->exists()){
        $defaults['terms_title'] = $terms["title"];
        $defaults['terms_content'] = $terms["content"];
      }

      $contacts = $listing->get('OwnerContacts');
      $defaults['OwnerContacts'] = array();

      if (count($contacts)) {
        $listing_contact_list = array();
        foreach ($contacts as $contact) {
          $listing_contact_list[] = $contact->getId();
        }

        $defaults['listing_contact_list'] = $listing_contact_list;
      }

      return $defaults;
    }
}
