<?php

namespace Transfercar\Mailchimp;

use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Middleware;

class MailchimpApi
{
    private function call($method, $url, $options)
    {
        $guzzle = new HttpClient(array(
            'base_uri' => 'https://us12.api.mailchimp.com/3.0/',
            'auth' => ['anystring', \sfConfig::get('app_mailchimp_apikey')],
            'timeout' => 5
        ));

        return $guzzle->request($method, $url, $options);
    }

    public function subscribe($listid, $email, $firstname, $lastname)
    {
        try {
            return $this->call('POST', sprintf('lists/%s/members/', $listid) , [
                'json'   => [
                    'email_address' => $email,
                    'status'        => 'subscribed',
                    'merge_fields'  => [
                        'FNAME' => $firstname,
                        'LNAME' => $lastname
                    ]
                ]
            ]);
        } catch (\GuzzleHttp\Exception\ClientException $e) {
            return $this->resubscribe($listid, $email);
        }

        return false;
    }

    public function resubscribe($listid, $email)
    {
        try {
            return $this->call('PATCH', sprintf('lists/%s/members/%s', $listid,  md5(strtolower($email))) , [
                'json'   => [
                    'status'        => 'subscribed'
                ]
            ]);
        } catch (\GuzzleHttp\Exception\ClientException $e) {
            return false;
        }

        return false;
    }

    public function unsubscribe($listid, $email)
    {
        try {
            return $this->call('PATCH', sprintf('lists/%s/members/%s', $listid,  md5(strtolower($email))) , [
                'json'   => [
                    'status'        => 'unsubscribed'
                ]
            ]);
        } catch (\GuzzleHttp\Exception\ClientException $e) {
            return false;
        }

        return false;
    }

    public function getBounced($listid)
    {
        return json_decode($this->call('GET', sprintf('list/%s/members?status=cleaned', $listid))->getBody(), true);
    }
}