<?php

namespace Transfercar\Mandrill;

use Mandrill;
use Swift_Message;
use sfConfig;
use Doctrine;
use Twilio\Rest\Client;

class TcMandrill
{
    private $mandrill;
    private $countryCode;
    private $i18n;

    public function __construct($countryCode = null)
    {
        $this->mandrill = new Mandrill('NicuAW0UFK9LeJkibkqqDQ');
        $this->countryCode = $countryCode ?: $_SERVER['TRANSFER_CAR_COUNTRY'];
    }

    public function search($email, $date, $tag)
    {
        try {
            $query = 'email:'.$email;
            $date_from = $date;
            $date_to = $date;
            $tags = array(
                $tag
            );
            $senders = array('noreply@'.str_replace(['www.', 'staging.'], '', $this->getDomain()));

            return $this->mandrill->messages->search($query, $date_from, $date_to, $tags, $senders);
        } catch (\Exception $e) {
            return;
        }
    }

    private function translateSubject($text, $lang)
    {
        if ($lang == 'en') {
            return $text;
        }

        $subjects = [
            'Transfercar Booking Details for {{DRIVER_FIRSTNAME}} {{DRIVER_LASTNAME}}' => 'Détails de la réservation pour {{DRIVER_FIRSTNAME}} {{DRIVER_LASTNAME}}',
            'Your relocation request has been accepted' => 'Votre demande de transfert a été acceptée',
            'Your relocation booking is confirmed' => 'Votre réservation est confirmée',
            'Your relocation request' => 'Votre demande de transfert',
            'New relocation requests' => 'Nouvelles demandes de transfert',
            'Your booking has been cancelled' => 'Votre réservation a été annulée',
            'Driver cancelled booking {{TCID}}' => 'Le conducteur a annulé la réservation  {{TCID}}',
            'Driver cancelled request {{TCID}}' => 'Le conducteur a annulé la demande  {{TCID}}',
            'Your request has been cancelled' => 'Votre demande a été annulée',
            'Someone else accepted the driver before you' => 'Quelqu\'un d\'autre a accepté le conducteur avant vous',
            'Your request has not been accepted, please try again' => 'Votre demande n\'a pas été acceptée, merci d\'essayer à nouveau',
            'Request {{REQUEST_ID}} has expired' => 'La demande {{REQUEST_ID}} a expiré',
            'Your request {{REQUEST_ID}} has expired, please try again' => 'Votre demande {{REQUEST_ID}} a expiré, merci d\'essayer à nouveau',
            'Transfercar relocation complete' => 'Le transfert Transfercar est terminé',
            'The request {{REQUEST_ID}} will expire at {{REQUEST_EXPIRE_TIME}}' => 'La demande {{REQUEST_ID}} va expirer à {{REQUEST_EXPIRE_TIME}}',
            'Your request is still being processed' => 'Votre demande est toujours en cours de traitement',
            'Welcome to Transfercar' => 'Bienvenue chez Transfercar',
            'New password' => 'Nouveau mot de passe',
            'Confirm your new e-mail on Transfercar' => 'Confirmer votre nouvel e-mail sur Transfercar'
        ];

        if (array_key_exists($text, $subjects)) {
            return $subjects[$text];
        }

        return $text;
    }
    /**
     * Send an email using a Mandrill template
     *
     * @param array $receiver
     * @param string $subject
     * @param array $data
     * @param string $template_name
     */
    public function sendTemplate($subject, $receiver, $data, $template_name = 'transfercar-default')
    {
        $aReceiver = array_key_exists('email', $receiver) ? $receiver : $receiver[0];
        $lang = isset($aReceiver['lang']) && $aReceiver['lang'] == 'fr' ? 'fr' : 'en';

        $subject = $this->translateSubject($subject, $lang);
        // prefix subject with Transfercar:
        $subject = strpos($subject, 'Transfercar') === false ? 'Transfercar: '.$subject : $subject;
        $mergeVars = [];
        if (isset($data['merge_vars'])) {
            $mergeVars = [];
            foreach ($data['merge_vars'] as $var) {
                $mergeVars[] = array(
                    'rcpt' => $var['rcpt'],
                    'vars' => $this->makeVarsStruct($var['vars'])
                );
            }

            unset($data['merge_vars']);
        }

        $message = array(
            'subject'      => $subject,
            'from_email'   => 'noreply@'.str_replace(['www.', 'staging.'], '', $this->getDomain()),
            'from_name'    => 'Transfercar',
            'to'           => $this->fixReceiver($receiver),
            'track_opens'  => true,
            'track_clicks' => true,
            'merge'        => true,
            'global_merge_vars' => $this->mergeGlobalVars($receiver, $data),
            'merge_vars'   => $this->mergeVars($receiver, $mergeVars),
            'google_analytics_campaign' => str_replace('-', '_', $template_name),
	    'google_analytics_domains' => [$this->getDomain()],
            //'tags' => [$template_name],
            'return_path_domain' => 'mandrillapp.'.str_replace(['www.', 'staging.'], '', $this->getDomain())
        );

        try {
            $this->mandrill->messages->sendTemplate($this->getTemplate($template_name, $receiver), array(), $message);
        } catch (\Exception $e) {
           // throw $e;
        }
    }

    private function getTemplate($template_name, $receiver)
    {
        $translated = [
            'booking-number-confirmed',
            'driver-request-confirmation',
            'request-update',
            'activation-code',
            'new-requests',
            'request-booking-details',
            'expiring-request', //
            'confirm-email',
            'request-expired-new',
            'request-rejected',
            'request-rejected-new',
            'booking-cancelled-by-ro',
            'driver-request-cancelled-by-ro',
            'booking-cancelled-by-driver', //
            'request-cancelled-by-driver', //
            //'driver-request-expired', //
            'request-canceled',
            'driver-request-canceled',
            'forgot-password',
            'fee-charged',
            'someone-else-accepted',
            'relocation-closed',
        ];

        if (!in_array($template_name, $translated)) {
            return $template_name;
        }

        $prefix = '2023-';

        $map = [
            'request-cancelled' => 'surplus-request-cancelled',
            'request-update' => 'request-accepted',
            'booking-number-confirmed' => 'booking-reference-updated',
            'booking-cancelled-by-ro' => 'driver-booking-cancelled-by-ro',
            'relocation-closed' => 'ro-relocation-review-fordriver',
            'fee-charged' => 'cancellation-fees-charged',
            'request-booking-details' => 'ro-request-accepted-details',
            'request-rejected-new' => 'request-rejected',
            'driver-request-cancelled-by-ro' => 'ro-onbehalfdriver-booking-cancelled',
            'request-cancelled-by-driver' => 'ro-request-cancelled-by-driver',
            'booking-cancelled-by-driver' => 'ro-booking-cancelled-by-driver'
        ];

        if (isset($map[$template_name])) {
            $template_name = $map[$template_name];
        }

        if (!array_key_exists('email', $receiver)) {
            $receiver = $receiver[0];
        }

        $lang = ''; // isset($receiver['lang']) && $receiver['lang'] == 'fr' ? '-fr' : '';

        return $prefix.$template_name.$lang;
    }

    public function removeEmailFromBlacklist($email, $subaccount = null)
    {
        $this->mandrill->rejects->delete($email, $subaccount);
    }

    private function fixReceiver($receivers)
    {
        // allow multiple receivers
        if (array_key_exists('email', $receivers))
        {
            $receivers = array($receivers);
        }

        $filtered = array();

        // remove hardbounced emails
        foreach ($receivers as $receiver)
        {
            // Useful for development, redirects to the directly to the configured email address
            if (sfConfig::get('app_mailer_redirect_address')) {
                $receiver = array('email' => sfConfig::get('app_mailer_redirect_address'), 'name' => $receiver['name']);
            }

            if (Doctrine::getTable('User')->isEmailBounced($receiver['email']))
            {
                continue;
            }

            $filtered[] = $receiver;
        }

        return $filtered;
    }

    /**
     * Send a message throught Mandrill using a Swift_Message instance
     *
     * This is useful while we are migrating our emails Mandrill
     * as we can still use the old code to build and send emails
     *
     */
    public function sendFromSwiftMessage(Swift_Message $msg)
    {
        $message = $this->mandrill->messages->parse($msg->toString());

        $message['track_opens']  = true;
        $message['track_clicks'] = true;

        $this->mandrill->messages->send($message, false, null, null);
    }

    public function getRejectList()
    {
        return $this->mandrill->rejects->getList();
    }

    /**
     * Transform a classic associative array into a Mandrill struct
     *
     * @param array $data
     * @return array
     */
    private function makeVarsStruct($data)
    {
        $struct = [];
        foreach ($data as $key => $value) {
            $struct[] = array(
                'name' => strtoupper($key),
                'content' => $value
            );
        }

        return $struct;
    }

    /**
     * Merge default vars with the array of vars given in parameter
     *
     * @var array $data An associative array of vars
     * @return array Merged vars
     */
    private function mergeGlobalVars($receiver, $data)
    {
        $data    = $this->makeVarsStruct($data);
        $default = $this->getDefaultVars($receiver);

        return array_merge($data, $default);
    }

    /**
     * Returns default Mandrill vars
     *
     * Emails have default variables such as the unsubscribe or profile link
     * This function returns them.
     *
     * @return array() Default variables
     */
    private function getDefaultVars($receiver)
    {
        return array(
            array(
                'name'    => 'UPDATE_PROFILE',
                'content' => $this->getLink('user/emailOptions')
            ),
            array(
                'name'    => 'COUNTRY',
                'content' => $this->getCountry(),
            ),
            // IS_NZ, IS_AU and IS_US are needed in mailchimp templates because we can't do COUNTRY='xxx'
            array(
                'name'    => 'IS_NZ',
                'content' => $this->getCountry() == 'New Zealand',
            ),
            array(
                'name'    => 'IS_AU',
                'content' => $this->getCountry() == 'Australia',
            ),
            array(
                'name'    => 'IS_US',
                'content' => $this->getCountry() == 'USA',
            ),
            array(
                'name'    => 'IS_SA',
                'content' => $this->getCountry() == 'South Africa',
            ),
            array(
                'name'    => 'IS_BR',
                'content' => $this->getCountry() == 'Brasil',
            ),
            array(
                'name'    => 'SUPPORT_EMAIL',
                'content' => $this->getSupportEmail()
            ),
            array(
                'name'    => 'WEB_LINK',
                'content' => $this->getLink('')
            ),
            array(
                'name'    => 'COMPANY',
                'content' => 'Transfercar'
            ),
            array(
                'name'    => 'DESCRIPTION',
                'content' => 'You are receiving this email because you are a registred Transfercar user'
            ),
        );
    }

    public function mergeVars($receivers, $data)
    {
        if (array_key_exists('email', $receivers)) {
            $receivers = [$receivers];
        }
        $final = [];
        foreach ($receivers as $receiver) {
            $rcpt = $this->getRcpt($receiver['email'], $data);

            $rcpt['vars'][] = array(
                'name'    => 'UNSUBSCRIBE',
                'content' => $this->getLink('emailUnsubscribe/'.$receiver['email'])
            );
            $rcpt['vars'][] = array(
                'name'    => 'EMAIL_OPT_LINK_REAL',
                'content' => $this->getLink('emailUnsubscribe/'.$receiver['email'])
            );

            $final[] = $rcpt;
        }

        return $final;
    }

    private function getRcpt($email, $data)
    {
        foreach ($data as $value) {
            if ($value['rcpt'] == $email) {
                return $value;
            }
        }

        return array(
            'rcpt' => $email,
            'vars' => []
        );
    }

    /**
     * Returns the absolute link of the url given in parameter
     *
     * @return string
     */
    public function getLink($url)
    {
        return 'http'.(sfConfig::get('app_tc_env') === 'prod' ? 's' : '').'://' . $this->getDomain().'/'.$url;
    }

    /**
     * Returns the current domain
     *
     * @return string
     */
    private function getDomain()
    {
        return sfConfig::get('app_domain_'.$this->countryCode);
    }

    /**
     * Returns the current country name
     *
     * @return string
     */
    private function getCountry()
    {
        switch ($this->countryCode) {
            case 'nz':
                return 'New Zealand';


            case 'au':
                return 'Australia';

            case 'us':
                return 'North America';

            case 'za':
                return 'South Africa';

            case 'br':
                return 'Brasil';
        }
    }

    /**
     * Returns the support email address
     *
     * @return string
     */
    private function getSupportEmail()
    {
        // remove subdomain (www.transfercar -> transfercar)
        $domain = sfConfig::get('app_domain_'.$this->countryCode);
        $domain = preg_replace('#^([\w]+\.)transfercar#', 'transfercar', $domain);

        return 'support@'. $domain;
    }
}
