<?php

namespace Transfercar\Map;

class TripsFetcher
{
    private $conn;

    public function __construct($conn)
    {
        $this->conn = $conn;
    }

    public function fetchTrips($country)
    {
        $this->conn->query('set sql_mode="STRICT_TRANS_TABLES,NO_ZERO_IN_DATE,NO_ZERO_DATE,ERROR_FOR_DIVISION_BY_ZERO,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION"');
        $query = $this->getQueryForCountry($country);

        $stmt = $this->conn->prepare($query);
        $stmt->execute(array($country, date('Y-m-d'), $country));

        $trips = array();

        while ($trip = $stmt->fetch())
        {
            $trips[$trip['name']] = array(
                'count' => max(1, $trip['nb_trips']),
                'lat' => floatval($trip['lat']),
                'long' => floatval($trip['lon']),
                'url'  => 'pickup='.$trip['slug'],
                'departures' => $this->getArrivals($trip['id'], $country)
            );
        }

        return $trips;
    }

    private function getQueryForCountry($country_id)
    {
        switch ($country_id) {
            case 1:
            case 2:
            case 5:
                return <<<EOF
SELECT id, name, lat, lon, sum(nb_trips) as nb_trips, slug
from (
SELECT loc.id, loc.name, loc.lat, loc.lon, sum(l.nb_listings) as nb_trips, ln.slug as slug
FROM locations loc
INNER JOIN location_category lc on lc.id=loc.category_id
INNER JOIN listing l on l.pickup_location_id=loc.id
INNER JOIN location_node ln on (ln.type_id=loc.id and ln.type='location')
WHERE lc.country_id = ?
AND l.date_return_by >= ?
AND l.status = 0
GROUP by loc.id, loc.name, loc.name, loc.lat, loc.lon, ln.slug 
UNION
SELECT loc2.id, loc2.name, loc2.lat, loc2.lon, count(*) as nb_trips, ln2.slug as slug
FROM locations loc2
INNER JOIN location_category lc2 on lc2.id=loc2.category_id
INNER JOIN ghost_listing l2 on l2.pickup_location_id=loc2.id
INNER JOIN location_node ln2 on (ln2.type_id=loc2.id and ln2.type='location')
WHERE lc2.country_id = ?
GROUP by loc2.id, loc2.name, loc2.name, loc2.lat, loc2.lon, ln2.slug
)t group by id, name, lat, lon,slug
EOF;
            default:
                return <<<EOF
SELECT id, name, lat, lon, sum(nb_trips) as nb_trips, slug
from (
SELECT d.id, d.description as name, loc.lat, loc.lon, sum(l.nb_listings) as nb_trips, ln.slug as slug
FROM location_category d
INNER JOIN locations loc on d.id=loc.category_id
INNER JOIN listing l on l.pickup_location_id=loc.id
INNER JOIN location_node ln on (ln.type_id=d.id and ln.type='category')
WHERE d.country_id = ?
AND l.date_return_by >= ?
AND l.status = 0
GROUP by d.description
UNION
SELECT d.id, d.description as name, loc.lat, loc.lon, count(*) as nb_trips, ln.slug as slug
FROM location_category d
INNER JOIN locations loc on d.id=loc.category_id
INNER JOIN ghost_listing l on l.pickup_location_id=loc.id
INNER JOIN location_node ln on (ln.type_id=d.id and ln.type='category')
WHERE d.country_id = ?
GROUP by d.description
)t group by id, name, lat, lon,slug
EOF;
        }
    }

    private function getArrivalsQuery($countryId)
    {
        ini_set('error_reporting', E_ALL);
        ini_set('display_errors', true);
        switch ($countryId) {
            case 1:
            case 2:
            case 5:
                return <<<EOF
SELECT d2.id, loc.id as pickup_id, loc.name as pickup_name, loc.lat pickup_lat, loc.lon pickup_lon, ln.slug as pickup_slug, loc2.name as dropoff_category_name, loc2.id as dropoff_id, loc2.name dropoff_name, loc2.lat dropoff_lat, loc2.lon dropoff_lon, ln2.slug as dropoff_slug, sum(l.nb_listings) as nb_trips
FROM locations loc
INNER JOIN location_category d on d.id=loc.category_id
INNER JOIN listing l on l.pickup_location_id=loc.id
INNER JOIN locations loc2 on l.dropoff_location_id=loc2.id
INNER JOIN location_category d2 on loc2.category_id=d2.id
INNER JOIN location_node ln on (ln.type_id=loc.id and ln.type='location')
INNER JOIN location_node ln2 on (ln2.type_id=loc2.id and ln2.type='location')
WHERE d.country_id = ?
AND l.date_return_by >= ?
AND l.status = 0
AND loc.id = ?
GROUP by loc.id, loc2.id
UNION
SELECT d2.id, loc.id as pickup_id, loc.name as pickup_name, loc.lat pickup_lat, loc.lon pickup_lon, ln.slug as pickup_slug, loc2.name as dropoff_category_name, loc2.id as dropoff_id, loc2.name dropoff_name, loc2.lat dropoff_lat, loc2.lon dropoff_lon, ln2.slug as dropoff_slug, count(*) as nb_trips
FROM locations loc
INNER JOIN location_category d on d.id=loc.category_id
INNER JOIN ghost_listing l on l.pickup_location_id=loc.id
INNER JOIN locations loc2 on l.dropoff_location_id=loc2.id
INNER JOIN location_category d2 on loc2.category_id=d2.id
INNER JOIN location_node ln on (ln.type_id=loc.id and ln.type='location')
INNER JOIN location_node ln2 on (ln2.type_id=loc2.id and ln2.type='location')
WHERE d.country_id = ?
AND loc.id = ?
GROUP by loc.id, loc2.id
EOF;

            case 3:
                return <<<EOF
SELECT d2.id, loc.id as pickup_id, loc.name as pickup_name, loc.lat pickup_lat, loc.lon pickup_lon, ln.slug as pickup_slug, d2.description as dropoff_category_name, loc2.id as dropoff_id, loc2.name dropoff_name, loc2.lat dropoff_lat, loc2.lon dropoff_lon, ln2.slug as dropoff_slug, sum(l.nb_listings) as nb_trips
FROM location_category d
INNER JOIN locations loc on d.id=loc.category_id
INNER JOIN listing l on l.pickup_location_id=loc.id
INNER JOIN locations loc2 on l.dropoff_location_id=loc2.id
INNER JOIN location_category d2 on loc2.category_id=d2.id
INNER JOIN location_node ln on (ln.type_id=loc.id and ln.type='location')
INNER JOIN location_node ln2 on (ln2.type_id=loc2.id and ln2.type='location')
WHERE d.country_id = ?
AND l.date_return_by >= ?
AND l.status = 0
AND d.id = ?
GROUP by loc.id, loc2.id
UNION
SELECT d2.id, loc.id as pickup_id, loc.name as pickup_name, loc.lat pickup_lat, loc.lon pickup_lon, ln.slug as pickup_slug, d2.description as dropoff_category_name, loc2.id as dropoff_id, loc2.name dropoff_name, loc2.lat dropoff_lat, loc2.lon dropoff_lon, ln2.slug as dropoff_slug, count(*) as nb_trips
FROM location_category d
INNER JOIN locations loc on d.id=loc.category_id
INNER JOIN ghost_listing l on l.pickup_location_id=loc.id
INNER JOIN locations loc2 on l.dropoff_location_id=loc2.id
INNER JOIN location_category d2 on loc2.category_id=d2.id
INNER JOIN location_node ln on (ln.type_id=loc.id and ln.type='location')
INNER JOIN location_node ln2 on (ln2.type_id=loc2.id and ln2.type='location')
WHERE d.country_id = ?
AND d.id = ?
GROUP by loc.id, loc2.id
EOF;
        }
    }

    private function getArrivals($id, $countryId)
    {

        $query = $this->getArrivalsQuery($countryId);

        $stmt = $this->conn->prepare($query);
        $stmt->execute(array($countryId, date('Y-m-d'), $id, $countryId, $id));

        $arrivals = [];
        while ($trip = $stmt->fetch())
        {
            if (!isset($arrivals[$trip['pickup_name']])) {
                $arrivals[$trip['pickup_name']] = array(
                    'pickup_name'     => $trip['pickup_name'],
                    'departure_count' => 0,
                    'url'             => 'pickup='.$trip['pickup_slug'],
                    'routes'          => [],
                    'lat'  => floatval($trip['pickup_lat']),
                    'long' => floatval($trip['pickup_lon'])
                );
            }

            $arrivals[$trip['pickup_name']]['departure_count'] += $trip['nb_trips'];
            $arrivals[$trip['pickup_name']]['routes'][] = [
                'dropoff_name'    => $trip['dropoff_name'],
                'arrival_count'   => $trip['nb_trips'],
                'arrival_state'   => $trip['dropoff_category_name'],
                'url'             => 'pickup='.$trip['pickup_slug'].'&dropoff='.$trip['dropoff_slug'],
                'lat'  => array(floatval($trip['pickup_lat']), floatval($trip['dropoff_lat'])),
                'long' => array(floatval($trip['pickup_lon']), floatval($trip['dropoff_lon'])),
            ];
        }

        return $arrivals;
    }
}
