<?php

namespace Transfercar\PhpAmqpLib;

use PhpAmqpLib\Connection\AMQPConnection;
use PhpAmqpLib\Message\AMQPMessage;

class tcRabbitmq
{
    private static $conn;
    private static $channels = array();

    public function __construct($host = 'localhost', $port = 5672, $username = 'guest', $password = 'guest', $path = '/')
    {
        if (!self::$conn) {
            $conn = new AMQPConnection($host, $port, $username, $password, $path);

            $this->makeChannel($conn, 'listing_created');
            $this->makeChannel($conn, 'listing_updated');
            $this->makeDelayedChannel($conn, 'accepted_requests', 300000);
            $this->makeDelayedChannel($conn, 'expire_requests',  86400000);
            $this->makeDelayedChannel($conn, 'expiring_requests', 72000000);

            self::$conn = $conn;
        }
    }

    public function makeChannel($conn, $queueName)
    {
        self::$channels[$queueName] = $ch = $conn->channel();
        $ch->confirm_select();
        $ch->queue_declare($queueName, false, true, false, false, false);
        $ch->queue_bind($queueName, 'amq.direct');
    }

    public function makeDelayedChannel($conn, $queueName, $ttl)
    {
        $this->makeChannel($conn, $queueName);

        self::$channels[$queueName.'_delayed'] = $delay_channel = $conn->channel();
        $delay_channel->confirm_select();

        $delay_channel->queue_declare($queueName.'_delayed', false, true, false, false, false, [
          'x-message-ttl' => array('I', $ttl), # Delay of 5 mins until the message is transferred in milliseconds.
          'x-dead-letter-exchange' => array('S', 'amq.direct'), # Exchange used to transfer the message from A to B.
          'x-dead-letter-routing-key' => array('S', $queueName) # Name of the queue we want the message transferred to.
        ]);
    }

    public function getConnection()
    {
        return self::$conn;
    }

    public function getChannel($name)
    {
        return self::$channels[$name];
    }
}