<?php

namespace Transfercar\Ranking;

use Listing;

class ListingRank
{
    private $conn;
    private $country_id;

    public function __construct($conn, $country_id)
    {
        $this->conn = $conn;
        $this->country_id = $country_id;
    }

    public function getRank(Listing $listing)
    {
        $ranks = $this->getRanks($this->country_id);
        $id = $listing->getId();

        if (empty($ranks)) {
            return array(
                'rank' => 1,
                'percentrank' => 1,
                'score' => $listing->get('score'),
                'score_debug' => $listing->get('score_debug')
            );
        }

        if (count($ranks) < 20) {
            return array(
                'rank' => 1,
                'percentrank' => 0.7,
                'score' => $listing->get('score'),
                'score_debug' => $listing->get('score_debug')
            );
        }

        if ($id && $row = array_filter($ranks, function($k) use ($id) { return (int) $k['id'] === (int) $id; })) {
            return array(
                'rank' => $k[0]['rank'],
                'percentrank' => $k[0]['percentrank'],
                'score' => $k[0]['curr'],
                'score_debug' => $listing->get('score_debug')
            );
        }

        $i = 0;
        $score = $listing->get('score');
        while ($i < count($ranks) && $score < $ranks[$i]['curr']) {
            $i++;
        }

        if ($i < count($ranks)) {
            return array(
                'rank' => $ranks[$i]['rank'],
                'percentrank' => $ranks[$i]['percentrank'],
                'score' => $score,
                'score_debug' => $listing->get('score_debug')
            );
        }

        return array(
            'rank' => count($ranks) + 1,
            'percentrank' => 0,
            'score' => $score,
            'score_debug' => $listing->get('score_debug')
        );
    }

    private function getRanks($country_id)
    {
        $query = <<<EOF
create temporary table tmp_score
    SELECT
        listing.id,
        listing.score
    FROM
        listing INNER JOIN vehicle_type v on listing.vehicle_type_id=v.id
    WHERE
        score is not null
        and status=0
        and v.country_id=:country_id
    ORDER BY score DESC;
EOF;

        $stmt = $this->conn->prepare($query);
        $stmt->execute(array(':country_id' => $country_id));

        $query = <<<EOF
create temporary table tmp_score2
    SELECT * from tmp_score
;
EOF;

        $stmt = $this->conn->prepare($query);
        $stmt->execute();

        $query = <<<EOF
SELECT
    tmp_score.id,
    tmp_score.score,
    @prev := @curr as prev,
    @curr := score as curr,
    @rank := IF(@prev > @curr, @rank+@ties, @rank) AS rank,
    @ties := IF(@prev = @curr, @ties+1, 1) AS ties,
    (1-@rank/@total) as percentrank
FROM
    tmp_score,
    (SELECT
        @curr := null,
        @prev := null,
        @rank := 0,
        @ties := 1,
        @total := count(*)
        FROM tmp_score2 l
        WHERE 1=1
        AND score is not null
    ) b

WHERE
    score is not null
ORDER BY
    score DESC
EOF;

        $stmt = $this->conn->prepare($query);
        $stmt->execute();

        return $stmt->fetchAll(\PDO::FETCH_ASSOC);
    }
}