<?php

namespace Transfercar\Ranking;

use Request;
use User;
use Doctrine;

class RoVoter implements VoterInterface
{
    private $debug = [];
    private $errors = [];

    /**
     * Calculate the upvotes for the given RO
     *
     * @see https://basecamp.com/1780292/projects/11507933/todos/230906429#comment_430543045
     * @param User $user
     * @return float The RO score
     */
    public function getUpVotes($user)
    {
        // score is calculated every 3 months. If not defined yet, calculate it
        if ($user->get('response_rate')) {
            return $user->get('response_rate');
        }

        $requests = $this->getNbRequests($user);
        $rejectedRequests = $this->getNbRequests($user, array(Request::STATUS_REJECTED, Request::STATUS_EXPIRED));

        $score = (1 - ($rejectedRequests / $requests));
        $this->debug[] = sprintf('RO upvotes = (1 - (%s / %s)) * 10 = %s', $requests, $rejectedRequests, round($score * 10, 2));

        return $score;
    }

    /**
     * Calculate the downvotes for the given RO
     *
     * @see https://basecamp.com/1780292/projects/11507933/todos/230906429#comment_430543045
     * @param User $user
     * @return float The RO score
     */
    public function getDownVotes($user)
    {
        $downvotes = 15 * (1 - $this->getUpVotes($user));
        // remove duplicate debug message
        array_pop($this->debug);
        $this->debug[] = sprintf('RO downvotes = %s', round($downvotes, 2));
        $this->errors['ro_score'] = true;

        return $downvotes;
    }

    public function getDebug()
    {
        return implode("\n", $this->debug);
    }


    public function getErrors()
    {
        $errorMessages = [
            'ro_score' => 'Improve your acceptance rate',
        ];

        return array_intersect_key($errorMessages, $this->errors);
    }

    private function getNbRequests($user, $statuses = null)
    {
        return Doctrine::getTable('Request')
            ->getNbRequests(
                $user,
                date('Y-m-d', strtotime('-3 months')),
                date('Y-m-d'),
                $statuses
        );
    }
}