<?php

namespace Transfercar\Search;

use sfPager;
use Doctrine_Query;
use Elastica;
use tcElasticaClient;
use tcElasticaListingResult;

class tcListingPager extends sfPager
{
    private $maxListingsPerPage,
            $maxSponsoredAdsPerPage,
            $listingCount,
            $numRelocations,
            $sponsoredAdsCount,
            $sortEvently = true,
            /** @var Doctrine_Query */
            $listingQuery,
            /** @var Doctrine_Query */
            $sponsoredAdsQuery,
            $results,
            $aggregations;

    public function __construct($maxListingPerPage = 20, $maxSponsoredAdsPerPage = 0)
    {
        parent::__construct(null, $maxListingPerPage);
        $this->maxListingsPerPage     = $maxListingPerPage;
        $this->maxSponsoredAdsPerPage = $maxSponsoredAdsPerPage;
    }

    public function setPage($page)
    {
        parent::setPage($page);

        return $this;
    }

    public function getMaxListingsPerPage()
    {
        return $this->maxListingsPerPage;
    }

    public function getMaxSponsoredAdsPerPage()
    {
        return $this->maxSponsoredAdsPerPage;
    }

    public function setListingQuery(Elastica\Query $listingQuery)
    {
        $this->listingQuery = $listingQuery;

        return $this;
    }

    public function setSponsoredAdsQuery(Doctrine_Query $sponsoredAdsQuery)
    {
        $this->sponsoredAdsQuery = $sponsoredAdsQuery;

        return $this;
    }

    public function init()
    {
        $sumQuery = clone $this->listingQuery;


        $search = (new tcElasticaClient())->getSearch($_SERVER['TRANSFER_CAR_COUNTRY']);

        // calculating offset
        $listingOffset = ($this->getPage() - 1) * $this->getMaxListingsPerPage();

        $this->listingQuery->setFrom($listingOffset);
        $this->listingQuery->setSize($this->getMaxPerPage());

        $this->addAggregations($this->listingQuery);
        $search->setQuery($this->listingQuery);

        $resultSet = $search->search();

        $this->listingCount   = $resultSet->getTotalHits();
        $this->numRelocations = $this->listingCount;
        $this->aggregations   = $resultSet->getAggregations();
        $this->numRelocations = $this->aggregations['nb_listings']['value'];

        /*
        if nb sponsored ads < getMaxSponsoredAdsPerPage():
            maxListingPerPage += diff

        else if nb de listing < getMaxListingPerPage():
            maxSponsoredAdsPerPage += diff

        */
        $nbListingsRemainingAtOffset     = $this->listingCount - $listingOffset;

        // total results is nb listings + nb ads
        $this->setNbResults($this->listingCount);
        $this->setLastPage(ceil($this->getNbResults() / $this->getMaxPerPage()));

        // $listings     = $this->listingQuery->execute();

        $listings     = array_map(
            function($v){
                return new tcElasticaListingResult($v->getHit());
            },
            $resultSet->getResults()
        );

        $this->results = new tcListingResults($listings, [], false, $this->getMaxSponsoredAdsPerPage());

        return $this;
    }

    public function getNumRelocations()
    {
        return $this->numRelocations;
    }

    public function getNumSponsoredAds()
    {
        return $this->sponsoredAdsCount;
    }

    public function getResults()
    {
        if (!$this->results) {
            $listings     = $this->listingQuery->execute();
            $sponsoredAds = $this->sponsoredAdsQuery->execute();

            $this->results = new tcListingResults($listings, $sponsoredAds, $this->sortEvently, $this->getMaxSponsoredAdsPerPage());
        }

        return $this->results;
    }

    public function retrieveObject($offset)
    {
        return false;
    }

    public function getAggregations()
    {
        return $this->aggregations;
    }

    private function addAggregations($query)
    {
        $aggregations = array(
            'pickup' => 'pickupLocation',
            'dropoff' => 'dropoffLocation',
            'transmission' => 'vehicleType.transmission',
            'vehicle_type'  => 'vehicleType.type',
            'vehicle_type_filter'  => 'vehicleType.filter_type',
            'min_age_filter'  => 'min_age_filter',
            'country' => 'country'
            // 'travellers' => 'vehicleType.travellers'
        );

        foreach ($aggregations as $name => $field)
        {
            $agg = new Elastica\Aggregation\Terms($name);
            $sumAgg = new Elastica\Aggregation\Sum('sum');
            $sumAgg->setField('nb_listings');

            if ($name == 'pickup' || $name == 'dropoff') {
                $agg->setScript("doc['".$field.".state.slug'].value + '_' + doc['".$field.".state.name'].value");
                $subAgg = new Elastica\Aggregation\Terms('locations');
                $subAgg->setScript("doc['".$field.".slug'].value + '_' + doc['".$field.".name'].value");
		$subAgg->addAggregation($sumAgg);
	        $agg->addAggregation($subAgg);
            } elseif ($name == 'travellers') {
                $agg->setScript("if (doc['vehicleType.travellers'].value == 1) { return doc['vehicleType.travellers'].value+'_'+doc['vehicleType.travellers'].value + '+ person'; } else { return doc['vehicleType.travellers'].value+'_'+doc['vehicleType.travellers'].value + '+ people'; }");
            } elseif ($name == 'xvehicle_type') {
                $agg->setScript("doc['".$field."'].value");
                $subAgg = new Elastica\Aggregation\Terms('locations');
                $subAgg->setScript("doc['".$field."'].value");
                $subAgg->addAggregation($sumAgg);
                $agg->addAggregation($subAgg);
            }else {
                $agg->setField($field);
                $agg->addAggregation($sumAgg);
                $agg->setMinimumDocumentCount(0);
            }

            $agg->setOrder('_term', 'asc');

            $query->addAggregation($agg);

        }

        $agg = new Elastica\Aggregation\Range('travellers');
        for ($i = 1; $i <= 6; $i++) {
            $agg->addRange($i);
        }

        $agg->setScript("doc['vehicleType.travellers'].value");
        // $agg->setScript("if (doc['vehicleType.travellers'].value == 1) { return doc['vehicleType.travellers'].value+'_'+doc['vehicleType.travellers'].value + '+ person'; } else { return doc['vehicleType.travellers'].value+'_'+doc['vehicleType.travellers'].value + '+ people'; }");

        $sumAgg = new Elastica\Aggregation\Sum('sum');
        $sumAgg->setField('nb_listings');
        $agg->addAggregation($sumAgg);
        $query->addAggregation($agg);


        $nestedAgg = new Elastica\Aggregation\Nested('custom_pickup', 'pickupLocation.custom_departures');
        $agg = new Elastica\Aggregation\Terms('pickup');

        $agg->setScript("doc['pickupLocation.custom_departures.slug'].value + '_' + doc['pickupLocation.custom_departures.name'].value");
        $agg->setOrder('_term', 'asc');

        $reverse = new Elastica\Aggregation\ReverseNested('sum');
        $sumAgg = new Elastica\Aggregation\Sum('sum');
        $sumAgg->setField('nb_listings');
        $reverse->addAggregation($sumAgg);

        $agg->addAggregation($reverse);
        $nestedAgg->addAggregation($agg);

        $query->addAggregation($nestedAgg);

        $nestedAgg = new Elastica\Aggregation\Nested('custom_dropoff', 'dropoffLocation.custom_departures');
        $agg = new Elastica\Aggregation\Terms('dropoff');
        $agg->setScript("doc['dropoffLocation.custom_departures.slug'].value + '_' + doc['dropoffLocation.custom_departures.name'].value");
        $agg->setOrder('_term', 'asc');

        $reverse = new Elastica\Aggregation\ReverseNested('sum');
        $sumAgg = new Elastica\Aggregation\Sum('sum');
        $sumAgg->setField('nb_listings');
        $reverse->addAggregation($sumAgg);

        $agg->addAggregation($reverse);

        $nestedAgg->addAggregation($agg);
        $query->addAggregation($nestedAgg);

        $agg = new Elastica\Aggregation\Sum('nb_listings');
        $agg->setField('nb_listings');

        $query->addAggregation($agg);
    }
}

