<?php

namespace Transfercar\Search;

use ArrayIterator;
use Doctrine;

class tcListingResults implements \IteratorAggregate
{

    private $iterator,
            $hasSponsoredAdsAtEnd = false;

    public function __construct($listings, $sponsoredAds, $sortEvently, $threshold)
    {
        $this->iterator = new ArrayIterator();

        Doctrine::getTable('AdListing')->incrementViewCounters($sponsoredAds);

        if ($sortEvently)
        {
            $this->sortSponsoredAdsEvenly($listings, $sponsoredAds, $threshold);
        }
        else
        {
            $this->sortSponsoredAdsEnd($listings, $sponsoredAds);
        }
    }

    public function getIterator()
    {
        return $this->iterator;
    }

    /**
     * Order relocations and sponsored ads evenly given a threshold parameter
     *
     * This allows to include sponsored ads in between relocation results
     *
     */
    public function sortSponsoredAdsEvenly($listings, $sponsoredAds, $threshold)
    {
        $nb = 0;
        foreach ($listings as $listing)
        {
            if ($nb > 0 && $nb % $threshold == 0 && count($sponsoredAds))
            {
                $this->iterator->append(new tcListingResult($sponsoredAds->getFirst()));
                $sponsoredAds->remove($sponsoredAds->key());
            }

            $this->iterator->append(new tcListingResult($listing));
            $nb++;
        }

        if (count($sponsoredAds))
        {
            $this->sortSponsoredAdsEnd(array(), $sponsoredAds);
        }
    }

    public function hasSponsoredAdsAtEnd()
    {
        return $this->hasSponsoredAdsAtEnd;
    }

    public function sortSponsoredAdsEnd($listings, $sponsoredAds)
    {
        foreach ($listings as $listing) {
            $this->iterator->append(new tcListingResult($listing));
        }

        $first = true;
        $this->hasSponsoredAdsAtEnd = true;
        foreach ($sponsoredAds as $sponsoredAd) {
            $this->iterator->append(new tcListingResult($sponsoredAd, $first));
            $first = false;
        }
    }
}