<?php

namespace Transfercar\Stripe;

use CCTransaction;
use Doctrine;
use Payments;
use CreditCard;

class TcStripeForm extends \BaseForm
{
    public function configure()
    {
        $this->setWidgets(array(
            'stripeToken' => new \sfWidgetFormInputHidden(),
            'save' => new \sfWidgetFormInputCheckbox(),
            'amount' => new \sfWidgetFormInputHidden(array(), array()),
            'receipt' => new \sfWidgetFormInputCheckbox()
        ));

        $this->widgetSchema->setNameFormat('payment[%s]');

        $this->widgetSchema->setLabels(array(
          'save' => 'Save this card for next time I credit my account',
          'amount' => 'Amount',
          'receipt' => 'Send payment receipt from payment operator to my e-mail'
        ));

        $this->setValidators(array(
            'stripeToken' => new \sfValidatorCallback(
                array(
                    'callback' => array($this, 'checkCreditCard'),
                    'required' => true,
                ), 
                array(
                    'invalid' => 'Your credit card was not approved. Please check your details again or try another credit card'
                )),
            'save' => new \sfValidatorPass(array('required' => false)),
            'amount' =>  new \sfValidatorPass(),//new \sfValidatorInteger(array('min' => 1), array('min' => 'Invalid value')),
            'receipt' => new \sfValidatorPass(array('required' => false))
        ));
    }


  public function checkCreditCard($validator, $token)
  {
      if ($token && $this->getOption('is_new_token', true)) 
      {
        $user_id = $this->getOption('user_id');
        $user = \Doctrine::getTable('User')->find($user_id);
        try {
          $token = $this->addStripeCard($user, $token);
        }
        catch (\Stripe\Error\Card $error) {
          $validator->setMessage('invalid', $error->getMessage());
          $error = new \sfValidatorError($validator, 'invalid');
          throw new \sfValidatorErrorSchema($validator, array('type' => $error));
        }
        catch (\Stripe\Error\Base $error) {
          $validator->setMessage('invalid', $error->getMessage());
          $error = new \sfValidatorError($validator, 'invalid');
          throw new \sfValidatorErrorSchema($validator, array('type' => $error));
        }
        catch (\Exception $error) {
          $validator->setMessage('invalid', $error->getMessage());
          $error = new \sfValidatorError($validator, 'invalid');
          throw new \sfValidatorErrorSchema($validator, array('type' => $error));
        }

        return $token;
      }

      return $token;
    }

    public function processPayment($description, $particular, $values, $card = null)
    {
        $stripe = new TcStripe();
        $is_error = false;
        try
        {
            $user_id = $this->getOption('user_id');
            $user = \Doctrine::getTable('User')->find($user_id);
            $card = $card ?: $this->saveCreditCard();
            $payment = $this->createPayment();
            $cct = $this->createTransaction($card->getId(), $payment->getId());
            $charge = $stripe->chargeCreditCard([
                'customer' => $user->get('stripe_token'),
                'source' => $values['stripeToken'],
                'amount' => $values['amount'] * 100,
                'description' => $description,
                'statement_descriptor'  => $particular,
                'currency'    => $this->getOption('currency')
            ]);
            $cct->status_message = $charge->description;
            $cct->unique_key = $charge->id;
            $cct->status = CCTransaction::STATUS_ACCEPTED;
            $cct->save();
            $payment->status = Payments::STATUS_ACCEPTED;
            $payment->save();
            return $cct;
        }  
        catch (\Stripe\Error\Card $ex)
        {
            $is_error = true;
            $cct->set('status', CCTransaction::STATUS_REJECTED);
            $cct->status_message = $ex->getMessage();
            $this->paymentError = $ex->getMessage();
            $payment->set('status', Payments::STATUS_REJECTED);
            $cct->save();
            $payment->save();
        }
        catch (\Stripe\Error\Base $ex)
        {   
            $is_error = true;
            $cct->set('status', CCTransaction::STATUS_REJECTED);
            $cct->status_message = $ex->getMessage();
            $this->paymentError = $ex->getMessage();
            $payment->set('status', Payments::STATUS_REJECTED);
            $cct->save();
            $payment->save();
        }

        return !$is_error;
    }

    public $paymentError;


    private function addStripeCard($user, $token)
    {
        $stripe = new TcStripe();
        $user_id = $this->getOption('user_id');
        $user = Doctrine::getTable('User')->find($user_id);

        if (!$user->get('stripe_token')) 
        {
            $customer = $stripe->createCustomer($user->get('email'), $user->full_name(), $user->get('is_owner') ? 'ro' : 'driver', $token);
            $user->set('stripe_token', $customer->id);
            $user->save();
            $card = $customer->sources->data[0];
        } 
        else 
        {
            $card = $stripe->addCard($user->get('stripe_token'), $token);
        }

        return $card->id;
    }

    public function saveCreditCard()
    {
        $card_data = $this->getValues();
        $user_id = $this->getOption('user_id');
        $user = Doctrine::getTable('User')->find($user_id);

        $stripe = new TcStripe();
        $card = $stripe->retrieveCard($user->get('stripe_token'), $card_data['stripeToken']);
        $id = '';
        if ($card instanceof \Stripe\Source){
            $id = $card->id;
            $card = $card->card;
        } else {
            $id = $card->id;
        }

        $card_save_number = $card->brand.' - **** ' . $card->last4;

        $ccard = new CreditCard();
        $ccard->card_type = 0;
        $ccard->expiry = null;
        $ccard->holder_name = $id;
        $ccard->number = $card_save_number;
        $ccard->csv = '';
        $ccard->owner_id = $user_id;
        $ccard->country = $card->country;
        $ccard->save();

        return $ccard;
    }


    private function createPayment()
    {
        $payment = new Payments();
        $payment->type = 'cc'; // credit card transaction
        $payment->date = date('Y-m-d H:i:s', time());
        $payment->amount = $this->getOption('credits');
        $payment->national_currency_amount = $this->getValue('amount');
        $payment->user_id = $this->getOption('user_id');
        $payment->status = Payments::STATUS_NEW;
        $payment->is_new_type_payment = "yes";
        $payment->country_id = $this->getOption('country_id');
        $payment->assigned_record_id = $this->getOption('user_id');
        $payment->save();

        return $payment;
    }

    private function createTransaction($card_id, $payment_id)
    {
        $cct = new CCTransaction();
        $cct->status = CCTransaction::STATUS_NEW;
        $cct->unique_key = implode('.', array($this->getOption('user_id'), $payment_id, time(), mt_rand()));
        $cct->credit_card_id = $card_id;
        $cct->payment_id = $payment_id;
        $cct->communication_log = array();
        $cct->save();

        return $cct;
    }
}
