<?php

namespace Transfercar\SwiftMailer;

// swiftmailer classes
use Swift_Events_ResponseListener;
use Swift_Events_SendListener;
use Swift_Events_ResponseEvent;
use Swift_Events_SendEvent;

use EmailLog;

class TcMailerPlugin implements Swift_Events_ResponseListener, Swift_Events_SendListener
{
    private $trackingId;

    public function responseReceived(Swift_Events_ResponseEvent $evt)
    {

        $response = $evt->getResponse();

        if (preg_match('/TrackingID:(.*)/', trim($response), $match)) {
            $this->trackingId = $match[1];
        }
    }

    /**
     * Not used.
     */
    public function beforeSendPerformed(Swift_Events_SendEvent $evt)
    {
    }

    /**
     * Invoked immediately after the Message is sent.
     *
     * @param Swift_Events_SendEvent $evt
     */
    public function sendPerformed(Swift_Events_SendEvent $evt)
    {
        $failures = array_flip($evt->getFailedRecipients());

        //write a log
        $elog = new EmailLog();

        $ids = $evt->getMessage()->getHeaders()->get('X-recipient-ids')->getValue();
        foreach (explode(',', $ids) as $id)
        {
            if ($id)
            {
                list($id, $address, $domain) = explode(':', $id);
                $elog->set('user_id', $id);
            }
            else
            {
                $domain = 'nz';
            }

            $elog->setTrackingId($this->trackingId);
            $elog->setDomain($domain);
            $elog->setSubject($evt->getMessage()->getSubject());
            $elog->setCreatedAt(date('Y-m-d H:i:s'));
            $elog->setStatus(array_key_exists($address, $failures) ? 'failed' : 'sent');
            $elog->save();

            $elog->free(true);
        }
    }
}