<?php

namespace Transfercar\User;

use sfEventDispatcher;
use sfStorage;
use sfEvent;
use sfConfig;
use sfLoader;

use DateTime;
use LocationCountry;

class TcFrontendUser extends TcBasicSecurityUser
{
	/*
    * Possible states of listing record
    */
	private $profile = null;
  private $country = null;
  private $forceShutdown = false;

  public function initialize(sfEventDispatcher $dispatcher, sfStorage $storage, $options = array())
  {

    if (sfConfig::get('app_default_culture') === 'en_US')
    {
      $options['default_culture'] = 'en_US';
    }

    parent::initialize($dispatcher, $storage, $options);
  }

	public function getAge($default = false)
	{
		$profile = $this->getAttribute("profile");
		if (!$profile) return $default;
        $birthDate = explode("-", $profile->date_of_birth);
        //get age from date or birthdate
        return (date("md", date("U", mktime(0, 0, 0, $birthDate[1], $birthDate[2], $birthDate[0]))) > date("md")
            ? ((date("Y") - $birthDate[0]) - 1)
            : (date("Y") - $birthDate[0]));
	}

  public function getId()
  {
    return $this->id;
  }

  public function getFullName()
  {
    $profile = $this->getAttribute("profile");
    return $profile ? $profile->getFullName() : null;
  }

	public function __get($name)
	{
		$profile = $this->getAttribute("profile", array());

		if (isset($profile->$name)) {
			return $profile->$name;
		}
		return null;
	}

  public function setCulture($culture)
  {
    if (true||$this->getUserDomain() == 'us') {
      if ($this->culture != $culture && $this->isAuthenticated()) {
        $profile = $this->getAttribute('profile');
        $profile->set('culture', $culture);
        $profile->save();
      }

      if ($this->culture && $this->culture != $culture) {
        $this->forceShutdown = true;
      }
    }

    parent::setCulture($culture);
  }

  public function getListingRequirements($listing)
  {
    if (!$this->isAuthenticated()) {
      return array();
    }

    $required = array();

    if($this->getAge() && $listing->get('min_age')) {
      if ($this->getAge() < $listing->get('min_age')) {
        $required[] = array("name" => "Minimum age", "value" => $listing->get('min_age'));
      }
    }

    $license = $listing->get('Owner')->get('license_type');
    $domain  = strtoupper($listing->getCountryDomain());

    $userLicense = strtoupper($this->license);
    $allowedLicenses = in_array($domain, array('NZ', 'AU')) ? array('NZ', 'AU') : array($domain);

    if ($userLicense && $license == 'full' && !in_array($userLicense, $allowedLicenses))
    {
      sfLoader::loadHelpers(array("Array"));
      $countries = COUNTRY_LIST();
      $value = '';
      if (in_array($domain, array('NZ', 'AU')))
      {
        $value =  "AU/NZ";
      }
      else{
        $value =  (isset($countries[$domain]) ? $countries[$domain] : $domain);
      }
      $required[] = array("name" => "Drivers license", "value" => $value);
    }

    return $required;
  }

  public function getCountry()
  {
    if (!$this->getAttribute('country'))
    {
      $this->checkCountry();
    }

    return $this->getAttribute('country');
  }

	/**
	 * Shortcut for flash messages
	 *
	 * @param string $message A message to display
	*/
  	public function flash($message)
    {
    	$this->setAttribute('flash_notice', $message);
    }


    public function checkCountry()
    {
      $x = isset($_SERVER['TRANSFER_CAR_COUNTRY']) ? $_SERVER['TRANSFER_CAR_COUNTRY'] : 'nz';
      $country = LocationCountry::retreiveByDomain($x);
	    $this->setAttribute("country", $country);
    }

  	public function getUserDomain()
    {
      if (!$this->getAttribute('country')) {
        $this->checkCountry();
      }

      if (!$this->getAttribute('domain')) {
        $country_domain = isset($_SERVER['TRANSFER_CAR_COUNTRY']) ? $_SERVER['TRANSFER_CAR_COUNTRY'] : 'nz';
        $this->setAttribute('domain', $country_domain);
      }

  		return $this->getAttribute('domain');
  	}

  	public function getUserSettings(){
  		$country_cnt = count(LocationCountry::getAll());

  		$user_settings = $this->getAttribute('profile')->UserSettings;
  		$user_settings = $user_settings->toArray();
  		$user_settings_country = array();
  		if (count($user_settings) > 0){
  			foreach ($user_settings as $item){
  				$user_settings_country[$item["setting"]][] = $item["country_id"];
  			}

  			foreach ($user_settings_country as $key => $item){
  				if (count($item) == $country_cnt) $user_settings_country[$key] = "all";
  				else $user_settings_country[$key] = $item[0];
  			}
  		}
		return $user_settings_country;
  	}

    public function login($user)
    {
      $this->setAttribute('profile', $user);

      // users can be logged in without a verified profile in the request process
      // So we only set the login date once the user has activated its account
      if ($user->isVerified())
      {
        $this->addNewTcFlash($user);
        $user->last_login = date('Y-m-d H:i:s', time());
        $user->save();
      }

      $this->setAuthenticated(true);
      $this->setAttribute('profile', $user);

      if (false && $user->get('culture')) {
        parent::setCulture($user->get('culture'));
      }

      $this->dispatcher->notify(new sfEvent($user, 'driver.logged_in'));
      $this->pushFlash('ga_event', array(
          'event'           => 'gaTriggerEvent',
          'gaEventCategory' => 'User',
          'gaEventAction'   => 'Login'
      ));
    }

    public function impersonate($user, $adminUser)
    {
        $this->login($user);
        $this->addCredentials('IS_IMPERSONATED');
        $this->setAttribute('admin_id', $adminUser->getId());
        $this->setAttribute('admin_name', $adminUser->get('name'));
    }

    private function addNewTcFlash($user)
    {
      if ($this->getUserDomain() == 'au') {
        return;
      }

      $last_login = new DateTime($user->get('last_login') ?: '2024-03-04');
      $new_tc_date = new DateTime('2024-03-04');
      if ($last_login < $new_tc_date) {
        $this->setFlash('new_tc', true);
      }
    }

    public function pushFlash($name, $value)
    {
      if (!($values = $this->getFlash($name)))
      {
        $values = array();
      }

      $values[] = $value;

      return $this->setFlash($name, $values);
    }

    public function logout()
    {
      $this->setAuthenticated(false);
      $this->getAttributeHolder()->clear();
      $this->forceShutdown = true;
    }

    public function shutdown($forceShutdown = false)
    {
      if ($this->forceShutdown || $forceShutdown || $this->getCulture() == 'fr_CA' || $this->getCulture() == 'zh' ||  $this->isAuthenticated() || $this->attributeHolder->getNames('symfony/user/sfUser/flash')) {
        parent::shutdown();
      }
    }
}
